#!/usr/bin/env sage
##################################################################
#
# example.sage -- an example SAGE script.
# 
# Example SAGE script.  You can use this example script in many
# different ways.
#
#  (*) sage example.sg
#  (*) ./example.sg  (assuming the SAGE directory is in your path)
#  (*) From within sage type
#                  load 'example.sg'
#  (*) From within sage type
#                  attach 'example.sg'
#      With attach, when you enter a blank line in SAGE, if 
#      example.sg has changed it is reloaded.
#
# As a tutorial and test of your install, you might want to
# try all these now. 
#
##################################################################

x,y,a,m = var('x,y,a,m')
psi = x*(1-x)*y*(1-y)
#psi = (y*(pi - y)*sin(x))^2
u = [psi.diff(y), - psi.diff(x)]
lap_u = [u[0].diff(x,2),u[1].diff(y,2)]
U_lapU = [u[i] - a*lap_u[i] for i in range(len(u))]
z = U_lapU[1].diff(x) - U_lapU[0].diff(y)

p = x*y
#p = cos(x)*cos(2*y)
grad_p = [p.diff(x),p.diff(y)]
f = [-m*lap_u[0] + grad_p[0] - z*u[1],
     -m*lap_u[1] + grad_p[1] + z*u[0] ]

T_LHS = (m*z + a*(u[0]*z.diff(x) + u[1]*z.diff(y))).simplify_exp()
T_RHS = (a*(f[1].diff(x) - f[0].diff(y)) + m*(u[1].diff(x) - u[0].diff(y))).simplify_exp()

print "psi:", psi
print "u:"
print u
print "div u:", u[0].diff(x) + u[1].diff(y)
print "z:"
print [z.simplify_exp()]
print "lap_u:"
print [lap_u[i].simplify_exp() for i in range(len(lap_u))]
print "p"
print [p]
print "grad_p"
print grad_p
print "T_LHS: ", [T_LHS]
print "T_RHS: ",[T_RHS]
print "T_LHS - T_RHS",[T_LHS.simplify_exp() - T_RHS.simplify_exp()]

###############################################################################
## Output
# psi: 
#                                 2             2  2
#                              sin (x) ( pi - y)  y
# u:
# [2*sin(x)^2*(pi - y)^2*y - 2*sin(x)^2*(pi - y)*y^2, 
#  -2*cos(x)*sin(x)*(pi - y)^2*y^2]
# div u: 
#                                        0
# z:
# [-a*(4*sin(x)^2*y^2 - 4*cos(x)^2*y^2 - 16*sin(x)^2*(pi - y)*y + 
#  16*cos(x)^2*(pi - y)*y + 4*sin(x)^2*(pi - y)^2 - 4*cos(x)^2*(pi - y)^2) + 
#  a*(-4*sin(x)^2*y^2 + 4*cos(x)^2*y^2 + 16*sin(x)^2*(pi - y)*y - 
#  16*cos(x)^2*(pi - y)*y - 4*sin(x)^2*(pi - y)^2 + 4*cos(x)^2*(pi - y)^2) + 
#  2*sin(x)^2*(pi - y)^2*y^2 - 2*cos(x)^2*(pi - y)^2*y^2 - 2*sin(x)^2*y^2 + 
#  8*sin(x)^2*(pi - y)*y - 2*sin(x)^2*(pi - y)^2]
# lap_u:
# [4*sin(x)^2*(pi - y)*y^2 - 4*cos(x)^2*(pi - y)*y^2 - 4*sin(x)^2*(pi - y)^2*y + 4*cos(x)^2*(pi - y)^2*y, 
# -4*cos(x)*sin(x)*y^2 + 16*cos(x)*sin(x)*(pi - y)*y - 4*cos(x)*sin(x)*(pi - y)^2]
# p
# [cos(x)*cos(2*y)]
# grad_p
# [-sin(x)*cos(2*y), -2*cos(x)*sin(2*y)]
#
#
#
#
# psi: 
#                                       3  3
#                                      x  y
# u:
# [3*x^3*y^2, -3*x^2*y^3]
# div u: 
#                                        0
# z:
# [-6*x*y^3 - 6*x^3*y + 72*a*x*y]
# lap_u:
# [18*x*y^2, -18*x^2*y]
# p
# [x*y]
# grad_p
# [y, x]
