// Copyright (C) 2008 Andy R Terrel
// Licensed under the GNU LGPL Version 3.0 or later.

#ifndef __FLUID_H
#define __FLUID_H

#include <dolfin.h>

namespace dolfin
{

  typedef unsigned int uint;

  class Fluid
  {
  public:
    Fluid (Mesh& mesh);

    Fluid (Mesh& mesh,MeshFunction<uint>& vel_subdomains, 
	   Array< Function* >& vel_bc_funcs);
      
    void toFile (std::string base_filename, bool print_pressure=true);

    inline Mesh& mesh () {return _mesh;}
    inline Function& velocity () {return _velocity;}
    inline Function& pressure () {return _pressure;}
    inline MeshFunction<uint>* vel_subdomains () {return _vel_subdomains;}     
    inline Array< Function* > vel_bc_funcs () {return _vel_bc_funcs;}     

  protected:
    Mesh& _mesh;
    MeshFunction<uint>* _vel_subdomains;
    Array< Function* > _vel_bc_funcs;
    Function _velocity;
    Function _pressure;
  };

  // Classes for marking pressure conditions (if necessary)
  class PinPoint : public SubDomain
  {
  public:
    PinPoint () : SubDomain(), pinpoint_set(false)
    { 
      pinpoint[0] = 0.0; pinpoint[1] = 0.0;
    }
     
    bool inside (const real *x, bool on_boundary) const
    {
      if (pinpoint_set) {
	return (fabs(x[1] - pinpoint[1]) < DOLFIN_EPS) && 
	       (fabs(x[0] - pinpoint[0]) < DOLFIN_EPS);
      } else {
	pinpoint_set = true;
	pinpoint[0] = x[0];
	pinpoint[1] = x[1];
	return true;
      }
    }
  private:
    mutable bool pinpoint_set;
    mutable real pinpoint[2];
  };

  class PressurePin : public Function
  {
  public:
    
    PressurePin (Mesh& mesh) : Function(mesh) {}
    
    real eval (const real* x) const
    {
    return 0.0;
    }
  };

}

#endif
