// Copyright (C) 2008 Andy R Terrel
// Licensed under the GNU LGPL Version 3.0 or later.

#ifndef __GRADE2SOLVER_H
#define __GRADE2SOLVER_H

#include <dolfin.h>
#include "Fluid.h"
#include "StokesSolver.h"

#include "forms/grade2/TaylorHood/P2THG2Vel.h"
#include "forms/grade2/TaylorHood/P2THG2GalZ.h"
#include "forms/grade2/TaylorHood/P2THG2SymZ.h"
#include "forms/grade2/TaylorHood/P2THG2StrZ.h"
#include "forms/grade2/TaylorHood/P2THG2SymStrZ.h"
#include "forms/grade2/TaylorHood/P2THG2RegZ.h"

#include "forms/grade2/ScottVogelius/P4SVG2Vel.h"
#include "forms/grade2/ScottVogelius/P4SVG2GalZ.h"
#include "forms/grade2/ScottVogelius/P4SVG2SymZ.h"
#include "forms/grade2/ScottVogelius/P4SVG2StrZ.h"
#include "forms/grade2/ScottVogelius/P4SVG2SymStrZ.h"
#include "forms/grade2/ScottVogelius/P4SVG2RegZ.h"

#include "forms/norms/P2VecL2Diff.h"
#include "forms/norms/P2L2Diff.h"
#include "forms/norms/P2VecL2.h"
#include "forms/norms/P2L2.h"
#include "forms/norms/P4VecL2Diff.h"
#include "forms/norms/P4L2Diff.h"
#include "forms/norms/P4VecL2.h"
#include "forms/norms/P4L2.h"


#include "forms/projections/P4VecDiff.h"
#include "forms/projections/P4Diff.h"
#include "forms/projections/P2VecDiff.h"
#include "forms/projections/P2Diff.h"
#include "forms/projections/P1Diff.h"

namespace dolfin
{

  class Grade2Solver : public StokesSolver
  {
  public:

    Grade2Solver ();

    Grade2Solver (const Grade2Solver& that);

    ~Grade2Solver ();

    Grade2Solver& operator= (const Grade2Solver& that);

    void solve (Fluid& fluid, Function& f, bool verbose=false);

    void solve (Fluid& fluid, Function& f,
		Function& v_ex, Function& p_ex, Function& z_ex);

  private:
    // Managed pointers
    Function* _z;
    Function* f_alpha;
    Function* f_mu;
    Function* f_delta;
    Function* f_epsilon;
    FacetNormal* f_norm;
    Function* f_rho;
    Function* f_w;
    MeshSize* f_h;

    Form* _a_transport;
    Form* _L_transport;

    // Unmanaged pointers
    Function* _v_ex;
    Function* _p_ex;
    Function* _z_ex;

    // Private functions
    void deletePtrs ();
    void formFactory (Fluid& fluid, Function& f);


  };
}

#endif
