// Copyright (C) 2008 Andy R Terrel
// Licensed under the GNU LGPL Version 3.0 or later.

#ifndef __STOKESSOLVER_H
#define __STOKESSOLVER_H

#include <dolfin.h>
#include "Fluid.h"
#include "forms/Stokes/TaylorHood/P2THStokes.h"
#include "forms/Stokes/ScottVogelius/P4SVStokes.h"
#include "forms/norms/P4VecDivL2.h"
#include "forms/projections/P4VecDiv.h"

namespace dolfin
{

  class StokesSolver : public Parametrized
  {
  public:

    StokesSolver ();

    StokesSolver (const StokesSolver& that);

    ~StokesSolver ();
    
    StokesSolver& operator= (const StokesSolver& that);

    virtual void solve (Fluid& fluid, Function& f);

  protected:
    Form* _a_stokes;
    Form* _L_stokes;
    Form* _a_div_proj;
    Form* _L_div_proj;

    virtual void deletePtrs ();

    virtual void destroyBCs (Array<DirichletBC*>& bcs);

    virtual void createBCs (Fluid& fluid, Array<DirichletBC*>& bcs);

    void mixedMethodSolve (Fluid& fluid, Array<DirichletBC *>& bcs,
			   Function& f);

    void scottVogeliusSolve (Fluid& fluid, Array<DirichletBC *>& bcs,
			     Function& f, Function& f_w);
    
  private:
    SubSystem _vel_subsystem;
    SubSystem _pres_subsystem;
    PinPoint* _pinpoint;
    PressurePin* _pressurepin;
  };
}

#endif
