#include "P4VecDivL2.h"
/// Constructor
UFC_P4VecDivL2Functional_finite_element_0_0::UFC_P4VecDivL2Functional_finite_element_0_0() : ufc::finite_element()
{
    // Do nothing
}

/// Destructor
UFC_P4VecDivL2Functional_finite_element_0_0::~UFC_P4VecDivL2Functional_finite_element_0_0()
{
    // Do nothing
}

/// Return a string identifying the finite element
const char* UFC_P4VecDivL2Functional_finite_element_0_0::signature() const
{
    return "Lagrange finite element of degree 4 on a triangle";
}

/// Return the cell shape
ufc::shape UFC_P4VecDivL2Functional_finite_element_0_0::cell_shape() const
{
    return ufc::triangle;
}

/// Return the dimension of the finite element function space
unsigned int UFC_P4VecDivL2Functional_finite_element_0_0::space_dimension() const
{
    return 15;
}

/// Return the rank of the value space
unsigned int UFC_P4VecDivL2Functional_finite_element_0_0::value_rank() const
{
    return 0;
}

/// Return the dimension of the value space for axis i
unsigned int UFC_P4VecDivL2Functional_finite_element_0_0::value_dimension(unsigned int i) const
{
    return 1;
}

/// Evaluate basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_basis(unsigned int i,
                                   double* values,
                                   const double* coordinates,
                                   const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_4_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
    const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
    const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
    const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
    const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
    const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
    const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
    const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
    const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
    // Table(s) of coefficients
    const static double coefficients0[15][15] = \
    {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
    {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
    {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
    {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
    {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
    {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
    {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14;
}

/// Evaluate all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_basis_all(double* values,
                                       const double* coordinates,
                                       const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
}

/// Evaluate order n derivatives of basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_basis_derivatives(unsigned int i,
                                               unsigned int n,
                                               double* values,
                                               const double* coordinates,
                                               const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_4_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
    const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
    const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
    const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
    const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
    const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
    const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
    const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
    const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
    // Table(s) of coefficients
    const static double coefficients0[15][15] = \
    {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
    {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
    {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
    {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
    {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
    {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
    {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[15][15] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.89897948556635, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 9.48683298050513, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4, 0, 7.07106781186548, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.29150262212918, 0, -2.99332590941916, 13.6626010212795, 0, 0.611010092660777, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.38178046004132, 0, 0, 12.5219806739988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.46410161513776, 0, 7.83836717690617, 0, 0, 8.40000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 10.9544511501033, 0, 0, -3.83325938999964, 0, 17.7482393492988, 0, 0.553283335172495, 0, 0, 0, 0, 0, 0},
    {4.73286382647969, 0, 3.3466401061363, 4.36435780471985, -1.12389939329608e-14, -5.07468037933239, 0, 17.0084012854152, 0, 1.52127765851133, 0, 0, 0, 0, 0},
    {0, 2.44948974278318, 0, 0, 9.14285714285716, 0, 0, 0, 14.8461497791618, 0, 0, 0, 0, 0, 0},
    {3.09838667696593, 0, 7.66811580507233, 0, 0, 10.733126291999, 0, 0, 0, 9.2951600308978, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[15][15] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.44948974278318, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.24264068711928, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 4.74341649025257, -0.912870929175277, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2, 6.12372435695794, 3.53553390593274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.3094010767585, 0, 8.16496580927726, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.64575131106459, 5.18459255872629, -1.49666295470958, 6.83130051063973, -1.05830052442584, 0.305505046330385, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.23606797749979, 2.19089023002066, 2.5298221281347, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.73205080756888, -5.09116882454314, 3.91918358845309, 0, 9.69948452238573, 4.20000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.00000000000001, 0, -2.82842712474619, 0, 0, 12.1243556529821, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.68328157299975, 5.47722557505166, -1.89736659610103, 7.4230748895809, -1.91662969499982, 0.663940002206988, 8.87411967464942, -1.07142857142857, 0.276641667586247, -0.0958314847499899, 0, 0, 0, 0, 0},
    {2.36643191323985, 2.89827534923789, 1.67332005306815, 2.18217890235992, 5.74704893215391, -2.53734018966619, 10.0623058987491, 8.50420064270761, -2.1957751641342, 0.760638829255664, 0, 0, 0, 0, 0},
    {2, 1.22474487139159, 3.53553390593274, -7.37711113563318, 4.57142857142858, 1.64957219768463, 0, 11.4997781699989, 7.4230748895809, -2.57142857142857, 0, 0, 0, 0, 0},
    {1.54919333848296, 6.6407830863536, 3.83405790253616, 0, -6.19677335393187, 5.36656314599949, 0, 0, 13.4164078649987, 4.6475800154489, 0, 0, 0, 0, 0},
    {-3.57770876399967, 0, 8.85437744847147, 0, -1.10153547165223e-14, -3.09838667696593, 0, 0, 0, 16.0996894379985, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
}

/// Evaluate order n derivatives of all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_basis_derivatives_all(unsigned int n,
                                                   double* values,
                                                   const double* coordinates,
                                                   const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
}

/// Evaluate linear functional for dof i on the function f
double UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_dof(unsigned int i,
                                   const ufc::function& f,
                                   const ufc::cell& c) const
{
    // The reference points, direction and weights:
    const static double X[15][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.75, 0.25}}, {{0.5, 0.5}}, {{0.25, 0.75}}, {{0, 0.25}}, {{0, 0.5}}, {{0, 0.75}}, {{0.25, 0}}, {{0.5, 0}}, {{0.75, 0}}, {{0.25, 0.25}}, {{0.5, 0.25}}, {{0.25, 0.5}}};
    const static double W[15][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[15][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
}

/// Evaluate linear functionals for all dofs on the function f
void UFC_P4VecDivL2Functional_finite_element_0_0::evaluate_dofs(double* values,
                                  const ufc::function& f,
                                  const ufc::cell& c) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Interpolate vertex values from dof values
void UFC_P4VecDivL2Functional_finite_element_0_0::interpolate_vertex_values(double* vertex_values,
                                              const double* dof_values,
                                              const ufc::cell& c) const
{
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
}

/// Return the number of sub elements (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_finite_element_0_0::num_sub_elements() const
{
    return 1;
}

/// Create a new finite element for sub element i (for a mixed element)
ufc::finite_element* UFC_P4VecDivL2Functional_finite_element_0_0::create_sub_element(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_finite_element_0_0();
}


/// Constructor
UFC_P4VecDivL2Functional_finite_element_0_1::UFC_P4VecDivL2Functional_finite_element_0_1() : ufc::finite_element()
{
    // Do nothing
}

/// Destructor
UFC_P4VecDivL2Functional_finite_element_0_1::~UFC_P4VecDivL2Functional_finite_element_0_1()
{
    // Do nothing
}

/// Return a string identifying the finite element
const char* UFC_P4VecDivL2Functional_finite_element_0_1::signature() const
{
    return "Lagrange finite element of degree 4 on a triangle";
}

/// Return the cell shape
ufc::shape UFC_P4VecDivL2Functional_finite_element_0_1::cell_shape() const
{
    return ufc::triangle;
}

/// Return the dimension of the finite element function space
unsigned int UFC_P4VecDivL2Functional_finite_element_0_1::space_dimension() const
{
    return 15;
}

/// Return the rank of the value space
unsigned int UFC_P4VecDivL2Functional_finite_element_0_1::value_rank() const
{
    return 0;
}

/// Return the dimension of the value space for axis i
unsigned int UFC_P4VecDivL2Functional_finite_element_0_1::value_dimension(unsigned int i) const
{
    return 1;
}

/// Evaluate basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_basis(unsigned int i,
                                   double* values,
                                   const double* coordinates,
                                   const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_4_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
    const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
    const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
    const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
    const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
    const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
    const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
    const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
    const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
    // Table(s) of coefficients
    const static double coefficients0[15][15] = \
    {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
    {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
    {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
    {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
    {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
    {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
    {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    const double coeff0_10 = coefficients0[dof][10];
    const double coeff0_11 = coefficients0[dof][11];
    const double coeff0_12 = coefficients0[dof][12];
    const double coeff0_13 = coefficients0[dof][13];
    const double coeff0_14 = coefficients0[dof][14];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14;
}

/// Evaluate all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_basis_all(double* values,
                                       const double* coordinates,
                                       const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
}

/// Evaluate order n derivatives of basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_basis_derivatives(unsigned int i,
                                               unsigned int n,
                                               double* values,
                                               const double* coordinates,
                                               const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
    const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
    const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
    const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
    const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
    const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
    const double psitilde_bs_2_0 = 1;
    const double psitilde_bs_2_1 = 3.5*y + 2.5;
    const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
    const double psitilde_bs_3_0 = 1;
    const double psitilde_bs_3_1 = 4.5*y + 3.5;
    const double psitilde_bs_4_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
    const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
    const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
    const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
    const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
    const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
    const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
    const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
    const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
    const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
    const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
    const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
    const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
    const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
    const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
    // Table(s) of coefficients
    const static double coefficients0[15][15] = \
    {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
    {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
    {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
    {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
    {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
    {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
    {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
    {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
    {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
    {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
    {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
    {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
    {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[15][15] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.89897948556635, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 9.48683298050513, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4, 0, 7.07106781186548, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.29150262212918, 0, -2.99332590941916, 13.6626010212795, 0, 0.611010092660777, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 4.38178046004132, 0, 0, 12.5219806739988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.46410161513776, 0, 7.83836717690617, 0, 0, 8.40000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 10.9544511501033, 0, 0, -3.83325938999964, 0, 17.7482393492988, 0, 0.553283335172495, 0, 0, 0, 0, 0, 0},
    {4.73286382647969, 0, 3.3466401061363, 4.36435780471985, -1.12389939329608e-14, -5.07468037933239, 0, 17.0084012854152, 0, 1.52127765851133, 0, 0, 0, 0, 0},
    {0, 2.44948974278318, 0, 0, 9.14285714285716, 0, 0, 0, 14.8461497791618, 0, 0, 0, 0, 0, 0},
    {3.09838667696593, 0, 7.66811580507233, 0, 0, 10.733126291999, 0, 0, 0, 9.2951600308978, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[15][15] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.44948974278318, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.24264068711928, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.58198889747161, 4.74341649025257, -0.912870929175277, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2, 6.12372435695794, 3.53553390593274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {-2.3094010767585, 0, 8.16496580927726, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.64575131106459, 5.18459255872629, -1.49666295470958, 6.83130051063973, -1.05830052442584, 0.305505046330385, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.23606797749979, 2.19089023002066, 2.5298221281347, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.73205080756888, -5.09116882454314, 3.91918358845309, 0, 9.69948452238573, 4.20000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.00000000000001, 0, -2.82842712474619, 0, 0, 12.1243556529821, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.68328157299975, 5.47722557505166, -1.89736659610103, 7.4230748895809, -1.91662969499982, 0.663940002206988, 8.87411967464942, -1.07142857142857, 0.276641667586247, -0.0958314847499899, 0, 0, 0, 0, 0},
    {2.36643191323985, 2.89827534923789, 1.67332005306815, 2.18217890235992, 5.74704893215391, -2.53734018966619, 10.0623058987491, 8.50420064270761, -2.1957751641342, 0.760638829255664, 0, 0, 0, 0, 0},
    {2, 1.22474487139159, 3.53553390593274, -7.37711113563318, 4.57142857142858, 1.64957219768463, 0, 11.4997781699989, 7.4230748895809, -2.57142857142857, 0, 0, 0, 0, 0},
    {1.54919333848296, 6.6407830863536, 3.83405790253616, 0, -6.19677335393187, 5.36656314599949, 0, 0, 13.4164078649987, 4.6475800154489, 0, 0, 0, 0, 0},
    {-3.57770876399967, 0, 8.85437744847147, 0, -1.10153547165223e-14, -3.09838667696593, 0, 0, 0, 16.0996894379985, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    double coeff0_10 = 0;
    double coeff0_11 = 0;
    double coeff0_12 = 0;
    double coeff0_13 = 0;
    double coeff0_14 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    double new_coeff0_10 = 0;
    double new_coeff0_11 = 0;
    double new_coeff0_12 = 0;
    double new_coeff0_13 = 0;
    double new_coeff0_14 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
      new_coeff0_10 = coefficients0[dof][10];
      new_coeff0_11 = coefficients0[dof][11];
      new_coeff0_12 = coefficients0[dof][12];
      new_coeff0_13 = coefficients0[dof][13];
      new_coeff0_14 = coefficients0[dof][14];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
        coeff0_10 = new_coeff0_10;
        coeff0_11 = new_coeff0_11;
        coeff0_12 = new_coeff0_12;
        coeff0_13 = new_coeff0_13;
        coeff0_14 = new_coeff0_14;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9];
          new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10];
          new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11];
          new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12];
          new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13];
          new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9];
          new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10];
          new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11];
          new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12];
          new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13];
          new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
}

/// Evaluate order n derivatives of all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_basis_derivatives_all(unsigned int n,
                                                   double* values,
                                                   const double* coordinates,
                                                   const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
}

/// Evaluate linear functional for dof i on the function f
double UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_dof(unsigned int i,
                                   const ufc::function& f,
                                   const ufc::cell& c) const
{
    // The reference points, direction and weights:
    const static double X[15][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.75, 0.25}}, {{0.5, 0.5}}, {{0.25, 0.75}}, {{0, 0.25}}, {{0, 0.5}}, {{0, 0.75}}, {{0.25, 0}}, {{0.5, 0}}, {{0.75, 0}}, {{0.25, 0.25}}, {{0.5, 0.25}}, {{0.25, 0.5}}};
    const static double W[15][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[15][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
}

/// Evaluate linear functionals for all dofs on the function f
void UFC_P4VecDivL2Functional_finite_element_0_1::evaluate_dofs(double* values,
                                  const ufc::function& f,
                                  const ufc::cell& c) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Interpolate vertex values from dof values
void UFC_P4VecDivL2Functional_finite_element_0_1::interpolate_vertex_values(double* vertex_values,
                                              const double* dof_values,
                                              const ufc::cell& c) const
{
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
}

/// Return the number of sub elements (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_finite_element_0_1::num_sub_elements() const
{
    return 1;
}

/// Create a new finite element for sub element i (for a mixed element)
ufc::finite_element* UFC_P4VecDivL2Functional_finite_element_0_1::create_sub_element(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_finite_element_0_1();
}


/// Constructor
UFC_P4VecDivL2Functional_finite_element_0::UFC_P4VecDivL2Functional_finite_element_0() : ufc::finite_element()
{
    // Do nothing
}

/// Destructor
UFC_P4VecDivL2Functional_finite_element_0::~UFC_P4VecDivL2Functional_finite_element_0()
{
    // Do nothing
}

/// Return a string identifying the finite element
const char* UFC_P4VecDivL2Functional_finite_element_0::signature() const
{
    return "Mixed finite element: [Lagrange finite element of degree 4 on a triangle, Lagrange finite element of degree 4 on a triangle]";
}

/// Return the cell shape
ufc::shape UFC_P4VecDivL2Functional_finite_element_0::cell_shape() const
{
    return ufc::triangle;
}

/// Return the dimension of the finite element function space
unsigned int UFC_P4VecDivL2Functional_finite_element_0::space_dimension() const
{
    return 30;
}

/// Return the rank of the value space
unsigned int UFC_P4VecDivL2Functional_finite_element_0::value_rank() const
{
    return 1;
}

/// Return the dimension of the value space for axis i
unsigned int UFC_P4VecDivL2Functional_finite_element_0::value_dimension(unsigned int i) const
{
    return 2;
}

/// Evaluate basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0::evaluate_basis(unsigned int i,
                                   double* values,
                                   const double* coordinates,
                                   const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    
    if (0 <= i && i <= 14)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
      const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
      const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
      const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
      const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
      const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
      const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
      const double psitilde_bs_2_0 = 1;
      const double psitilde_bs_2_1 = 3.5*y + 2.5;
      const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
      const double psitilde_bs_3_0 = 1;
      const double psitilde_bs_3_1 = 4.5*y + 3.5;
      const double psitilde_bs_4_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
      const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
      const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
      const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
      const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
      const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
      const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
      const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
      const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
      const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
      const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
      const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
      const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
      const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
      const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
      // Table(s) of coefficients
      const static double coefficients0[15][15] =   \
      {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
      {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
      {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
      {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
      {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
      {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
      {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
      {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
      {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
      {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
      {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff0_4 =   coefficients0[dof][4];
      const double coeff0_5 =   coefficients0[dof][5];
      const double coeff0_6 =   coefficients0[dof][6];
      const double coeff0_7 =   coefficients0[dof][7];
      const double coeff0_8 =   coefficients0[dof][8];
      const double coeff0_9 =   coefficients0[dof][9];
      const double coeff0_10 =   coefficients0[dof][10];
      const double coeff0_11 =   coefficients0[dof][11];
      const double coeff0_12 =   coefficients0[dof][12];
      const double coeff0_13 =   coefficients0[dof][13];
      const double coeff0_14 =   coefficients0[dof][14];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14;
    }
    
    if (15 <= i && i <= 29)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 15;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
      const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
      const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
      const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
      const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
      const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
      const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
      const double psitilde_bs_2_0 = 1;
      const double psitilde_bs_2_1 = 3.5*y + 2.5;
      const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
      const double psitilde_bs_3_0 = 1;
      const double psitilde_bs_3_1 = 4.5*y + 3.5;
      const double psitilde_bs_4_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
      const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
      const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
      const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
      const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
      const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
      const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
      const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
      const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
      const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
      const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
      const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
      const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
      const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
      const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
      // Table(s) of coefficients
      const static double coefficients0[15][15] =   \
      {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
      {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
      {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
      {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
      {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
      {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
      {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
      {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
      {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
      {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
      {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff0_4 =   coefficients0[dof][4];
      const double coeff0_5 =   coefficients0[dof][5];
      const double coeff0_6 =   coefficients0[dof][6];
      const double coeff0_7 =   coefficients0[dof][7];
      const double coeff0_8 =   coefficients0[dof][8];
      const double coeff0_9 =   coefficients0[dof][9];
      const double coeff0_10 =   coefficients0[dof][10];
      const double coeff0_11 =   coefficients0[dof][11];
      const double coeff0_12 =   coefficients0[dof][12];
      const double coeff0_13 =   coefficients0[dof][13];
      const double coeff0_14 =   coefficients0[dof][14];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9 + coeff0_10*basisvalue10 + coeff0_11*basisvalue11 + coeff0_12*basisvalue12 + coeff0_13*basisvalue13 + coeff0_14*basisvalue14;
    }
    
}

/// Evaluate all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0::evaluate_basis_all(double* values,
                                       const double* coordinates,
                                       const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
}

/// Evaluate order n derivatives of basis function i at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0::evaluate_basis_derivatives(unsigned int i,
                                               unsigned int n,
                                               double* values,
                                               const double* coordinates,
                                               const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
      
    // Compute determinant of Jacobian
    const double detJ = J_00*J_11 - J_01*J_10;
    
    // Compute inverse of Jacobian
    
    // Get coordinates and map to the reference (UFC) element
    double x = (element_coordinates[0][1]*element_coordinates[2][0] -\
                element_coordinates[0][0]*element_coordinates[2][1] +\
                J_11*coordinates[0] - J_01*coordinates[1]) / detJ;
    double y = (element_coordinates[1][1]*element_coordinates[0][0] -\
                element_coordinates[1][0]*element_coordinates[0][1] -\
                J_10*coordinates[0] + J_00*coordinates[1]) / detJ;
    
    // Map coordinates to the reference square
    if (std::abs(y - 1.0) < 1e-14)
      x = -1.0;
    else
      x = 2.0 *x/(1.0 - y) - 1.0;
    y = 2.0*y - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 2;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 1)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[2][2] =  {{J_11 / detJ, -J_01 / detJ}, {-J_10 / detJ, J_00 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 2*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 14)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
      const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
      const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
      const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
      const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
      const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
      const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
      const double psitilde_bs_2_0 = 1;
      const double psitilde_bs_2_1 = 3.5*y + 2.5;
      const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
      const double psitilde_bs_3_0 = 1;
      const double psitilde_bs_3_1 = 4.5*y + 3.5;
      const double psitilde_bs_4_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
      const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
      const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
      const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
      const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
      const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
      const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
      const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
      const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
      const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
      const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
      const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
      const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
      const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
      const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
      // Table(s) of coefficients
      const static double coefficients0[15][15] =   \
      {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
      {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
      {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
      {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
      {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
      {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
      {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
      {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
      {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
      {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
      {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[15][15] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.89897948556635, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 9.48683298050513, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4, 0, 7.07106781186548, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.29150262212918, 0, -2.99332590941916, 13.6626010212795, 0, 0.611010092660777, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 4.38178046004132, 0, 0, 12.5219806739988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.46410161513776, 0, 7.83836717690617, 0, 0, 8.40000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 10.9544511501033, 0, 0, -3.83325938999964, 0, 17.7482393492988, 0, 0.553283335172495, 0, 0, 0, 0, 0, 0},
      {4.73286382647969, 0, 3.3466401061363, 4.36435780471985, -1.12389939329608e-14, -5.07468037933239, 0, 17.0084012854152, 0, 1.52127765851133, 0, 0, 0, 0, 0},
      {0, 2.44948974278318, 0, 0, 9.14285714285716, 0, 0, 0, 14.8461497791618, 0, 0, 0, 0, 0, 0},
      {3.09838667696593, 0, 7.66811580507233, 0, 0, 10.733126291999, 0, 0, 0, 9.2951600308978, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats1[15][15] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.44948974278318, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.24264068711928, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.58198889747161, 4.74341649025257, -0.912870929175277, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2, 6.12372435695794, 3.53553390593274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {-2.3094010767585, 0, 8.16496580927726, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.64575131106459, 5.18459255872629, -1.49666295470958, 6.83130051063973, -1.05830052442584, 0.305505046330385, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.23606797749979, 2.19089023002066, 2.5298221281347, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0, 9.69948452238573, 4.20000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.00000000000001, 0, -2.82842712474619, 0, 0, 12.1243556529821, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.68328157299975, 5.47722557505166, -1.89736659610103, 7.4230748895809, -1.91662969499982, 0.663940002206988, 8.87411967464942, -1.07142857142857, 0.276641667586247, -0.0958314847499899, 0, 0, 0, 0, 0},
      {2.36643191323985, 2.89827534923789, 1.67332005306815, 2.18217890235992, 5.74704893215391, -2.53734018966619, 10.0623058987491, 8.50420064270761, -2.1957751641342, 0.760638829255664, 0, 0, 0, 0, 0},
      {2, 1.22474487139159, 3.53553390593274, -7.37711113563318, 4.57142857142858, 1.64957219768463, 0, 11.4997781699989, 7.4230748895809, -2.57142857142857, 0, 0, 0, 0, 0},
      {1.54919333848296, 6.6407830863536, 3.83405790253616, 0, -6.19677335393187, 5.36656314599949, 0, 0, 13.4164078649987, 4.6475800154489, 0, 0, 0, 0, 0},
      {-3.57770876399967, 0, 8.85437744847147, 0, -1.10153547165223e-14, -3.09838667696593, 0, 0, 0, 16.0996894379985, 0, 0, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff0_4 = 0;
      double coeff0_5 = 0;
      double coeff0_6 = 0;
      double coeff0_7 = 0;
      double coeff0_8 = 0;
      double coeff0_9 = 0;
      double coeff0_10 = 0;
      double coeff0_11 = 0;
      double coeff0_12 = 0;
      double coeff0_13 = 0;
      double coeff0_14 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff0_4 = 0;
      double new_coeff0_5 = 0;
      double new_coeff0_6 = 0;
      double new_coeff0_7 = 0;
      double new_coeff0_8 = 0;
      double new_coeff0_9 = 0;
      double new_coeff0_10 = 0;
      double new_coeff0_11 = 0;
      double new_coeff0_12 = 0;
      double new_coeff0_13 = 0;
      double new_coeff0_14 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff0_4 = coefficients0[dof][4];
        new_coeff0_5 = coefficients0[dof][5];
        new_coeff0_6 = coefficients0[dof][6];
        new_coeff0_7 = coefficients0[dof][7];
        new_coeff0_8 = coefficients0[dof][8];
        new_coeff0_9 = coefficients0[dof][9];
        new_coeff0_10 = coefficients0[dof][10];
        new_coeff0_11 = coefficients0[dof][11];
        new_coeff0_12 = coefficients0[dof][12];
        new_coeff0_13 = coefficients0[dof][13];
        new_coeff0_14 = coefficients0[dof][14];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff0_4 = new_coeff0_4;
          coeff0_5 = new_coeff0_5;
          coeff0_6 = new_coeff0_6;
          coeff0_7 = new_coeff0_7;
          coeff0_8 = new_coeff0_8;
          coeff0_9 = new_coeff0_9;
          coeff0_10 = new_coeff0_10;
          coeff0_11 = new_coeff0_11;
          coeff0_12 = new_coeff0_12;
          coeff0_13 = new_coeff0_13;
          coeff0_14 = new_coeff0_14;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3];
            new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4];
            new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5];
            new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6];
            new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7];
            new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8];
            new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9];
            new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10];
            new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11];
            new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12];
            new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13];
            new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3];
            new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4];
            new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5];
            new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6];
            new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7];
            new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8];
            new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9];
            new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10];
            new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11];
            new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12];
            new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13];
            new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (15 <= i && i <= 29)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 15;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_y_3 = scalings_y_2*(0.5 - 0.5*y);
      const double scalings_y_4 = scalings_y_3*(0.5 - 0.5*y);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
      const double psitilde_a_3 = 1.66666666666667*x*psitilde_a_2 - 0.666666666666667*psitilde_a_1;
      const double psitilde_a_4 = 1.75*x*psitilde_a_3 - 0.75*psitilde_a_2;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.111111111111111*psitilde_bs_0_1 + 1.66666666666667*y*psitilde_bs_0_1 - 0.555555555555556*psitilde_bs_0_0;
      const double psitilde_bs_0_3 = 0.05*psitilde_bs_0_2 + 1.75*y*psitilde_bs_0_2 - 0.7*psitilde_bs_0_1;
      const double psitilde_bs_0_4 = 0.0285714285714286*psitilde_bs_0_3 + 1.8*y*psitilde_bs_0_3 - 0.771428571428571*psitilde_bs_0_2;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_1_2 = 0.54*psitilde_bs_1_1 + 2.1*y*psitilde_bs_1_1 - 0.56*psitilde_bs_1_0;
      const double psitilde_bs_1_3 = 0.285714285714286*psitilde_bs_1_2 + 2*y*psitilde_bs_1_2 - 0.714285714285714*psitilde_bs_1_1;
      const double psitilde_bs_2_0 = 1;
      const double psitilde_bs_2_1 = 3.5*y + 2.5;
      const double psitilde_bs_2_2 = 1.02040816326531*psitilde_bs_2_1 + 2.57142857142857*y*psitilde_bs_2_1 - 0.551020408163265*psitilde_bs_2_0;
      const double psitilde_bs_3_0 = 1;
      const double psitilde_bs_3_1 = 4.5*y + 3.5;
      const double psitilde_bs_4_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.707106781186548*psitilde_a_0*scalings_y_0*psitilde_bs_0_0;
      const double basisvalue1 = 1.73205080756888*psitilde_a_1*scalings_y_1*psitilde_bs_1_0;
      const double basisvalue2 = psitilde_a_0*scalings_y_0*psitilde_bs_0_1;
      const double basisvalue3 = 2.73861278752583*psitilde_a_2*scalings_y_2*psitilde_bs_2_0;
      const double basisvalue4 = 2.12132034355964*psitilde_a_1*scalings_y_1*psitilde_bs_1_1;
      const double basisvalue5 = 1.22474487139159*psitilde_a_0*scalings_y_0*psitilde_bs_0_2;
      const double basisvalue6 = 3.74165738677394*psitilde_a_3*scalings_y_3*psitilde_bs_3_0;
      const double basisvalue7 = 3.16227766016838*psitilde_a_2*scalings_y_2*psitilde_bs_2_1;
      const double basisvalue8 = 2.44948974278318*psitilde_a_1*scalings_y_1*psitilde_bs_1_2;
      const double basisvalue9 = 1.4142135623731*psitilde_a_0*scalings_y_0*psitilde_bs_0_3;
      const double basisvalue10 = 4.74341649025257*psitilde_a_4*scalings_y_4*psitilde_bs_4_0;
      const double basisvalue11 = 4.18330013267038*psitilde_a_3*scalings_y_3*psitilde_bs_3_1;
      const double basisvalue12 = 3.53553390593274*psitilde_a_2*scalings_y_2*psitilde_bs_2_2;
      const double basisvalue13 = 2.73861278752583*psitilde_a_1*scalings_y_1*psitilde_bs_1_3;
      const double basisvalue14 = 1.58113883008419*psitilde_a_0*scalings_y_0*psitilde_bs_0_4;
    
      // Table(s) of coefficients
      const static double coefficients0[15][15] =   \
      {{0, -0.0412393049421161, -0.0238095238095238, 0.0289800294976278, 0.0224478343233824, 0.0129602631893289, -0.0395942580610999, -0.0334632556631574, -0.025920526378658, -0.014965222882255, 0.0321247254366312, 0.0283313448138523, 0.023944356611608, 0.0185472188784818, 0.0107082418122104},
      {0, 0.0412393049421161, -0.0238095238095238, 0.0289800294976279, -0.0224478343233825, 0.012960263189329, 0.0395942580610999, -0.0334632556631574, 0.025920526378658, -0.014965222882255, 0.0321247254366312, -0.0283313448138523, 0.0239443566116079, -0.0185472188784818, 0.0107082418122104},
      {0, 0, 0.0476190476190476, 0, 0, 0.038880789567987, 0, 0, 0, 0.0598608915290199, 0, 0, 0, 0, 0.0535412090610519},
      {0.125707872210942, 0.131965775814772, -0.0253968253968253, 0.139104141588614, -0.0718330698348239, 0.0311046316543896, 0.0633508128977599, 0.0267706045305259, -0.0622092633087791, 0.0478887132232159, 0, 0.0566626896277046, -0.0838052481406279, 0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527355, 0.0109971479845642, 0.00634920634920629, 0, 0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, -0.139104141588614, 0.107082418122104},
      {0.125707872210942, 0.0439885919382572, 0.126984126984127, 0, 0.0359165349174119, 0.155523158271948, 0, 0, 0.103682105514632, -0.011972178305804, 0, 0, 0, 0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.131965775814772, -0.0253968253968254, 0.139104141588614, 0.0718330698348239, 0.0311046316543896, -0.0633508128977599, 0.0267706045305259, 0.0622092633087791, 0.0478887132232159, 0, -0.0566626896277046, -0.0838052481406278, -0.0834624849531682, -0.0535412090610519},
      {-0.0314269680527354, -0.0109971479845644, 0.00634920634920625, 0, -0.188561808316413, -0.163299316185545, 0, 0.0936971158568409, 0, -0.0419026240703139, 0, 0, 0.0838052481406278, 0.139104141588614, 0.107082418122104},
      {0.125707872210942, -0.0439885919382573, 0.126984126984127, 0, -0.0359165349174119, 0.155523158271948, 0, 0, -0.103682105514632, -0.011972178305804, 0, 0, 0, -0.0927360943924091, -0.107082418122104},
      {0.125707872210942, -0.0879771838765143, -0.101587301587302, 0.0927360943924091, 0.107749604752236, 0.0725774738602423, 0.0791885161221998, -0.013385302265263, -0.0518410527573159, -0.0419026240703139, -0.128498901746525, -0.0566626896277046, -0.011972178305804, 0.0092736094392409, 0.0107082418122104},
      {-0.0314269680527354, 0, -0.0126984126984127, -0.243432247780074, 0, 0.0544331053951817, 0, 0.0936971158568408, 0, -0.0419026240703139, 0.192748352619787, 0, -0.023944356611608, 0, 0.0107082418122104},
      {0.125707872210942, 0.0879771838765145, -0.101587301587302, 0.0927360943924091, -0.107749604752236, 0.0725774738602423, -0.0791885161221999, -0.013385302265263, 0.0518410527573159, -0.0419026240703139, -0.128498901746525, 0.0566626896277046, -0.011972178305804, -0.00927360943924091, 0.0107082418122104},
      {0.251415744421884, -0.351908735506058, -0.203174603174603, -0.139104141588614, -0.107749604752236, -0.0622092633087791, 0.19005243869328, -0.0267706045305259, 0.124418526617558, 0.155638317975452, 0, 0.169988068883114, 0.0838052481406278, -0.0278208283177227, -0.0535412090610519},
      {0.251415744421884, 0.351908735506058, -0.203174603174603, -0.139104141588614, 0.107749604752236, -0.0622092633087792, -0.19005243869328, -0.0267706045305259, -0.124418526617558, 0.155638317975452, 0, -0.169988068883114, 0.0838052481406278, 0.0278208283177228, -0.0535412090610519},
      {0.251415744421883, 0, 0.406349206349206, 0, 0, -0.186627789926337, 0, -0.187394231713682, 0, -0.203527031198668, 0, 0, -0.167610496281256, 0, 0.107082418122104}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[15][15] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.89897948556635, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 9.48683298050513, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4, 0, 7.07106781186548, 0, 0, 1.59872115546023e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.29150262212918, 0, -2.99332590941916, 13.6626010212795, 0, 0.611010092660777, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 4.38178046004132, 0, 0, 12.5219806739988, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.46410161513776, 0, 7.83836717690617, 0, 0, 8.40000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 10.9544511501033, 0, 0, -3.83325938999964, 0, 17.7482393492988, 0, 0.553283335172495, 0, 0, 0, 0, 0, 0},
      {4.73286382647969, 0, 3.3466401061363, 4.36435780471985, -1.12389939329608e-14, -5.07468037933239, 0, 17.0084012854152, 0, 1.52127765851133, 0, 0, 0, 0, 0},
      {0, 2.44948974278318, 0, 0, 9.14285714285716, 0, 0, 0, 14.8461497791618, 0, 0, 0, 0, 0, 0},
      {3.09838667696593, 0, 7.66811580507233, 0, 0, 10.733126291999, 0, 0, 0, 9.2951600308978, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats1[15][15] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.44948974278318, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.24264068711928, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.58198889747161, 4.74341649025257, -0.912870929175277, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2, 6.12372435695794, 3.53553390593274, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {-2.3094010767585, 0, 8.16496580927726, 0, 0, 1.24344978758018e-14, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.64575131106459, 5.18459255872629, -1.49666295470958, 6.83130051063973, -1.05830052442584, 0.305505046330385, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.23606797749979, 2.19089023002066, 2.5298221281347, 8.08290376865476, 6.26099033699941, -1.80739222823013, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {1.73205080756888, -5.09116882454314, 3.91918358845309, 0, 9.69948452238573, 4.20000000000001, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.00000000000001, 0, -2.82842712474619, 0, 0, 12.1243556529821, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.68328157299975, 5.47722557505166, -1.89736659610103, 7.4230748895809, -1.91662969499982, 0.663940002206988, 8.87411967464942, -1.07142857142857, 0.276641667586247, -0.0958314847499899, 0, 0, 0, 0, 0},
      {2.36643191323985, 2.89827534923789, 1.67332005306815, 2.18217890235992, 5.74704893215391, -2.53734018966619, 10.0623058987491, 8.50420064270761, -2.1957751641342, 0.760638829255664, 0, 0, 0, 0, 0},
      {2, 1.22474487139159, 3.53553390593274, -7.37711113563318, 4.57142857142858, 1.64957219768463, 0, 11.4997781699989, 7.4230748895809, -2.57142857142857, 0, 0, 0, 0, 0},
      {1.54919333848296, 6.6407830863536, 3.83405790253616, 0, -6.19677335393187, 5.36656314599949, 0, 0, 13.4164078649987, 4.6475800154489, 0, 0, 0, 0, 0},
      {-3.57770876399967, 0, 8.85437744847147, 0, -1.10153547165223e-14, -3.09838667696593, 0, 0, 0, 16.0996894379985, 0, 0, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff0_4 = 0;
      double coeff0_5 = 0;
      double coeff0_6 = 0;
      double coeff0_7 = 0;
      double coeff0_8 = 0;
      double coeff0_9 = 0;
      double coeff0_10 = 0;
      double coeff0_11 = 0;
      double coeff0_12 = 0;
      double coeff0_13 = 0;
      double coeff0_14 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff0_4 = 0;
      double new_coeff0_5 = 0;
      double new_coeff0_6 = 0;
      double new_coeff0_7 = 0;
      double new_coeff0_8 = 0;
      double new_coeff0_9 = 0;
      double new_coeff0_10 = 0;
      double new_coeff0_11 = 0;
      double new_coeff0_12 = 0;
      double new_coeff0_13 = 0;
      double new_coeff0_14 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff0_4 = coefficients0[dof][4];
        new_coeff0_5 = coefficients0[dof][5];
        new_coeff0_6 = coefficients0[dof][6];
        new_coeff0_7 = coefficients0[dof][7];
        new_coeff0_8 = coefficients0[dof][8];
        new_coeff0_9 = coefficients0[dof][9];
        new_coeff0_10 = coefficients0[dof][10];
        new_coeff0_11 = coefficients0[dof][11];
        new_coeff0_12 = coefficients0[dof][12];
        new_coeff0_13 = coefficients0[dof][13];
        new_coeff0_14 = coefficients0[dof][14];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff0_4 = new_coeff0_4;
          coeff0_5 = new_coeff0_5;
          coeff0_6 = new_coeff0_6;
          coeff0_7 = new_coeff0_7;
          coeff0_8 = new_coeff0_8;
          coeff0_9 = new_coeff0_9;
          coeff0_10 = new_coeff0_10;
          coeff0_11 = new_coeff0_11;
          coeff0_12 = new_coeff0_12;
          coeff0_13 = new_coeff0_13;
          coeff0_14 = new_coeff0_14;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0] + coeff0_10*dmats0[10][0] + coeff0_11*dmats0[11][0] + coeff0_12*dmats0[12][0] + coeff0_13*dmats0[13][0] + coeff0_14*dmats0[14][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1] + coeff0_10*dmats0[10][1] + coeff0_11*dmats0[11][1] + coeff0_12*dmats0[12][1] + coeff0_13*dmats0[13][1] + coeff0_14*dmats0[14][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2] + coeff0_10*dmats0[10][2] + coeff0_11*dmats0[11][2] + coeff0_12*dmats0[12][2] + coeff0_13*dmats0[13][2] + coeff0_14*dmats0[14][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3] + coeff0_10*dmats0[10][3] + coeff0_11*dmats0[11][3] + coeff0_12*dmats0[12][3] + coeff0_13*dmats0[13][3] + coeff0_14*dmats0[14][3];
            new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4] + coeff0_10*dmats0[10][4] + coeff0_11*dmats0[11][4] + coeff0_12*dmats0[12][4] + coeff0_13*dmats0[13][4] + coeff0_14*dmats0[14][4];
            new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5] + coeff0_10*dmats0[10][5] + coeff0_11*dmats0[11][5] + coeff0_12*dmats0[12][5] + coeff0_13*dmats0[13][5] + coeff0_14*dmats0[14][5];
            new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6] + coeff0_10*dmats0[10][6] + coeff0_11*dmats0[11][6] + coeff0_12*dmats0[12][6] + coeff0_13*dmats0[13][6] + coeff0_14*dmats0[14][6];
            new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7] + coeff0_10*dmats0[10][7] + coeff0_11*dmats0[11][7] + coeff0_12*dmats0[12][7] + coeff0_13*dmats0[13][7] + coeff0_14*dmats0[14][7];
            new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8] + coeff0_10*dmats0[10][8] + coeff0_11*dmats0[11][8] + coeff0_12*dmats0[12][8] + coeff0_13*dmats0[13][8] + coeff0_14*dmats0[14][8];
            new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9] + coeff0_10*dmats0[10][9] + coeff0_11*dmats0[11][9] + coeff0_12*dmats0[12][9] + coeff0_13*dmats0[13][9] + coeff0_14*dmats0[14][9];
            new_coeff0_10 = coeff0_0*dmats0[0][10] + coeff0_1*dmats0[1][10] + coeff0_2*dmats0[2][10] + coeff0_3*dmats0[3][10] + coeff0_4*dmats0[4][10] + coeff0_5*dmats0[5][10] + coeff0_6*dmats0[6][10] + coeff0_7*dmats0[7][10] + coeff0_8*dmats0[8][10] + coeff0_9*dmats0[9][10] + coeff0_10*dmats0[10][10] + coeff0_11*dmats0[11][10] + coeff0_12*dmats0[12][10] + coeff0_13*dmats0[13][10] + coeff0_14*dmats0[14][10];
            new_coeff0_11 = coeff0_0*dmats0[0][11] + coeff0_1*dmats0[1][11] + coeff0_2*dmats0[2][11] + coeff0_3*dmats0[3][11] + coeff0_4*dmats0[4][11] + coeff0_5*dmats0[5][11] + coeff0_6*dmats0[6][11] + coeff0_7*dmats0[7][11] + coeff0_8*dmats0[8][11] + coeff0_9*dmats0[9][11] + coeff0_10*dmats0[10][11] + coeff0_11*dmats0[11][11] + coeff0_12*dmats0[12][11] + coeff0_13*dmats0[13][11] + coeff0_14*dmats0[14][11];
            new_coeff0_12 = coeff0_0*dmats0[0][12] + coeff0_1*dmats0[1][12] + coeff0_2*dmats0[2][12] + coeff0_3*dmats0[3][12] + coeff0_4*dmats0[4][12] + coeff0_5*dmats0[5][12] + coeff0_6*dmats0[6][12] + coeff0_7*dmats0[7][12] + coeff0_8*dmats0[8][12] + coeff0_9*dmats0[9][12] + coeff0_10*dmats0[10][12] + coeff0_11*dmats0[11][12] + coeff0_12*dmats0[12][12] + coeff0_13*dmats0[13][12] + coeff0_14*dmats0[14][12];
            new_coeff0_13 = coeff0_0*dmats0[0][13] + coeff0_1*dmats0[1][13] + coeff0_2*dmats0[2][13] + coeff0_3*dmats0[3][13] + coeff0_4*dmats0[4][13] + coeff0_5*dmats0[5][13] + coeff0_6*dmats0[6][13] + coeff0_7*dmats0[7][13] + coeff0_8*dmats0[8][13] + coeff0_9*dmats0[9][13] + coeff0_10*dmats0[10][13] + coeff0_11*dmats0[11][13] + coeff0_12*dmats0[12][13] + coeff0_13*dmats0[13][13] + coeff0_14*dmats0[14][13];
            new_coeff0_14 = coeff0_0*dmats0[0][14] + coeff0_1*dmats0[1][14] + coeff0_2*dmats0[2][14] + coeff0_3*dmats0[3][14] + coeff0_4*dmats0[4][14] + coeff0_5*dmats0[5][14] + coeff0_6*dmats0[6][14] + coeff0_7*dmats0[7][14] + coeff0_8*dmats0[8][14] + coeff0_9*dmats0[9][14] + coeff0_10*dmats0[10][14] + coeff0_11*dmats0[11][14] + coeff0_12*dmats0[12][14] + coeff0_13*dmats0[13][14] + coeff0_14*dmats0[14][14];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0] + coeff0_10*dmats1[10][0] + coeff0_11*dmats1[11][0] + coeff0_12*dmats1[12][0] + coeff0_13*dmats1[13][0] + coeff0_14*dmats1[14][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1] + coeff0_10*dmats1[10][1] + coeff0_11*dmats1[11][1] + coeff0_12*dmats1[12][1] + coeff0_13*dmats1[13][1] + coeff0_14*dmats1[14][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2] + coeff0_10*dmats1[10][2] + coeff0_11*dmats1[11][2] + coeff0_12*dmats1[12][2] + coeff0_13*dmats1[13][2] + coeff0_14*dmats1[14][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3] + coeff0_10*dmats1[10][3] + coeff0_11*dmats1[11][3] + coeff0_12*dmats1[12][3] + coeff0_13*dmats1[13][3] + coeff0_14*dmats1[14][3];
            new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4] + coeff0_10*dmats1[10][4] + coeff0_11*dmats1[11][4] + coeff0_12*dmats1[12][4] + coeff0_13*dmats1[13][4] + coeff0_14*dmats1[14][4];
            new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5] + coeff0_10*dmats1[10][5] + coeff0_11*dmats1[11][5] + coeff0_12*dmats1[12][5] + coeff0_13*dmats1[13][5] + coeff0_14*dmats1[14][5];
            new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6] + coeff0_10*dmats1[10][6] + coeff0_11*dmats1[11][6] + coeff0_12*dmats1[12][6] + coeff0_13*dmats1[13][6] + coeff0_14*dmats1[14][6];
            new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7] + coeff0_10*dmats1[10][7] + coeff0_11*dmats1[11][7] + coeff0_12*dmats1[12][7] + coeff0_13*dmats1[13][7] + coeff0_14*dmats1[14][7];
            new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8] + coeff0_10*dmats1[10][8] + coeff0_11*dmats1[11][8] + coeff0_12*dmats1[12][8] + coeff0_13*dmats1[13][8] + coeff0_14*dmats1[14][8];
            new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9] + coeff0_10*dmats1[10][9] + coeff0_11*dmats1[11][9] + coeff0_12*dmats1[12][9] + coeff0_13*dmats1[13][9] + coeff0_14*dmats1[14][9];
            new_coeff0_10 = coeff0_0*dmats1[0][10] + coeff0_1*dmats1[1][10] + coeff0_2*dmats1[2][10] + coeff0_3*dmats1[3][10] + coeff0_4*dmats1[4][10] + coeff0_5*dmats1[5][10] + coeff0_6*dmats1[6][10] + coeff0_7*dmats1[7][10] + coeff0_8*dmats1[8][10] + coeff0_9*dmats1[9][10] + coeff0_10*dmats1[10][10] + coeff0_11*dmats1[11][10] + coeff0_12*dmats1[12][10] + coeff0_13*dmats1[13][10] + coeff0_14*dmats1[14][10];
            new_coeff0_11 = coeff0_0*dmats1[0][11] + coeff0_1*dmats1[1][11] + coeff0_2*dmats1[2][11] + coeff0_3*dmats1[3][11] + coeff0_4*dmats1[4][11] + coeff0_5*dmats1[5][11] + coeff0_6*dmats1[6][11] + coeff0_7*dmats1[7][11] + coeff0_8*dmats1[8][11] + coeff0_9*dmats1[9][11] + coeff0_10*dmats1[10][11] + coeff0_11*dmats1[11][11] + coeff0_12*dmats1[12][11] + coeff0_13*dmats1[13][11] + coeff0_14*dmats1[14][11];
            new_coeff0_12 = coeff0_0*dmats1[0][12] + coeff0_1*dmats1[1][12] + coeff0_2*dmats1[2][12] + coeff0_3*dmats1[3][12] + coeff0_4*dmats1[4][12] + coeff0_5*dmats1[5][12] + coeff0_6*dmats1[6][12] + coeff0_7*dmats1[7][12] + coeff0_8*dmats1[8][12] + coeff0_9*dmats1[9][12] + coeff0_10*dmats1[10][12] + coeff0_11*dmats1[11][12] + coeff0_12*dmats1[12][12] + coeff0_13*dmats1[13][12] + coeff0_14*dmats1[14][12];
            new_coeff0_13 = coeff0_0*dmats1[0][13] + coeff0_1*dmats1[1][13] + coeff0_2*dmats1[2][13] + coeff0_3*dmats1[3][13] + coeff0_4*dmats1[4][13] + coeff0_5*dmats1[5][13] + coeff0_6*dmats1[6][13] + coeff0_7*dmats1[7][13] + coeff0_8*dmats1[8][13] + coeff0_9*dmats1[9][13] + coeff0_10*dmats1[10][13] + coeff0_11*dmats1[11][13] + coeff0_12*dmats1[12][13] + coeff0_13*dmats1[13][13] + coeff0_14*dmats1[14][13];
            new_coeff0_14 = coeff0_0*dmats1[0][14] + coeff0_1*dmats1[1][14] + coeff0_2*dmats1[2][14] + coeff0_3*dmats1[3][14] + coeff0_4*dmats1[4][14] + coeff0_5*dmats1[5][14] + coeff0_6*dmats1[6][14] + coeff0_7*dmats1[7][14] + coeff0_8*dmats1[8][14] + coeff0_9*dmats1[9][14] + coeff0_10*dmats1[10][14] + coeff0_11*dmats1[11][14] + coeff0_12*dmats1[12][14] + coeff0_13*dmats1[13][14] + coeff0_14*dmats1[14][14];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9 + new_coeff0_10*basisvalue10 + new_coeff0_11*basisvalue11 + new_coeff0_12*basisvalue12 + new_coeff0_13*basisvalue13 + new_coeff0_14*basisvalue14;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
}

/// Evaluate order n derivatives of all basis functions at given point in cell
void UFC_P4VecDivL2Functional_finite_element_0::evaluate_basis_derivatives_all(unsigned int n,
                                                   double* values,
                                                   const double* coordinates,
                                                   const ufc::cell& c) const
{
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
}

/// Evaluate linear functional for dof i on the function f
double UFC_P4VecDivL2Functional_finite_element_0::evaluate_dof(unsigned int i,
                                   const ufc::function& f,
                                   const ufc::cell& c) const
{
    // The reference points, direction and weights:
    const static double X[30][1][2] = {{{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.75, 0.25}}, {{0.5, 0.5}}, {{0.25, 0.75}}, {{0, 0.25}}, {{0, 0.5}}, {{0, 0.75}}, {{0.25, 0}}, {{0.5, 0}}, {{0.75, 0}}, {{0.25, 0.25}}, {{0.5, 0.25}}, {{0.25, 0.5}}, {{0, 0}}, {{1, 0}}, {{0, 1}}, {{0.75, 0.25}}, {{0.5, 0.5}}, {{0.25, 0.75}}, {{0, 0.25}}, {{0, 0.5}}, {{0, 0.75}}, {{0.25, 0}}, {{0.5, 0}}, {{0.75, 0}}, {{0.25, 0.25}}, {{0.5, 0.25}}, {{0.25, 0.5}}};
    const static double W[30][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[30][1][2] = {{{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{1, 0}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}, {{0, 1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    
    // Compute affine mapping y = F(X)
    double y[2];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1];
    
    // Evaluate function at physical points
    double values[2];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 2; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
}

/// Evaluate linear functionals for all dofs on the function f
void UFC_P4VecDivL2Functional_finite_element_0::evaluate_dofs(double* values,
                                  const ufc::function& f,
                                  const ufc::cell& c) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Interpolate vertex values from dof values
void UFC_P4VecDivL2Functional_finite_element_0::interpolate_vertex_values(double* vertex_values,
                                              const double* dof_values,
                                              const ufc::cell& c) const
{
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[2] = dof_values[1];
    vertex_values[4] = dof_values[2];
    // Evaluate at vertices and use affine mapping
    vertex_values[1] = dof_values[15];
    vertex_values[3] = dof_values[16];
    vertex_values[5] = dof_values[17];
}

/// Return the number of sub elements (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_finite_element_0::num_sub_elements() const
{
    return 2;
}

/// Create a new finite element for sub element i (for a mixed element)
ufc::finite_element* UFC_P4VecDivL2Functional_finite_element_0::create_sub_element(unsigned int i) const
{
    switch ( i )
    {
    case 0:
      return new UFC_P4VecDivL2Functional_finite_element_0_0();
      break;
    case 1:
      return new UFC_P4VecDivL2Functional_finite_element_0_1();
      break;
    }
    return 0;
}

/// Constructor
UFC_P4VecDivL2Functional_dof_map_0_0::UFC_P4VecDivL2Functional_dof_map_0_0() : ufc::dof_map()
{
    __global_dimension = 0;
}

/// Destructor
UFC_P4VecDivL2Functional_dof_map_0_0::~UFC_P4VecDivL2Functional_dof_map_0_0()
{
    // Do nothing
}

/// Return a string identifying the dof map
const char* UFC_P4VecDivL2Functional_dof_map_0_0::signature() const
{
    return "FFC dof map for Lagrange finite element of degree 4 on a triangle";
}

/// Return true iff mesh entities of topological dimension d are needed
bool UFC_P4VecDivL2Functional_dof_map_0_0::needs_mesh_entities(unsigned int d) const
{
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    }
    return false;
}

/// Initialize dof map for mesh (return true iff init_cell() is needed)
bool UFC_P4VecDivL2Functional_dof_map_0_0::init_mesh(const ufc::mesh& m)
{
    __global_dimension = m.num_entities[0] + 3*m.num_entities[1] + 3*m.num_entities[2];
    return false;
}

/// Initialize dof map for given cell
void UFC_P4VecDivL2Functional_dof_map_0_0::init_cell(const ufc::mesh& m,
                              const ufc::cell& c)
{
    // Do nothing
}

/// Finish initialization of dof map for cells
void UFC_P4VecDivL2Functional_dof_map_0_0::init_cell_finalize()
{
    // Do nothing
}

/// Return the dimension of the global finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::global_dimension() const
{
    return __global_dimension;
}

/// Return the dimension of the local finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::local_dimension() const
{
    return 15;
}

// Return the geometric dimension of the coordinates this dof map provides
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::geometric_dimension() const
{
    return 2;
}

/// Return the number of dofs on each cell facet
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::num_facet_dofs() const
{
    return 5;
}

/// Return the number of dofs associated with each cell entity of dimension d
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::num_entity_dofs(unsigned int d) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the local-to-global mapping of dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0_0::tabulate_dofs(unsigned int* dofs,
                                  const ufc::mesh& m,
                                  const ufc::cell& c) const
{
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + 3*c.entity_indices[1][0];
    dofs[4] = offset + 3*c.entity_indices[1][0] + 1;
    dofs[5] = offset + 3*c.entity_indices[1][0] + 2;
    dofs[6] = offset + 3*c.entity_indices[1][1];
    dofs[7] = offset + 3*c.entity_indices[1][1] + 1;
    dofs[8] = offset + 3*c.entity_indices[1][1] + 2;
    dofs[9] = offset + 3*c.entity_indices[1][2];
    dofs[10] = offset + 3*c.entity_indices[1][2] + 1;
    dofs[11] = offset + 3*c.entity_indices[1][2] + 2;
    offset = offset + 3*m.num_entities[1];
    dofs[12] = offset + 3*c.entity_indices[2][0];
    dofs[13] = offset + 3*c.entity_indices[2][0] + 1;
    dofs[14] = offset + 3*c.entity_indices[2][0] + 2;
}

/// Tabulate the local-to-local mapping from facet dofs to cell dofs
void UFC_P4VecDivL2Functional_dof_map_0_0::tabulate_facet_dofs(unsigned int* dofs,
                                        unsigned int facet) const
{
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 6;
      dofs[3] = 7;
      dofs[4] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 9;
      dofs[3] = 10;
      dofs[4] = 11;
      break;
    }
}

/// Tabulate the local-to-local mapping of dofs on entity (d, i)
void UFC_P4VecDivL2Functional_dof_map_0_0::tabulate_entity_dofs(unsigned int* dofs,
                                  unsigned int d, unsigned int i) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the coordinates of all dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0_0::tabulate_coordinates(double** coordinates,
                                         const ufc::cell& c) const
{
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.75*x[1][0] + 0.25*x[2][0];
    coordinates[3][1] = 0.75*x[1][1] + 0.25*x[2][1];
    coordinates[4][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.25*x[1][0] + 0.75*x[2][0];
    coordinates[5][1] = 0.25*x[1][1] + 0.75*x[2][1];
    coordinates[6][0] = 0.75*x[0][0] + 0.25*x[2][0];
    coordinates[6][1] = 0.75*x[0][1] + 0.25*x[2][1];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][0] = 0.25*x[0][0] + 0.75*x[2][0];
    coordinates[8][1] = 0.25*x[0][1] + 0.75*x[2][1];
    coordinates[9][0] = 0.75*x[0][0] + 0.25*x[1][0];
    coordinates[9][1] = 0.75*x[0][1] + 0.25*x[1][1];
    coordinates[10][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[10][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[11][0] = 0.25*x[0][0] + 0.75*x[1][0];
    coordinates[11][1] = 0.25*x[0][1] + 0.75*x[1][1];
    coordinates[12][0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    coordinates[12][1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    coordinates[13][0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    coordinates[13][1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    coordinates[14][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    coordinates[14][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
}

/// Return the number of sub dof maps (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_dof_map_0_0::num_sub_dof_maps() const
{
    return 1;
}

/// Create a new dof_map for sub dof map i (for a mixed element)
ufc::dof_map* UFC_P4VecDivL2Functional_dof_map_0_0::create_sub_dof_map(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_dof_map_0_0();
}


/// Constructor
UFC_P4VecDivL2Functional_dof_map_0_1::UFC_P4VecDivL2Functional_dof_map_0_1() : ufc::dof_map()
{
    __global_dimension = 0;
}

/// Destructor
UFC_P4VecDivL2Functional_dof_map_0_1::~UFC_P4VecDivL2Functional_dof_map_0_1()
{
    // Do nothing
}

/// Return a string identifying the dof map
const char* UFC_P4VecDivL2Functional_dof_map_0_1::signature() const
{
    return "FFC dof map for Lagrange finite element of degree 4 on a triangle";
}

/// Return true iff mesh entities of topological dimension d are needed
bool UFC_P4VecDivL2Functional_dof_map_0_1::needs_mesh_entities(unsigned int d) const
{
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    }
    return false;
}

/// Initialize dof map for mesh (return true iff init_cell() is needed)
bool UFC_P4VecDivL2Functional_dof_map_0_1::init_mesh(const ufc::mesh& m)
{
    __global_dimension = m.num_entities[0] + 3*m.num_entities[1] + 3*m.num_entities[2];
    return false;
}

/// Initialize dof map for given cell
void UFC_P4VecDivL2Functional_dof_map_0_1::init_cell(const ufc::mesh& m,
                              const ufc::cell& c)
{
    // Do nothing
}

/// Finish initialization of dof map for cells
void UFC_P4VecDivL2Functional_dof_map_0_1::init_cell_finalize()
{
    // Do nothing
}

/// Return the dimension of the global finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::global_dimension() const
{
    return __global_dimension;
}

/// Return the dimension of the local finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::local_dimension() const
{
    return 15;
}

// Return the geometric dimension of the coordinates this dof map provides
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::geometric_dimension() const
{
    return 2;
}

/// Return the number of dofs on each cell facet
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::num_facet_dofs() const
{
    return 5;
}

/// Return the number of dofs associated with each cell entity of dimension d
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::num_entity_dofs(unsigned int d) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the local-to-global mapping of dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0_1::tabulate_dofs(unsigned int* dofs,
                                  const ufc::mesh& m,
                                  const ufc::cell& c) const
{
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + 3*c.entity_indices[1][0];
    dofs[4] = offset + 3*c.entity_indices[1][0] + 1;
    dofs[5] = offset + 3*c.entity_indices[1][0] + 2;
    dofs[6] = offset + 3*c.entity_indices[1][1];
    dofs[7] = offset + 3*c.entity_indices[1][1] + 1;
    dofs[8] = offset + 3*c.entity_indices[1][1] + 2;
    dofs[9] = offset + 3*c.entity_indices[1][2];
    dofs[10] = offset + 3*c.entity_indices[1][2] + 1;
    dofs[11] = offset + 3*c.entity_indices[1][2] + 2;
    offset = offset + 3*m.num_entities[1];
    dofs[12] = offset + 3*c.entity_indices[2][0];
    dofs[13] = offset + 3*c.entity_indices[2][0] + 1;
    dofs[14] = offset + 3*c.entity_indices[2][0] + 2;
}

/// Tabulate the local-to-local mapping from facet dofs to cell dofs
void UFC_P4VecDivL2Functional_dof_map_0_1::tabulate_facet_dofs(unsigned int* dofs,
                                        unsigned int facet) const
{
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 6;
      dofs[3] = 7;
      dofs[4] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 9;
      dofs[3] = 10;
      dofs[4] = 11;
      break;
    }
}

/// Tabulate the local-to-local mapping of dofs on entity (d, i)
void UFC_P4VecDivL2Functional_dof_map_0_1::tabulate_entity_dofs(unsigned int* dofs,
                                  unsigned int d, unsigned int i) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the coordinates of all dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0_1::tabulate_coordinates(double** coordinates,
                                         const ufc::cell& c) const
{
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.75*x[1][0] + 0.25*x[2][0];
    coordinates[3][1] = 0.75*x[1][1] + 0.25*x[2][1];
    coordinates[4][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.25*x[1][0] + 0.75*x[2][0];
    coordinates[5][1] = 0.25*x[1][1] + 0.75*x[2][1];
    coordinates[6][0] = 0.75*x[0][0] + 0.25*x[2][0];
    coordinates[6][1] = 0.75*x[0][1] + 0.25*x[2][1];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][0] = 0.25*x[0][0] + 0.75*x[2][0];
    coordinates[8][1] = 0.25*x[0][1] + 0.75*x[2][1];
    coordinates[9][0] = 0.75*x[0][0] + 0.25*x[1][0];
    coordinates[9][1] = 0.75*x[0][1] + 0.25*x[1][1];
    coordinates[10][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[10][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[11][0] = 0.25*x[0][0] + 0.75*x[1][0];
    coordinates[11][1] = 0.25*x[0][1] + 0.75*x[1][1];
    coordinates[12][0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    coordinates[12][1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    coordinates[13][0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    coordinates[13][1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    coordinates[14][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    coordinates[14][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
}

/// Return the number of sub dof maps (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_dof_map_0_1::num_sub_dof_maps() const
{
    return 1;
}

/// Create a new dof_map for sub dof map i (for a mixed element)
ufc::dof_map* UFC_P4VecDivL2Functional_dof_map_0_1::create_sub_dof_map(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_dof_map_0_1();
}


/// Constructor
UFC_P4VecDivL2Functional_dof_map_0::UFC_P4VecDivL2Functional_dof_map_0() : ufc::dof_map()
{
    __global_dimension = 0;
}

/// Destructor
UFC_P4VecDivL2Functional_dof_map_0::~UFC_P4VecDivL2Functional_dof_map_0()
{
    // Do nothing
}

/// Return a string identifying the dof map
const char* UFC_P4VecDivL2Functional_dof_map_0::signature() const
{
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 4 on a triangle, Lagrange finite element of degree 4 on a triangle]";
}

/// Return true iff mesh entities of topological dimension d are needed
bool UFC_P4VecDivL2Functional_dof_map_0::needs_mesh_entities(unsigned int d) const
{
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return true;
      break;
    }
    return false;
}

/// Initialize dof map for mesh (return true iff init_cell() is needed)
bool UFC_P4VecDivL2Functional_dof_map_0::init_mesh(const ufc::mesh& m)
{
    __global_dimension = 2*m.num_entities[0] + 6*m.num_entities[1] + 6*m.num_entities[2];
    return false;
}

/// Initialize dof map for given cell
void UFC_P4VecDivL2Functional_dof_map_0::init_cell(const ufc::mesh& m,
                              const ufc::cell& c)
{
    // Do nothing
}

/// Finish initialization of dof map for cells
void UFC_P4VecDivL2Functional_dof_map_0::init_cell_finalize()
{
    // Do nothing
}

/// Return the dimension of the global finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0::global_dimension() const
{
    return __global_dimension;
}

/// Return the dimension of the local finite element function space
unsigned int UFC_P4VecDivL2Functional_dof_map_0::local_dimension() const
{
    return 30;
}

// Return the geometric dimension of the coordinates this dof map provides
unsigned int UFC_P4VecDivL2Functional_dof_map_0::geometric_dimension() const
{
    return 2;
}

/// Return the number of dofs on each cell facet
unsigned int UFC_P4VecDivL2Functional_dof_map_0::num_facet_dofs() const
{
    return 10;
}

/// Return the number of dofs associated with each cell entity of dimension d
unsigned int UFC_P4VecDivL2Functional_dof_map_0::num_entity_dofs(unsigned int d) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the local-to-global mapping of dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0::tabulate_dofs(unsigned int* dofs,
                                  const ufc::mesh& m,
                                  const ufc::cell& c) const
{
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    unsigned int offset = m.num_entities[0];
    dofs[3] = offset + 3*c.entity_indices[1][0];
    dofs[4] = offset + 3*c.entity_indices[1][0] + 1;
    dofs[5] = offset + 3*c.entity_indices[1][0] + 2;
    dofs[6] = offset + 3*c.entity_indices[1][1];
    dofs[7] = offset + 3*c.entity_indices[1][1] + 1;
    dofs[8] = offset + 3*c.entity_indices[1][1] + 2;
    dofs[9] = offset + 3*c.entity_indices[1][2];
    dofs[10] = offset + 3*c.entity_indices[1][2] + 1;
    dofs[11] = offset + 3*c.entity_indices[1][2] + 2;
    offset = offset + 3*m.num_entities[1];
    dofs[12] = offset + 3*c.entity_indices[2][0];
    dofs[13] = offset + 3*c.entity_indices[2][0] + 1;
    dofs[14] = offset + 3*c.entity_indices[2][0] + 2;
    offset = offset + 3*m.num_entities[2];
    dofs[15] = offset + c.entity_indices[0][0];
    dofs[16] = offset + c.entity_indices[0][1];
    dofs[17] = offset + c.entity_indices[0][2];
    offset = offset + m.num_entities[0];
    dofs[18] = offset + 3*c.entity_indices[1][0];
    dofs[19] = offset + 3*c.entity_indices[1][0] + 1;
    dofs[20] = offset + 3*c.entity_indices[1][0] + 2;
    dofs[21] = offset + 3*c.entity_indices[1][1];
    dofs[22] = offset + 3*c.entity_indices[1][1] + 1;
    dofs[23] = offset + 3*c.entity_indices[1][1] + 2;
    dofs[24] = offset + 3*c.entity_indices[1][2];
    dofs[25] = offset + 3*c.entity_indices[1][2] + 1;
    dofs[26] = offset + 3*c.entity_indices[1][2] + 2;
    offset = offset + 3*m.num_entities[1];
    dofs[27] = offset + 3*c.entity_indices[2][0];
    dofs[28] = offset + 3*c.entity_indices[2][0] + 1;
    dofs[29] = offset + 3*c.entity_indices[2][0] + 2;
}

/// Tabulate the local-to-local mapping from facet dofs to cell dofs
void UFC_P4VecDivL2Functional_dof_map_0::tabulate_facet_dofs(unsigned int* dofs,
                                        unsigned int facet) const
{
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 16;
      dofs[6] = 17;
      dofs[7] = 18;
      dofs[8] = 19;
      dofs[9] = 20;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 6;
      dofs[3] = 7;
      dofs[4] = 8;
      dofs[5] = 15;
      dofs[6] = 17;
      dofs[7] = 21;
      dofs[8] = 22;
      dofs[9] = 23;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 9;
      dofs[3] = 10;
      dofs[4] = 11;
      dofs[5] = 15;
      dofs[6] = 16;
      dofs[7] = 24;
      dofs[8] = 25;
      dofs[9] = 26;
      break;
    }
}

/// Tabulate the local-to-local mapping of dofs on entity (d, i)
void UFC_P4VecDivL2Functional_dof_map_0::tabulate_entity_dofs(unsigned int* dofs,
                                  unsigned int d, unsigned int i) const
{
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
}

/// Tabulate the coordinates of all dofs on a cell
void UFC_P4VecDivL2Functional_dof_map_0::tabulate_coordinates(double** coordinates,
                                         const ufc::cell& c) const
{
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[3][0] = 0.75*x[1][0] + 0.25*x[2][0];
    coordinates[3][1] = 0.75*x[1][1] + 0.25*x[2][1];
    coordinates[4][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[5][0] = 0.25*x[1][0] + 0.75*x[2][0];
    coordinates[5][1] = 0.25*x[1][1] + 0.75*x[2][1];
    coordinates[6][0] = 0.75*x[0][0] + 0.25*x[2][0];
    coordinates[6][1] = 0.75*x[0][1] + 0.25*x[2][1];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][0] = 0.25*x[0][0] + 0.75*x[2][0];
    coordinates[8][1] = 0.25*x[0][1] + 0.75*x[2][1];
    coordinates[9][0] = 0.75*x[0][0] + 0.25*x[1][0];
    coordinates[9][1] = 0.75*x[0][1] + 0.25*x[1][1];
    coordinates[10][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[10][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[11][0] = 0.25*x[0][0] + 0.75*x[1][0];
    coordinates[11][1] = 0.25*x[0][1] + 0.75*x[1][1];
    coordinates[12][0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    coordinates[12][1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    coordinates[13][0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    coordinates[13][1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    coordinates[14][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    coordinates[14][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
    coordinates[15][0] = x[0][0];
    coordinates[15][1] = x[0][1];
    coordinates[16][0] = x[1][0];
    coordinates[16][1] = x[1][1];
    coordinates[17][0] = x[2][0];
    coordinates[17][1] = x[2][1];
    coordinates[18][0] = 0.75*x[1][0] + 0.25*x[2][0];
    coordinates[18][1] = 0.75*x[1][1] + 0.25*x[2][1];
    coordinates[19][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[19][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[20][0] = 0.25*x[1][0] + 0.75*x[2][0];
    coordinates[20][1] = 0.25*x[1][1] + 0.75*x[2][1];
    coordinates[21][0] = 0.75*x[0][0] + 0.25*x[2][0];
    coordinates[21][1] = 0.75*x[0][1] + 0.25*x[2][1];
    coordinates[22][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[22][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[23][0] = 0.25*x[0][0] + 0.75*x[2][0];
    coordinates[23][1] = 0.25*x[0][1] + 0.75*x[2][1];
    coordinates[24][0] = 0.75*x[0][0] + 0.25*x[1][0];
    coordinates[24][1] = 0.75*x[0][1] + 0.25*x[1][1];
    coordinates[25][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[25][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[26][0] = 0.25*x[0][0] + 0.75*x[1][0];
    coordinates[26][1] = 0.25*x[0][1] + 0.75*x[1][1];
    coordinates[27][0] = 0.5*x[0][0] + 0.25*x[1][0] + 0.25*x[2][0];
    coordinates[27][1] = 0.5*x[0][1] + 0.25*x[1][1] + 0.25*x[2][1];
    coordinates[28][0] = 0.25*x[0][0] + 0.5*x[1][0] + 0.25*x[2][0];
    coordinates[28][1] = 0.25*x[0][1] + 0.5*x[1][1] + 0.25*x[2][1];
    coordinates[29][0] = 0.25*x[0][0] + 0.25*x[1][0] + 0.5*x[2][0];
    coordinates[29][1] = 0.25*x[0][1] + 0.25*x[1][1] + 0.5*x[2][1];
}

/// Return the number of sub dof maps (for a mixed element)
unsigned int UFC_P4VecDivL2Functional_dof_map_0::num_sub_dof_maps() const
{
    return 2;
}

/// Create a new dof_map for sub dof map i (for a mixed element)
ufc::dof_map* UFC_P4VecDivL2Functional_dof_map_0::create_sub_dof_map(unsigned int i) const
{
    switch ( i )
    {
    case 0:
      return new UFC_P4VecDivL2Functional_dof_map_0_0();
      break;
    case 1:
      return new UFC_P4VecDivL2Functional_dof_map_0_1();
      break;
    }
    return 0;
}


/// Constructor
UFC_P4VecDivL2Functional_cell_integral_0::UFC_P4VecDivL2Functional_cell_integral_0() : ufc::cell_integral()
{
    // Do nothing
}

/// Destructor
UFC_P4VecDivL2Functional_cell_integral_0::~UFC_P4VecDivL2Functional_cell_integral_0()
{
    // Do nothing
}

/// Tabulate the tensor for the contribution from a local cell
void UFC_P4VecDivL2Functional_cell_integral_0::tabulate_tensor(double* A,
                                    const double * const * w,
                                    const ufc::cell& c) const
{
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
      
    // Compute determinant of Jacobian
    double detJ = J_00*J_11 - J_01*J_10;
      
    // Compute inverse of Jacobian
    const double Jinv_00 =  J_11 / detJ;
    const double Jinv_01 = -J_01 / detJ;
    const double Jinv_10 = -J_10 / detJ;
    const double Jinv_11 =  J_00 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Number of operations to compute element tensor = 17087
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_0_0_2 = w[0][2];
    const double c0_0_0_3 = w[0][3];
    const double c0_0_0_4 = w[0][4];
    const double c0_0_0_5 = w[0][5];
    const double c0_0_0_6 = w[0][6];
    const double c0_0_0_7 = w[0][7];
    const double c0_0_0_8 = w[0][8];
    const double c0_0_0_9 = w[0][9];
    const double c0_0_0_10 = w[0][10];
    const double c0_0_0_11 = w[0][11];
    const double c0_0_0_12 = w[0][12];
    const double c0_0_0_13 = w[0][13];
    const double c0_0_0_14 = w[0][14];
    const double c0_0_0_15 = w[0][15];
    const double c0_0_0_16 = w[0][16];
    const double c0_0_0_17 = w[0][17];
    const double c0_0_0_18 = w[0][18];
    const double c0_0_0_19 = w[0][19];
    const double c0_0_0_20 = w[0][20];
    const double c0_0_0_21 = w[0][21];
    const double c0_0_0_22 = w[0][22];
    const double c0_0_0_23 = w[0][23];
    const double c0_0_0_24 = w[0][24];
    const double c0_0_0_25 = w[0][25];
    const double c0_0_0_26 = w[0][26];
    const double c0_0_0_27 = w[0][27];
    const double c0_0_0_28 = w[0][28];
    const double c0_0_0_29 = w[0][29];
    const double c0_0_1_0 = w[0][0];
    const double c0_0_1_1 = w[0][1];
    const double c0_0_1_2 = w[0][2];
    const double c0_0_1_3 = w[0][3];
    const double c0_0_1_4 = w[0][4];
    const double c0_0_1_5 = w[0][5];
    const double c0_0_1_6 = w[0][6];
    const double c0_0_1_7 = w[0][7];
    const double c0_0_1_8 = w[0][8];
    const double c0_0_1_9 = w[0][9];
    const double c0_0_1_10 = w[0][10];
    const double c0_0_1_11 = w[0][11];
    const double c0_0_1_12 = w[0][12];
    const double c0_0_1_13 = w[0][13];
    const double c0_0_1_14 = w[0][14];
    const double c0_0_1_15 = w[0][15];
    const double c0_0_1_16 = w[0][16];
    const double c0_0_1_17 = w[0][17];
    const double c0_0_1_18 = w[0][18];
    const double c0_0_1_19 = w[0][19];
    const double c0_0_1_20 = w[0][20];
    const double c0_0_1_21 = w[0][21];
    const double c0_0_1_22 = w[0][22];
    const double c0_0_1_23 = w[0][23];
    const double c0_0_1_24 = w[0][24];
    const double c0_0_1_25 = w[0][25];
    const double c0_0_1_26 = w[0][26];
    const double c0_0_1_27 = w[0][27];
    const double c0_0_1_28 = w[0][28];
    const double c0_0_1_29 = w[0][29];
    
    // Compute geometry tensors
    // Number of operations to compute decalrations = 11392
    const double G0_0_0_0_0_0_0 = det*c0_0_0_0*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_0_0_0_0_1_0 = det*c0_0_0_0*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_0_0_0_1_0_0 = det*c0_0_0_0*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_0_0_0_2_1_0 = det*c0_0_0_0*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_0_0_0_3_0_0 = det*c0_0_0_0*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_0_0_0_3_1_0 = det*c0_0_0_0*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_0_0_0_4_0_0 = det*c0_0_0_0*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_0_0_0_4_1_0 = det*c0_0_0_0*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_0_0_0_5_0_0 = det*c0_0_0_0*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_0_0_0_5_1_0 = det*c0_0_0_0*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_0_0_0_6_0_0 = det*c0_0_0_0*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_0_0_0_6_1_0 = det*c0_0_0_0*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_0_0_0_7_0_0 = det*c0_0_0_0*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_0_0_0_7_1_0 = det*c0_0_0_0*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_0_0_0_8_0_0 = det*c0_0_0_0*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_0_0_0_8_1_0 = det*c0_0_0_0*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_0_0_0_9_0_0 = det*c0_0_0_0*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_0_0_0_9_1_0 = det*c0_0_0_0*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_0_0_0_10_0_0 = det*c0_0_0_0*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_0_0_0_10_1_0 = det*c0_0_0_0*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_0_0_0_11_0_0 = det*c0_0_0_0*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_0_0_0_11_1_0 = det*c0_0_0_0*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_0_0_0_12_0_0 = det*c0_0_0_0*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_0_0_0_12_1_0 = det*c0_0_0_0*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_0_0_0_13_0_0 = det*c0_0_0_0*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_0_0_0_13_1_0 = det*c0_0_0_0*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_0_0_0_14_0_0 = det*c0_0_0_0*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_0_0_0_14_1_0 = det*c0_0_0_0*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_0_0_0_15_0_1 = det*c0_0_0_0*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_0_0_0_15_1_1 = det*c0_0_0_0*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_0_0_0_16_0_1 = det*c0_0_0_0*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_0_0_0_17_1_1 = det*c0_0_0_0*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_0_0_0_18_0_1 = det*c0_0_0_0*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_0_0_0_18_1_1 = det*c0_0_0_0*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_0_0_0_19_0_1 = det*c0_0_0_0*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_0_0_0_19_1_1 = det*c0_0_0_0*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_0_0_0_20_0_1 = det*c0_0_0_0*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_0_0_0_20_1_1 = det*c0_0_0_0*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_0_0_0_21_0_1 = det*c0_0_0_0*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_0_0_0_21_1_1 = det*c0_0_0_0*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_0_0_0_22_0_1 = det*c0_0_0_0*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_0_0_0_22_1_1 = det*c0_0_0_0*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_0_0_0_23_0_1 = det*c0_0_0_0*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_0_0_0_23_1_1 = det*c0_0_0_0*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_0_0_0_24_0_1 = det*c0_0_0_0*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_0_0_0_24_1_1 = det*c0_0_0_0*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_0_0_0_25_0_1 = det*c0_0_0_0*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_0_0_0_25_1_1 = det*c0_0_0_0*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_0_0_0_26_0_1 = det*c0_0_0_0*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_0_0_0_26_1_1 = det*c0_0_0_0*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_0_0_0_27_0_1 = det*c0_0_0_0*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_0_0_0_27_1_1 = det*c0_0_0_0*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_0_0_0_28_0_1 = det*c0_0_0_0*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_0_0_0_28_1_1 = det*c0_0_0_0*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_0_0_0_29_0_1 = det*c0_0_0_0*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_0_0_0_29_1_1 = det*c0_0_0_0*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_0_1_0_0_0_0 = det*c0_0_0_0*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_0_1_0_0_1_0 = det*c0_0_0_0*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_0_1_0_1_0_0 = det*c0_0_0_0*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_0_1_0_2_1_0 = det*c0_0_0_0*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_0_1_0_3_0_0 = det*c0_0_0_0*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_0_1_0_3_1_0 = det*c0_0_0_0*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_0_1_0_4_0_0 = det*c0_0_0_0*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_0_1_0_4_1_0 = det*c0_0_0_0*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_0_1_0_5_0_0 = det*c0_0_0_0*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_0_1_0_5_1_0 = det*c0_0_0_0*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_0_1_0_6_0_0 = det*c0_0_0_0*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_0_1_0_6_1_0 = det*c0_0_0_0*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_0_1_0_7_0_0 = det*c0_0_0_0*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_0_1_0_7_1_0 = det*c0_0_0_0*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_0_1_0_8_0_0 = det*c0_0_0_0*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_0_1_0_8_1_0 = det*c0_0_0_0*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_0_1_0_9_0_0 = det*c0_0_0_0*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_0_1_0_9_1_0 = det*c0_0_0_0*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_0_1_0_10_0_0 = det*c0_0_0_0*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_0_1_0_10_1_0 = det*c0_0_0_0*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_0_1_0_11_0_0 = det*c0_0_0_0*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_0_1_0_11_1_0 = det*c0_0_0_0*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_0_1_0_12_0_0 = det*c0_0_0_0*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_0_1_0_12_1_0 = det*c0_0_0_0*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_0_1_0_13_0_0 = det*c0_0_0_0*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_0_1_0_13_1_0 = det*c0_0_0_0*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_0_1_0_14_0_0 = det*c0_0_0_0*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_0_1_0_14_1_0 = det*c0_0_0_0*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_0_1_0_15_0_1 = det*c0_0_0_0*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_0_1_0_15_1_1 = det*c0_0_0_0*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_0_1_0_16_0_1 = det*c0_0_0_0*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_0_1_0_17_1_1 = det*c0_0_0_0*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_0_1_0_18_0_1 = det*c0_0_0_0*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_0_1_0_18_1_1 = det*c0_0_0_0*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_0_1_0_19_0_1 = det*c0_0_0_0*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_0_1_0_19_1_1 = det*c0_0_0_0*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_0_1_0_20_0_1 = det*c0_0_0_0*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_0_1_0_20_1_1 = det*c0_0_0_0*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_0_1_0_21_0_1 = det*c0_0_0_0*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_0_1_0_21_1_1 = det*c0_0_0_0*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_0_1_0_22_0_1 = det*c0_0_0_0*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_0_1_0_22_1_1 = det*c0_0_0_0*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_0_1_0_23_0_1 = det*c0_0_0_0*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_0_1_0_23_1_1 = det*c0_0_0_0*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_0_1_0_24_0_1 = det*c0_0_0_0*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_0_1_0_24_1_1 = det*c0_0_0_0*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_0_1_0_25_0_1 = det*c0_0_0_0*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_0_1_0_25_1_1 = det*c0_0_0_0*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_0_1_0_26_0_1 = det*c0_0_0_0*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_0_1_0_26_1_1 = det*c0_0_0_0*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_0_1_0_27_0_1 = det*c0_0_0_0*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_0_1_0_27_1_1 = det*c0_0_0_0*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_0_1_0_28_0_1 = det*c0_0_0_0*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_0_1_0_28_1_1 = det*c0_0_0_0*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_0_1_0_29_0_1 = det*c0_0_0_0*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_0_1_0_29_1_1 = det*c0_0_0_0*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_1_0_0_0_0_0 = det*c0_0_0_1*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_1_0_0_0_1_0 = det*c0_0_0_1*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_1_0_0_1_0_0 = det*c0_0_0_1*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_1_0_0_2_1_0 = det*c0_0_0_1*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_1_0_0_3_0_0 = det*c0_0_0_1*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_1_0_0_3_1_0 = det*c0_0_0_1*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_1_0_0_4_0_0 = det*c0_0_0_1*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_1_0_0_4_1_0 = det*c0_0_0_1*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_1_0_0_5_0_0 = det*c0_0_0_1*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_1_0_0_5_1_0 = det*c0_0_0_1*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_1_0_0_6_0_0 = det*c0_0_0_1*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_1_0_0_7_0_0 = det*c0_0_0_1*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_1_0_0_8_0_0 = det*c0_0_0_1*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_1_0_0_9_0_0 = det*c0_0_0_1*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_1_0_0_9_1_0 = det*c0_0_0_1*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_1_0_0_10_0_0 = det*c0_0_0_1*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_1_0_0_10_1_0 = det*c0_0_0_1*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_1_0_0_11_0_0 = det*c0_0_0_1*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_1_0_0_11_1_0 = det*c0_0_0_1*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_1_0_0_12_0_0 = det*c0_0_0_1*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_1_0_0_13_0_0 = det*c0_0_0_1*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_1_0_0_14_0_0 = det*c0_0_0_1*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_1_0_0_15_0_1 = det*c0_0_0_1*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_1_0_0_15_1_1 = det*c0_0_0_1*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_1_0_0_16_0_1 = det*c0_0_0_1*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_1_0_0_17_1_1 = det*c0_0_0_1*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_1_0_0_18_0_1 = det*c0_0_0_1*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_1_0_0_18_1_1 = det*c0_0_0_1*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_1_0_0_19_0_1 = det*c0_0_0_1*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_1_0_0_19_1_1 = det*c0_0_0_1*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_1_0_0_20_0_1 = det*c0_0_0_1*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_1_0_0_20_1_1 = det*c0_0_0_1*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_1_0_0_21_0_1 = det*c0_0_0_1*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_1_0_0_22_0_1 = det*c0_0_0_1*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_1_0_0_23_0_1 = det*c0_0_0_1*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_1_0_0_24_0_1 = det*c0_0_0_1*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_1_0_0_24_1_1 = det*c0_0_0_1*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_1_0_0_25_0_1 = det*c0_0_0_1*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_1_0_0_25_1_1 = det*c0_0_0_1*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_1_0_0_26_0_1 = det*c0_0_0_1*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_1_0_0_26_1_1 = det*c0_0_0_1*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_1_0_0_27_0_1 = det*c0_0_0_1*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_1_0_0_28_0_1 = det*c0_0_0_1*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_1_0_0_29_0_1 = det*c0_0_0_1*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_2_1_0_0_0_0 = det*c0_0_0_2*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_2_1_0_0_1_0 = det*c0_0_0_2*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_2_1_0_1_0_0 = det*c0_0_0_2*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_2_1_0_2_1_0 = det*c0_0_0_2*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_2_1_0_3_0_0 = det*c0_0_0_2*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_2_1_0_3_1_0 = det*c0_0_0_2*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_2_1_0_4_0_0 = det*c0_0_0_2*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_2_1_0_4_1_0 = det*c0_0_0_2*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_2_1_0_5_0_0 = det*c0_0_0_2*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_2_1_0_5_1_0 = det*c0_0_0_2*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_2_1_0_6_0_0 = det*c0_0_0_2*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_2_1_0_6_1_0 = det*c0_0_0_2*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_2_1_0_7_0_0 = det*c0_0_0_2*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_2_1_0_7_1_0 = det*c0_0_0_2*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_2_1_0_8_0_0 = det*c0_0_0_2*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_2_1_0_8_1_0 = det*c0_0_0_2*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_2_1_0_9_1_0 = det*c0_0_0_2*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_2_1_0_10_1_0 = det*c0_0_0_2*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_2_1_0_11_1_0 = det*c0_0_0_2*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_2_1_0_12_1_0 = det*c0_0_0_2*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_2_1_0_13_1_0 = det*c0_0_0_2*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_2_1_0_14_1_0 = det*c0_0_0_2*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_2_1_0_15_0_1 = det*c0_0_0_2*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_2_1_0_15_1_1 = det*c0_0_0_2*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_2_1_0_16_0_1 = det*c0_0_0_2*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_2_1_0_17_1_1 = det*c0_0_0_2*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_2_1_0_18_0_1 = det*c0_0_0_2*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_2_1_0_18_1_1 = det*c0_0_0_2*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_2_1_0_19_0_1 = det*c0_0_0_2*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_2_1_0_19_1_1 = det*c0_0_0_2*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_2_1_0_20_0_1 = det*c0_0_0_2*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_2_1_0_20_1_1 = det*c0_0_0_2*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_2_1_0_21_0_1 = det*c0_0_0_2*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_2_1_0_21_1_1 = det*c0_0_0_2*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_2_1_0_22_0_1 = det*c0_0_0_2*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_2_1_0_22_1_1 = det*c0_0_0_2*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_2_1_0_23_0_1 = det*c0_0_0_2*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_2_1_0_23_1_1 = det*c0_0_0_2*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_2_1_0_24_1_1 = det*c0_0_0_2*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_2_1_0_25_1_1 = det*c0_0_0_2*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_2_1_0_26_1_1 = det*c0_0_0_2*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_2_1_0_27_1_1 = det*c0_0_0_2*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_2_1_0_28_1_1 = det*c0_0_0_2*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_2_1_0_29_1_1 = det*c0_0_0_2*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_3_0_0_0_0_0 = det*c0_0_0_3*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_3_0_0_0_1_0 = det*c0_0_0_3*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_3_0_0_1_0_0 = det*c0_0_0_3*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_3_0_0_2_1_0 = det*c0_0_0_3*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_3_0_0_3_0_0 = det*c0_0_0_3*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_3_0_0_3_1_0 = det*c0_0_0_3*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_3_0_0_4_0_0 = det*c0_0_0_3*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_3_0_0_4_1_0 = det*c0_0_0_3*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_3_0_0_5_0_0 = det*c0_0_0_3*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_3_0_0_5_1_0 = det*c0_0_0_3*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_3_0_0_6_0_0 = det*c0_0_0_3*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_3_0_0_6_1_0 = det*c0_0_0_3*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_3_0_0_7_0_0 = det*c0_0_0_3*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_3_0_0_7_1_0 = det*c0_0_0_3*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_3_0_0_8_0_0 = det*c0_0_0_3*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_3_0_0_8_1_0 = det*c0_0_0_3*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_3_0_0_9_0_0 = det*c0_0_0_3*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_3_0_0_9_1_0 = det*c0_0_0_3*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_3_0_0_10_0_0 = det*c0_0_0_3*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_3_0_0_10_1_0 = det*c0_0_0_3*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_3_0_0_11_0_0 = det*c0_0_0_3*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_3_0_0_11_1_0 = det*c0_0_0_3*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_3_0_0_12_0_0 = det*c0_0_0_3*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_3_0_0_12_1_0 = det*c0_0_0_3*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_3_0_0_13_0_0 = det*c0_0_0_3*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_3_0_0_13_1_0 = det*c0_0_0_3*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_3_0_0_14_0_0 = det*c0_0_0_3*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_3_0_0_14_1_0 = det*c0_0_0_3*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_3_0_0_15_0_1 = det*c0_0_0_3*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_3_0_0_15_1_1 = det*c0_0_0_3*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_3_0_0_16_0_1 = det*c0_0_0_3*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_3_0_0_17_1_1 = det*c0_0_0_3*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_3_0_0_18_0_1 = det*c0_0_0_3*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_3_0_0_18_1_1 = det*c0_0_0_3*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_3_0_0_19_0_1 = det*c0_0_0_3*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_3_0_0_19_1_1 = det*c0_0_0_3*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_3_0_0_20_0_1 = det*c0_0_0_3*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_3_0_0_20_1_1 = det*c0_0_0_3*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_3_0_0_21_0_1 = det*c0_0_0_3*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_3_0_0_21_1_1 = det*c0_0_0_3*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_3_0_0_22_0_1 = det*c0_0_0_3*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_3_0_0_22_1_1 = det*c0_0_0_3*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_3_0_0_23_0_1 = det*c0_0_0_3*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_3_0_0_23_1_1 = det*c0_0_0_3*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_3_0_0_24_0_1 = det*c0_0_0_3*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_3_0_0_24_1_1 = det*c0_0_0_3*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_3_0_0_25_0_1 = det*c0_0_0_3*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_3_0_0_25_1_1 = det*c0_0_0_3*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_3_0_0_26_0_1 = det*c0_0_0_3*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_3_0_0_26_1_1 = det*c0_0_0_3*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_3_0_0_27_0_1 = det*c0_0_0_3*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_3_0_0_27_1_1 = det*c0_0_0_3*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_3_0_0_28_0_1 = det*c0_0_0_3*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_3_0_0_28_1_1 = det*c0_0_0_3*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_3_0_0_29_0_1 = det*c0_0_0_3*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_3_0_0_29_1_1 = det*c0_0_0_3*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_3_1_0_0_0_0 = det*c0_0_0_3*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_3_1_0_0_1_0 = det*c0_0_0_3*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_3_1_0_1_0_0 = det*c0_0_0_3*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_3_1_0_2_1_0 = det*c0_0_0_3*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_3_1_0_3_0_0 = det*c0_0_0_3*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_3_1_0_3_1_0 = det*c0_0_0_3*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_3_1_0_4_0_0 = det*c0_0_0_3*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_3_1_0_4_1_0 = det*c0_0_0_3*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_3_1_0_5_0_0 = det*c0_0_0_3*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_3_1_0_5_1_0 = det*c0_0_0_3*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_3_1_0_6_0_0 = det*c0_0_0_3*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_3_1_0_7_0_0 = det*c0_0_0_3*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_3_1_0_8_0_0 = det*c0_0_0_3*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_3_1_0_9_0_0 = det*c0_0_0_3*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_3_1_0_9_1_0 = det*c0_0_0_3*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_3_1_0_10_0_0 = det*c0_0_0_3*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_3_1_0_10_1_0 = det*c0_0_0_3*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_3_1_0_11_0_0 = det*c0_0_0_3*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_3_1_0_11_1_0 = det*c0_0_0_3*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_3_1_0_12_0_0 = det*c0_0_0_3*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_3_1_0_13_0_0 = det*c0_0_0_3*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_3_1_0_14_0_0 = det*c0_0_0_3*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_3_1_0_15_0_1 = det*c0_0_0_3*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_3_1_0_15_1_1 = det*c0_0_0_3*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_3_1_0_16_0_1 = det*c0_0_0_3*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_3_1_0_17_1_1 = det*c0_0_0_3*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_3_1_0_18_0_1 = det*c0_0_0_3*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_3_1_0_18_1_1 = det*c0_0_0_3*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_3_1_0_19_0_1 = det*c0_0_0_3*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_3_1_0_19_1_1 = det*c0_0_0_3*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_3_1_0_20_0_1 = det*c0_0_0_3*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_3_1_0_20_1_1 = det*c0_0_0_3*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_3_1_0_21_0_1 = det*c0_0_0_3*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_3_1_0_22_0_1 = det*c0_0_0_3*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_3_1_0_23_0_1 = det*c0_0_0_3*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_3_1_0_24_0_1 = det*c0_0_0_3*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_3_1_0_24_1_1 = det*c0_0_0_3*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_3_1_0_25_0_1 = det*c0_0_0_3*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_3_1_0_25_1_1 = det*c0_0_0_3*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_3_1_0_26_0_1 = det*c0_0_0_3*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_3_1_0_26_1_1 = det*c0_0_0_3*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_3_1_0_27_0_1 = det*c0_0_0_3*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_3_1_0_28_0_1 = det*c0_0_0_3*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_3_1_0_29_0_1 = det*c0_0_0_3*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_4_0_0_0_0_0 = det*c0_0_0_4*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_4_0_0_0_1_0 = det*c0_0_0_4*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_4_0_0_1_0_0 = det*c0_0_0_4*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_4_0_0_2_1_0 = det*c0_0_0_4*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_4_0_0_3_0_0 = det*c0_0_0_4*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_4_0_0_3_1_0 = det*c0_0_0_4*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_4_0_0_4_0_0 = det*c0_0_0_4*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_4_0_0_4_1_0 = det*c0_0_0_4*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_4_0_0_5_0_0 = det*c0_0_0_4*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_4_0_0_5_1_0 = det*c0_0_0_4*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_4_0_0_6_0_0 = det*c0_0_0_4*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_4_0_0_6_1_0 = det*c0_0_0_4*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_4_0_0_7_0_0 = det*c0_0_0_4*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_4_0_0_7_1_0 = det*c0_0_0_4*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_4_0_0_8_0_0 = det*c0_0_0_4*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_4_0_0_8_1_0 = det*c0_0_0_4*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_4_0_0_9_0_0 = det*c0_0_0_4*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_4_0_0_9_1_0 = det*c0_0_0_4*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_4_0_0_10_0_0 = det*c0_0_0_4*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_4_0_0_10_1_0 = det*c0_0_0_4*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_4_0_0_11_0_0 = det*c0_0_0_4*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_4_0_0_11_1_0 = det*c0_0_0_4*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_4_0_0_12_0_0 = det*c0_0_0_4*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_4_0_0_12_1_0 = det*c0_0_0_4*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_4_0_0_13_0_0 = det*c0_0_0_4*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_4_0_0_13_1_0 = det*c0_0_0_4*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_4_0_0_14_0_0 = det*c0_0_0_4*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_4_0_0_14_1_0 = det*c0_0_0_4*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_4_0_0_15_0_1 = det*c0_0_0_4*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_4_0_0_15_1_1 = det*c0_0_0_4*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_4_0_0_16_0_1 = det*c0_0_0_4*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_4_0_0_17_1_1 = det*c0_0_0_4*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_4_0_0_18_0_1 = det*c0_0_0_4*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_4_0_0_18_1_1 = det*c0_0_0_4*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_4_0_0_19_0_1 = det*c0_0_0_4*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_4_0_0_19_1_1 = det*c0_0_0_4*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_4_0_0_20_0_1 = det*c0_0_0_4*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_4_0_0_20_1_1 = det*c0_0_0_4*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_4_0_0_21_0_1 = det*c0_0_0_4*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_4_0_0_21_1_1 = det*c0_0_0_4*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_4_0_0_22_0_1 = det*c0_0_0_4*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_4_0_0_22_1_1 = det*c0_0_0_4*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_4_0_0_23_0_1 = det*c0_0_0_4*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_4_0_0_23_1_1 = det*c0_0_0_4*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_4_0_0_24_0_1 = det*c0_0_0_4*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_4_0_0_24_1_1 = det*c0_0_0_4*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_4_0_0_25_0_1 = det*c0_0_0_4*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_4_0_0_25_1_1 = det*c0_0_0_4*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_4_0_0_26_0_1 = det*c0_0_0_4*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_4_0_0_26_1_1 = det*c0_0_0_4*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_4_0_0_27_0_1 = det*c0_0_0_4*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_4_0_0_27_1_1 = det*c0_0_0_4*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_4_0_0_28_0_1 = det*c0_0_0_4*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_4_0_0_28_1_1 = det*c0_0_0_4*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_4_0_0_29_0_1 = det*c0_0_0_4*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_4_0_0_29_1_1 = det*c0_0_0_4*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_4_1_0_0_0_0 = det*c0_0_0_4*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_4_1_0_0_1_0 = det*c0_0_0_4*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_4_1_0_1_0_0 = det*c0_0_0_4*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_4_1_0_2_1_0 = det*c0_0_0_4*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_4_1_0_3_0_0 = det*c0_0_0_4*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_4_1_0_3_1_0 = det*c0_0_0_4*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_4_1_0_4_0_0 = det*c0_0_0_4*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_4_1_0_4_1_0 = det*c0_0_0_4*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_4_1_0_5_0_0 = det*c0_0_0_4*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_4_1_0_5_1_0 = det*c0_0_0_4*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_4_1_0_6_0_0 = det*c0_0_0_4*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_4_1_0_6_1_0 = det*c0_0_0_4*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_4_1_0_7_0_0 = det*c0_0_0_4*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_4_1_0_7_1_0 = det*c0_0_0_4*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_4_1_0_8_0_0 = det*c0_0_0_4*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_4_1_0_8_1_0 = det*c0_0_0_4*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_4_1_0_9_0_0 = det*c0_0_0_4*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_4_1_0_9_1_0 = det*c0_0_0_4*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_4_1_0_10_0_0 = det*c0_0_0_4*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_4_1_0_10_1_0 = det*c0_0_0_4*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_4_1_0_11_0_0 = det*c0_0_0_4*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_4_1_0_11_1_0 = det*c0_0_0_4*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_4_1_0_12_0_0 = det*c0_0_0_4*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_4_1_0_12_1_0 = det*c0_0_0_4*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_4_1_0_13_0_0 = det*c0_0_0_4*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_4_1_0_13_1_0 = det*c0_0_0_4*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_4_1_0_14_0_0 = det*c0_0_0_4*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_4_1_0_14_1_0 = det*c0_0_0_4*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_4_1_0_15_0_1 = det*c0_0_0_4*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_4_1_0_15_1_1 = det*c0_0_0_4*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_4_1_0_16_0_1 = det*c0_0_0_4*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_4_1_0_17_1_1 = det*c0_0_0_4*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_4_1_0_18_0_1 = det*c0_0_0_4*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_4_1_0_18_1_1 = det*c0_0_0_4*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_4_1_0_19_0_1 = det*c0_0_0_4*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_4_1_0_19_1_1 = det*c0_0_0_4*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_4_1_0_20_0_1 = det*c0_0_0_4*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_4_1_0_20_1_1 = det*c0_0_0_4*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_4_1_0_21_0_1 = det*c0_0_0_4*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_4_1_0_21_1_1 = det*c0_0_0_4*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_4_1_0_22_0_1 = det*c0_0_0_4*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_4_1_0_22_1_1 = det*c0_0_0_4*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_4_1_0_23_0_1 = det*c0_0_0_4*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_4_1_0_23_1_1 = det*c0_0_0_4*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_4_1_0_24_0_1 = det*c0_0_0_4*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_4_1_0_24_1_1 = det*c0_0_0_4*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_4_1_0_25_0_1 = det*c0_0_0_4*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_4_1_0_25_1_1 = det*c0_0_0_4*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_4_1_0_26_0_1 = det*c0_0_0_4*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_4_1_0_26_1_1 = det*c0_0_0_4*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_4_1_0_27_0_1 = det*c0_0_0_4*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_4_1_0_27_1_1 = det*c0_0_0_4*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_4_1_0_28_0_1 = det*c0_0_0_4*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_4_1_0_28_1_1 = det*c0_0_0_4*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_4_1_0_29_0_1 = det*c0_0_0_4*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_4_1_0_29_1_1 = det*c0_0_0_4*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_5_0_0_0_0_0 = det*c0_0_0_5*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_5_0_0_0_1_0 = det*c0_0_0_5*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_5_0_0_1_0_0 = det*c0_0_0_5*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_5_0_0_2_1_0 = det*c0_0_0_5*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_5_0_0_3_0_0 = det*c0_0_0_5*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_5_0_0_3_1_0 = det*c0_0_0_5*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_5_0_0_4_0_0 = det*c0_0_0_5*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_5_0_0_4_1_0 = det*c0_0_0_5*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_5_0_0_5_0_0 = det*c0_0_0_5*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_5_0_0_5_1_0 = det*c0_0_0_5*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_5_0_0_6_0_0 = det*c0_0_0_5*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_5_0_0_6_1_0 = det*c0_0_0_5*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_5_0_0_7_0_0 = det*c0_0_0_5*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_5_0_0_7_1_0 = det*c0_0_0_5*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_5_0_0_8_0_0 = det*c0_0_0_5*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_5_0_0_8_1_0 = det*c0_0_0_5*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_5_0_0_9_1_0 = det*c0_0_0_5*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_5_0_0_10_1_0 = det*c0_0_0_5*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_5_0_0_11_1_0 = det*c0_0_0_5*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_5_0_0_12_1_0 = det*c0_0_0_5*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_5_0_0_13_1_0 = det*c0_0_0_5*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_5_0_0_14_1_0 = det*c0_0_0_5*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_5_0_0_15_0_1 = det*c0_0_0_5*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_5_0_0_15_1_1 = det*c0_0_0_5*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_5_0_0_16_0_1 = det*c0_0_0_5*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_5_0_0_17_1_1 = det*c0_0_0_5*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_5_0_0_18_0_1 = det*c0_0_0_5*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_5_0_0_18_1_1 = det*c0_0_0_5*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_5_0_0_19_0_1 = det*c0_0_0_5*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_5_0_0_19_1_1 = det*c0_0_0_5*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_5_0_0_20_0_1 = det*c0_0_0_5*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_5_0_0_20_1_1 = det*c0_0_0_5*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_5_0_0_21_0_1 = det*c0_0_0_5*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_5_0_0_21_1_1 = det*c0_0_0_5*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_5_0_0_22_0_1 = det*c0_0_0_5*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_5_0_0_22_1_1 = det*c0_0_0_5*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_5_0_0_23_0_1 = det*c0_0_0_5*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_5_0_0_23_1_1 = det*c0_0_0_5*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_5_0_0_24_1_1 = det*c0_0_0_5*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_5_0_0_25_1_1 = det*c0_0_0_5*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_5_0_0_26_1_1 = det*c0_0_0_5*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_5_0_0_27_1_1 = det*c0_0_0_5*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_5_0_0_28_1_1 = det*c0_0_0_5*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_5_0_0_29_1_1 = det*c0_0_0_5*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_5_1_0_0_0_0 = det*c0_0_0_5*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_5_1_0_0_1_0 = det*c0_0_0_5*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_5_1_0_1_0_0 = det*c0_0_0_5*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_5_1_0_2_1_0 = det*c0_0_0_5*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_5_1_0_3_0_0 = det*c0_0_0_5*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_5_1_0_3_1_0 = det*c0_0_0_5*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_5_1_0_4_0_0 = det*c0_0_0_5*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_5_1_0_4_1_0 = det*c0_0_0_5*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_5_1_0_5_0_0 = det*c0_0_0_5*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_5_1_0_5_1_0 = det*c0_0_0_5*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_5_1_0_6_0_0 = det*c0_0_0_5*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_5_1_0_6_1_0 = det*c0_0_0_5*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_5_1_0_7_0_0 = det*c0_0_0_5*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_5_1_0_7_1_0 = det*c0_0_0_5*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_5_1_0_8_0_0 = det*c0_0_0_5*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_5_1_0_8_1_0 = det*c0_0_0_5*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_5_1_0_9_0_0 = det*c0_0_0_5*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_5_1_0_9_1_0 = det*c0_0_0_5*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_5_1_0_10_0_0 = det*c0_0_0_5*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_5_1_0_10_1_0 = det*c0_0_0_5*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_5_1_0_11_0_0 = det*c0_0_0_5*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_5_1_0_11_1_0 = det*c0_0_0_5*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_5_1_0_12_0_0 = det*c0_0_0_5*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_5_1_0_12_1_0 = det*c0_0_0_5*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_5_1_0_13_0_0 = det*c0_0_0_5*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_5_1_0_13_1_0 = det*c0_0_0_5*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_5_1_0_14_0_0 = det*c0_0_0_5*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_5_1_0_14_1_0 = det*c0_0_0_5*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_5_1_0_15_0_1 = det*c0_0_0_5*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_5_1_0_15_1_1 = det*c0_0_0_5*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_5_1_0_16_0_1 = det*c0_0_0_5*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_5_1_0_17_1_1 = det*c0_0_0_5*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_5_1_0_18_0_1 = det*c0_0_0_5*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_5_1_0_18_1_1 = det*c0_0_0_5*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_5_1_0_19_0_1 = det*c0_0_0_5*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_5_1_0_19_1_1 = det*c0_0_0_5*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_5_1_0_20_0_1 = det*c0_0_0_5*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_5_1_0_20_1_1 = det*c0_0_0_5*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_5_1_0_21_0_1 = det*c0_0_0_5*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_5_1_0_21_1_1 = det*c0_0_0_5*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_5_1_0_22_0_1 = det*c0_0_0_5*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_5_1_0_22_1_1 = det*c0_0_0_5*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_5_1_0_23_0_1 = det*c0_0_0_5*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_5_1_0_23_1_1 = det*c0_0_0_5*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_5_1_0_24_0_1 = det*c0_0_0_5*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_5_1_0_24_1_1 = det*c0_0_0_5*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_5_1_0_25_0_1 = det*c0_0_0_5*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_5_1_0_25_1_1 = det*c0_0_0_5*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_5_1_0_26_0_1 = det*c0_0_0_5*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_5_1_0_26_1_1 = det*c0_0_0_5*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_5_1_0_27_0_1 = det*c0_0_0_5*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_5_1_0_27_1_1 = det*c0_0_0_5*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_5_1_0_28_0_1 = det*c0_0_0_5*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_5_1_0_28_1_1 = det*c0_0_0_5*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_5_1_0_29_0_1 = det*c0_0_0_5*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_5_1_0_29_1_1 = det*c0_0_0_5*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_6_0_0_0_0_0 = det*c0_0_0_6*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_6_0_0_0_1_0 = det*c0_0_0_6*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_6_0_0_1_0_0 = det*c0_0_0_6*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_6_0_0_2_1_0 = det*c0_0_0_6*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_6_0_0_3_0_0 = det*c0_0_0_6*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_6_0_0_3_1_0 = det*c0_0_0_6*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_6_0_0_4_0_0 = det*c0_0_0_6*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_6_0_0_4_1_0 = det*c0_0_0_6*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_6_0_0_5_0_0 = det*c0_0_0_6*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_6_0_0_5_1_0 = det*c0_0_0_6*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_6_0_0_6_0_0 = det*c0_0_0_6*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_6_0_0_6_1_0 = det*c0_0_0_6*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_6_0_0_7_0_0 = det*c0_0_0_6*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_6_0_0_7_1_0 = det*c0_0_0_6*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_6_0_0_8_0_0 = det*c0_0_0_6*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_6_0_0_8_1_0 = det*c0_0_0_6*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_6_0_0_9_0_0 = det*c0_0_0_6*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_6_0_0_9_1_0 = det*c0_0_0_6*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_6_0_0_10_0_0 = det*c0_0_0_6*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_6_0_0_10_1_0 = det*c0_0_0_6*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_6_0_0_11_0_0 = det*c0_0_0_6*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_6_0_0_11_1_0 = det*c0_0_0_6*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_6_0_0_12_0_0 = det*c0_0_0_6*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_6_0_0_12_1_0 = det*c0_0_0_6*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_6_0_0_13_0_0 = det*c0_0_0_6*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_6_0_0_13_1_0 = det*c0_0_0_6*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_6_0_0_14_0_0 = det*c0_0_0_6*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_6_0_0_14_1_0 = det*c0_0_0_6*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_6_0_0_15_0_1 = det*c0_0_0_6*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_6_0_0_15_1_1 = det*c0_0_0_6*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_6_0_0_16_0_1 = det*c0_0_0_6*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_6_0_0_17_1_1 = det*c0_0_0_6*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_6_0_0_18_0_1 = det*c0_0_0_6*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_6_0_0_18_1_1 = det*c0_0_0_6*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_6_0_0_19_0_1 = det*c0_0_0_6*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_6_0_0_19_1_1 = det*c0_0_0_6*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_6_0_0_20_0_1 = det*c0_0_0_6*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_6_0_0_20_1_1 = det*c0_0_0_6*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_6_0_0_21_0_1 = det*c0_0_0_6*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_6_0_0_21_1_1 = det*c0_0_0_6*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_6_0_0_22_0_1 = det*c0_0_0_6*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_6_0_0_22_1_1 = det*c0_0_0_6*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_6_0_0_23_0_1 = det*c0_0_0_6*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_6_0_0_23_1_1 = det*c0_0_0_6*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_6_0_0_24_0_1 = det*c0_0_0_6*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_6_0_0_24_1_1 = det*c0_0_0_6*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_6_0_0_25_0_1 = det*c0_0_0_6*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_6_0_0_25_1_1 = det*c0_0_0_6*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_6_0_0_26_0_1 = det*c0_0_0_6*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_6_0_0_26_1_1 = det*c0_0_0_6*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_6_0_0_27_0_1 = det*c0_0_0_6*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_6_0_0_27_1_1 = det*c0_0_0_6*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_6_0_0_28_0_1 = det*c0_0_0_6*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_6_0_0_28_1_1 = det*c0_0_0_6*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_6_0_0_29_0_1 = det*c0_0_0_6*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_6_0_0_29_1_1 = det*c0_0_0_6*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_6_1_0_0_0_0 = det*c0_0_0_6*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_6_1_0_0_1_0 = det*c0_0_0_6*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_6_1_0_2_1_0 = det*c0_0_0_6*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_6_1_0_3_0_0 = det*c0_0_0_6*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_6_1_0_4_0_0 = det*c0_0_0_6*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_6_1_0_4_1_0 = det*c0_0_0_6*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_6_1_0_5_0_0 = det*c0_0_0_6*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_6_1_0_5_1_0 = det*c0_0_0_6*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_6_1_0_6_0_0 = det*c0_0_0_6*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_6_1_0_6_1_0 = det*c0_0_0_6*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_6_1_0_7_0_0 = det*c0_0_0_6*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_6_1_0_7_1_0 = det*c0_0_0_6*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_6_1_0_8_0_0 = det*c0_0_0_6*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_6_1_0_8_1_0 = det*c0_0_0_6*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_6_1_0_9_0_0 = det*c0_0_0_6*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_6_1_0_9_1_0 = det*c0_0_0_6*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_6_1_0_10_0_0 = det*c0_0_0_6*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_6_1_0_10_1_0 = det*c0_0_0_6*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_6_1_0_11_0_0 = det*c0_0_0_6*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_6_1_0_12_0_0 = det*c0_0_0_6*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_6_1_0_12_1_0 = det*c0_0_0_6*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_6_1_0_13_0_0 = det*c0_0_0_6*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_6_1_0_13_1_0 = det*c0_0_0_6*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_6_1_0_14_0_0 = det*c0_0_0_6*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_6_1_0_14_1_0 = det*c0_0_0_6*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_6_1_0_15_0_1 = det*c0_0_0_6*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_6_1_0_15_1_1 = det*c0_0_0_6*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_6_1_0_17_1_1 = det*c0_0_0_6*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_6_1_0_18_0_1 = det*c0_0_0_6*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_6_1_0_19_0_1 = det*c0_0_0_6*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_6_1_0_19_1_1 = det*c0_0_0_6*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_6_1_0_20_0_1 = det*c0_0_0_6*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_6_1_0_20_1_1 = det*c0_0_0_6*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_6_1_0_21_0_1 = det*c0_0_0_6*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_6_1_0_21_1_1 = det*c0_0_0_6*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_6_1_0_22_0_1 = det*c0_0_0_6*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_6_1_0_22_1_1 = det*c0_0_0_6*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_6_1_0_23_0_1 = det*c0_0_0_6*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_6_1_0_23_1_1 = det*c0_0_0_6*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_6_1_0_24_0_1 = det*c0_0_0_6*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_6_1_0_24_1_1 = det*c0_0_0_6*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_6_1_0_25_0_1 = det*c0_0_0_6*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_6_1_0_25_1_1 = det*c0_0_0_6*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_6_1_0_26_0_1 = det*c0_0_0_6*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_6_1_0_27_0_1 = det*c0_0_0_6*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_6_1_0_27_1_1 = det*c0_0_0_6*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_6_1_0_28_0_1 = det*c0_0_0_6*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_6_1_0_28_1_1 = det*c0_0_0_6*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_6_1_0_29_0_1 = det*c0_0_0_6*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_6_1_0_29_1_1 = det*c0_0_0_6*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_7_0_0_0_0_0 = det*c0_0_0_7*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_7_0_0_0_1_0 = det*c0_0_0_7*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_7_0_0_1_0_0 = det*c0_0_0_7*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_7_0_0_2_1_0 = det*c0_0_0_7*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_7_0_0_3_0_0 = det*c0_0_0_7*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_7_0_0_3_1_0 = det*c0_0_0_7*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_7_0_0_4_0_0 = det*c0_0_0_7*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_7_0_0_4_1_0 = det*c0_0_0_7*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_7_0_0_5_0_0 = det*c0_0_0_7*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_7_0_0_5_1_0 = det*c0_0_0_7*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_7_0_0_6_0_0 = det*c0_0_0_7*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_7_0_0_6_1_0 = det*c0_0_0_7*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_7_0_0_7_0_0 = det*c0_0_0_7*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_7_0_0_7_1_0 = det*c0_0_0_7*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_7_0_0_8_0_0 = det*c0_0_0_7*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_7_0_0_8_1_0 = det*c0_0_0_7*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_7_0_0_9_0_0 = det*c0_0_0_7*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_7_0_0_9_1_0 = det*c0_0_0_7*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_7_0_0_10_0_0 = det*c0_0_0_7*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_7_0_0_10_1_0 = det*c0_0_0_7*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_7_0_0_11_0_0 = det*c0_0_0_7*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_7_0_0_11_1_0 = det*c0_0_0_7*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_7_0_0_12_0_0 = det*c0_0_0_7*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_7_0_0_12_1_0 = det*c0_0_0_7*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_7_0_0_13_0_0 = det*c0_0_0_7*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_7_0_0_13_1_0 = det*c0_0_0_7*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_7_0_0_14_0_0 = det*c0_0_0_7*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_7_0_0_14_1_0 = det*c0_0_0_7*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_7_0_0_15_0_1 = det*c0_0_0_7*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_7_0_0_15_1_1 = det*c0_0_0_7*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_7_0_0_16_0_1 = det*c0_0_0_7*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_7_0_0_17_1_1 = det*c0_0_0_7*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_7_0_0_18_0_1 = det*c0_0_0_7*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_7_0_0_18_1_1 = det*c0_0_0_7*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_7_0_0_19_0_1 = det*c0_0_0_7*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_7_0_0_19_1_1 = det*c0_0_0_7*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_7_0_0_20_0_1 = det*c0_0_0_7*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_7_0_0_20_1_1 = det*c0_0_0_7*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_7_0_0_21_0_1 = det*c0_0_0_7*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_7_0_0_21_1_1 = det*c0_0_0_7*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_7_0_0_22_0_1 = det*c0_0_0_7*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_7_0_0_22_1_1 = det*c0_0_0_7*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_7_0_0_23_0_1 = det*c0_0_0_7*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_7_0_0_23_1_1 = det*c0_0_0_7*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_7_0_0_24_0_1 = det*c0_0_0_7*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_7_0_0_24_1_1 = det*c0_0_0_7*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_7_0_0_25_0_1 = det*c0_0_0_7*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_7_0_0_25_1_1 = det*c0_0_0_7*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_7_0_0_26_0_1 = det*c0_0_0_7*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_7_0_0_26_1_1 = det*c0_0_0_7*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_7_0_0_27_0_1 = det*c0_0_0_7*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_7_0_0_27_1_1 = det*c0_0_0_7*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_7_0_0_28_0_1 = det*c0_0_0_7*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_7_0_0_28_1_1 = det*c0_0_0_7*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_7_0_0_29_0_1 = det*c0_0_0_7*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_7_0_0_29_1_1 = det*c0_0_0_7*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_7_1_0_0_0_0 = det*c0_0_0_7*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_7_1_0_0_1_0 = det*c0_0_0_7*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_7_1_0_2_1_0 = det*c0_0_0_7*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_7_1_0_3_0_0 = det*c0_0_0_7*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_7_1_0_4_0_0 = det*c0_0_0_7*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_7_1_0_4_1_0 = det*c0_0_0_7*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_7_1_0_5_0_0 = det*c0_0_0_7*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_7_1_0_5_1_0 = det*c0_0_0_7*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_7_1_0_6_0_0 = det*c0_0_0_7*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_7_1_0_6_1_0 = det*c0_0_0_7*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_7_1_0_7_0_0 = det*c0_0_0_7*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_7_1_0_7_1_0 = det*c0_0_0_7*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_7_1_0_8_0_0 = det*c0_0_0_7*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_7_1_0_8_1_0 = det*c0_0_0_7*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_7_1_0_9_0_0 = det*c0_0_0_7*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_7_1_0_9_1_0 = det*c0_0_0_7*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_7_1_0_10_0_0 = det*c0_0_0_7*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_7_1_0_10_1_0 = det*c0_0_0_7*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_7_1_0_11_0_0 = det*c0_0_0_7*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_7_1_0_12_0_0 = det*c0_0_0_7*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_7_1_0_12_1_0 = det*c0_0_0_7*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_7_1_0_13_0_0 = det*c0_0_0_7*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_7_1_0_13_1_0 = det*c0_0_0_7*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_7_1_0_14_0_0 = det*c0_0_0_7*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_7_1_0_14_1_0 = det*c0_0_0_7*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_7_1_0_15_0_1 = det*c0_0_0_7*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_7_1_0_15_1_1 = det*c0_0_0_7*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_7_1_0_17_1_1 = det*c0_0_0_7*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_7_1_0_18_0_1 = det*c0_0_0_7*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_7_1_0_19_0_1 = det*c0_0_0_7*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_7_1_0_19_1_1 = det*c0_0_0_7*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_7_1_0_20_0_1 = det*c0_0_0_7*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_7_1_0_20_1_1 = det*c0_0_0_7*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_7_1_0_21_0_1 = det*c0_0_0_7*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_7_1_0_21_1_1 = det*c0_0_0_7*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_7_1_0_22_0_1 = det*c0_0_0_7*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_7_1_0_22_1_1 = det*c0_0_0_7*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_7_1_0_23_0_1 = det*c0_0_0_7*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_7_1_0_23_1_1 = det*c0_0_0_7*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_7_1_0_24_0_1 = det*c0_0_0_7*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_7_1_0_24_1_1 = det*c0_0_0_7*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_7_1_0_25_0_1 = det*c0_0_0_7*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_7_1_0_25_1_1 = det*c0_0_0_7*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_7_1_0_26_0_1 = det*c0_0_0_7*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_7_1_0_27_0_1 = det*c0_0_0_7*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_7_1_0_27_1_1 = det*c0_0_0_7*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_7_1_0_28_0_1 = det*c0_0_0_7*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_7_1_0_28_1_1 = det*c0_0_0_7*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_7_1_0_29_0_1 = det*c0_0_0_7*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_7_1_0_29_1_1 = det*c0_0_0_7*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_8_0_0_0_0_0 = det*c0_0_0_8*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_8_0_0_0_1_0 = det*c0_0_0_8*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_8_0_0_1_0_0 = det*c0_0_0_8*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_8_0_0_2_1_0 = det*c0_0_0_8*c0_0_1_2*Jinv_00*Jinv_10;
    const double G0_8_0_0_3_0_0 = det*c0_0_0_8*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_8_0_0_3_1_0 = det*c0_0_0_8*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_8_0_0_4_0_0 = det*c0_0_0_8*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_8_0_0_4_1_0 = det*c0_0_0_8*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_8_0_0_5_0_0 = det*c0_0_0_8*c0_0_1_5*Jinv_00*Jinv_00;
    const double G0_8_0_0_5_1_0 = det*c0_0_0_8*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_8_0_0_6_0_0 = det*c0_0_0_8*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_8_0_0_6_1_0 = det*c0_0_0_8*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_8_0_0_7_0_0 = det*c0_0_0_8*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_8_0_0_7_1_0 = det*c0_0_0_8*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_8_0_0_8_0_0 = det*c0_0_0_8*c0_0_1_8*Jinv_00*Jinv_00;
    const double G0_8_0_0_8_1_0 = det*c0_0_0_8*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_8_0_0_9_1_0 = det*c0_0_0_8*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_8_0_0_10_1_0 = det*c0_0_0_8*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_8_0_0_11_1_0 = det*c0_0_0_8*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_8_0_0_12_1_0 = det*c0_0_0_8*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_8_0_0_13_1_0 = det*c0_0_0_8*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_8_0_0_14_1_0 = det*c0_0_0_8*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_8_0_0_15_0_1 = det*c0_0_0_8*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_8_0_0_15_1_1 = det*c0_0_0_8*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_8_0_0_16_0_1 = det*c0_0_0_8*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_8_0_0_17_1_1 = det*c0_0_0_8*c0_0_1_17*Jinv_00*Jinv_11;
    const double G0_8_0_0_18_0_1 = det*c0_0_0_8*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_8_0_0_18_1_1 = det*c0_0_0_8*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_8_0_0_19_0_1 = det*c0_0_0_8*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_8_0_0_19_1_1 = det*c0_0_0_8*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_8_0_0_20_0_1 = det*c0_0_0_8*c0_0_1_20*Jinv_00*Jinv_01;
    const double G0_8_0_0_20_1_1 = det*c0_0_0_8*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_8_0_0_21_0_1 = det*c0_0_0_8*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_8_0_0_21_1_1 = det*c0_0_0_8*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_8_0_0_22_0_1 = det*c0_0_0_8*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_8_0_0_22_1_1 = det*c0_0_0_8*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_8_0_0_23_0_1 = det*c0_0_0_8*c0_0_1_23*Jinv_00*Jinv_01;
    const double G0_8_0_0_23_1_1 = det*c0_0_0_8*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_8_0_0_24_1_1 = det*c0_0_0_8*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_8_0_0_25_1_1 = det*c0_0_0_8*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_8_0_0_26_1_1 = det*c0_0_0_8*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_8_0_0_27_1_1 = det*c0_0_0_8*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_8_0_0_28_1_1 = det*c0_0_0_8*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_8_0_0_29_1_1 = det*c0_0_0_8*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_8_1_0_0_0_0 = det*c0_0_0_8*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_8_1_0_0_1_0 = det*c0_0_0_8*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_8_1_0_2_1_0 = det*c0_0_0_8*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_8_1_0_3_0_0 = det*c0_0_0_8*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_8_1_0_4_0_0 = det*c0_0_0_8*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_8_1_0_4_1_0 = det*c0_0_0_8*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_8_1_0_5_0_0 = det*c0_0_0_8*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_8_1_0_5_1_0 = det*c0_0_0_8*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_8_1_0_6_0_0 = det*c0_0_0_8*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_8_1_0_6_1_0 = det*c0_0_0_8*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_8_1_0_7_0_0 = det*c0_0_0_8*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_8_1_0_7_1_0 = det*c0_0_0_8*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_8_1_0_8_0_0 = det*c0_0_0_8*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_8_1_0_8_1_0 = det*c0_0_0_8*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_8_1_0_9_0_0 = det*c0_0_0_8*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_8_1_0_9_1_0 = det*c0_0_0_8*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_8_1_0_10_0_0 = det*c0_0_0_8*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_8_1_0_10_1_0 = det*c0_0_0_8*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_8_1_0_11_0_0 = det*c0_0_0_8*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_8_1_0_12_0_0 = det*c0_0_0_8*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_8_1_0_12_1_0 = det*c0_0_0_8*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_8_1_0_13_0_0 = det*c0_0_0_8*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_8_1_0_13_1_0 = det*c0_0_0_8*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_8_1_0_14_0_0 = det*c0_0_0_8*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_8_1_0_14_1_0 = det*c0_0_0_8*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_8_1_0_15_0_1 = det*c0_0_0_8*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_8_1_0_15_1_1 = det*c0_0_0_8*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_8_1_0_17_1_1 = det*c0_0_0_8*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_8_1_0_18_0_1 = det*c0_0_0_8*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_8_1_0_19_0_1 = det*c0_0_0_8*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_8_1_0_19_1_1 = det*c0_0_0_8*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_8_1_0_20_0_1 = det*c0_0_0_8*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_8_1_0_20_1_1 = det*c0_0_0_8*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_8_1_0_21_0_1 = det*c0_0_0_8*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_8_1_0_21_1_1 = det*c0_0_0_8*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_8_1_0_22_0_1 = det*c0_0_0_8*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_8_1_0_22_1_1 = det*c0_0_0_8*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_8_1_0_23_0_1 = det*c0_0_0_8*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_8_1_0_23_1_1 = det*c0_0_0_8*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_8_1_0_24_0_1 = det*c0_0_0_8*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_8_1_0_24_1_1 = det*c0_0_0_8*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_8_1_0_25_0_1 = det*c0_0_0_8*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_8_1_0_25_1_1 = det*c0_0_0_8*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_8_1_0_26_0_1 = det*c0_0_0_8*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_8_1_0_27_0_1 = det*c0_0_0_8*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_8_1_0_27_1_1 = det*c0_0_0_8*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_8_1_0_28_0_1 = det*c0_0_0_8*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_8_1_0_28_1_1 = det*c0_0_0_8*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_8_1_0_29_0_1 = det*c0_0_0_8*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_8_1_0_29_1_1 = det*c0_0_0_8*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_9_0_0_0_0_0 = det*c0_0_0_9*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_9_0_0_0_1_0 = det*c0_0_0_9*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_9_0_0_1_0_0 = det*c0_0_0_9*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_9_0_0_3_0_0 = det*c0_0_0_9*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_9_0_0_3_1_0 = det*c0_0_0_9*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_9_0_0_4_0_0 = det*c0_0_0_9*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_9_0_0_4_1_0 = det*c0_0_0_9*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_9_0_0_5_1_0 = det*c0_0_0_9*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_9_0_0_6_0_0 = det*c0_0_0_9*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_9_0_0_6_1_0 = det*c0_0_0_9*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_9_0_0_7_0_0 = det*c0_0_0_9*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_9_0_0_7_1_0 = det*c0_0_0_9*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_9_0_0_8_1_0 = det*c0_0_0_9*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_9_0_0_9_0_0 = det*c0_0_0_9*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_9_0_0_9_1_0 = det*c0_0_0_9*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_9_0_0_10_0_0 = det*c0_0_0_9*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_9_0_0_10_1_0 = det*c0_0_0_9*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_9_0_0_11_0_0 = det*c0_0_0_9*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_9_0_0_11_1_0 = det*c0_0_0_9*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_9_0_0_12_0_0 = det*c0_0_0_9*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_9_0_0_12_1_0 = det*c0_0_0_9*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_9_0_0_13_0_0 = det*c0_0_0_9*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_9_0_0_13_1_0 = det*c0_0_0_9*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_9_0_0_14_0_0 = det*c0_0_0_9*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_9_0_0_14_1_0 = det*c0_0_0_9*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_9_0_0_15_0_1 = det*c0_0_0_9*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_9_0_0_15_1_1 = det*c0_0_0_9*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_9_0_0_16_0_1 = det*c0_0_0_9*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_9_0_0_18_0_1 = det*c0_0_0_9*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_9_0_0_18_1_1 = det*c0_0_0_9*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_9_0_0_19_0_1 = det*c0_0_0_9*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_9_0_0_19_1_1 = det*c0_0_0_9*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_9_0_0_20_1_1 = det*c0_0_0_9*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_9_0_0_21_0_1 = det*c0_0_0_9*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_9_0_0_21_1_1 = det*c0_0_0_9*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_9_0_0_22_0_1 = det*c0_0_0_9*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_9_0_0_22_1_1 = det*c0_0_0_9*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_9_0_0_23_1_1 = det*c0_0_0_9*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_9_0_0_24_0_1 = det*c0_0_0_9*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_9_0_0_24_1_1 = det*c0_0_0_9*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_9_0_0_25_0_1 = det*c0_0_0_9*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_9_0_0_25_1_1 = det*c0_0_0_9*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_9_0_0_26_0_1 = det*c0_0_0_9*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_9_0_0_26_1_1 = det*c0_0_0_9*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_9_0_0_27_0_1 = det*c0_0_0_9*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_9_0_0_27_1_1 = det*c0_0_0_9*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_9_0_0_28_0_1 = det*c0_0_0_9*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_9_0_0_28_1_1 = det*c0_0_0_9*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_9_0_0_29_0_1 = det*c0_0_0_9*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_9_0_0_29_1_1 = det*c0_0_0_9*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_9_1_0_0_0_0 = det*c0_0_0_9*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_9_1_0_0_1_0 = det*c0_0_0_9*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_9_1_0_1_0_0 = det*c0_0_0_9*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_9_1_0_2_1_0 = det*c0_0_0_9*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_9_1_0_3_0_0 = det*c0_0_0_9*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_9_1_0_3_1_0 = det*c0_0_0_9*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_9_1_0_4_0_0 = det*c0_0_0_9*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_9_1_0_4_1_0 = det*c0_0_0_9*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_9_1_0_5_0_0 = det*c0_0_0_9*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_9_1_0_5_1_0 = det*c0_0_0_9*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_9_1_0_6_0_0 = det*c0_0_0_9*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_9_1_0_6_1_0 = det*c0_0_0_9*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_9_1_0_7_0_0 = det*c0_0_0_9*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_9_1_0_7_1_0 = det*c0_0_0_9*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_9_1_0_8_0_0 = det*c0_0_0_9*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_9_1_0_8_1_0 = det*c0_0_0_9*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_9_1_0_9_0_0 = det*c0_0_0_9*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_9_1_0_9_1_0 = det*c0_0_0_9*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_9_1_0_10_0_0 = det*c0_0_0_9*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_9_1_0_10_1_0 = det*c0_0_0_9*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_9_1_0_11_0_0 = det*c0_0_0_9*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_9_1_0_11_1_0 = det*c0_0_0_9*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_9_1_0_12_0_0 = det*c0_0_0_9*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_9_1_0_12_1_0 = det*c0_0_0_9*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_9_1_0_13_0_0 = det*c0_0_0_9*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_9_1_0_13_1_0 = det*c0_0_0_9*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_9_1_0_14_0_0 = det*c0_0_0_9*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_9_1_0_14_1_0 = det*c0_0_0_9*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_9_1_0_15_0_1 = det*c0_0_0_9*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_9_1_0_15_1_1 = det*c0_0_0_9*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_9_1_0_16_0_1 = det*c0_0_0_9*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_9_1_0_17_1_1 = det*c0_0_0_9*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_9_1_0_18_0_1 = det*c0_0_0_9*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_9_1_0_18_1_1 = det*c0_0_0_9*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_9_1_0_19_0_1 = det*c0_0_0_9*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_9_1_0_19_1_1 = det*c0_0_0_9*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_9_1_0_20_0_1 = det*c0_0_0_9*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_9_1_0_20_1_1 = det*c0_0_0_9*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_9_1_0_21_0_1 = det*c0_0_0_9*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_9_1_0_21_1_1 = det*c0_0_0_9*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_9_1_0_22_0_1 = det*c0_0_0_9*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_9_1_0_22_1_1 = det*c0_0_0_9*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_9_1_0_23_0_1 = det*c0_0_0_9*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_9_1_0_23_1_1 = det*c0_0_0_9*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_9_1_0_24_0_1 = det*c0_0_0_9*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_9_1_0_24_1_1 = det*c0_0_0_9*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_9_1_0_25_0_1 = det*c0_0_0_9*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_9_1_0_25_1_1 = det*c0_0_0_9*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_9_1_0_26_0_1 = det*c0_0_0_9*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_9_1_0_26_1_1 = det*c0_0_0_9*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_9_1_0_27_0_1 = det*c0_0_0_9*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_9_1_0_27_1_1 = det*c0_0_0_9*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_9_1_0_28_0_1 = det*c0_0_0_9*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_9_1_0_28_1_1 = det*c0_0_0_9*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_9_1_0_29_0_1 = det*c0_0_0_9*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_9_1_0_29_1_1 = det*c0_0_0_9*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_10_0_0_0_0_0 = det*c0_0_0_10*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_10_0_0_0_1_0 = det*c0_0_0_10*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_10_0_0_1_0_0 = det*c0_0_0_10*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_10_0_0_3_0_0 = det*c0_0_0_10*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_10_0_0_3_1_0 = det*c0_0_0_10*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_10_0_0_4_0_0 = det*c0_0_0_10*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_10_0_0_4_1_0 = det*c0_0_0_10*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_10_0_0_5_1_0 = det*c0_0_0_10*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_10_0_0_6_0_0 = det*c0_0_0_10*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_10_0_0_6_1_0 = det*c0_0_0_10*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_10_0_0_7_0_0 = det*c0_0_0_10*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_10_0_0_7_1_0 = det*c0_0_0_10*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_10_0_0_8_1_0 = det*c0_0_0_10*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_10_0_0_9_0_0 = det*c0_0_0_10*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_10_0_0_9_1_0 = det*c0_0_0_10*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_10_0_0_10_0_0 = det*c0_0_0_10*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_10_0_0_10_1_0 = det*c0_0_0_10*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_10_0_0_11_0_0 = det*c0_0_0_10*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_10_0_0_11_1_0 = det*c0_0_0_10*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_10_0_0_12_0_0 = det*c0_0_0_10*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_10_0_0_12_1_0 = det*c0_0_0_10*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_10_0_0_13_0_0 = det*c0_0_0_10*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_10_0_0_13_1_0 = det*c0_0_0_10*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_10_0_0_14_0_0 = det*c0_0_0_10*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_10_0_0_14_1_0 = det*c0_0_0_10*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_10_0_0_15_0_1 = det*c0_0_0_10*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_10_0_0_15_1_1 = det*c0_0_0_10*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_10_0_0_16_0_1 = det*c0_0_0_10*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_10_0_0_18_0_1 = det*c0_0_0_10*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_10_0_0_18_1_1 = det*c0_0_0_10*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_10_0_0_19_0_1 = det*c0_0_0_10*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_10_0_0_19_1_1 = det*c0_0_0_10*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_10_0_0_20_1_1 = det*c0_0_0_10*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_10_0_0_21_0_1 = det*c0_0_0_10*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_10_0_0_21_1_1 = det*c0_0_0_10*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_10_0_0_22_0_1 = det*c0_0_0_10*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_10_0_0_22_1_1 = det*c0_0_0_10*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_10_0_0_23_1_1 = det*c0_0_0_10*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_10_0_0_24_0_1 = det*c0_0_0_10*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_10_0_0_24_1_1 = det*c0_0_0_10*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_10_0_0_25_0_1 = det*c0_0_0_10*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_10_0_0_25_1_1 = det*c0_0_0_10*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_10_0_0_26_0_1 = det*c0_0_0_10*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_10_0_0_26_1_1 = det*c0_0_0_10*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_10_0_0_27_0_1 = det*c0_0_0_10*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_10_0_0_27_1_1 = det*c0_0_0_10*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_10_0_0_28_0_1 = det*c0_0_0_10*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_10_0_0_28_1_1 = det*c0_0_0_10*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_10_0_0_29_0_1 = det*c0_0_0_10*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_10_0_0_29_1_1 = det*c0_0_0_10*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_10_1_0_0_0_0 = det*c0_0_0_10*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_10_1_0_0_1_0 = det*c0_0_0_10*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_10_1_0_1_0_0 = det*c0_0_0_10*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_10_1_0_2_1_0 = det*c0_0_0_10*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_10_1_0_3_0_0 = det*c0_0_0_10*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_10_1_0_3_1_0 = det*c0_0_0_10*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_10_1_0_4_0_0 = det*c0_0_0_10*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_10_1_0_4_1_0 = det*c0_0_0_10*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_10_1_0_5_0_0 = det*c0_0_0_10*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_10_1_0_5_1_0 = det*c0_0_0_10*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_10_1_0_6_0_0 = det*c0_0_0_10*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_10_1_0_6_1_0 = det*c0_0_0_10*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_10_1_0_7_0_0 = det*c0_0_0_10*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_10_1_0_7_1_0 = det*c0_0_0_10*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_10_1_0_8_0_0 = det*c0_0_0_10*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_10_1_0_8_1_0 = det*c0_0_0_10*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_10_1_0_9_0_0 = det*c0_0_0_10*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_10_1_0_9_1_0 = det*c0_0_0_10*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_10_1_0_10_0_0 = det*c0_0_0_10*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_10_1_0_10_1_0 = det*c0_0_0_10*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_10_1_0_11_0_0 = det*c0_0_0_10*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_10_1_0_11_1_0 = det*c0_0_0_10*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_10_1_0_12_0_0 = det*c0_0_0_10*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_10_1_0_12_1_0 = det*c0_0_0_10*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_10_1_0_13_0_0 = det*c0_0_0_10*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_10_1_0_13_1_0 = det*c0_0_0_10*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_10_1_0_14_0_0 = det*c0_0_0_10*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_10_1_0_14_1_0 = det*c0_0_0_10*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_10_1_0_15_0_1 = det*c0_0_0_10*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_10_1_0_15_1_1 = det*c0_0_0_10*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_10_1_0_16_0_1 = det*c0_0_0_10*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_10_1_0_17_1_1 = det*c0_0_0_10*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_10_1_0_18_0_1 = det*c0_0_0_10*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_10_1_0_18_1_1 = det*c0_0_0_10*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_10_1_0_19_0_1 = det*c0_0_0_10*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_10_1_0_19_1_1 = det*c0_0_0_10*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_10_1_0_20_0_1 = det*c0_0_0_10*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_10_1_0_20_1_1 = det*c0_0_0_10*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_10_1_0_21_0_1 = det*c0_0_0_10*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_10_1_0_21_1_1 = det*c0_0_0_10*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_10_1_0_22_0_1 = det*c0_0_0_10*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_10_1_0_22_1_1 = det*c0_0_0_10*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_10_1_0_23_0_1 = det*c0_0_0_10*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_10_1_0_23_1_1 = det*c0_0_0_10*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_10_1_0_24_0_1 = det*c0_0_0_10*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_10_1_0_24_1_1 = det*c0_0_0_10*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_10_1_0_25_0_1 = det*c0_0_0_10*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_10_1_0_25_1_1 = det*c0_0_0_10*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_10_1_0_26_0_1 = det*c0_0_0_10*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_10_1_0_26_1_1 = det*c0_0_0_10*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_10_1_0_27_0_1 = det*c0_0_0_10*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_10_1_0_27_1_1 = det*c0_0_0_10*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_10_1_0_28_0_1 = det*c0_0_0_10*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_10_1_0_28_1_1 = det*c0_0_0_10*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_10_1_0_29_0_1 = det*c0_0_0_10*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_10_1_0_29_1_1 = det*c0_0_0_10*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_11_0_0_0_0_0 = det*c0_0_0_11*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_11_0_0_0_1_0 = det*c0_0_0_11*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_11_0_0_1_0_0 = det*c0_0_0_11*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_11_0_0_3_0_0 = det*c0_0_0_11*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_11_0_0_3_1_0 = det*c0_0_0_11*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_11_0_0_4_0_0 = det*c0_0_0_11*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_11_0_0_4_1_0 = det*c0_0_0_11*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_11_0_0_5_1_0 = det*c0_0_0_11*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_11_0_0_6_0_0 = det*c0_0_0_11*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_11_0_0_6_1_0 = det*c0_0_0_11*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_11_0_0_7_0_0 = det*c0_0_0_11*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_11_0_0_7_1_0 = det*c0_0_0_11*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_11_0_0_8_1_0 = det*c0_0_0_11*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_11_0_0_9_0_0 = det*c0_0_0_11*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_11_0_0_9_1_0 = det*c0_0_0_11*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_11_0_0_10_0_0 = det*c0_0_0_11*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_11_0_0_10_1_0 = det*c0_0_0_11*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_11_0_0_11_0_0 = det*c0_0_0_11*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_11_0_0_11_1_0 = det*c0_0_0_11*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_11_0_0_12_0_0 = det*c0_0_0_11*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_11_0_0_12_1_0 = det*c0_0_0_11*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_11_0_0_13_0_0 = det*c0_0_0_11*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_11_0_0_13_1_0 = det*c0_0_0_11*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_11_0_0_14_0_0 = det*c0_0_0_11*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_11_0_0_14_1_0 = det*c0_0_0_11*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_11_0_0_15_0_1 = det*c0_0_0_11*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_11_0_0_15_1_1 = det*c0_0_0_11*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_11_0_0_16_0_1 = det*c0_0_0_11*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_11_0_0_18_0_1 = det*c0_0_0_11*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_11_0_0_18_1_1 = det*c0_0_0_11*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_11_0_0_19_0_1 = det*c0_0_0_11*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_11_0_0_19_1_1 = det*c0_0_0_11*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_11_0_0_20_1_1 = det*c0_0_0_11*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_11_0_0_21_0_1 = det*c0_0_0_11*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_11_0_0_21_1_1 = det*c0_0_0_11*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_11_0_0_22_0_1 = det*c0_0_0_11*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_11_0_0_22_1_1 = det*c0_0_0_11*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_11_0_0_23_1_1 = det*c0_0_0_11*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_11_0_0_24_0_1 = det*c0_0_0_11*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_11_0_0_24_1_1 = det*c0_0_0_11*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_11_0_0_25_0_1 = det*c0_0_0_11*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_11_0_0_25_1_1 = det*c0_0_0_11*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_11_0_0_26_0_1 = det*c0_0_0_11*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_11_0_0_26_1_1 = det*c0_0_0_11*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_11_0_0_27_0_1 = det*c0_0_0_11*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_11_0_0_27_1_1 = det*c0_0_0_11*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_11_0_0_28_0_1 = det*c0_0_0_11*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_11_0_0_28_1_1 = det*c0_0_0_11*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_11_0_0_29_0_1 = det*c0_0_0_11*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_11_0_0_29_1_1 = det*c0_0_0_11*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_11_1_0_0_0_0 = det*c0_0_0_11*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_11_1_0_0_1_0 = det*c0_0_0_11*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_11_1_0_1_0_0 = det*c0_0_0_11*c0_0_1_1*Jinv_10*Jinv_00;
    const double G0_11_1_0_2_1_0 = det*c0_0_0_11*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_11_1_0_3_0_0 = det*c0_0_0_11*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_11_1_0_3_1_0 = det*c0_0_0_11*c0_0_1_3*Jinv_10*Jinv_10;
    const double G0_11_1_0_4_0_0 = det*c0_0_0_11*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_11_1_0_4_1_0 = det*c0_0_0_11*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_11_1_0_5_0_0 = det*c0_0_0_11*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_11_1_0_5_1_0 = det*c0_0_0_11*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_11_1_0_6_0_0 = det*c0_0_0_11*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_11_1_0_7_0_0 = det*c0_0_0_11*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_11_1_0_8_0_0 = det*c0_0_0_11*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_11_1_0_9_0_0 = det*c0_0_0_11*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_11_1_0_9_1_0 = det*c0_0_0_11*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_11_1_0_10_0_0 = det*c0_0_0_11*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_11_1_0_10_1_0 = det*c0_0_0_11*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_11_1_0_11_0_0 = det*c0_0_0_11*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_11_1_0_11_1_0 = det*c0_0_0_11*c0_0_1_11*Jinv_10*Jinv_10;
    const double G0_11_1_0_12_0_0 = det*c0_0_0_11*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_11_1_0_13_0_0 = det*c0_0_0_11*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_11_1_0_14_0_0 = det*c0_0_0_11*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_11_1_0_15_0_1 = det*c0_0_0_11*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_11_1_0_15_1_1 = det*c0_0_0_11*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_11_1_0_16_0_1 = det*c0_0_0_11*c0_0_1_16*Jinv_10*Jinv_01;
    const double G0_11_1_0_17_1_1 = det*c0_0_0_11*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_11_1_0_18_0_1 = det*c0_0_0_11*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_11_1_0_18_1_1 = det*c0_0_0_11*c0_0_1_18*Jinv_10*Jinv_11;
    const double G0_11_1_0_19_0_1 = det*c0_0_0_11*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_11_1_0_19_1_1 = det*c0_0_0_11*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_11_1_0_20_0_1 = det*c0_0_0_11*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_11_1_0_20_1_1 = det*c0_0_0_11*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_11_1_0_21_0_1 = det*c0_0_0_11*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_11_1_0_22_0_1 = det*c0_0_0_11*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_11_1_0_23_0_1 = det*c0_0_0_11*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_11_1_0_24_0_1 = det*c0_0_0_11*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_11_1_0_24_1_1 = det*c0_0_0_11*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_11_1_0_25_0_1 = det*c0_0_0_11*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_11_1_0_25_1_1 = det*c0_0_0_11*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_11_1_0_26_0_1 = det*c0_0_0_11*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_11_1_0_26_1_1 = det*c0_0_0_11*c0_0_1_26*Jinv_10*Jinv_11;
    const double G0_11_1_0_27_0_1 = det*c0_0_0_11*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_11_1_0_28_0_1 = det*c0_0_0_11*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_11_1_0_29_0_1 = det*c0_0_0_11*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_12_0_0_0_0_0 = det*c0_0_0_12*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_12_0_0_0_1_0 = det*c0_0_0_12*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_12_0_0_1_0_0 = det*c0_0_0_12*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_12_0_0_3_0_0 = det*c0_0_0_12*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_12_0_0_3_1_0 = det*c0_0_0_12*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_12_0_0_4_0_0 = det*c0_0_0_12*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_12_0_0_4_1_0 = det*c0_0_0_12*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_12_0_0_5_1_0 = det*c0_0_0_12*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_12_0_0_6_0_0 = det*c0_0_0_12*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_12_0_0_6_1_0 = det*c0_0_0_12*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_12_0_0_7_0_0 = det*c0_0_0_12*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_12_0_0_7_1_0 = det*c0_0_0_12*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_12_0_0_8_1_0 = det*c0_0_0_12*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_12_0_0_9_0_0 = det*c0_0_0_12*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_12_0_0_9_1_0 = det*c0_0_0_12*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_12_0_0_10_0_0 = det*c0_0_0_12*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_12_0_0_10_1_0 = det*c0_0_0_12*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_12_0_0_11_0_0 = det*c0_0_0_12*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_12_0_0_11_1_0 = det*c0_0_0_12*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_12_0_0_12_0_0 = det*c0_0_0_12*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_12_0_0_12_1_0 = det*c0_0_0_12*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_12_0_0_13_0_0 = det*c0_0_0_12*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_12_0_0_13_1_0 = det*c0_0_0_12*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_12_0_0_14_0_0 = det*c0_0_0_12*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_12_0_0_14_1_0 = det*c0_0_0_12*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_12_0_0_15_0_1 = det*c0_0_0_12*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_12_0_0_15_1_1 = det*c0_0_0_12*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_12_0_0_16_0_1 = det*c0_0_0_12*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_12_0_0_18_0_1 = det*c0_0_0_12*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_12_0_0_18_1_1 = det*c0_0_0_12*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_12_0_0_19_0_1 = det*c0_0_0_12*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_12_0_0_19_1_1 = det*c0_0_0_12*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_12_0_0_20_1_1 = det*c0_0_0_12*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_12_0_0_21_0_1 = det*c0_0_0_12*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_12_0_0_21_1_1 = det*c0_0_0_12*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_12_0_0_22_0_1 = det*c0_0_0_12*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_12_0_0_22_1_1 = det*c0_0_0_12*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_12_0_0_23_1_1 = det*c0_0_0_12*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_12_0_0_24_0_1 = det*c0_0_0_12*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_12_0_0_24_1_1 = det*c0_0_0_12*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_12_0_0_25_0_1 = det*c0_0_0_12*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_12_0_0_25_1_1 = det*c0_0_0_12*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_12_0_0_26_0_1 = det*c0_0_0_12*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_12_0_0_26_1_1 = det*c0_0_0_12*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_12_0_0_27_0_1 = det*c0_0_0_12*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_12_0_0_27_1_1 = det*c0_0_0_12*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_12_0_0_28_0_1 = det*c0_0_0_12*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_12_0_0_28_1_1 = det*c0_0_0_12*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_12_0_0_29_0_1 = det*c0_0_0_12*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_12_0_0_29_1_1 = det*c0_0_0_12*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_12_1_0_0_0_0 = det*c0_0_0_12*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_12_1_0_0_1_0 = det*c0_0_0_12*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_12_1_0_2_1_0 = det*c0_0_0_12*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_12_1_0_3_0_0 = det*c0_0_0_12*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_12_1_0_4_0_0 = det*c0_0_0_12*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_12_1_0_4_1_0 = det*c0_0_0_12*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_12_1_0_5_0_0 = det*c0_0_0_12*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_12_1_0_5_1_0 = det*c0_0_0_12*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_12_1_0_6_0_0 = det*c0_0_0_12*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_12_1_0_6_1_0 = det*c0_0_0_12*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_12_1_0_7_0_0 = det*c0_0_0_12*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_12_1_0_7_1_0 = det*c0_0_0_12*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_12_1_0_8_0_0 = det*c0_0_0_12*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_12_1_0_8_1_0 = det*c0_0_0_12*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_12_1_0_9_0_0 = det*c0_0_0_12*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_12_1_0_9_1_0 = det*c0_0_0_12*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_12_1_0_10_0_0 = det*c0_0_0_12*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_12_1_0_10_1_0 = det*c0_0_0_12*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_12_1_0_11_0_0 = det*c0_0_0_12*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_12_1_0_12_0_0 = det*c0_0_0_12*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_12_1_0_12_1_0 = det*c0_0_0_12*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_12_1_0_13_0_0 = det*c0_0_0_12*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_12_1_0_13_1_0 = det*c0_0_0_12*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_12_1_0_14_0_0 = det*c0_0_0_12*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_12_1_0_14_1_0 = det*c0_0_0_12*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_12_1_0_15_0_1 = det*c0_0_0_12*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_12_1_0_15_1_1 = det*c0_0_0_12*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_12_1_0_17_1_1 = det*c0_0_0_12*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_12_1_0_18_0_1 = det*c0_0_0_12*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_12_1_0_19_0_1 = det*c0_0_0_12*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_12_1_0_19_1_1 = det*c0_0_0_12*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_12_1_0_20_0_1 = det*c0_0_0_12*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_12_1_0_20_1_1 = det*c0_0_0_12*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_12_1_0_21_0_1 = det*c0_0_0_12*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_12_1_0_21_1_1 = det*c0_0_0_12*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_12_1_0_22_0_1 = det*c0_0_0_12*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_12_1_0_22_1_1 = det*c0_0_0_12*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_12_1_0_23_0_1 = det*c0_0_0_12*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_12_1_0_23_1_1 = det*c0_0_0_12*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_12_1_0_24_0_1 = det*c0_0_0_12*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_12_1_0_24_1_1 = det*c0_0_0_12*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_12_1_0_25_0_1 = det*c0_0_0_12*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_12_1_0_25_1_1 = det*c0_0_0_12*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_12_1_0_26_0_1 = det*c0_0_0_12*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_12_1_0_27_0_1 = det*c0_0_0_12*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_12_1_0_27_1_1 = det*c0_0_0_12*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_12_1_0_28_0_1 = det*c0_0_0_12*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_12_1_0_28_1_1 = det*c0_0_0_12*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_12_1_0_29_0_1 = det*c0_0_0_12*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_12_1_0_29_1_1 = det*c0_0_0_12*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_13_0_0_0_0_0 = det*c0_0_0_13*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_13_0_0_0_1_0 = det*c0_0_0_13*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_13_0_0_1_0_0 = det*c0_0_0_13*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_13_0_0_3_0_0 = det*c0_0_0_13*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_13_0_0_3_1_0 = det*c0_0_0_13*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_13_0_0_4_0_0 = det*c0_0_0_13*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_13_0_0_4_1_0 = det*c0_0_0_13*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_13_0_0_5_1_0 = det*c0_0_0_13*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_13_0_0_6_0_0 = det*c0_0_0_13*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_13_0_0_6_1_0 = det*c0_0_0_13*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_13_0_0_7_0_0 = det*c0_0_0_13*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_13_0_0_7_1_0 = det*c0_0_0_13*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_13_0_0_8_1_0 = det*c0_0_0_13*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_13_0_0_9_0_0 = det*c0_0_0_13*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_13_0_0_9_1_0 = det*c0_0_0_13*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_13_0_0_10_0_0 = det*c0_0_0_13*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_13_0_0_10_1_0 = det*c0_0_0_13*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_13_0_0_11_0_0 = det*c0_0_0_13*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_13_0_0_11_1_0 = det*c0_0_0_13*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_13_0_0_12_0_0 = det*c0_0_0_13*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_13_0_0_12_1_0 = det*c0_0_0_13*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_13_0_0_13_0_0 = det*c0_0_0_13*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_13_0_0_13_1_0 = det*c0_0_0_13*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_13_0_0_14_0_0 = det*c0_0_0_13*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_13_0_0_14_1_0 = det*c0_0_0_13*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_13_0_0_15_0_1 = det*c0_0_0_13*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_13_0_0_15_1_1 = det*c0_0_0_13*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_13_0_0_16_0_1 = det*c0_0_0_13*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_13_0_0_18_0_1 = det*c0_0_0_13*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_13_0_0_18_1_1 = det*c0_0_0_13*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_13_0_0_19_0_1 = det*c0_0_0_13*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_13_0_0_19_1_1 = det*c0_0_0_13*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_13_0_0_20_1_1 = det*c0_0_0_13*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_13_0_0_21_0_1 = det*c0_0_0_13*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_13_0_0_21_1_1 = det*c0_0_0_13*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_13_0_0_22_0_1 = det*c0_0_0_13*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_13_0_0_22_1_1 = det*c0_0_0_13*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_13_0_0_23_1_1 = det*c0_0_0_13*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_13_0_0_24_0_1 = det*c0_0_0_13*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_13_0_0_24_1_1 = det*c0_0_0_13*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_13_0_0_25_0_1 = det*c0_0_0_13*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_13_0_0_25_1_1 = det*c0_0_0_13*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_13_0_0_26_0_1 = det*c0_0_0_13*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_13_0_0_26_1_1 = det*c0_0_0_13*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_13_0_0_27_0_1 = det*c0_0_0_13*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_13_0_0_27_1_1 = det*c0_0_0_13*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_13_0_0_28_0_1 = det*c0_0_0_13*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_13_0_0_28_1_1 = det*c0_0_0_13*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_13_0_0_29_0_1 = det*c0_0_0_13*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_13_0_0_29_1_1 = det*c0_0_0_13*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_13_1_0_0_0_0 = det*c0_0_0_13*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_13_1_0_0_1_0 = det*c0_0_0_13*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_13_1_0_2_1_0 = det*c0_0_0_13*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_13_1_0_3_0_0 = det*c0_0_0_13*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_13_1_0_4_0_0 = det*c0_0_0_13*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_13_1_0_4_1_0 = det*c0_0_0_13*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_13_1_0_5_0_0 = det*c0_0_0_13*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_13_1_0_5_1_0 = det*c0_0_0_13*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_13_1_0_6_0_0 = det*c0_0_0_13*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_13_1_0_6_1_0 = det*c0_0_0_13*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_13_1_0_7_0_0 = det*c0_0_0_13*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_13_1_0_7_1_0 = det*c0_0_0_13*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_13_1_0_8_0_0 = det*c0_0_0_13*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_13_1_0_8_1_0 = det*c0_0_0_13*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_13_1_0_9_0_0 = det*c0_0_0_13*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_13_1_0_9_1_0 = det*c0_0_0_13*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_13_1_0_10_0_0 = det*c0_0_0_13*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_13_1_0_10_1_0 = det*c0_0_0_13*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_13_1_0_11_0_0 = det*c0_0_0_13*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_13_1_0_12_0_0 = det*c0_0_0_13*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_13_1_0_12_1_0 = det*c0_0_0_13*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_13_1_0_13_0_0 = det*c0_0_0_13*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_13_1_0_13_1_0 = det*c0_0_0_13*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_13_1_0_14_0_0 = det*c0_0_0_13*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_13_1_0_14_1_0 = det*c0_0_0_13*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_13_1_0_15_0_1 = det*c0_0_0_13*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_13_1_0_15_1_1 = det*c0_0_0_13*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_13_1_0_17_1_1 = det*c0_0_0_13*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_13_1_0_18_0_1 = det*c0_0_0_13*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_13_1_0_19_0_1 = det*c0_0_0_13*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_13_1_0_19_1_1 = det*c0_0_0_13*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_13_1_0_20_0_1 = det*c0_0_0_13*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_13_1_0_20_1_1 = det*c0_0_0_13*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_13_1_0_21_0_1 = det*c0_0_0_13*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_13_1_0_21_1_1 = det*c0_0_0_13*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_13_1_0_22_0_1 = det*c0_0_0_13*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_13_1_0_22_1_1 = det*c0_0_0_13*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_13_1_0_23_0_1 = det*c0_0_0_13*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_13_1_0_23_1_1 = det*c0_0_0_13*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_13_1_0_24_0_1 = det*c0_0_0_13*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_13_1_0_24_1_1 = det*c0_0_0_13*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_13_1_0_25_0_1 = det*c0_0_0_13*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_13_1_0_25_1_1 = det*c0_0_0_13*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_13_1_0_26_0_1 = det*c0_0_0_13*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_13_1_0_27_0_1 = det*c0_0_0_13*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_13_1_0_27_1_1 = det*c0_0_0_13*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_13_1_0_28_0_1 = det*c0_0_0_13*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_13_1_0_28_1_1 = det*c0_0_0_13*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_13_1_0_29_0_1 = det*c0_0_0_13*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_13_1_0_29_1_1 = det*c0_0_0_13*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_14_0_0_0_0_0 = det*c0_0_0_14*c0_0_1_0*Jinv_00*Jinv_00;
    const double G0_14_0_0_0_1_0 = det*c0_0_0_14*c0_0_1_0*Jinv_00*Jinv_10;
    const double G0_14_0_0_1_0_0 = det*c0_0_0_14*c0_0_1_1*Jinv_00*Jinv_00;
    const double G0_14_0_0_3_0_0 = det*c0_0_0_14*c0_0_1_3*Jinv_00*Jinv_00;
    const double G0_14_0_0_3_1_0 = det*c0_0_0_14*c0_0_1_3*Jinv_00*Jinv_10;
    const double G0_14_0_0_4_0_0 = det*c0_0_0_14*c0_0_1_4*Jinv_00*Jinv_00;
    const double G0_14_0_0_4_1_0 = det*c0_0_0_14*c0_0_1_4*Jinv_00*Jinv_10;
    const double G0_14_0_0_5_1_0 = det*c0_0_0_14*c0_0_1_5*Jinv_00*Jinv_10;
    const double G0_14_0_0_6_0_0 = det*c0_0_0_14*c0_0_1_6*Jinv_00*Jinv_00;
    const double G0_14_0_0_6_1_0 = det*c0_0_0_14*c0_0_1_6*Jinv_00*Jinv_10;
    const double G0_14_0_0_7_0_0 = det*c0_0_0_14*c0_0_1_7*Jinv_00*Jinv_00;
    const double G0_14_0_0_7_1_0 = det*c0_0_0_14*c0_0_1_7*Jinv_00*Jinv_10;
    const double G0_14_0_0_8_1_0 = det*c0_0_0_14*c0_0_1_8*Jinv_00*Jinv_10;
    const double G0_14_0_0_9_0_0 = det*c0_0_0_14*c0_0_1_9*Jinv_00*Jinv_00;
    const double G0_14_0_0_9_1_0 = det*c0_0_0_14*c0_0_1_9*Jinv_00*Jinv_10;
    const double G0_14_0_0_10_0_0 = det*c0_0_0_14*c0_0_1_10*Jinv_00*Jinv_00;
    const double G0_14_0_0_10_1_0 = det*c0_0_0_14*c0_0_1_10*Jinv_00*Jinv_10;
    const double G0_14_0_0_11_0_0 = det*c0_0_0_14*c0_0_1_11*Jinv_00*Jinv_00;
    const double G0_14_0_0_11_1_0 = det*c0_0_0_14*c0_0_1_11*Jinv_00*Jinv_10;
    const double G0_14_0_0_12_0_0 = det*c0_0_0_14*c0_0_1_12*Jinv_00*Jinv_00;
    const double G0_14_0_0_12_1_0 = det*c0_0_0_14*c0_0_1_12*Jinv_00*Jinv_10;
    const double G0_14_0_0_13_0_0 = det*c0_0_0_14*c0_0_1_13*Jinv_00*Jinv_00;
    const double G0_14_0_0_13_1_0 = det*c0_0_0_14*c0_0_1_13*Jinv_00*Jinv_10;
    const double G0_14_0_0_14_0_0 = det*c0_0_0_14*c0_0_1_14*Jinv_00*Jinv_00;
    const double G0_14_0_0_14_1_0 = det*c0_0_0_14*c0_0_1_14*Jinv_00*Jinv_10;
    const double G0_14_0_0_15_0_1 = det*c0_0_0_14*c0_0_1_15*Jinv_00*Jinv_01;
    const double G0_14_0_0_15_1_1 = det*c0_0_0_14*c0_0_1_15*Jinv_00*Jinv_11;
    const double G0_14_0_0_16_0_1 = det*c0_0_0_14*c0_0_1_16*Jinv_00*Jinv_01;
    const double G0_14_0_0_18_0_1 = det*c0_0_0_14*c0_0_1_18*Jinv_00*Jinv_01;
    const double G0_14_0_0_18_1_1 = det*c0_0_0_14*c0_0_1_18*Jinv_00*Jinv_11;
    const double G0_14_0_0_19_0_1 = det*c0_0_0_14*c0_0_1_19*Jinv_00*Jinv_01;
    const double G0_14_0_0_19_1_1 = det*c0_0_0_14*c0_0_1_19*Jinv_00*Jinv_11;
    const double G0_14_0_0_20_1_1 = det*c0_0_0_14*c0_0_1_20*Jinv_00*Jinv_11;
    const double G0_14_0_0_21_0_1 = det*c0_0_0_14*c0_0_1_21*Jinv_00*Jinv_01;
    const double G0_14_0_0_21_1_1 = det*c0_0_0_14*c0_0_1_21*Jinv_00*Jinv_11;
    const double G0_14_0_0_22_0_1 = det*c0_0_0_14*c0_0_1_22*Jinv_00*Jinv_01;
    const double G0_14_0_0_22_1_1 = det*c0_0_0_14*c0_0_1_22*Jinv_00*Jinv_11;
    const double G0_14_0_0_23_1_1 = det*c0_0_0_14*c0_0_1_23*Jinv_00*Jinv_11;
    const double G0_14_0_0_24_0_1 = det*c0_0_0_14*c0_0_1_24*Jinv_00*Jinv_01;
    const double G0_14_0_0_24_1_1 = det*c0_0_0_14*c0_0_1_24*Jinv_00*Jinv_11;
    const double G0_14_0_0_25_0_1 = det*c0_0_0_14*c0_0_1_25*Jinv_00*Jinv_01;
    const double G0_14_0_0_25_1_1 = det*c0_0_0_14*c0_0_1_25*Jinv_00*Jinv_11;
    const double G0_14_0_0_26_0_1 = det*c0_0_0_14*c0_0_1_26*Jinv_00*Jinv_01;
    const double G0_14_0_0_26_1_1 = det*c0_0_0_14*c0_0_1_26*Jinv_00*Jinv_11;
    const double G0_14_0_0_27_0_1 = det*c0_0_0_14*c0_0_1_27*Jinv_00*Jinv_01;
    const double G0_14_0_0_27_1_1 = det*c0_0_0_14*c0_0_1_27*Jinv_00*Jinv_11;
    const double G0_14_0_0_28_0_1 = det*c0_0_0_14*c0_0_1_28*Jinv_00*Jinv_01;
    const double G0_14_0_0_28_1_1 = det*c0_0_0_14*c0_0_1_28*Jinv_00*Jinv_11;
    const double G0_14_0_0_29_0_1 = det*c0_0_0_14*c0_0_1_29*Jinv_00*Jinv_01;
    const double G0_14_0_0_29_1_1 = det*c0_0_0_14*c0_0_1_29*Jinv_00*Jinv_11;
    const double G0_14_1_0_0_0_0 = det*c0_0_0_14*c0_0_1_0*Jinv_10*Jinv_00;
    const double G0_14_1_0_0_1_0 = det*c0_0_0_14*c0_0_1_0*Jinv_10*Jinv_10;
    const double G0_14_1_0_2_1_0 = det*c0_0_0_14*c0_0_1_2*Jinv_10*Jinv_10;
    const double G0_14_1_0_3_0_0 = det*c0_0_0_14*c0_0_1_3*Jinv_10*Jinv_00;
    const double G0_14_1_0_4_0_0 = det*c0_0_0_14*c0_0_1_4*Jinv_10*Jinv_00;
    const double G0_14_1_0_4_1_0 = det*c0_0_0_14*c0_0_1_4*Jinv_10*Jinv_10;
    const double G0_14_1_0_5_0_0 = det*c0_0_0_14*c0_0_1_5*Jinv_10*Jinv_00;
    const double G0_14_1_0_5_1_0 = det*c0_0_0_14*c0_0_1_5*Jinv_10*Jinv_10;
    const double G0_14_1_0_6_0_0 = det*c0_0_0_14*c0_0_1_6*Jinv_10*Jinv_00;
    const double G0_14_1_0_6_1_0 = det*c0_0_0_14*c0_0_1_6*Jinv_10*Jinv_10;
    const double G0_14_1_0_7_0_0 = det*c0_0_0_14*c0_0_1_7*Jinv_10*Jinv_00;
    const double G0_14_1_0_7_1_0 = det*c0_0_0_14*c0_0_1_7*Jinv_10*Jinv_10;
    const double G0_14_1_0_8_0_0 = det*c0_0_0_14*c0_0_1_8*Jinv_10*Jinv_00;
    const double G0_14_1_0_8_1_0 = det*c0_0_0_14*c0_0_1_8*Jinv_10*Jinv_10;
    const double G0_14_1_0_9_0_0 = det*c0_0_0_14*c0_0_1_9*Jinv_10*Jinv_00;
    const double G0_14_1_0_9_1_0 = det*c0_0_0_14*c0_0_1_9*Jinv_10*Jinv_10;
    const double G0_14_1_0_10_0_0 = det*c0_0_0_14*c0_0_1_10*Jinv_10*Jinv_00;
    const double G0_14_1_0_10_1_0 = det*c0_0_0_14*c0_0_1_10*Jinv_10*Jinv_10;
    const double G0_14_1_0_11_0_0 = det*c0_0_0_14*c0_0_1_11*Jinv_10*Jinv_00;
    const double G0_14_1_0_12_0_0 = det*c0_0_0_14*c0_0_1_12*Jinv_10*Jinv_00;
    const double G0_14_1_0_12_1_0 = det*c0_0_0_14*c0_0_1_12*Jinv_10*Jinv_10;
    const double G0_14_1_0_13_0_0 = det*c0_0_0_14*c0_0_1_13*Jinv_10*Jinv_00;
    const double G0_14_1_0_13_1_0 = det*c0_0_0_14*c0_0_1_13*Jinv_10*Jinv_10;
    const double G0_14_1_0_14_0_0 = det*c0_0_0_14*c0_0_1_14*Jinv_10*Jinv_00;
    const double G0_14_1_0_14_1_0 = det*c0_0_0_14*c0_0_1_14*Jinv_10*Jinv_10;
    const double G0_14_1_0_15_0_1 = det*c0_0_0_14*c0_0_1_15*Jinv_10*Jinv_01;
    const double G0_14_1_0_15_1_1 = det*c0_0_0_14*c0_0_1_15*Jinv_10*Jinv_11;
    const double G0_14_1_0_17_1_1 = det*c0_0_0_14*c0_0_1_17*Jinv_10*Jinv_11;
    const double G0_14_1_0_18_0_1 = det*c0_0_0_14*c0_0_1_18*Jinv_10*Jinv_01;
    const double G0_14_1_0_19_0_1 = det*c0_0_0_14*c0_0_1_19*Jinv_10*Jinv_01;
    const double G0_14_1_0_19_1_1 = det*c0_0_0_14*c0_0_1_19*Jinv_10*Jinv_11;
    const double G0_14_1_0_20_0_1 = det*c0_0_0_14*c0_0_1_20*Jinv_10*Jinv_01;
    const double G0_14_1_0_20_1_1 = det*c0_0_0_14*c0_0_1_20*Jinv_10*Jinv_11;
    const double G0_14_1_0_21_0_1 = det*c0_0_0_14*c0_0_1_21*Jinv_10*Jinv_01;
    const double G0_14_1_0_21_1_1 = det*c0_0_0_14*c0_0_1_21*Jinv_10*Jinv_11;
    const double G0_14_1_0_22_0_1 = det*c0_0_0_14*c0_0_1_22*Jinv_10*Jinv_01;
    const double G0_14_1_0_22_1_1 = det*c0_0_0_14*c0_0_1_22*Jinv_10*Jinv_11;
    const double G0_14_1_0_23_0_1 = det*c0_0_0_14*c0_0_1_23*Jinv_10*Jinv_01;
    const double G0_14_1_0_23_1_1 = det*c0_0_0_14*c0_0_1_23*Jinv_10*Jinv_11;
    const double G0_14_1_0_24_0_1 = det*c0_0_0_14*c0_0_1_24*Jinv_10*Jinv_01;
    const double G0_14_1_0_24_1_1 = det*c0_0_0_14*c0_0_1_24*Jinv_10*Jinv_11;
    const double G0_14_1_0_25_0_1 = det*c0_0_0_14*c0_0_1_25*Jinv_10*Jinv_01;
    const double G0_14_1_0_25_1_1 = det*c0_0_0_14*c0_0_1_25*Jinv_10*Jinv_11;
    const double G0_14_1_0_26_0_1 = det*c0_0_0_14*c0_0_1_26*Jinv_10*Jinv_01;
    const double G0_14_1_0_27_0_1 = det*c0_0_0_14*c0_0_1_27*Jinv_10*Jinv_01;
    const double G0_14_1_0_27_1_1 = det*c0_0_0_14*c0_0_1_27*Jinv_10*Jinv_11;
    const double G0_14_1_0_28_0_1 = det*c0_0_0_14*c0_0_1_28*Jinv_10*Jinv_01;
    const double G0_14_1_0_28_1_1 = det*c0_0_0_14*c0_0_1_28*Jinv_10*Jinv_11;
    const double G0_14_1_0_29_0_1 = det*c0_0_0_14*c0_0_1_29*Jinv_10*Jinv_01;
    const double G0_14_1_0_29_1_1 = det*c0_0_0_14*c0_0_1_29*Jinv_10*Jinv_11;
    const double G0_15_0_1_0_0_0 = det*c0_0_0_15*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_15_0_1_0_1_0 = det*c0_0_0_15*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_15_0_1_1_0_0 = det*c0_0_0_15*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_15_0_1_2_1_0 = det*c0_0_0_15*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_15_0_1_3_0_0 = det*c0_0_0_15*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_15_0_1_3_1_0 = det*c0_0_0_15*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_15_0_1_4_0_0 = det*c0_0_0_15*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_15_0_1_4_1_0 = det*c0_0_0_15*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_15_0_1_5_0_0 = det*c0_0_0_15*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_15_0_1_5_1_0 = det*c0_0_0_15*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_15_0_1_6_0_0 = det*c0_0_0_15*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_15_0_1_6_1_0 = det*c0_0_0_15*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_15_0_1_7_0_0 = det*c0_0_0_15*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_15_0_1_7_1_0 = det*c0_0_0_15*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_15_0_1_8_0_0 = det*c0_0_0_15*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_15_0_1_8_1_0 = det*c0_0_0_15*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_15_0_1_9_0_0 = det*c0_0_0_15*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_15_0_1_9_1_0 = det*c0_0_0_15*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_15_0_1_10_0_0 = det*c0_0_0_15*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_15_0_1_10_1_0 = det*c0_0_0_15*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_15_0_1_11_0_0 = det*c0_0_0_15*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_15_0_1_11_1_0 = det*c0_0_0_15*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_15_0_1_12_0_0 = det*c0_0_0_15*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_15_0_1_12_1_0 = det*c0_0_0_15*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_15_0_1_13_0_0 = det*c0_0_0_15*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_15_0_1_13_1_0 = det*c0_0_0_15*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_15_0_1_14_0_0 = det*c0_0_0_15*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_15_0_1_14_1_0 = det*c0_0_0_15*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_15_0_1_15_0_1 = det*c0_0_0_15*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_15_0_1_15_1_1 = det*c0_0_0_15*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_15_0_1_16_0_1 = det*c0_0_0_15*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_15_0_1_17_1_1 = det*c0_0_0_15*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_15_0_1_18_0_1 = det*c0_0_0_15*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_15_0_1_18_1_1 = det*c0_0_0_15*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_15_0_1_19_0_1 = det*c0_0_0_15*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_15_0_1_19_1_1 = det*c0_0_0_15*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_15_0_1_20_0_1 = det*c0_0_0_15*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_15_0_1_20_1_1 = det*c0_0_0_15*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_15_0_1_21_0_1 = det*c0_0_0_15*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_15_0_1_21_1_1 = det*c0_0_0_15*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_15_0_1_22_0_1 = det*c0_0_0_15*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_15_0_1_22_1_1 = det*c0_0_0_15*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_15_0_1_23_0_1 = det*c0_0_0_15*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_15_0_1_23_1_1 = det*c0_0_0_15*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_15_0_1_24_0_1 = det*c0_0_0_15*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_15_0_1_24_1_1 = det*c0_0_0_15*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_15_0_1_25_0_1 = det*c0_0_0_15*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_15_0_1_25_1_1 = det*c0_0_0_15*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_15_0_1_26_0_1 = det*c0_0_0_15*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_15_0_1_26_1_1 = det*c0_0_0_15*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_15_0_1_27_0_1 = det*c0_0_0_15*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_15_0_1_27_1_1 = det*c0_0_0_15*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_15_0_1_28_0_1 = det*c0_0_0_15*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_15_0_1_28_1_1 = det*c0_0_0_15*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_15_0_1_29_0_1 = det*c0_0_0_15*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_15_0_1_29_1_1 = det*c0_0_0_15*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_15_1_1_0_0_0 = det*c0_0_0_15*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_15_1_1_0_1_0 = det*c0_0_0_15*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_15_1_1_1_0_0 = det*c0_0_0_15*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_15_1_1_2_1_0 = det*c0_0_0_15*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_15_1_1_3_0_0 = det*c0_0_0_15*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_15_1_1_3_1_0 = det*c0_0_0_15*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_15_1_1_4_0_0 = det*c0_0_0_15*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_15_1_1_4_1_0 = det*c0_0_0_15*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_15_1_1_5_0_0 = det*c0_0_0_15*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_15_1_1_5_1_0 = det*c0_0_0_15*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_15_1_1_6_0_0 = det*c0_0_0_15*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_15_1_1_6_1_0 = det*c0_0_0_15*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_15_1_1_7_0_0 = det*c0_0_0_15*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_15_1_1_7_1_0 = det*c0_0_0_15*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_15_1_1_8_0_0 = det*c0_0_0_15*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_15_1_1_8_1_0 = det*c0_0_0_15*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_15_1_1_9_0_0 = det*c0_0_0_15*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_15_1_1_9_1_0 = det*c0_0_0_15*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_15_1_1_10_0_0 = det*c0_0_0_15*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_15_1_1_10_1_0 = det*c0_0_0_15*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_15_1_1_11_0_0 = det*c0_0_0_15*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_15_1_1_11_1_0 = det*c0_0_0_15*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_15_1_1_12_0_0 = det*c0_0_0_15*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_15_1_1_12_1_0 = det*c0_0_0_15*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_15_1_1_13_0_0 = det*c0_0_0_15*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_15_1_1_13_1_0 = det*c0_0_0_15*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_15_1_1_14_0_0 = det*c0_0_0_15*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_15_1_1_14_1_0 = det*c0_0_0_15*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_15_1_1_15_0_1 = det*c0_0_0_15*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_15_1_1_15_1_1 = det*c0_0_0_15*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_15_1_1_16_0_1 = det*c0_0_0_15*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_15_1_1_17_1_1 = det*c0_0_0_15*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_15_1_1_18_0_1 = det*c0_0_0_15*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_15_1_1_18_1_1 = det*c0_0_0_15*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_15_1_1_19_0_1 = det*c0_0_0_15*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_15_1_1_19_1_1 = det*c0_0_0_15*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_15_1_1_20_0_1 = det*c0_0_0_15*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_15_1_1_20_1_1 = det*c0_0_0_15*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_15_1_1_21_0_1 = det*c0_0_0_15*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_15_1_1_21_1_1 = det*c0_0_0_15*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_15_1_1_22_0_1 = det*c0_0_0_15*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_15_1_1_22_1_1 = det*c0_0_0_15*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_15_1_1_23_0_1 = det*c0_0_0_15*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_15_1_1_23_1_1 = det*c0_0_0_15*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_15_1_1_24_0_1 = det*c0_0_0_15*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_15_1_1_24_1_1 = det*c0_0_0_15*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_15_1_1_25_0_1 = det*c0_0_0_15*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_15_1_1_25_1_1 = det*c0_0_0_15*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_15_1_1_26_0_1 = det*c0_0_0_15*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_15_1_1_26_1_1 = det*c0_0_0_15*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_15_1_1_27_0_1 = det*c0_0_0_15*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_15_1_1_27_1_1 = det*c0_0_0_15*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_15_1_1_28_0_1 = det*c0_0_0_15*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_15_1_1_28_1_1 = det*c0_0_0_15*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_15_1_1_29_0_1 = det*c0_0_0_15*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_15_1_1_29_1_1 = det*c0_0_0_15*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_16_0_1_0_0_0 = det*c0_0_0_16*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_16_0_1_0_1_0 = det*c0_0_0_16*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_16_0_1_1_0_0 = det*c0_0_0_16*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_16_0_1_2_1_0 = det*c0_0_0_16*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_16_0_1_3_0_0 = det*c0_0_0_16*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_16_0_1_3_1_0 = det*c0_0_0_16*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_16_0_1_4_0_0 = det*c0_0_0_16*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_16_0_1_4_1_0 = det*c0_0_0_16*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_16_0_1_5_0_0 = det*c0_0_0_16*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_16_0_1_5_1_0 = det*c0_0_0_16*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_16_0_1_6_0_0 = det*c0_0_0_16*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_16_0_1_7_0_0 = det*c0_0_0_16*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_16_0_1_8_0_0 = det*c0_0_0_16*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_16_0_1_9_0_0 = det*c0_0_0_16*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_16_0_1_9_1_0 = det*c0_0_0_16*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_16_0_1_10_0_0 = det*c0_0_0_16*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_16_0_1_10_1_0 = det*c0_0_0_16*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_16_0_1_11_0_0 = det*c0_0_0_16*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_16_0_1_11_1_0 = det*c0_0_0_16*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_16_0_1_12_0_0 = det*c0_0_0_16*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_16_0_1_13_0_0 = det*c0_0_0_16*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_16_0_1_14_0_0 = det*c0_0_0_16*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_16_0_1_15_0_1 = det*c0_0_0_16*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_16_0_1_15_1_1 = det*c0_0_0_16*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_16_0_1_16_0_1 = det*c0_0_0_16*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_16_0_1_17_1_1 = det*c0_0_0_16*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_16_0_1_18_0_1 = det*c0_0_0_16*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_16_0_1_18_1_1 = det*c0_0_0_16*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_16_0_1_19_0_1 = det*c0_0_0_16*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_16_0_1_19_1_1 = det*c0_0_0_16*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_16_0_1_20_0_1 = det*c0_0_0_16*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_16_0_1_20_1_1 = det*c0_0_0_16*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_16_0_1_21_0_1 = det*c0_0_0_16*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_16_0_1_22_0_1 = det*c0_0_0_16*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_16_0_1_23_0_1 = det*c0_0_0_16*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_16_0_1_24_0_1 = det*c0_0_0_16*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_16_0_1_24_1_1 = det*c0_0_0_16*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_16_0_1_25_0_1 = det*c0_0_0_16*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_16_0_1_25_1_1 = det*c0_0_0_16*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_16_0_1_26_0_1 = det*c0_0_0_16*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_16_0_1_26_1_1 = det*c0_0_0_16*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_16_0_1_27_0_1 = det*c0_0_0_16*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_16_0_1_28_0_1 = det*c0_0_0_16*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_16_0_1_29_0_1 = det*c0_0_0_16*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_17_1_1_0_0_0 = det*c0_0_0_17*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_17_1_1_0_1_0 = det*c0_0_0_17*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_17_1_1_1_0_0 = det*c0_0_0_17*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_17_1_1_2_1_0 = det*c0_0_0_17*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_17_1_1_3_0_0 = det*c0_0_0_17*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_17_1_1_3_1_0 = det*c0_0_0_17*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_17_1_1_4_0_0 = det*c0_0_0_17*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_17_1_1_4_1_0 = det*c0_0_0_17*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_17_1_1_5_0_0 = det*c0_0_0_17*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_17_1_1_5_1_0 = det*c0_0_0_17*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_17_1_1_6_0_0 = det*c0_0_0_17*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_17_1_1_6_1_0 = det*c0_0_0_17*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_17_1_1_7_0_0 = det*c0_0_0_17*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_17_1_1_7_1_0 = det*c0_0_0_17*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_17_1_1_8_0_0 = det*c0_0_0_17*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_17_1_1_8_1_0 = det*c0_0_0_17*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_17_1_1_9_1_0 = det*c0_0_0_17*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_17_1_1_10_1_0 = det*c0_0_0_17*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_17_1_1_11_1_0 = det*c0_0_0_17*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_17_1_1_12_1_0 = det*c0_0_0_17*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_17_1_1_13_1_0 = det*c0_0_0_17*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_17_1_1_14_1_0 = det*c0_0_0_17*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_17_1_1_15_0_1 = det*c0_0_0_17*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_17_1_1_15_1_1 = det*c0_0_0_17*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_17_1_1_16_0_1 = det*c0_0_0_17*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_17_1_1_17_1_1 = det*c0_0_0_17*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_17_1_1_18_0_1 = det*c0_0_0_17*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_17_1_1_18_1_1 = det*c0_0_0_17*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_17_1_1_19_0_1 = det*c0_0_0_17*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_17_1_1_19_1_1 = det*c0_0_0_17*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_17_1_1_20_0_1 = det*c0_0_0_17*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_17_1_1_20_1_1 = det*c0_0_0_17*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_17_1_1_21_0_1 = det*c0_0_0_17*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_17_1_1_21_1_1 = det*c0_0_0_17*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_17_1_1_22_0_1 = det*c0_0_0_17*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_17_1_1_22_1_1 = det*c0_0_0_17*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_17_1_1_23_0_1 = det*c0_0_0_17*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_17_1_1_23_1_1 = det*c0_0_0_17*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_17_1_1_24_1_1 = det*c0_0_0_17*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_17_1_1_25_1_1 = det*c0_0_0_17*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_17_1_1_26_1_1 = det*c0_0_0_17*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_17_1_1_27_1_1 = det*c0_0_0_17*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_17_1_1_28_1_1 = det*c0_0_0_17*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_17_1_1_29_1_1 = det*c0_0_0_17*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_18_0_1_0_0_0 = det*c0_0_0_18*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_18_0_1_0_1_0 = det*c0_0_0_18*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_18_0_1_1_0_0 = det*c0_0_0_18*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_18_0_1_2_1_0 = det*c0_0_0_18*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_18_0_1_3_0_0 = det*c0_0_0_18*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_18_0_1_3_1_0 = det*c0_0_0_18*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_18_0_1_4_0_0 = det*c0_0_0_18*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_18_0_1_4_1_0 = det*c0_0_0_18*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_18_0_1_5_0_0 = det*c0_0_0_18*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_18_0_1_5_1_0 = det*c0_0_0_18*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_18_0_1_6_0_0 = det*c0_0_0_18*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_18_0_1_6_1_0 = det*c0_0_0_18*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_18_0_1_7_0_0 = det*c0_0_0_18*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_18_0_1_7_1_0 = det*c0_0_0_18*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_18_0_1_8_0_0 = det*c0_0_0_18*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_18_0_1_8_1_0 = det*c0_0_0_18*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_18_0_1_9_0_0 = det*c0_0_0_18*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_18_0_1_9_1_0 = det*c0_0_0_18*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_18_0_1_10_0_0 = det*c0_0_0_18*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_18_0_1_10_1_0 = det*c0_0_0_18*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_18_0_1_11_0_0 = det*c0_0_0_18*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_18_0_1_11_1_0 = det*c0_0_0_18*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_18_0_1_12_0_0 = det*c0_0_0_18*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_18_0_1_12_1_0 = det*c0_0_0_18*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_18_0_1_13_0_0 = det*c0_0_0_18*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_18_0_1_13_1_0 = det*c0_0_0_18*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_18_0_1_14_0_0 = det*c0_0_0_18*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_18_0_1_14_1_0 = det*c0_0_0_18*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_18_0_1_15_0_1 = det*c0_0_0_18*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_18_0_1_15_1_1 = det*c0_0_0_18*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_18_0_1_16_0_1 = det*c0_0_0_18*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_18_0_1_17_1_1 = det*c0_0_0_18*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_18_0_1_18_0_1 = det*c0_0_0_18*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_18_0_1_18_1_1 = det*c0_0_0_18*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_18_0_1_19_0_1 = det*c0_0_0_18*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_18_0_1_19_1_1 = det*c0_0_0_18*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_18_0_1_20_0_1 = det*c0_0_0_18*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_18_0_1_20_1_1 = det*c0_0_0_18*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_18_0_1_21_0_1 = det*c0_0_0_18*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_18_0_1_21_1_1 = det*c0_0_0_18*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_18_0_1_22_0_1 = det*c0_0_0_18*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_18_0_1_22_1_1 = det*c0_0_0_18*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_18_0_1_23_0_1 = det*c0_0_0_18*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_18_0_1_23_1_1 = det*c0_0_0_18*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_18_0_1_24_0_1 = det*c0_0_0_18*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_18_0_1_24_1_1 = det*c0_0_0_18*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_18_0_1_25_0_1 = det*c0_0_0_18*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_18_0_1_25_1_1 = det*c0_0_0_18*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_18_0_1_26_0_1 = det*c0_0_0_18*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_18_0_1_26_1_1 = det*c0_0_0_18*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_18_0_1_27_0_1 = det*c0_0_0_18*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_18_0_1_27_1_1 = det*c0_0_0_18*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_18_0_1_28_0_1 = det*c0_0_0_18*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_18_0_1_28_1_1 = det*c0_0_0_18*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_18_0_1_29_0_1 = det*c0_0_0_18*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_18_0_1_29_1_1 = det*c0_0_0_18*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_18_1_1_0_0_0 = det*c0_0_0_18*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_18_1_1_0_1_0 = det*c0_0_0_18*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_18_1_1_1_0_0 = det*c0_0_0_18*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_18_1_1_2_1_0 = det*c0_0_0_18*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_18_1_1_3_0_0 = det*c0_0_0_18*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_18_1_1_3_1_0 = det*c0_0_0_18*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_18_1_1_4_0_0 = det*c0_0_0_18*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_18_1_1_4_1_0 = det*c0_0_0_18*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_18_1_1_5_0_0 = det*c0_0_0_18*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_18_1_1_5_1_0 = det*c0_0_0_18*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_18_1_1_6_0_0 = det*c0_0_0_18*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_18_1_1_7_0_0 = det*c0_0_0_18*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_18_1_1_8_0_0 = det*c0_0_0_18*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_18_1_1_9_0_0 = det*c0_0_0_18*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_18_1_1_9_1_0 = det*c0_0_0_18*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_18_1_1_10_0_0 = det*c0_0_0_18*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_18_1_1_10_1_0 = det*c0_0_0_18*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_18_1_1_11_0_0 = det*c0_0_0_18*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_18_1_1_11_1_0 = det*c0_0_0_18*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_18_1_1_12_0_0 = det*c0_0_0_18*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_18_1_1_13_0_0 = det*c0_0_0_18*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_18_1_1_14_0_0 = det*c0_0_0_18*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_18_1_1_15_0_1 = det*c0_0_0_18*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_18_1_1_15_1_1 = det*c0_0_0_18*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_18_1_1_16_0_1 = det*c0_0_0_18*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_18_1_1_17_1_1 = det*c0_0_0_18*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_18_1_1_18_0_1 = det*c0_0_0_18*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_18_1_1_18_1_1 = det*c0_0_0_18*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_18_1_1_19_0_1 = det*c0_0_0_18*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_18_1_1_19_1_1 = det*c0_0_0_18*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_18_1_1_20_0_1 = det*c0_0_0_18*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_18_1_1_20_1_1 = det*c0_0_0_18*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_18_1_1_21_0_1 = det*c0_0_0_18*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_18_1_1_22_0_1 = det*c0_0_0_18*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_18_1_1_23_0_1 = det*c0_0_0_18*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_18_1_1_24_0_1 = det*c0_0_0_18*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_18_1_1_24_1_1 = det*c0_0_0_18*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_18_1_1_25_0_1 = det*c0_0_0_18*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_18_1_1_25_1_1 = det*c0_0_0_18*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_18_1_1_26_0_1 = det*c0_0_0_18*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_18_1_1_26_1_1 = det*c0_0_0_18*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_18_1_1_27_0_1 = det*c0_0_0_18*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_18_1_1_28_0_1 = det*c0_0_0_18*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_18_1_1_29_0_1 = det*c0_0_0_18*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_19_0_1_0_0_0 = det*c0_0_0_19*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_19_0_1_0_1_0 = det*c0_0_0_19*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_19_0_1_1_0_0 = det*c0_0_0_19*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_19_0_1_2_1_0 = det*c0_0_0_19*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_19_0_1_3_0_0 = det*c0_0_0_19*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_19_0_1_3_1_0 = det*c0_0_0_19*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_19_0_1_4_0_0 = det*c0_0_0_19*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_19_0_1_4_1_0 = det*c0_0_0_19*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_19_0_1_5_0_0 = det*c0_0_0_19*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_19_0_1_5_1_0 = det*c0_0_0_19*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_19_0_1_6_0_0 = det*c0_0_0_19*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_19_0_1_6_1_0 = det*c0_0_0_19*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_19_0_1_7_0_0 = det*c0_0_0_19*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_19_0_1_7_1_0 = det*c0_0_0_19*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_19_0_1_8_0_0 = det*c0_0_0_19*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_19_0_1_8_1_0 = det*c0_0_0_19*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_19_0_1_9_0_0 = det*c0_0_0_19*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_19_0_1_9_1_0 = det*c0_0_0_19*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_19_0_1_10_0_0 = det*c0_0_0_19*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_19_0_1_10_1_0 = det*c0_0_0_19*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_19_0_1_11_0_0 = det*c0_0_0_19*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_19_0_1_11_1_0 = det*c0_0_0_19*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_19_0_1_12_0_0 = det*c0_0_0_19*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_19_0_1_12_1_0 = det*c0_0_0_19*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_19_0_1_13_0_0 = det*c0_0_0_19*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_19_0_1_13_1_0 = det*c0_0_0_19*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_19_0_1_14_0_0 = det*c0_0_0_19*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_19_0_1_14_1_0 = det*c0_0_0_19*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_19_0_1_15_0_1 = det*c0_0_0_19*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_19_0_1_15_1_1 = det*c0_0_0_19*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_19_0_1_16_0_1 = det*c0_0_0_19*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_19_0_1_17_1_1 = det*c0_0_0_19*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_19_0_1_18_0_1 = det*c0_0_0_19*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_19_0_1_18_1_1 = det*c0_0_0_19*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_19_0_1_19_0_1 = det*c0_0_0_19*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_19_0_1_19_1_1 = det*c0_0_0_19*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_19_0_1_20_0_1 = det*c0_0_0_19*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_19_0_1_20_1_1 = det*c0_0_0_19*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_19_0_1_21_0_1 = det*c0_0_0_19*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_19_0_1_21_1_1 = det*c0_0_0_19*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_19_0_1_22_0_1 = det*c0_0_0_19*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_19_0_1_22_1_1 = det*c0_0_0_19*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_19_0_1_23_0_1 = det*c0_0_0_19*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_19_0_1_23_1_1 = det*c0_0_0_19*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_19_0_1_24_0_1 = det*c0_0_0_19*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_19_0_1_24_1_1 = det*c0_0_0_19*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_19_0_1_25_0_1 = det*c0_0_0_19*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_19_0_1_25_1_1 = det*c0_0_0_19*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_19_0_1_26_0_1 = det*c0_0_0_19*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_19_0_1_26_1_1 = det*c0_0_0_19*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_19_0_1_27_0_1 = det*c0_0_0_19*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_19_0_1_27_1_1 = det*c0_0_0_19*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_19_0_1_28_0_1 = det*c0_0_0_19*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_19_0_1_28_1_1 = det*c0_0_0_19*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_19_0_1_29_0_1 = det*c0_0_0_19*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_19_0_1_29_1_1 = det*c0_0_0_19*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_19_1_1_0_0_0 = det*c0_0_0_19*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_19_1_1_0_1_0 = det*c0_0_0_19*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_19_1_1_1_0_0 = det*c0_0_0_19*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_19_1_1_2_1_0 = det*c0_0_0_19*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_19_1_1_3_0_0 = det*c0_0_0_19*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_19_1_1_3_1_0 = det*c0_0_0_19*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_19_1_1_4_0_0 = det*c0_0_0_19*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_19_1_1_4_1_0 = det*c0_0_0_19*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_19_1_1_5_0_0 = det*c0_0_0_19*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_19_1_1_5_1_0 = det*c0_0_0_19*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_19_1_1_6_0_0 = det*c0_0_0_19*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_19_1_1_6_1_0 = det*c0_0_0_19*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_19_1_1_7_0_0 = det*c0_0_0_19*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_19_1_1_7_1_0 = det*c0_0_0_19*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_19_1_1_8_0_0 = det*c0_0_0_19*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_19_1_1_8_1_0 = det*c0_0_0_19*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_19_1_1_9_0_0 = det*c0_0_0_19*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_19_1_1_9_1_0 = det*c0_0_0_19*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_19_1_1_10_0_0 = det*c0_0_0_19*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_19_1_1_10_1_0 = det*c0_0_0_19*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_19_1_1_11_0_0 = det*c0_0_0_19*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_19_1_1_11_1_0 = det*c0_0_0_19*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_19_1_1_12_0_0 = det*c0_0_0_19*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_19_1_1_12_1_0 = det*c0_0_0_19*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_19_1_1_13_0_0 = det*c0_0_0_19*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_19_1_1_13_1_0 = det*c0_0_0_19*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_19_1_1_14_0_0 = det*c0_0_0_19*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_19_1_1_14_1_0 = det*c0_0_0_19*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_19_1_1_15_0_1 = det*c0_0_0_19*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_19_1_1_15_1_1 = det*c0_0_0_19*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_19_1_1_16_0_1 = det*c0_0_0_19*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_19_1_1_17_1_1 = det*c0_0_0_19*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_19_1_1_18_0_1 = det*c0_0_0_19*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_19_1_1_18_1_1 = det*c0_0_0_19*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_19_1_1_19_0_1 = det*c0_0_0_19*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_19_1_1_19_1_1 = det*c0_0_0_19*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_19_1_1_20_0_1 = det*c0_0_0_19*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_19_1_1_20_1_1 = det*c0_0_0_19*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_19_1_1_21_0_1 = det*c0_0_0_19*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_19_1_1_21_1_1 = det*c0_0_0_19*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_19_1_1_22_0_1 = det*c0_0_0_19*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_19_1_1_22_1_1 = det*c0_0_0_19*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_19_1_1_23_0_1 = det*c0_0_0_19*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_19_1_1_23_1_1 = det*c0_0_0_19*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_19_1_1_24_0_1 = det*c0_0_0_19*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_19_1_1_24_1_1 = det*c0_0_0_19*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_19_1_1_25_0_1 = det*c0_0_0_19*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_19_1_1_25_1_1 = det*c0_0_0_19*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_19_1_1_26_0_1 = det*c0_0_0_19*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_19_1_1_26_1_1 = det*c0_0_0_19*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_19_1_1_27_0_1 = det*c0_0_0_19*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_19_1_1_27_1_1 = det*c0_0_0_19*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_19_1_1_28_0_1 = det*c0_0_0_19*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_19_1_1_28_1_1 = det*c0_0_0_19*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_19_1_1_29_0_1 = det*c0_0_0_19*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_19_1_1_29_1_1 = det*c0_0_0_19*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_20_0_1_0_0_0 = det*c0_0_0_20*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_20_0_1_0_1_0 = det*c0_0_0_20*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_20_0_1_1_0_0 = det*c0_0_0_20*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_20_0_1_2_1_0 = det*c0_0_0_20*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_20_0_1_3_0_0 = det*c0_0_0_20*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_20_0_1_3_1_0 = det*c0_0_0_20*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_20_0_1_4_0_0 = det*c0_0_0_20*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_20_0_1_4_1_0 = det*c0_0_0_20*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_20_0_1_5_0_0 = det*c0_0_0_20*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_20_0_1_5_1_0 = det*c0_0_0_20*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_20_0_1_6_0_0 = det*c0_0_0_20*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_20_0_1_6_1_0 = det*c0_0_0_20*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_20_0_1_7_0_0 = det*c0_0_0_20*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_20_0_1_7_1_0 = det*c0_0_0_20*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_20_0_1_8_0_0 = det*c0_0_0_20*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_20_0_1_8_1_0 = det*c0_0_0_20*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_20_0_1_9_1_0 = det*c0_0_0_20*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_20_0_1_10_1_0 = det*c0_0_0_20*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_20_0_1_11_1_0 = det*c0_0_0_20*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_20_0_1_12_1_0 = det*c0_0_0_20*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_20_0_1_13_1_0 = det*c0_0_0_20*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_20_0_1_14_1_0 = det*c0_0_0_20*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_20_0_1_15_0_1 = det*c0_0_0_20*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_20_0_1_15_1_1 = det*c0_0_0_20*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_20_0_1_16_0_1 = det*c0_0_0_20*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_20_0_1_17_1_1 = det*c0_0_0_20*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_20_0_1_18_0_1 = det*c0_0_0_20*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_20_0_1_18_1_1 = det*c0_0_0_20*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_20_0_1_19_0_1 = det*c0_0_0_20*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_20_0_1_19_1_1 = det*c0_0_0_20*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_20_0_1_20_0_1 = det*c0_0_0_20*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_20_0_1_20_1_1 = det*c0_0_0_20*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_20_0_1_21_0_1 = det*c0_0_0_20*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_20_0_1_21_1_1 = det*c0_0_0_20*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_20_0_1_22_0_1 = det*c0_0_0_20*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_20_0_1_22_1_1 = det*c0_0_0_20*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_20_0_1_23_0_1 = det*c0_0_0_20*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_20_0_1_23_1_1 = det*c0_0_0_20*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_20_0_1_24_1_1 = det*c0_0_0_20*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_20_0_1_25_1_1 = det*c0_0_0_20*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_20_0_1_26_1_1 = det*c0_0_0_20*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_20_0_1_27_1_1 = det*c0_0_0_20*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_20_0_1_28_1_1 = det*c0_0_0_20*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_20_0_1_29_1_1 = det*c0_0_0_20*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_20_1_1_0_0_0 = det*c0_0_0_20*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_20_1_1_0_1_0 = det*c0_0_0_20*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_20_1_1_1_0_0 = det*c0_0_0_20*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_20_1_1_2_1_0 = det*c0_0_0_20*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_20_1_1_3_0_0 = det*c0_0_0_20*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_20_1_1_3_1_0 = det*c0_0_0_20*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_20_1_1_4_0_0 = det*c0_0_0_20*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_20_1_1_4_1_0 = det*c0_0_0_20*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_20_1_1_5_0_0 = det*c0_0_0_20*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_20_1_1_5_1_0 = det*c0_0_0_20*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_20_1_1_6_0_0 = det*c0_0_0_20*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_20_1_1_6_1_0 = det*c0_0_0_20*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_20_1_1_7_0_0 = det*c0_0_0_20*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_20_1_1_7_1_0 = det*c0_0_0_20*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_20_1_1_8_0_0 = det*c0_0_0_20*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_20_1_1_8_1_0 = det*c0_0_0_20*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_20_1_1_9_0_0 = det*c0_0_0_20*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_20_1_1_9_1_0 = det*c0_0_0_20*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_20_1_1_10_0_0 = det*c0_0_0_20*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_20_1_1_10_1_0 = det*c0_0_0_20*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_20_1_1_11_0_0 = det*c0_0_0_20*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_20_1_1_11_1_0 = det*c0_0_0_20*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_20_1_1_12_0_0 = det*c0_0_0_20*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_20_1_1_12_1_0 = det*c0_0_0_20*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_20_1_1_13_0_0 = det*c0_0_0_20*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_20_1_1_13_1_0 = det*c0_0_0_20*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_20_1_1_14_0_0 = det*c0_0_0_20*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_20_1_1_14_1_0 = det*c0_0_0_20*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_20_1_1_15_0_1 = det*c0_0_0_20*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_20_1_1_15_1_1 = det*c0_0_0_20*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_20_1_1_16_0_1 = det*c0_0_0_20*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_20_1_1_17_1_1 = det*c0_0_0_20*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_20_1_1_18_0_1 = det*c0_0_0_20*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_20_1_1_18_1_1 = det*c0_0_0_20*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_20_1_1_19_0_1 = det*c0_0_0_20*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_20_1_1_19_1_1 = det*c0_0_0_20*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_20_1_1_20_0_1 = det*c0_0_0_20*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_20_1_1_20_1_1 = det*c0_0_0_20*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_20_1_1_21_0_1 = det*c0_0_0_20*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_20_1_1_21_1_1 = det*c0_0_0_20*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_20_1_1_22_0_1 = det*c0_0_0_20*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_20_1_1_22_1_1 = det*c0_0_0_20*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_20_1_1_23_0_1 = det*c0_0_0_20*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_20_1_1_23_1_1 = det*c0_0_0_20*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_20_1_1_24_0_1 = det*c0_0_0_20*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_20_1_1_24_1_1 = det*c0_0_0_20*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_20_1_1_25_0_1 = det*c0_0_0_20*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_20_1_1_25_1_1 = det*c0_0_0_20*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_20_1_1_26_0_1 = det*c0_0_0_20*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_20_1_1_26_1_1 = det*c0_0_0_20*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_20_1_1_27_0_1 = det*c0_0_0_20*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_20_1_1_27_1_1 = det*c0_0_0_20*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_20_1_1_28_0_1 = det*c0_0_0_20*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_20_1_1_28_1_1 = det*c0_0_0_20*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_20_1_1_29_0_1 = det*c0_0_0_20*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_20_1_1_29_1_1 = det*c0_0_0_20*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_21_0_1_0_0_0 = det*c0_0_0_21*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_21_0_1_0_1_0 = det*c0_0_0_21*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_21_0_1_1_0_0 = det*c0_0_0_21*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_21_0_1_2_1_0 = det*c0_0_0_21*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_21_0_1_3_0_0 = det*c0_0_0_21*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_21_0_1_3_1_0 = det*c0_0_0_21*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_21_0_1_4_0_0 = det*c0_0_0_21*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_21_0_1_4_1_0 = det*c0_0_0_21*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_21_0_1_5_0_0 = det*c0_0_0_21*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_21_0_1_5_1_0 = det*c0_0_0_21*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_21_0_1_6_0_0 = det*c0_0_0_21*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_21_0_1_6_1_0 = det*c0_0_0_21*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_21_0_1_7_0_0 = det*c0_0_0_21*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_21_0_1_7_1_0 = det*c0_0_0_21*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_21_0_1_8_0_0 = det*c0_0_0_21*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_21_0_1_8_1_0 = det*c0_0_0_21*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_21_0_1_9_0_0 = det*c0_0_0_21*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_21_0_1_9_1_0 = det*c0_0_0_21*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_21_0_1_10_0_0 = det*c0_0_0_21*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_21_0_1_10_1_0 = det*c0_0_0_21*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_21_0_1_11_0_0 = det*c0_0_0_21*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_21_0_1_11_1_0 = det*c0_0_0_21*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_21_0_1_12_0_0 = det*c0_0_0_21*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_21_0_1_12_1_0 = det*c0_0_0_21*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_21_0_1_13_0_0 = det*c0_0_0_21*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_21_0_1_13_1_0 = det*c0_0_0_21*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_21_0_1_14_0_0 = det*c0_0_0_21*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_21_0_1_14_1_0 = det*c0_0_0_21*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_21_0_1_15_0_1 = det*c0_0_0_21*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_21_0_1_15_1_1 = det*c0_0_0_21*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_21_0_1_16_0_1 = det*c0_0_0_21*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_21_0_1_17_1_1 = det*c0_0_0_21*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_21_0_1_18_0_1 = det*c0_0_0_21*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_21_0_1_18_1_1 = det*c0_0_0_21*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_21_0_1_19_0_1 = det*c0_0_0_21*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_21_0_1_19_1_1 = det*c0_0_0_21*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_21_0_1_20_0_1 = det*c0_0_0_21*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_21_0_1_20_1_1 = det*c0_0_0_21*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_21_0_1_21_0_1 = det*c0_0_0_21*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_21_0_1_21_1_1 = det*c0_0_0_21*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_21_0_1_22_0_1 = det*c0_0_0_21*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_21_0_1_22_1_1 = det*c0_0_0_21*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_21_0_1_23_0_1 = det*c0_0_0_21*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_21_0_1_23_1_1 = det*c0_0_0_21*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_21_0_1_24_0_1 = det*c0_0_0_21*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_21_0_1_24_1_1 = det*c0_0_0_21*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_21_0_1_25_0_1 = det*c0_0_0_21*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_21_0_1_25_1_1 = det*c0_0_0_21*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_21_0_1_26_0_1 = det*c0_0_0_21*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_21_0_1_26_1_1 = det*c0_0_0_21*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_21_0_1_27_0_1 = det*c0_0_0_21*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_21_0_1_27_1_1 = det*c0_0_0_21*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_21_0_1_28_0_1 = det*c0_0_0_21*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_21_0_1_28_1_1 = det*c0_0_0_21*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_21_0_1_29_0_1 = det*c0_0_0_21*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_21_0_1_29_1_1 = det*c0_0_0_21*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_21_1_1_0_0_0 = det*c0_0_0_21*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_21_1_1_0_1_0 = det*c0_0_0_21*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_21_1_1_2_1_0 = det*c0_0_0_21*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_21_1_1_3_0_0 = det*c0_0_0_21*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_21_1_1_4_0_0 = det*c0_0_0_21*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_21_1_1_4_1_0 = det*c0_0_0_21*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_21_1_1_5_0_0 = det*c0_0_0_21*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_21_1_1_5_1_0 = det*c0_0_0_21*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_21_1_1_6_0_0 = det*c0_0_0_21*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_21_1_1_6_1_0 = det*c0_0_0_21*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_21_1_1_7_0_0 = det*c0_0_0_21*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_21_1_1_7_1_0 = det*c0_0_0_21*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_21_1_1_8_0_0 = det*c0_0_0_21*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_21_1_1_8_1_0 = det*c0_0_0_21*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_21_1_1_9_0_0 = det*c0_0_0_21*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_21_1_1_9_1_0 = det*c0_0_0_21*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_21_1_1_10_0_0 = det*c0_0_0_21*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_21_1_1_10_1_0 = det*c0_0_0_21*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_21_1_1_11_0_0 = det*c0_0_0_21*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_21_1_1_12_0_0 = det*c0_0_0_21*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_21_1_1_12_1_0 = det*c0_0_0_21*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_21_1_1_13_0_0 = det*c0_0_0_21*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_21_1_1_13_1_0 = det*c0_0_0_21*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_21_1_1_14_0_0 = det*c0_0_0_21*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_21_1_1_14_1_0 = det*c0_0_0_21*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_21_1_1_15_0_1 = det*c0_0_0_21*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_21_1_1_15_1_1 = det*c0_0_0_21*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_21_1_1_17_1_1 = det*c0_0_0_21*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_21_1_1_18_0_1 = det*c0_0_0_21*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_21_1_1_19_0_1 = det*c0_0_0_21*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_21_1_1_19_1_1 = det*c0_0_0_21*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_21_1_1_20_0_1 = det*c0_0_0_21*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_21_1_1_20_1_1 = det*c0_0_0_21*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_21_1_1_21_0_1 = det*c0_0_0_21*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_21_1_1_21_1_1 = det*c0_0_0_21*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_21_1_1_22_0_1 = det*c0_0_0_21*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_21_1_1_22_1_1 = det*c0_0_0_21*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_21_1_1_23_0_1 = det*c0_0_0_21*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_21_1_1_23_1_1 = det*c0_0_0_21*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_21_1_1_24_0_1 = det*c0_0_0_21*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_21_1_1_24_1_1 = det*c0_0_0_21*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_21_1_1_25_0_1 = det*c0_0_0_21*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_21_1_1_25_1_1 = det*c0_0_0_21*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_21_1_1_26_0_1 = det*c0_0_0_21*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_21_1_1_27_0_1 = det*c0_0_0_21*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_21_1_1_27_1_1 = det*c0_0_0_21*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_21_1_1_28_0_1 = det*c0_0_0_21*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_21_1_1_28_1_1 = det*c0_0_0_21*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_21_1_1_29_0_1 = det*c0_0_0_21*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_21_1_1_29_1_1 = det*c0_0_0_21*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_22_0_1_0_0_0 = det*c0_0_0_22*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_22_0_1_0_1_0 = det*c0_0_0_22*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_22_0_1_1_0_0 = det*c0_0_0_22*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_22_0_1_2_1_0 = det*c0_0_0_22*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_22_0_1_3_0_0 = det*c0_0_0_22*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_22_0_1_3_1_0 = det*c0_0_0_22*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_22_0_1_4_0_0 = det*c0_0_0_22*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_22_0_1_4_1_0 = det*c0_0_0_22*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_22_0_1_5_0_0 = det*c0_0_0_22*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_22_0_1_5_1_0 = det*c0_0_0_22*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_22_0_1_6_0_0 = det*c0_0_0_22*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_22_0_1_6_1_0 = det*c0_0_0_22*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_22_0_1_7_0_0 = det*c0_0_0_22*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_22_0_1_7_1_0 = det*c0_0_0_22*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_22_0_1_8_0_0 = det*c0_0_0_22*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_22_0_1_8_1_0 = det*c0_0_0_22*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_22_0_1_9_0_0 = det*c0_0_0_22*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_22_0_1_9_1_0 = det*c0_0_0_22*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_22_0_1_10_0_0 = det*c0_0_0_22*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_22_0_1_10_1_0 = det*c0_0_0_22*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_22_0_1_11_0_0 = det*c0_0_0_22*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_22_0_1_11_1_0 = det*c0_0_0_22*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_22_0_1_12_0_0 = det*c0_0_0_22*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_22_0_1_12_1_0 = det*c0_0_0_22*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_22_0_1_13_0_0 = det*c0_0_0_22*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_22_0_1_13_1_0 = det*c0_0_0_22*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_22_0_1_14_0_0 = det*c0_0_0_22*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_22_0_1_14_1_0 = det*c0_0_0_22*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_22_0_1_15_0_1 = det*c0_0_0_22*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_22_0_1_15_1_1 = det*c0_0_0_22*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_22_0_1_16_0_1 = det*c0_0_0_22*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_22_0_1_17_1_1 = det*c0_0_0_22*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_22_0_1_18_0_1 = det*c0_0_0_22*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_22_0_1_18_1_1 = det*c0_0_0_22*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_22_0_1_19_0_1 = det*c0_0_0_22*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_22_0_1_19_1_1 = det*c0_0_0_22*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_22_0_1_20_0_1 = det*c0_0_0_22*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_22_0_1_20_1_1 = det*c0_0_0_22*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_22_0_1_21_0_1 = det*c0_0_0_22*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_22_0_1_21_1_1 = det*c0_0_0_22*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_22_0_1_22_0_1 = det*c0_0_0_22*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_22_0_1_22_1_1 = det*c0_0_0_22*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_22_0_1_23_0_1 = det*c0_0_0_22*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_22_0_1_23_1_1 = det*c0_0_0_22*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_22_0_1_24_0_1 = det*c0_0_0_22*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_22_0_1_24_1_1 = det*c0_0_0_22*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_22_0_1_25_0_1 = det*c0_0_0_22*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_22_0_1_25_1_1 = det*c0_0_0_22*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_22_0_1_26_0_1 = det*c0_0_0_22*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_22_0_1_26_1_1 = det*c0_0_0_22*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_22_0_1_27_0_1 = det*c0_0_0_22*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_22_0_1_27_1_1 = det*c0_0_0_22*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_22_0_1_28_0_1 = det*c0_0_0_22*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_22_0_1_28_1_1 = det*c0_0_0_22*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_22_0_1_29_0_1 = det*c0_0_0_22*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_22_0_1_29_1_1 = det*c0_0_0_22*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_22_1_1_0_0_0 = det*c0_0_0_22*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_22_1_1_0_1_0 = det*c0_0_0_22*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_22_1_1_2_1_0 = det*c0_0_0_22*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_22_1_1_3_0_0 = det*c0_0_0_22*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_22_1_1_4_0_0 = det*c0_0_0_22*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_22_1_1_4_1_0 = det*c0_0_0_22*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_22_1_1_5_0_0 = det*c0_0_0_22*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_22_1_1_5_1_0 = det*c0_0_0_22*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_22_1_1_6_0_0 = det*c0_0_0_22*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_22_1_1_6_1_0 = det*c0_0_0_22*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_22_1_1_7_0_0 = det*c0_0_0_22*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_22_1_1_7_1_0 = det*c0_0_0_22*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_22_1_1_8_0_0 = det*c0_0_0_22*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_22_1_1_8_1_0 = det*c0_0_0_22*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_22_1_1_9_0_0 = det*c0_0_0_22*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_22_1_1_9_1_0 = det*c0_0_0_22*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_22_1_1_10_0_0 = det*c0_0_0_22*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_22_1_1_10_1_0 = det*c0_0_0_22*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_22_1_1_11_0_0 = det*c0_0_0_22*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_22_1_1_12_0_0 = det*c0_0_0_22*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_22_1_1_12_1_0 = det*c0_0_0_22*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_22_1_1_13_0_0 = det*c0_0_0_22*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_22_1_1_13_1_0 = det*c0_0_0_22*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_22_1_1_14_0_0 = det*c0_0_0_22*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_22_1_1_14_1_0 = det*c0_0_0_22*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_22_1_1_15_0_1 = det*c0_0_0_22*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_22_1_1_15_1_1 = det*c0_0_0_22*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_22_1_1_17_1_1 = det*c0_0_0_22*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_22_1_1_18_0_1 = det*c0_0_0_22*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_22_1_1_19_0_1 = det*c0_0_0_22*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_22_1_1_19_1_1 = det*c0_0_0_22*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_22_1_1_20_0_1 = det*c0_0_0_22*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_22_1_1_20_1_1 = det*c0_0_0_22*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_22_1_1_21_0_1 = det*c0_0_0_22*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_22_1_1_21_1_1 = det*c0_0_0_22*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_22_1_1_22_0_1 = det*c0_0_0_22*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_22_1_1_22_1_1 = det*c0_0_0_22*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_22_1_1_23_0_1 = det*c0_0_0_22*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_22_1_1_23_1_1 = det*c0_0_0_22*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_22_1_1_24_0_1 = det*c0_0_0_22*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_22_1_1_24_1_1 = det*c0_0_0_22*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_22_1_1_25_0_1 = det*c0_0_0_22*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_22_1_1_25_1_1 = det*c0_0_0_22*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_22_1_1_26_0_1 = det*c0_0_0_22*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_22_1_1_27_0_1 = det*c0_0_0_22*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_22_1_1_27_1_1 = det*c0_0_0_22*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_22_1_1_28_0_1 = det*c0_0_0_22*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_22_1_1_28_1_1 = det*c0_0_0_22*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_22_1_1_29_0_1 = det*c0_0_0_22*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_22_1_1_29_1_1 = det*c0_0_0_22*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_23_0_1_0_0_0 = det*c0_0_0_23*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_23_0_1_0_1_0 = det*c0_0_0_23*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_23_0_1_1_0_0 = det*c0_0_0_23*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_23_0_1_2_1_0 = det*c0_0_0_23*c0_0_1_2*Jinv_01*Jinv_10;
    const double G0_23_0_1_3_0_0 = det*c0_0_0_23*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_23_0_1_3_1_0 = det*c0_0_0_23*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_23_0_1_4_0_0 = det*c0_0_0_23*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_23_0_1_4_1_0 = det*c0_0_0_23*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_23_0_1_5_0_0 = det*c0_0_0_23*c0_0_1_5*Jinv_01*Jinv_00;
    const double G0_23_0_1_5_1_0 = det*c0_0_0_23*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_23_0_1_6_0_0 = det*c0_0_0_23*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_23_0_1_6_1_0 = det*c0_0_0_23*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_23_0_1_7_0_0 = det*c0_0_0_23*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_23_0_1_7_1_0 = det*c0_0_0_23*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_23_0_1_8_0_0 = det*c0_0_0_23*c0_0_1_8*Jinv_01*Jinv_00;
    const double G0_23_0_1_8_1_0 = det*c0_0_0_23*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_23_0_1_9_1_0 = det*c0_0_0_23*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_23_0_1_10_1_0 = det*c0_0_0_23*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_23_0_1_11_1_0 = det*c0_0_0_23*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_23_0_1_12_1_0 = det*c0_0_0_23*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_23_0_1_13_1_0 = det*c0_0_0_23*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_23_0_1_14_1_0 = det*c0_0_0_23*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_23_0_1_15_0_1 = det*c0_0_0_23*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_23_0_1_15_1_1 = det*c0_0_0_23*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_23_0_1_16_0_1 = det*c0_0_0_23*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_23_0_1_17_1_1 = det*c0_0_0_23*c0_0_1_17*Jinv_01*Jinv_11;
    const double G0_23_0_1_18_0_1 = det*c0_0_0_23*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_23_0_1_18_1_1 = det*c0_0_0_23*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_23_0_1_19_0_1 = det*c0_0_0_23*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_23_0_1_19_1_1 = det*c0_0_0_23*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_23_0_1_20_0_1 = det*c0_0_0_23*c0_0_1_20*Jinv_01*Jinv_01;
    const double G0_23_0_1_20_1_1 = det*c0_0_0_23*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_23_0_1_21_0_1 = det*c0_0_0_23*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_23_0_1_21_1_1 = det*c0_0_0_23*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_23_0_1_22_0_1 = det*c0_0_0_23*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_23_0_1_22_1_1 = det*c0_0_0_23*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_23_0_1_23_0_1 = det*c0_0_0_23*c0_0_1_23*Jinv_01*Jinv_01;
    const double G0_23_0_1_23_1_1 = det*c0_0_0_23*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_23_0_1_24_1_1 = det*c0_0_0_23*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_23_0_1_25_1_1 = det*c0_0_0_23*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_23_0_1_26_1_1 = det*c0_0_0_23*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_23_0_1_27_1_1 = det*c0_0_0_23*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_23_0_1_28_1_1 = det*c0_0_0_23*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_23_0_1_29_1_1 = det*c0_0_0_23*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_23_1_1_0_0_0 = det*c0_0_0_23*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_23_1_1_0_1_0 = det*c0_0_0_23*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_23_1_1_2_1_0 = det*c0_0_0_23*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_23_1_1_3_0_0 = det*c0_0_0_23*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_23_1_1_4_0_0 = det*c0_0_0_23*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_23_1_1_4_1_0 = det*c0_0_0_23*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_23_1_1_5_0_0 = det*c0_0_0_23*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_23_1_1_5_1_0 = det*c0_0_0_23*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_23_1_1_6_0_0 = det*c0_0_0_23*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_23_1_1_6_1_0 = det*c0_0_0_23*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_23_1_1_7_0_0 = det*c0_0_0_23*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_23_1_1_7_1_0 = det*c0_0_0_23*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_23_1_1_8_0_0 = det*c0_0_0_23*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_23_1_1_8_1_0 = det*c0_0_0_23*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_23_1_1_9_0_0 = det*c0_0_0_23*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_23_1_1_9_1_0 = det*c0_0_0_23*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_23_1_1_10_0_0 = det*c0_0_0_23*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_23_1_1_10_1_0 = det*c0_0_0_23*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_23_1_1_11_0_0 = det*c0_0_0_23*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_23_1_1_12_0_0 = det*c0_0_0_23*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_23_1_1_12_1_0 = det*c0_0_0_23*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_23_1_1_13_0_0 = det*c0_0_0_23*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_23_1_1_13_1_0 = det*c0_0_0_23*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_23_1_1_14_0_0 = det*c0_0_0_23*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_23_1_1_14_1_0 = det*c0_0_0_23*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_23_1_1_15_0_1 = det*c0_0_0_23*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_23_1_1_15_1_1 = det*c0_0_0_23*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_23_1_1_17_1_1 = det*c0_0_0_23*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_23_1_1_18_0_1 = det*c0_0_0_23*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_23_1_1_19_0_1 = det*c0_0_0_23*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_23_1_1_19_1_1 = det*c0_0_0_23*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_23_1_1_20_0_1 = det*c0_0_0_23*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_23_1_1_20_1_1 = det*c0_0_0_23*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_23_1_1_21_0_1 = det*c0_0_0_23*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_23_1_1_21_1_1 = det*c0_0_0_23*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_23_1_1_22_0_1 = det*c0_0_0_23*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_23_1_1_22_1_1 = det*c0_0_0_23*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_23_1_1_23_0_1 = det*c0_0_0_23*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_23_1_1_23_1_1 = det*c0_0_0_23*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_23_1_1_24_0_1 = det*c0_0_0_23*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_23_1_1_24_1_1 = det*c0_0_0_23*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_23_1_1_25_0_1 = det*c0_0_0_23*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_23_1_1_25_1_1 = det*c0_0_0_23*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_23_1_1_26_0_1 = det*c0_0_0_23*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_23_1_1_27_0_1 = det*c0_0_0_23*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_23_1_1_27_1_1 = det*c0_0_0_23*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_23_1_1_28_0_1 = det*c0_0_0_23*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_23_1_1_28_1_1 = det*c0_0_0_23*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_23_1_1_29_0_1 = det*c0_0_0_23*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_23_1_1_29_1_1 = det*c0_0_0_23*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_24_0_1_0_0_0 = det*c0_0_0_24*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_24_0_1_0_1_0 = det*c0_0_0_24*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_24_0_1_1_0_0 = det*c0_0_0_24*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_24_0_1_3_0_0 = det*c0_0_0_24*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_24_0_1_3_1_0 = det*c0_0_0_24*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_24_0_1_4_0_0 = det*c0_0_0_24*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_24_0_1_4_1_0 = det*c0_0_0_24*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_24_0_1_5_1_0 = det*c0_0_0_24*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_24_0_1_6_0_0 = det*c0_0_0_24*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_24_0_1_6_1_0 = det*c0_0_0_24*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_24_0_1_7_0_0 = det*c0_0_0_24*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_24_0_1_7_1_0 = det*c0_0_0_24*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_24_0_1_8_1_0 = det*c0_0_0_24*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_24_0_1_9_0_0 = det*c0_0_0_24*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_24_0_1_9_1_0 = det*c0_0_0_24*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_24_0_1_10_0_0 = det*c0_0_0_24*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_24_0_1_10_1_0 = det*c0_0_0_24*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_24_0_1_11_0_0 = det*c0_0_0_24*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_24_0_1_11_1_0 = det*c0_0_0_24*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_24_0_1_12_0_0 = det*c0_0_0_24*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_24_0_1_12_1_0 = det*c0_0_0_24*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_24_0_1_13_0_0 = det*c0_0_0_24*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_24_0_1_13_1_0 = det*c0_0_0_24*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_24_0_1_14_0_0 = det*c0_0_0_24*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_24_0_1_14_1_0 = det*c0_0_0_24*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_24_0_1_15_0_1 = det*c0_0_0_24*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_24_0_1_15_1_1 = det*c0_0_0_24*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_24_0_1_16_0_1 = det*c0_0_0_24*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_24_0_1_18_0_1 = det*c0_0_0_24*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_24_0_1_18_1_1 = det*c0_0_0_24*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_24_0_1_19_0_1 = det*c0_0_0_24*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_24_0_1_19_1_1 = det*c0_0_0_24*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_24_0_1_20_1_1 = det*c0_0_0_24*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_24_0_1_21_0_1 = det*c0_0_0_24*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_24_0_1_21_1_1 = det*c0_0_0_24*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_24_0_1_22_0_1 = det*c0_0_0_24*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_24_0_1_22_1_1 = det*c0_0_0_24*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_24_0_1_23_1_1 = det*c0_0_0_24*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_24_0_1_24_0_1 = det*c0_0_0_24*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_24_0_1_24_1_1 = det*c0_0_0_24*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_24_0_1_25_0_1 = det*c0_0_0_24*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_24_0_1_25_1_1 = det*c0_0_0_24*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_24_0_1_26_0_1 = det*c0_0_0_24*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_24_0_1_26_1_1 = det*c0_0_0_24*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_24_0_1_27_0_1 = det*c0_0_0_24*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_24_0_1_27_1_1 = det*c0_0_0_24*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_24_0_1_28_0_1 = det*c0_0_0_24*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_24_0_1_28_1_1 = det*c0_0_0_24*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_24_0_1_29_0_1 = det*c0_0_0_24*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_24_0_1_29_1_1 = det*c0_0_0_24*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_24_1_1_0_0_0 = det*c0_0_0_24*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_24_1_1_0_1_0 = det*c0_0_0_24*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_24_1_1_1_0_0 = det*c0_0_0_24*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_24_1_1_2_1_0 = det*c0_0_0_24*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_24_1_1_3_0_0 = det*c0_0_0_24*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_24_1_1_3_1_0 = det*c0_0_0_24*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_24_1_1_4_0_0 = det*c0_0_0_24*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_24_1_1_4_1_0 = det*c0_0_0_24*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_24_1_1_5_0_0 = det*c0_0_0_24*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_24_1_1_5_1_0 = det*c0_0_0_24*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_24_1_1_6_0_0 = det*c0_0_0_24*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_24_1_1_6_1_0 = det*c0_0_0_24*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_24_1_1_7_0_0 = det*c0_0_0_24*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_24_1_1_7_1_0 = det*c0_0_0_24*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_24_1_1_8_0_0 = det*c0_0_0_24*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_24_1_1_8_1_0 = det*c0_0_0_24*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_24_1_1_9_0_0 = det*c0_0_0_24*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_24_1_1_9_1_0 = det*c0_0_0_24*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_24_1_1_10_0_0 = det*c0_0_0_24*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_24_1_1_10_1_0 = det*c0_0_0_24*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_24_1_1_11_0_0 = det*c0_0_0_24*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_24_1_1_11_1_0 = det*c0_0_0_24*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_24_1_1_12_0_0 = det*c0_0_0_24*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_24_1_1_12_1_0 = det*c0_0_0_24*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_24_1_1_13_0_0 = det*c0_0_0_24*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_24_1_1_13_1_0 = det*c0_0_0_24*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_24_1_1_14_0_0 = det*c0_0_0_24*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_24_1_1_14_1_0 = det*c0_0_0_24*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_24_1_1_15_0_1 = det*c0_0_0_24*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_24_1_1_15_1_1 = det*c0_0_0_24*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_24_1_1_16_0_1 = det*c0_0_0_24*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_24_1_1_17_1_1 = det*c0_0_0_24*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_24_1_1_18_0_1 = det*c0_0_0_24*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_24_1_1_18_1_1 = det*c0_0_0_24*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_24_1_1_19_0_1 = det*c0_0_0_24*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_24_1_1_19_1_1 = det*c0_0_0_24*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_24_1_1_20_0_1 = det*c0_0_0_24*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_24_1_1_20_1_1 = det*c0_0_0_24*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_24_1_1_21_0_1 = det*c0_0_0_24*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_24_1_1_21_1_1 = det*c0_0_0_24*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_24_1_1_22_0_1 = det*c0_0_0_24*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_24_1_1_22_1_1 = det*c0_0_0_24*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_24_1_1_23_0_1 = det*c0_0_0_24*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_24_1_1_23_1_1 = det*c0_0_0_24*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_24_1_1_24_0_1 = det*c0_0_0_24*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_24_1_1_24_1_1 = det*c0_0_0_24*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_24_1_1_25_0_1 = det*c0_0_0_24*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_24_1_1_25_1_1 = det*c0_0_0_24*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_24_1_1_26_0_1 = det*c0_0_0_24*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_24_1_1_26_1_1 = det*c0_0_0_24*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_24_1_1_27_0_1 = det*c0_0_0_24*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_24_1_1_27_1_1 = det*c0_0_0_24*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_24_1_1_28_0_1 = det*c0_0_0_24*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_24_1_1_28_1_1 = det*c0_0_0_24*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_24_1_1_29_0_1 = det*c0_0_0_24*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_24_1_1_29_1_1 = det*c0_0_0_24*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_25_0_1_0_0_0 = det*c0_0_0_25*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_25_0_1_0_1_0 = det*c0_0_0_25*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_25_0_1_1_0_0 = det*c0_0_0_25*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_25_0_1_3_0_0 = det*c0_0_0_25*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_25_0_1_3_1_0 = det*c0_0_0_25*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_25_0_1_4_0_0 = det*c0_0_0_25*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_25_0_1_4_1_0 = det*c0_0_0_25*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_25_0_1_5_1_0 = det*c0_0_0_25*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_25_0_1_6_0_0 = det*c0_0_0_25*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_25_0_1_6_1_0 = det*c0_0_0_25*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_25_0_1_7_0_0 = det*c0_0_0_25*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_25_0_1_7_1_0 = det*c0_0_0_25*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_25_0_1_8_1_0 = det*c0_0_0_25*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_25_0_1_9_0_0 = det*c0_0_0_25*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_25_0_1_9_1_0 = det*c0_0_0_25*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_25_0_1_10_0_0 = det*c0_0_0_25*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_25_0_1_10_1_0 = det*c0_0_0_25*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_25_0_1_11_0_0 = det*c0_0_0_25*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_25_0_1_11_1_0 = det*c0_0_0_25*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_25_0_1_12_0_0 = det*c0_0_0_25*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_25_0_1_12_1_0 = det*c0_0_0_25*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_25_0_1_13_0_0 = det*c0_0_0_25*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_25_0_1_13_1_0 = det*c0_0_0_25*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_25_0_1_14_0_0 = det*c0_0_0_25*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_25_0_1_14_1_0 = det*c0_0_0_25*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_25_0_1_15_0_1 = det*c0_0_0_25*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_25_0_1_15_1_1 = det*c0_0_0_25*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_25_0_1_16_0_1 = det*c0_0_0_25*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_25_0_1_18_0_1 = det*c0_0_0_25*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_25_0_1_18_1_1 = det*c0_0_0_25*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_25_0_1_19_0_1 = det*c0_0_0_25*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_25_0_1_19_1_1 = det*c0_0_0_25*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_25_0_1_20_1_1 = det*c0_0_0_25*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_25_0_1_21_0_1 = det*c0_0_0_25*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_25_0_1_21_1_1 = det*c0_0_0_25*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_25_0_1_22_0_1 = det*c0_0_0_25*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_25_0_1_22_1_1 = det*c0_0_0_25*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_25_0_1_23_1_1 = det*c0_0_0_25*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_25_0_1_24_0_1 = det*c0_0_0_25*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_25_0_1_24_1_1 = det*c0_0_0_25*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_25_0_1_25_0_1 = det*c0_0_0_25*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_25_0_1_25_1_1 = det*c0_0_0_25*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_25_0_1_26_0_1 = det*c0_0_0_25*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_25_0_1_26_1_1 = det*c0_0_0_25*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_25_0_1_27_0_1 = det*c0_0_0_25*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_25_0_1_27_1_1 = det*c0_0_0_25*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_25_0_1_28_0_1 = det*c0_0_0_25*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_25_0_1_28_1_1 = det*c0_0_0_25*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_25_0_1_29_0_1 = det*c0_0_0_25*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_25_0_1_29_1_1 = det*c0_0_0_25*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_25_1_1_0_0_0 = det*c0_0_0_25*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_25_1_1_0_1_0 = det*c0_0_0_25*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_25_1_1_1_0_0 = det*c0_0_0_25*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_25_1_1_2_1_0 = det*c0_0_0_25*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_25_1_1_3_0_0 = det*c0_0_0_25*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_25_1_1_3_1_0 = det*c0_0_0_25*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_25_1_1_4_0_0 = det*c0_0_0_25*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_25_1_1_4_1_0 = det*c0_0_0_25*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_25_1_1_5_0_0 = det*c0_0_0_25*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_25_1_1_5_1_0 = det*c0_0_0_25*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_25_1_1_6_0_0 = det*c0_0_0_25*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_25_1_1_6_1_0 = det*c0_0_0_25*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_25_1_1_7_0_0 = det*c0_0_0_25*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_25_1_1_7_1_0 = det*c0_0_0_25*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_25_1_1_8_0_0 = det*c0_0_0_25*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_25_1_1_8_1_0 = det*c0_0_0_25*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_25_1_1_9_0_0 = det*c0_0_0_25*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_25_1_1_9_1_0 = det*c0_0_0_25*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_25_1_1_10_0_0 = det*c0_0_0_25*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_25_1_1_10_1_0 = det*c0_0_0_25*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_25_1_1_11_0_0 = det*c0_0_0_25*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_25_1_1_11_1_0 = det*c0_0_0_25*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_25_1_1_12_0_0 = det*c0_0_0_25*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_25_1_1_12_1_0 = det*c0_0_0_25*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_25_1_1_13_0_0 = det*c0_0_0_25*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_25_1_1_13_1_0 = det*c0_0_0_25*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_25_1_1_14_0_0 = det*c0_0_0_25*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_25_1_1_14_1_0 = det*c0_0_0_25*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_25_1_1_15_0_1 = det*c0_0_0_25*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_25_1_1_15_1_1 = det*c0_0_0_25*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_25_1_1_16_0_1 = det*c0_0_0_25*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_25_1_1_17_1_1 = det*c0_0_0_25*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_25_1_1_18_0_1 = det*c0_0_0_25*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_25_1_1_18_1_1 = det*c0_0_0_25*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_25_1_1_19_0_1 = det*c0_0_0_25*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_25_1_1_19_1_1 = det*c0_0_0_25*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_25_1_1_20_0_1 = det*c0_0_0_25*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_25_1_1_20_1_1 = det*c0_0_0_25*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_25_1_1_21_0_1 = det*c0_0_0_25*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_25_1_1_21_1_1 = det*c0_0_0_25*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_25_1_1_22_0_1 = det*c0_0_0_25*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_25_1_1_22_1_1 = det*c0_0_0_25*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_25_1_1_23_0_1 = det*c0_0_0_25*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_25_1_1_23_1_1 = det*c0_0_0_25*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_25_1_1_24_0_1 = det*c0_0_0_25*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_25_1_1_24_1_1 = det*c0_0_0_25*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_25_1_1_25_0_1 = det*c0_0_0_25*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_25_1_1_25_1_1 = det*c0_0_0_25*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_25_1_1_26_0_1 = det*c0_0_0_25*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_25_1_1_26_1_1 = det*c0_0_0_25*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_25_1_1_27_0_1 = det*c0_0_0_25*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_25_1_1_27_1_1 = det*c0_0_0_25*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_25_1_1_28_0_1 = det*c0_0_0_25*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_25_1_1_28_1_1 = det*c0_0_0_25*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_25_1_1_29_0_1 = det*c0_0_0_25*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_25_1_1_29_1_1 = det*c0_0_0_25*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_26_0_1_0_0_0 = det*c0_0_0_26*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_26_0_1_0_1_0 = det*c0_0_0_26*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_26_0_1_1_0_0 = det*c0_0_0_26*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_26_0_1_3_0_0 = det*c0_0_0_26*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_26_0_1_3_1_0 = det*c0_0_0_26*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_26_0_1_4_0_0 = det*c0_0_0_26*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_26_0_1_4_1_0 = det*c0_0_0_26*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_26_0_1_5_1_0 = det*c0_0_0_26*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_26_0_1_6_0_0 = det*c0_0_0_26*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_26_0_1_6_1_0 = det*c0_0_0_26*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_26_0_1_7_0_0 = det*c0_0_0_26*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_26_0_1_7_1_0 = det*c0_0_0_26*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_26_0_1_8_1_0 = det*c0_0_0_26*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_26_0_1_9_0_0 = det*c0_0_0_26*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_26_0_1_9_1_0 = det*c0_0_0_26*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_26_0_1_10_0_0 = det*c0_0_0_26*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_26_0_1_10_1_0 = det*c0_0_0_26*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_26_0_1_11_0_0 = det*c0_0_0_26*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_26_0_1_11_1_0 = det*c0_0_0_26*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_26_0_1_12_0_0 = det*c0_0_0_26*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_26_0_1_12_1_0 = det*c0_0_0_26*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_26_0_1_13_0_0 = det*c0_0_0_26*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_26_0_1_13_1_0 = det*c0_0_0_26*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_26_0_1_14_0_0 = det*c0_0_0_26*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_26_0_1_14_1_0 = det*c0_0_0_26*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_26_0_1_15_0_1 = det*c0_0_0_26*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_26_0_1_15_1_1 = det*c0_0_0_26*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_26_0_1_16_0_1 = det*c0_0_0_26*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_26_0_1_18_0_1 = det*c0_0_0_26*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_26_0_1_18_1_1 = det*c0_0_0_26*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_26_0_1_19_0_1 = det*c0_0_0_26*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_26_0_1_19_1_1 = det*c0_0_0_26*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_26_0_1_20_1_1 = det*c0_0_0_26*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_26_0_1_21_0_1 = det*c0_0_0_26*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_26_0_1_21_1_1 = det*c0_0_0_26*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_26_0_1_22_0_1 = det*c0_0_0_26*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_26_0_1_22_1_1 = det*c0_0_0_26*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_26_0_1_23_1_1 = det*c0_0_0_26*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_26_0_1_24_0_1 = det*c0_0_0_26*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_26_0_1_24_1_1 = det*c0_0_0_26*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_26_0_1_25_0_1 = det*c0_0_0_26*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_26_0_1_25_1_1 = det*c0_0_0_26*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_26_0_1_26_0_1 = det*c0_0_0_26*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_26_0_1_26_1_1 = det*c0_0_0_26*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_26_0_1_27_0_1 = det*c0_0_0_26*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_26_0_1_27_1_1 = det*c0_0_0_26*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_26_0_1_28_0_1 = det*c0_0_0_26*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_26_0_1_28_1_1 = det*c0_0_0_26*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_26_0_1_29_0_1 = det*c0_0_0_26*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_26_0_1_29_1_1 = det*c0_0_0_26*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_26_1_1_0_0_0 = det*c0_0_0_26*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_26_1_1_0_1_0 = det*c0_0_0_26*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_26_1_1_1_0_0 = det*c0_0_0_26*c0_0_1_1*Jinv_11*Jinv_00;
    const double G0_26_1_1_2_1_0 = det*c0_0_0_26*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_26_1_1_3_0_0 = det*c0_0_0_26*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_26_1_1_3_1_0 = det*c0_0_0_26*c0_0_1_3*Jinv_11*Jinv_10;
    const double G0_26_1_1_4_0_0 = det*c0_0_0_26*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_26_1_1_4_1_0 = det*c0_0_0_26*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_26_1_1_5_0_0 = det*c0_0_0_26*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_26_1_1_5_1_0 = det*c0_0_0_26*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_26_1_1_6_0_0 = det*c0_0_0_26*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_26_1_1_7_0_0 = det*c0_0_0_26*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_26_1_1_8_0_0 = det*c0_0_0_26*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_26_1_1_9_0_0 = det*c0_0_0_26*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_26_1_1_9_1_0 = det*c0_0_0_26*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_26_1_1_10_0_0 = det*c0_0_0_26*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_26_1_1_10_1_0 = det*c0_0_0_26*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_26_1_1_11_0_0 = det*c0_0_0_26*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_26_1_1_11_1_0 = det*c0_0_0_26*c0_0_1_11*Jinv_11*Jinv_10;
    const double G0_26_1_1_12_0_0 = det*c0_0_0_26*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_26_1_1_13_0_0 = det*c0_0_0_26*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_26_1_1_14_0_0 = det*c0_0_0_26*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_26_1_1_15_0_1 = det*c0_0_0_26*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_26_1_1_15_1_1 = det*c0_0_0_26*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_26_1_1_16_0_1 = det*c0_0_0_26*c0_0_1_16*Jinv_11*Jinv_01;
    const double G0_26_1_1_17_1_1 = det*c0_0_0_26*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_26_1_1_18_0_1 = det*c0_0_0_26*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_26_1_1_18_1_1 = det*c0_0_0_26*c0_0_1_18*Jinv_11*Jinv_11;
    const double G0_26_1_1_19_0_1 = det*c0_0_0_26*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_26_1_1_19_1_1 = det*c0_0_0_26*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_26_1_1_20_0_1 = det*c0_0_0_26*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_26_1_1_20_1_1 = det*c0_0_0_26*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_26_1_1_21_0_1 = det*c0_0_0_26*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_26_1_1_22_0_1 = det*c0_0_0_26*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_26_1_1_23_0_1 = det*c0_0_0_26*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_26_1_1_24_0_1 = det*c0_0_0_26*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_26_1_1_24_1_1 = det*c0_0_0_26*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_26_1_1_25_0_1 = det*c0_0_0_26*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_26_1_1_25_1_1 = det*c0_0_0_26*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_26_1_1_26_0_1 = det*c0_0_0_26*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_26_1_1_26_1_1 = det*c0_0_0_26*c0_0_1_26*Jinv_11*Jinv_11;
    const double G0_26_1_1_27_0_1 = det*c0_0_0_26*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_26_1_1_28_0_1 = det*c0_0_0_26*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_26_1_1_29_0_1 = det*c0_0_0_26*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_27_0_1_0_0_0 = det*c0_0_0_27*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_27_0_1_0_1_0 = det*c0_0_0_27*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_27_0_1_1_0_0 = det*c0_0_0_27*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_27_0_1_3_0_0 = det*c0_0_0_27*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_27_0_1_3_1_0 = det*c0_0_0_27*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_27_0_1_4_0_0 = det*c0_0_0_27*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_27_0_1_4_1_0 = det*c0_0_0_27*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_27_0_1_5_1_0 = det*c0_0_0_27*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_27_0_1_6_0_0 = det*c0_0_0_27*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_27_0_1_6_1_0 = det*c0_0_0_27*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_27_0_1_7_0_0 = det*c0_0_0_27*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_27_0_1_7_1_0 = det*c0_0_0_27*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_27_0_1_8_1_0 = det*c0_0_0_27*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_27_0_1_9_0_0 = det*c0_0_0_27*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_27_0_1_9_1_0 = det*c0_0_0_27*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_27_0_1_10_0_0 = det*c0_0_0_27*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_27_0_1_10_1_0 = det*c0_0_0_27*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_27_0_1_11_0_0 = det*c0_0_0_27*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_27_0_1_11_1_0 = det*c0_0_0_27*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_27_0_1_12_0_0 = det*c0_0_0_27*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_27_0_1_12_1_0 = det*c0_0_0_27*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_27_0_1_13_0_0 = det*c0_0_0_27*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_27_0_1_13_1_0 = det*c0_0_0_27*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_27_0_1_14_0_0 = det*c0_0_0_27*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_27_0_1_14_1_0 = det*c0_0_0_27*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_27_0_1_15_0_1 = det*c0_0_0_27*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_27_0_1_15_1_1 = det*c0_0_0_27*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_27_0_1_16_0_1 = det*c0_0_0_27*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_27_0_1_18_0_1 = det*c0_0_0_27*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_27_0_1_18_1_1 = det*c0_0_0_27*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_27_0_1_19_0_1 = det*c0_0_0_27*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_27_0_1_19_1_1 = det*c0_0_0_27*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_27_0_1_20_1_1 = det*c0_0_0_27*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_27_0_1_21_0_1 = det*c0_0_0_27*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_27_0_1_21_1_1 = det*c0_0_0_27*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_27_0_1_22_0_1 = det*c0_0_0_27*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_27_0_1_22_1_1 = det*c0_0_0_27*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_27_0_1_23_1_1 = det*c0_0_0_27*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_27_0_1_24_0_1 = det*c0_0_0_27*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_27_0_1_24_1_1 = det*c0_0_0_27*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_27_0_1_25_0_1 = det*c0_0_0_27*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_27_0_1_25_1_1 = det*c0_0_0_27*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_27_0_1_26_0_1 = det*c0_0_0_27*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_27_0_1_26_1_1 = det*c0_0_0_27*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_27_0_1_27_0_1 = det*c0_0_0_27*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_27_0_1_27_1_1 = det*c0_0_0_27*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_27_0_1_28_0_1 = det*c0_0_0_27*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_27_0_1_28_1_1 = det*c0_0_0_27*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_27_0_1_29_0_1 = det*c0_0_0_27*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_27_0_1_29_1_1 = det*c0_0_0_27*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_27_1_1_0_0_0 = det*c0_0_0_27*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_27_1_1_0_1_0 = det*c0_0_0_27*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_27_1_1_2_1_0 = det*c0_0_0_27*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_27_1_1_3_0_0 = det*c0_0_0_27*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_27_1_1_4_0_0 = det*c0_0_0_27*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_27_1_1_4_1_0 = det*c0_0_0_27*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_27_1_1_5_0_0 = det*c0_0_0_27*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_27_1_1_5_1_0 = det*c0_0_0_27*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_27_1_1_6_0_0 = det*c0_0_0_27*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_27_1_1_6_1_0 = det*c0_0_0_27*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_27_1_1_7_0_0 = det*c0_0_0_27*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_27_1_1_7_1_0 = det*c0_0_0_27*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_27_1_1_8_0_0 = det*c0_0_0_27*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_27_1_1_8_1_0 = det*c0_0_0_27*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_27_1_1_9_0_0 = det*c0_0_0_27*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_27_1_1_9_1_0 = det*c0_0_0_27*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_27_1_1_10_0_0 = det*c0_0_0_27*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_27_1_1_10_1_0 = det*c0_0_0_27*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_27_1_1_11_0_0 = det*c0_0_0_27*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_27_1_1_12_0_0 = det*c0_0_0_27*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_27_1_1_12_1_0 = det*c0_0_0_27*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_27_1_1_13_0_0 = det*c0_0_0_27*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_27_1_1_13_1_0 = det*c0_0_0_27*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_27_1_1_14_0_0 = det*c0_0_0_27*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_27_1_1_14_1_0 = det*c0_0_0_27*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_27_1_1_15_0_1 = det*c0_0_0_27*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_27_1_1_15_1_1 = det*c0_0_0_27*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_27_1_1_17_1_1 = det*c0_0_0_27*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_27_1_1_18_0_1 = det*c0_0_0_27*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_27_1_1_19_0_1 = det*c0_0_0_27*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_27_1_1_19_1_1 = det*c0_0_0_27*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_27_1_1_20_0_1 = det*c0_0_0_27*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_27_1_1_20_1_1 = det*c0_0_0_27*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_27_1_1_21_0_1 = det*c0_0_0_27*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_27_1_1_21_1_1 = det*c0_0_0_27*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_27_1_1_22_0_1 = det*c0_0_0_27*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_27_1_1_22_1_1 = det*c0_0_0_27*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_27_1_1_23_0_1 = det*c0_0_0_27*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_27_1_1_23_1_1 = det*c0_0_0_27*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_27_1_1_24_0_1 = det*c0_0_0_27*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_27_1_1_24_1_1 = det*c0_0_0_27*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_27_1_1_25_0_1 = det*c0_0_0_27*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_27_1_1_25_1_1 = det*c0_0_0_27*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_27_1_1_26_0_1 = det*c0_0_0_27*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_27_1_1_27_0_1 = det*c0_0_0_27*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_27_1_1_27_1_1 = det*c0_0_0_27*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_27_1_1_28_0_1 = det*c0_0_0_27*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_27_1_1_28_1_1 = det*c0_0_0_27*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_27_1_1_29_0_1 = det*c0_0_0_27*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_27_1_1_29_1_1 = det*c0_0_0_27*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_28_0_1_0_0_0 = det*c0_0_0_28*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_28_0_1_0_1_0 = det*c0_0_0_28*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_28_0_1_1_0_0 = det*c0_0_0_28*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_28_0_1_3_0_0 = det*c0_0_0_28*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_28_0_1_3_1_0 = det*c0_0_0_28*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_28_0_1_4_0_0 = det*c0_0_0_28*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_28_0_1_4_1_0 = det*c0_0_0_28*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_28_0_1_5_1_0 = det*c0_0_0_28*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_28_0_1_6_0_0 = det*c0_0_0_28*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_28_0_1_6_1_0 = det*c0_0_0_28*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_28_0_1_7_0_0 = det*c0_0_0_28*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_28_0_1_7_1_0 = det*c0_0_0_28*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_28_0_1_8_1_0 = det*c0_0_0_28*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_28_0_1_9_0_0 = det*c0_0_0_28*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_28_0_1_9_1_0 = det*c0_0_0_28*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_28_0_1_10_0_0 = det*c0_0_0_28*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_28_0_1_10_1_0 = det*c0_0_0_28*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_28_0_1_11_0_0 = det*c0_0_0_28*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_28_0_1_11_1_0 = det*c0_0_0_28*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_28_0_1_12_0_0 = det*c0_0_0_28*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_28_0_1_12_1_0 = det*c0_0_0_28*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_28_0_1_13_0_0 = det*c0_0_0_28*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_28_0_1_13_1_0 = det*c0_0_0_28*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_28_0_1_14_0_0 = det*c0_0_0_28*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_28_0_1_14_1_0 = det*c0_0_0_28*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_28_0_1_15_0_1 = det*c0_0_0_28*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_28_0_1_15_1_1 = det*c0_0_0_28*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_28_0_1_16_0_1 = det*c0_0_0_28*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_28_0_1_18_0_1 = det*c0_0_0_28*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_28_0_1_18_1_1 = det*c0_0_0_28*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_28_0_1_19_0_1 = det*c0_0_0_28*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_28_0_1_19_1_1 = det*c0_0_0_28*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_28_0_1_20_1_1 = det*c0_0_0_28*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_28_0_1_21_0_1 = det*c0_0_0_28*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_28_0_1_21_1_1 = det*c0_0_0_28*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_28_0_1_22_0_1 = det*c0_0_0_28*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_28_0_1_22_1_1 = det*c0_0_0_28*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_28_0_1_23_1_1 = det*c0_0_0_28*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_28_0_1_24_0_1 = det*c0_0_0_28*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_28_0_1_24_1_1 = det*c0_0_0_28*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_28_0_1_25_0_1 = det*c0_0_0_28*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_28_0_1_25_1_1 = det*c0_0_0_28*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_28_0_1_26_0_1 = det*c0_0_0_28*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_28_0_1_26_1_1 = det*c0_0_0_28*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_28_0_1_27_0_1 = det*c0_0_0_28*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_28_0_1_27_1_1 = det*c0_0_0_28*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_28_0_1_28_0_1 = det*c0_0_0_28*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_28_0_1_28_1_1 = det*c0_0_0_28*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_28_0_1_29_0_1 = det*c0_0_0_28*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_28_0_1_29_1_1 = det*c0_0_0_28*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_28_1_1_0_0_0 = det*c0_0_0_28*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_28_1_1_0_1_0 = det*c0_0_0_28*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_28_1_1_2_1_0 = det*c0_0_0_28*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_28_1_1_3_0_0 = det*c0_0_0_28*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_28_1_1_4_0_0 = det*c0_0_0_28*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_28_1_1_4_1_0 = det*c0_0_0_28*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_28_1_1_5_0_0 = det*c0_0_0_28*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_28_1_1_5_1_0 = det*c0_0_0_28*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_28_1_1_6_0_0 = det*c0_0_0_28*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_28_1_1_6_1_0 = det*c0_0_0_28*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_28_1_1_7_0_0 = det*c0_0_0_28*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_28_1_1_7_1_0 = det*c0_0_0_28*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_28_1_1_8_0_0 = det*c0_0_0_28*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_28_1_1_8_1_0 = det*c0_0_0_28*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_28_1_1_9_0_0 = det*c0_0_0_28*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_28_1_1_9_1_0 = det*c0_0_0_28*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_28_1_1_10_0_0 = det*c0_0_0_28*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_28_1_1_10_1_0 = det*c0_0_0_28*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_28_1_1_11_0_0 = det*c0_0_0_28*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_28_1_1_12_0_0 = det*c0_0_0_28*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_28_1_1_12_1_0 = det*c0_0_0_28*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_28_1_1_13_0_0 = det*c0_0_0_28*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_28_1_1_13_1_0 = det*c0_0_0_28*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_28_1_1_14_0_0 = det*c0_0_0_28*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_28_1_1_14_1_0 = det*c0_0_0_28*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_28_1_1_15_0_1 = det*c0_0_0_28*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_28_1_1_15_1_1 = det*c0_0_0_28*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_28_1_1_17_1_1 = det*c0_0_0_28*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_28_1_1_18_0_1 = det*c0_0_0_28*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_28_1_1_19_0_1 = det*c0_0_0_28*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_28_1_1_19_1_1 = det*c0_0_0_28*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_28_1_1_20_0_1 = det*c0_0_0_28*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_28_1_1_20_1_1 = det*c0_0_0_28*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_28_1_1_21_0_1 = det*c0_0_0_28*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_28_1_1_21_1_1 = det*c0_0_0_28*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_28_1_1_22_0_1 = det*c0_0_0_28*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_28_1_1_22_1_1 = det*c0_0_0_28*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_28_1_1_23_0_1 = det*c0_0_0_28*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_28_1_1_23_1_1 = det*c0_0_0_28*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_28_1_1_24_0_1 = det*c0_0_0_28*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_28_1_1_24_1_1 = det*c0_0_0_28*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_28_1_1_25_0_1 = det*c0_0_0_28*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_28_1_1_25_1_1 = det*c0_0_0_28*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_28_1_1_26_0_1 = det*c0_0_0_28*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_28_1_1_27_0_1 = det*c0_0_0_28*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_28_1_1_27_1_1 = det*c0_0_0_28*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_28_1_1_28_0_1 = det*c0_0_0_28*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_28_1_1_28_1_1 = det*c0_0_0_28*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_28_1_1_29_0_1 = det*c0_0_0_28*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_28_1_1_29_1_1 = det*c0_0_0_28*c0_0_1_29*Jinv_11*Jinv_11;
    const double G0_29_0_1_0_0_0 = det*c0_0_0_29*c0_0_1_0*Jinv_01*Jinv_00;
    const double G0_29_0_1_0_1_0 = det*c0_0_0_29*c0_0_1_0*Jinv_01*Jinv_10;
    const double G0_29_0_1_1_0_0 = det*c0_0_0_29*c0_0_1_1*Jinv_01*Jinv_00;
    const double G0_29_0_1_3_0_0 = det*c0_0_0_29*c0_0_1_3*Jinv_01*Jinv_00;
    const double G0_29_0_1_3_1_0 = det*c0_0_0_29*c0_0_1_3*Jinv_01*Jinv_10;
    const double G0_29_0_1_4_0_0 = det*c0_0_0_29*c0_0_1_4*Jinv_01*Jinv_00;
    const double G0_29_0_1_4_1_0 = det*c0_0_0_29*c0_0_1_4*Jinv_01*Jinv_10;
    const double G0_29_0_1_5_1_0 = det*c0_0_0_29*c0_0_1_5*Jinv_01*Jinv_10;
    const double G0_29_0_1_6_0_0 = det*c0_0_0_29*c0_0_1_6*Jinv_01*Jinv_00;
    const double G0_29_0_1_6_1_0 = det*c0_0_0_29*c0_0_1_6*Jinv_01*Jinv_10;
    const double G0_29_0_1_7_0_0 = det*c0_0_0_29*c0_0_1_7*Jinv_01*Jinv_00;
    const double G0_29_0_1_7_1_0 = det*c0_0_0_29*c0_0_1_7*Jinv_01*Jinv_10;
    const double G0_29_0_1_8_1_0 = det*c0_0_0_29*c0_0_1_8*Jinv_01*Jinv_10;
    const double G0_29_0_1_9_0_0 = det*c0_0_0_29*c0_0_1_9*Jinv_01*Jinv_00;
    const double G0_29_0_1_9_1_0 = det*c0_0_0_29*c0_0_1_9*Jinv_01*Jinv_10;
    const double G0_29_0_1_10_0_0 = det*c0_0_0_29*c0_0_1_10*Jinv_01*Jinv_00;
    const double G0_29_0_1_10_1_0 = det*c0_0_0_29*c0_0_1_10*Jinv_01*Jinv_10;
    const double G0_29_0_1_11_0_0 = det*c0_0_0_29*c0_0_1_11*Jinv_01*Jinv_00;
    const double G0_29_0_1_11_1_0 = det*c0_0_0_29*c0_0_1_11*Jinv_01*Jinv_10;
    const double G0_29_0_1_12_0_0 = det*c0_0_0_29*c0_0_1_12*Jinv_01*Jinv_00;
    const double G0_29_0_1_12_1_0 = det*c0_0_0_29*c0_0_1_12*Jinv_01*Jinv_10;
    const double G0_29_0_1_13_0_0 = det*c0_0_0_29*c0_0_1_13*Jinv_01*Jinv_00;
    const double G0_29_0_1_13_1_0 = det*c0_0_0_29*c0_0_1_13*Jinv_01*Jinv_10;
    const double G0_29_0_1_14_0_0 = det*c0_0_0_29*c0_0_1_14*Jinv_01*Jinv_00;
    const double G0_29_0_1_14_1_0 = det*c0_0_0_29*c0_0_1_14*Jinv_01*Jinv_10;
    const double G0_29_0_1_15_0_1 = det*c0_0_0_29*c0_0_1_15*Jinv_01*Jinv_01;
    const double G0_29_0_1_15_1_1 = det*c0_0_0_29*c0_0_1_15*Jinv_01*Jinv_11;
    const double G0_29_0_1_16_0_1 = det*c0_0_0_29*c0_0_1_16*Jinv_01*Jinv_01;
    const double G0_29_0_1_18_0_1 = det*c0_0_0_29*c0_0_1_18*Jinv_01*Jinv_01;
    const double G0_29_0_1_18_1_1 = det*c0_0_0_29*c0_0_1_18*Jinv_01*Jinv_11;
    const double G0_29_0_1_19_0_1 = det*c0_0_0_29*c0_0_1_19*Jinv_01*Jinv_01;
    const double G0_29_0_1_19_1_1 = det*c0_0_0_29*c0_0_1_19*Jinv_01*Jinv_11;
    const double G0_29_0_1_20_1_1 = det*c0_0_0_29*c0_0_1_20*Jinv_01*Jinv_11;
    const double G0_29_0_1_21_0_1 = det*c0_0_0_29*c0_0_1_21*Jinv_01*Jinv_01;
    const double G0_29_0_1_21_1_1 = det*c0_0_0_29*c0_0_1_21*Jinv_01*Jinv_11;
    const double G0_29_0_1_22_0_1 = det*c0_0_0_29*c0_0_1_22*Jinv_01*Jinv_01;
    const double G0_29_0_1_22_1_1 = det*c0_0_0_29*c0_0_1_22*Jinv_01*Jinv_11;
    const double G0_29_0_1_23_1_1 = det*c0_0_0_29*c0_0_1_23*Jinv_01*Jinv_11;
    const double G0_29_0_1_24_0_1 = det*c0_0_0_29*c0_0_1_24*Jinv_01*Jinv_01;
    const double G0_29_0_1_24_1_1 = det*c0_0_0_29*c0_0_1_24*Jinv_01*Jinv_11;
    const double G0_29_0_1_25_0_1 = det*c0_0_0_29*c0_0_1_25*Jinv_01*Jinv_01;
    const double G0_29_0_1_25_1_1 = det*c0_0_0_29*c0_0_1_25*Jinv_01*Jinv_11;
    const double G0_29_0_1_26_0_1 = det*c0_0_0_29*c0_0_1_26*Jinv_01*Jinv_01;
    const double G0_29_0_1_26_1_1 = det*c0_0_0_29*c0_0_1_26*Jinv_01*Jinv_11;
    const double G0_29_0_1_27_0_1 = det*c0_0_0_29*c0_0_1_27*Jinv_01*Jinv_01;
    const double G0_29_0_1_27_1_1 = det*c0_0_0_29*c0_0_1_27*Jinv_01*Jinv_11;
    const double G0_29_0_1_28_0_1 = det*c0_0_0_29*c0_0_1_28*Jinv_01*Jinv_01;
    const double G0_29_0_1_28_1_1 = det*c0_0_0_29*c0_0_1_28*Jinv_01*Jinv_11;
    const double G0_29_0_1_29_0_1 = det*c0_0_0_29*c0_0_1_29*Jinv_01*Jinv_01;
    const double G0_29_0_1_29_1_1 = det*c0_0_0_29*c0_0_1_29*Jinv_01*Jinv_11;
    const double G0_29_1_1_0_0_0 = det*c0_0_0_29*c0_0_1_0*Jinv_11*Jinv_00;
    const double G0_29_1_1_0_1_0 = det*c0_0_0_29*c0_0_1_0*Jinv_11*Jinv_10;
    const double G0_29_1_1_2_1_0 = det*c0_0_0_29*c0_0_1_2*Jinv_11*Jinv_10;
    const double G0_29_1_1_3_0_0 = det*c0_0_0_29*c0_0_1_3*Jinv_11*Jinv_00;
    const double G0_29_1_1_4_0_0 = det*c0_0_0_29*c0_0_1_4*Jinv_11*Jinv_00;
    const double G0_29_1_1_4_1_0 = det*c0_0_0_29*c0_0_1_4*Jinv_11*Jinv_10;
    const double G0_29_1_1_5_0_0 = det*c0_0_0_29*c0_0_1_5*Jinv_11*Jinv_00;
    const double G0_29_1_1_5_1_0 = det*c0_0_0_29*c0_0_1_5*Jinv_11*Jinv_10;
    const double G0_29_1_1_6_0_0 = det*c0_0_0_29*c0_0_1_6*Jinv_11*Jinv_00;
    const double G0_29_1_1_6_1_0 = det*c0_0_0_29*c0_0_1_6*Jinv_11*Jinv_10;
    const double G0_29_1_1_7_0_0 = det*c0_0_0_29*c0_0_1_7*Jinv_11*Jinv_00;
    const double G0_29_1_1_7_1_0 = det*c0_0_0_29*c0_0_1_7*Jinv_11*Jinv_10;
    const double G0_29_1_1_8_0_0 = det*c0_0_0_29*c0_0_1_8*Jinv_11*Jinv_00;
    const double G0_29_1_1_8_1_0 = det*c0_0_0_29*c0_0_1_8*Jinv_11*Jinv_10;
    const double G0_29_1_1_9_0_0 = det*c0_0_0_29*c0_0_1_9*Jinv_11*Jinv_00;
    const double G0_29_1_1_9_1_0 = det*c0_0_0_29*c0_0_1_9*Jinv_11*Jinv_10;
    const double G0_29_1_1_10_0_0 = det*c0_0_0_29*c0_0_1_10*Jinv_11*Jinv_00;
    const double G0_29_1_1_10_1_0 = det*c0_0_0_29*c0_0_1_10*Jinv_11*Jinv_10;
    const double G0_29_1_1_11_0_0 = det*c0_0_0_29*c0_0_1_11*Jinv_11*Jinv_00;
    const double G0_29_1_1_12_0_0 = det*c0_0_0_29*c0_0_1_12*Jinv_11*Jinv_00;
    const double G0_29_1_1_12_1_0 = det*c0_0_0_29*c0_0_1_12*Jinv_11*Jinv_10;
    const double G0_29_1_1_13_0_0 = det*c0_0_0_29*c0_0_1_13*Jinv_11*Jinv_00;
    const double G0_29_1_1_13_1_0 = det*c0_0_0_29*c0_0_1_13*Jinv_11*Jinv_10;
    const double G0_29_1_1_14_0_0 = det*c0_0_0_29*c0_0_1_14*Jinv_11*Jinv_00;
    const double G0_29_1_1_14_1_0 = det*c0_0_0_29*c0_0_1_14*Jinv_11*Jinv_10;
    const double G0_29_1_1_15_0_1 = det*c0_0_0_29*c0_0_1_15*Jinv_11*Jinv_01;
    const double G0_29_1_1_15_1_1 = det*c0_0_0_29*c0_0_1_15*Jinv_11*Jinv_11;
    const double G0_29_1_1_17_1_1 = det*c0_0_0_29*c0_0_1_17*Jinv_11*Jinv_11;
    const double G0_29_1_1_18_0_1 = det*c0_0_0_29*c0_0_1_18*Jinv_11*Jinv_01;
    const double G0_29_1_1_19_0_1 = det*c0_0_0_29*c0_0_1_19*Jinv_11*Jinv_01;
    const double G0_29_1_1_19_1_1 = det*c0_0_0_29*c0_0_1_19*Jinv_11*Jinv_11;
    const double G0_29_1_1_20_0_1 = det*c0_0_0_29*c0_0_1_20*Jinv_11*Jinv_01;
    const double G0_29_1_1_20_1_1 = det*c0_0_0_29*c0_0_1_20*Jinv_11*Jinv_11;
    const double G0_29_1_1_21_0_1 = det*c0_0_0_29*c0_0_1_21*Jinv_11*Jinv_01;
    const double G0_29_1_1_21_1_1 = det*c0_0_0_29*c0_0_1_21*Jinv_11*Jinv_11;
    const double G0_29_1_1_22_0_1 = det*c0_0_0_29*c0_0_1_22*Jinv_11*Jinv_01;
    const double G0_29_1_1_22_1_1 = det*c0_0_0_29*c0_0_1_22*Jinv_11*Jinv_11;
    const double G0_29_1_1_23_0_1 = det*c0_0_0_29*c0_0_1_23*Jinv_11*Jinv_01;
    const double G0_29_1_1_23_1_1 = det*c0_0_0_29*c0_0_1_23*Jinv_11*Jinv_11;
    const double G0_29_1_1_24_0_1 = det*c0_0_0_29*c0_0_1_24*Jinv_11*Jinv_01;
    const double G0_29_1_1_24_1_1 = det*c0_0_0_29*c0_0_1_24*Jinv_11*Jinv_11;
    const double G0_29_1_1_25_0_1 = det*c0_0_0_29*c0_0_1_25*Jinv_11*Jinv_01;
    const double G0_29_1_1_25_1_1 = det*c0_0_0_29*c0_0_1_25*Jinv_11*Jinv_11;
    const double G0_29_1_1_26_0_1 = det*c0_0_0_29*c0_0_1_26*Jinv_11*Jinv_01;
    const double G0_29_1_1_27_0_1 = det*c0_0_0_29*c0_0_1_27*Jinv_11*Jinv_01;
    const double G0_29_1_1_27_1_1 = det*c0_0_0_29*c0_0_1_27*Jinv_11*Jinv_11;
    const double G0_29_1_1_28_0_1 = det*c0_0_0_29*c0_0_1_28*Jinv_11*Jinv_01;
    const double G0_29_1_1_28_1_1 = det*c0_0_0_29*c0_0_1_28*Jinv_11*Jinv_11;
    const double G0_29_1_1_29_0_1 = det*c0_0_0_29*c0_0_1_29*Jinv_11*Jinv_01;
    const double G0_29_1_1_29_1_1 = det*c0_0_0_29*c0_0_1_29*Jinv_11*Jinv_11;
    
    // Compute element tensor
    // Number of operations to compute tensor = 5695
    A[0] = 0.373015873015873*G0_0_0_0_0_0_0 + 0.373015873015873*G0_0_0_0_0_1_0 + 0.0566137566137565*G0_0_0_0_1_0_0 + 0.0566137566137566*G0_0_0_0_2_1_0 + 0.0423280423280418*G0_0_0_0_3_0_0 + 0.042328042328042*G0_0_0_0_3_1_0 + 0.0423280423280421*G0_0_0_0_4_0_0 + 0.0423280423280422*G0_0_0_0_4_1_0 + 0.0423280423280421*G0_0_0_0_5_0_0 + 0.042328042328042*G0_0_0_0_5_1_0 + 0.126984126984127*G0_0_0_0_6_0_0 - 0.651851851851852*G0_0_0_0_6_1_0 + 0.0423280423280421*G0_0_0_0_7_0_0 + 0.467724867724868*G0_0_0_0_7_1_0 - 0.042328042328042*G0_0_0_0_8_0_0 - 0.245502645502646*G0_0_0_0_8_1_0 - 0.651851851851852*G0_0_0_0_9_0_0 + 0.126984126984127*G0_0_0_0_9_1_0 + 0.467724867724868*G0_0_0_0_10_0_0 + 0.0423280423280423*G0_0_0_0_10_1_0 - 0.245502645502646*G0_0_0_0_11_0_0 - 0.0423280423280419*G0_0_0_0_11_1_0 - 0.0846560846560848*G0_0_0_0_12_0_0 - 0.084656084656085*G0_0_0_0_12_1_0 - 0.0846560846560835*G0_0_0_0_13_0_0 - 0.0846560846560848*G0_0_0_0_13_1_0 - 0.0846560846560841*G0_0_0_0_14_0_0 - 0.0846560846560835*G0_0_0_0_14_1_0 + 0.373015873015873*G0_0_0_0_15_0_1 + 0.373015873015873*G0_0_0_0_15_1_1 + 0.0566137566137565*G0_0_0_0_16_0_1 + 0.0566137566137566*G0_0_0_0_17_1_1 + 0.0423280423280418*G0_0_0_0_18_0_1 + 0.042328042328042*G0_0_0_0_18_1_1 + 0.0423280423280421*G0_0_0_0_19_0_1 + 0.0423280423280422*G0_0_0_0_19_1_1 + 0.0423280423280421*G0_0_0_0_20_0_1 + 0.042328042328042*G0_0_0_0_20_1_1 + 0.126984126984127*G0_0_0_0_21_0_1 - 0.651851851851852*G0_0_0_0_21_1_1 + 0.0423280423280421*G0_0_0_0_22_0_1 + 0.467724867724868*G0_0_0_0_22_1_1 - 0.042328042328042*G0_0_0_0_23_0_1 - 0.245502645502646*G0_0_0_0_23_1_1 - 0.651851851851852*G0_0_0_0_24_0_1 + 0.126984126984127*G0_0_0_0_24_1_1 + 0.467724867724868*G0_0_0_0_25_0_1 + 0.0423280423280423*G0_0_0_0_25_1_1 - 0.245502645502646*G0_0_0_0_26_0_1 - 0.0423280423280419*G0_0_0_0_26_1_1 - 0.0846560846560848*G0_0_0_0_27_0_1 - 0.084656084656085*G0_0_0_0_27_1_1 - 0.0846560846560835*G0_0_0_0_28_0_1 - 0.0846560846560848*G0_0_0_0_28_1_1 - 0.0846560846560841*G0_0_0_0_29_0_1 - 0.0846560846560835*G0_0_0_0_29_1_1 + 0.373015873015873*G0_0_1_0_0_0_0 + 0.373015873015873*G0_0_1_0_0_1_0 + 0.0566137566137567*G0_0_1_0_1_0_0 + 0.0566137566137567*G0_0_1_0_2_1_0 + 0.0423280423280421*G0_0_1_0_3_0_0 + 0.0423280423280425*G0_0_1_0_3_1_0 + 0.0423280423280417*G0_0_1_0_4_0_0 + 0.0423280423280419*G0_0_1_0_4_1_0 + 0.0423280423280423*G0_0_1_0_5_0_0 + 0.0423280423280419*G0_0_1_0_5_1_0 + 0.126984126984126*G0_0_1_0_6_0_0 - 0.651851851851852*G0_0_1_0_6_1_0 + 0.0423280423280421*G0_0_1_0_7_0_0 + 0.467724867724868*G0_0_1_0_7_1_0 - 0.0423280423280422*G0_0_1_0_8_0_0 - 0.245502645502646*G0_0_1_0_8_1_0 - 0.651851851851852*G0_0_1_0_9_0_0 + 0.126984126984127*G0_0_1_0_9_1_0 + 0.467724867724868*G0_0_1_0_10_0_0 + 0.0423280423280427*G0_0_1_0_10_1_0 - 0.245502645502646*G0_0_1_0_11_0_0 - 0.0423280423280424*G0_0_1_0_11_1_0 - 0.0846560846560841*G0_0_1_0_12_0_0 - 0.0846560846560852*G0_0_1_0_12_1_0 - 0.084656084656084*G0_0_1_0_13_0_0 - 0.0846560846560849*G0_0_1_0_13_1_0 - 0.0846560846560838*G0_0_1_0_14_0_0 - 0.0846560846560832*G0_0_1_0_14_1_0 + 0.373015873015873*G0_0_1_0_15_0_1 + 0.373015873015873*G0_0_1_0_15_1_1 + 0.0566137566137567*G0_0_1_0_16_0_1 + 0.0566137566137567*G0_0_1_0_17_1_1 + 0.0423280423280421*G0_0_1_0_18_0_1 + 0.0423280423280425*G0_0_1_0_18_1_1 + 0.0423280423280417*G0_0_1_0_19_0_1 + 0.0423280423280419*G0_0_1_0_19_1_1 + 0.0423280423280423*G0_0_1_0_20_0_1 + 0.0423280423280419*G0_0_1_0_20_1_1 + 0.126984126984126*G0_0_1_0_21_0_1 - 0.651851851851852*G0_0_1_0_21_1_1 + 0.0423280423280421*G0_0_1_0_22_0_1 + 0.467724867724868*G0_0_1_0_22_1_1 - 0.0423280423280422*G0_0_1_0_23_0_1 - 0.245502645502646*G0_0_1_0_23_1_1 - 0.651851851851852*G0_0_1_0_24_0_1 + 0.126984126984127*G0_0_1_0_24_1_1 + 0.467724867724868*G0_0_1_0_25_0_1 + 0.0423280423280427*G0_0_1_0_25_1_1 - 0.245502645502646*G0_0_1_0_26_0_1 - 0.0423280423280424*G0_0_1_0_26_1_1 - 0.0846560846560841*G0_0_1_0_27_0_1 - 0.0846560846560852*G0_0_1_0_27_1_1 - 0.084656084656084*G0_0_1_0_28_0_1 - 0.0846560846560849*G0_0_1_0_28_1_1 - 0.0846560846560838*G0_0_1_0_29_0_1 - 0.0846560846560832*G0_0_1_0_29_1_1 + 0.0566137566137565*G0_1_0_0_0_0_0 + 0.0566137566137567*G0_1_0_0_0_1_0 + 0.373015873015873*G0_1_0_0_1_0_0 - 0.0566137566137567*G0_1_0_0_2_1_0 + 0.126984126984127*G0_1_0_0_3_0_0 + 0.778835978835978*G0_1_0_0_3_1_0 + 0.0423280423280423*G0_1_0_0_4_0_0 - 0.425396825396825*G0_1_0_0_4_1_0 - 0.0423280423280424*G0_1_0_0_5_0_0 + 0.203174603174603*G0_1_0_0_5_1_0 + 0.0423280423280421*G0_1_0_0_6_0_0 + 0.0423280423280423*G0_1_0_0_7_0_0 + 0.0423280423280424*G0_1_0_0_8_0_0 - 0.245502645502645*G0_1_0_0_9_0_0 - 0.203174603174603*G0_1_0_0_9_1_0 + 0.467724867724867*G0_1_0_0_10_0_0 + 0.425396825396824*G0_1_0_0_10_1_0 - 0.651851851851851*G0_1_0_0_11_0_0 - 0.778835978835978*G0_1_0_0_11_1_0 - 0.084656084656084*G0_1_0_0_12_0_0 - 0.0846560846560853*G0_1_0_0_13_0_0 - 0.0846560846560846*G0_1_0_0_14_0_0 + 0.0566137566137565*G0_1_0_0_15_0_1 + 0.0566137566137567*G0_1_0_0_15_1_1 + 0.373015873015873*G0_1_0_0_16_0_1 - 0.0566137566137567*G0_1_0_0_17_1_1 + 0.126984126984127*G0_1_0_0_18_0_1 + 0.778835978835978*G0_1_0_0_18_1_1 + 0.0423280423280423*G0_1_0_0_19_0_1 - 0.425396825396825*G0_1_0_0_19_1_1 - 0.0423280423280424*G0_1_0_0_20_0_1 + 0.203174603174603*G0_1_0_0_20_1_1 + 0.0423280423280421*G0_1_0_0_21_0_1 + 0.0423280423280423*G0_1_0_0_22_0_1 + 0.0423280423280424*G0_1_0_0_23_0_1 - 0.245502645502645*G0_1_0_0_24_0_1 - 0.203174603174603*G0_1_0_0_24_1_1 + 0.467724867724867*G0_1_0_0_25_0_1 + 0.425396825396824*G0_1_0_0_25_1_1 - 0.651851851851851*G0_1_0_0_26_0_1 - 0.778835978835978*G0_1_0_0_26_1_1 - 0.084656084656084*G0_1_0_0_27_0_1 - 0.0846560846560853*G0_1_0_0_28_0_1 - 0.0846560846560846*G0_1_0_0_29_0_1 + 0.0566137566137566*G0_2_1_0_0_0_0 + 0.0566137566137567*G0_2_1_0_0_1_0 - 0.0566137566137566*G0_2_1_0_1_0_0 + 0.373015873015873*G0_2_1_0_2_1_0 + 0.203174603174602*G0_2_1_0_3_0_0 - 0.0423280423280427*G0_2_1_0_3_1_0 - 0.425396825396825*G0_2_1_0_4_0_0 + 0.0423280423280416*G0_2_1_0_4_1_0 + 0.77883597883598*G0_2_1_0_5_0_0 + 0.126984126984128*G0_2_1_0_5_1_0 - 0.203174603174603*G0_2_1_0_6_0_0 - 0.245502645502646*G0_2_1_0_6_1_0 + 0.425396825396825*G0_2_1_0_7_0_0 + 0.467724867724867*G0_2_1_0_7_1_0 - 0.77883597883598*G0_2_1_0_8_0_0 - 0.651851851851852*G0_2_1_0_8_1_0 + 0.0423280423280423*G0_2_1_0_9_1_0 + 0.042328042328043*G0_2_1_0_10_1_0 + 0.0423280423280428*G0_2_1_0_11_1_0 - 0.084656084656085*G0_2_1_0_12_1_0 - 0.0846560846560848*G0_2_1_0_13_1_0 - 0.0846560846560851*G0_2_1_0_14_1_0 + 0.0566137566137566*G0_2_1_0_15_0_1 + 0.0566137566137567*G0_2_1_0_15_1_1 - 0.0566137566137566*G0_2_1_0_16_0_1 + 0.373015873015873*G0_2_1_0_17_1_1 + 0.203174603174602*G0_2_1_0_18_0_1 - 0.0423280423280427*G0_2_1_0_18_1_1 - 0.425396825396825*G0_2_1_0_19_0_1 + 0.0423280423280416*G0_2_1_0_19_1_1 + 0.77883597883598*G0_2_1_0_20_0_1 + 0.126984126984128*G0_2_1_0_20_1_1 - 0.203174603174603*G0_2_1_0_21_0_1 - 0.245502645502646*G0_2_1_0_21_1_1 + 0.425396825396825*G0_2_1_0_22_0_1 + 0.467724867724867*G0_2_1_0_22_1_1 - 0.77883597883598*G0_2_1_0_23_0_1 - 0.651851851851852*G0_2_1_0_23_1_1 + 0.0423280423280423*G0_2_1_0_24_1_1 + 0.042328042328043*G0_2_1_0_25_1_1 + 0.0423280423280428*G0_2_1_0_26_1_1 - 0.084656084656085*G0_2_1_0_27_1_1 - 0.0846560846560848*G0_2_1_0_28_1_1 - 0.0846560846560851*G0_2_1_0_29_1_1 + 0.0423280423280418*G0_3_0_0_0_0_0 + 0.0423280423280421*G0_3_0_0_0_1_0 + 0.126984126984127*G0_3_0_0_1_0_0 + 0.203174603174602*G0_3_0_0_2_1_0 + 1.82857142857143*G0_3_0_0_3_0_0 + 1.04973544973545*G0_3_0_0_3_1_0 - 0.660317460317462*G0_3_0_0_4_0_0 + 0.821164021164017*G0_3_0_0_4_1_0 + 0.338624338624337*G0_3_0_0_5_0_0 - 0.440211640211641*G0_3_0_0_5_1_0 - 0.203174603174603*G0_3_0_0_6_0_0 - 0.101587301587302*G0_3_0_0_6_1_0 - 0.152380952380956*G0_3_0_0_7_0_0 - 0.0423280423280443*G0_3_0_0_7_1_0 - 0.338624338624337*G0_3_0_0_8_0_0 - 0.101587301587299*G0_3_0_0_8_1_0 - 0.135449735449733*G0_3_0_0_9_0_0 - 0.237037037037035*G0_3_0_0_9_1_0 + 0.1015873015873*G0_3_0_0_10_0_0 + 0.465608465608465*G0_3_0_0_10_1_0 - 0.135449735449735*G0_3_0_0_11_0_0 - 1.04973544973545*G0_3_0_0_11_1_0 + 0.812698412698412*G0_3_0_0_12_0_0 + 0.338624338624334*G0_3_0_0_12_1_0 - 2.43809523809524*G0_3_0_0_13_0_0 - 1.28677248677248*G0_3_0_0_13_1_0 + 0.812698412698417*G0_3_0_0_14_0_0 + 0.338624338624343*G0_3_0_0_14_1_0 + 0.0423280423280418*G0_3_0_0_15_0_1 + 0.0423280423280421*G0_3_0_0_15_1_1 + 0.126984126984127*G0_3_0_0_16_0_1 + 0.203174603174602*G0_3_0_0_17_1_1 + 1.82857142857143*G0_3_0_0_18_0_1 + 1.04973544973545*G0_3_0_0_18_1_1 - 0.660317460317462*G0_3_0_0_19_0_1 + 0.821164021164017*G0_3_0_0_19_1_1 + 0.338624338624337*G0_3_0_0_20_0_1 - 0.440211640211641*G0_3_0_0_20_1_1 - 0.203174603174603*G0_3_0_0_21_0_1 - 0.101587301587302*G0_3_0_0_21_1_1 - 0.152380952380956*G0_3_0_0_22_0_1 - 0.0423280423280443*G0_3_0_0_22_1_1 - 0.338624338624337*G0_3_0_0_23_0_1 - 0.101587301587299*G0_3_0_0_23_1_1 - 0.135449735449733*G0_3_0_0_24_0_1 - 0.237037037037035*G0_3_0_0_24_1_1 + 0.1015873015873*G0_3_0_0_25_0_1 + 0.465608465608465*G0_3_0_0_25_1_1 - 0.135449735449735*G0_3_0_0_26_0_1 - 1.04973544973545*G0_3_0_0_26_1_1 + 0.812698412698412*G0_3_0_0_27_0_1 + 0.338624338624334*G0_3_0_0_27_1_1 - 2.43809523809524*G0_3_0_0_28_0_1 - 1.28677248677248*G0_3_0_0_28_1_1 + 0.812698412698417*G0_3_0_0_29_0_1 + 0.338624338624343*G0_3_0_0_29_1_1 + 0.042328042328042*G0_3_1_0_0_0_0 + 0.0423280423280425*G0_3_1_0_0_1_0 + 0.778835978835978*G0_3_1_0_1_0_0 - 0.0423280423280427*G0_3_1_0_2_1_0 + 1.04973544973545*G0_3_1_0_3_0_0 + 2.0994708994709*G0_3_1_0_3_1_0 - 0.296296296296297*G0_3_1_0_4_0_0 - 0.761904761904764*G0_3_1_0_4_1_0 + 0.101587301587301*G0_3_1_0_5_0_0 + 0.338624338624338*G0_3_1_0_5_1_0 - 0.101587301587301*G0_3_1_0_6_0_0 - 0.042328042328044*G0_3_1_0_7_0_0 - 0.101587301587301*G0_3_1_0_8_0_0 - 0.237037037037036*G0_3_1_0_9_0_0 - 0.338624338624337*G0_3_1_0_9_1_0 + 0.465608465608464*G0_3_1_0_10_0_0 + 0.761904761904759*G0_3_1_0_10_1_0 - 1.04973544973545*G0_3_1_0_11_0_0 - 2.0994708994709*G0_3_1_0_11_1_0 + 0.338624338624338*G0_3_1_0_12_0_0 - 1.28677248677249*G0_3_1_0_13_0_0 + 0.338624338624341*G0_3_1_0_14_0_0 + 0.042328042328042*G0_3_1_0_15_0_1 + 0.0423280423280425*G0_3_1_0_15_1_1 + 0.778835978835978*G0_3_1_0_16_0_1 - 0.0423280423280427*G0_3_1_0_17_1_1 + 1.04973544973545*G0_3_1_0_18_0_1 + 2.0994708994709*G0_3_1_0_18_1_1 - 0.296296296296297*G0_3_1_0_19_0_1 - 0.761904761904764*G0_3_1_0_19_1_1 + 0.101587301587301*G0_3_1_0_20_0_1 + 0.338624338624338*G0_3_1_0_20_1_1 - 0.101587301587301*G0_3_1_0_21_0_1 - 0.042328042328044*G0_3_1_0_22_0_1 - 0.101587301587301*G0_3_1_0_23_0_1 - 0.237037037037036*G0_3_1_0_24_0_1 - 0.338624338624337*G0_3_1_0_24_1_1 + 0.465608465608464*G0_3_1_0_25_0_1 + 0.761904761904759*G0_3_1_0_25_1_1 - 1.04973544973545*G0_3_1_0_26_0_1 - 2.0994708994709*G0_3_1_0_26_1_1 + 0.338624338624338*G0_3_1_0_27_0_1 - 1.28677248677249*G0_3_1_0_28_0_1 + 0.338624338624341*G0_3_1_0_29_0_1 + 0.0423280423280421*G0_4_0_0_0_0_0 + 0.0423280423280417*G0_4_0_0_0_1_0 + 0.0423280423280423*G0_4_0_0_1_0_0 - 0.425396825396825*G0_4_0_0_2_1_0 - 0.660317460317462*G0_4_0_0_3_0_0 - 0.296296296296297*G0_4_0_0_3_1_0 + 2.45079365079365*G0_4_0_0_4_0_0 + 0.97142857142857*G0_4_0_0_4_1_0 - 0.761904761904761*G0_4_0_0_5_0_0 + 0.821164021164023*G0_4_0_0_5_1_0 - 0.152380952380952*G0_4_0_0_6_0_0 - 0.11005291005291*G0_4_0_0_6_1_0 + 0.393650793650795*G0_4_0_0_7_0_0 + 0.196825396825397*G0_4_0_0_7_1_0 + 0.761904761904761*G0_4_0_0_8_0_0 + 0.296296296296296*G0_4_0_0_8_1_0 - 0.0677248677248682*G0_4_0_0_9_0_0 - 0.110052910052911*G0_4_0_0_9_1_0 + 0.050793650793651*G0_4_0_0_10_0_0 + 0.196825396825396*G0_4_0_0_10_1_0 - 0.0677248677248674*G0_4_0_0_11_0_0 + 0.296296296296296*G0_4_0_0_11_1_0 + 0.406349206349208*G0_4_0_0_12_0_0 + 0.457142857142862*G0_4_0_0_12_1_0 + 0.406349206349207*G0_4_0_0_13_0_0 - 1.16825396825396*G0_4_0_0_13_1_0 - 2.84444444444444*G0_4_0_0_14_0_0 - 1.16825396825397*G0_4_0_0_14_1_0 + 0.0423280423280421*G0_4_0_0_15_0_1 + 0.0423280423280417*G0_4_0_0_15_1_1 + 0.0423280423280423*G0_4_0_0_16_0_1 - 0.425396825396825*G0_4_0_0_17_1_1 - 0.660317460317462*G0_4_0_0_18_0_1 - 0.296296296296297*G0_4_0_0_18_1_1 + 2.45079365079365*G0_4_0_0_19_0_1 + 0.97142857142857*G0_4_0_0_19_1_1 - 0.761904761904761*G0_4_0_0_20_0_1 + 0.821164021164023*G0_4_0_0_20_1_1 - 0.152380952380952*G0_4_0_0_21_0_1 - 0.11005291005291*G0_4_0_0_21_1_1 + 0.393650793650795*G0_4_0_0_22_0_1 + 0.196825396825397*G0_4_0_0_22_1_1 + 0.761904761904761*G0_4_0_0_23_0_1 + 0.296296296296296*G0_4_0_0_23_1_1 - 0.0677248677248682*G0_4_0_0_24_0_1 - 0.110052910052911*G0_4_0_0_24_1_1 + 0.050793650793651*G0_4_0_0_25_0_1 + 0.196825396825396*G0_4_0_0_25_1_1 - 0.0677248677248674*G0_4_0_0_26_0_1 + 0.296296296296296*G0_4_0_0_26_1_1 + 0.406349206349208*G0_4_0_0_27_0_1 + 0.457142857142862*G0_4_0_0_27_1_1 + 0.406349206349207*G0_4_0_0_28_0_1 - 1.16825396825396*G0_4_0_0_28_1_1 - 2.84444444444444*G0_4_0_0_29_0_1 - 1.16825396825397*G0_4_0_0_29_1_1 + 0.0423280423280422*G0_4_1_0_0_0_0 + 0.0423280423280419*G0_4_1_0_0_1_0 - 0.425396825396825*G0_4_1_0_1_0_0 + 0.0423280423280416*G0_4_1_0_2_1_0 + 0.821164021164017*G0_4_1_0_3_0_0 - 0.761904761904764*G0_4_1_0_3_1_0 + 0.97142857142857*G0_4_1_0_4_0_0 + 2.45079365079365*G0_4_1_0_4_1_0 - 0.296296296296298*G0_4_1_0_5_0_0 - 0.66031746031746*G0_4_1_0_5_1_0 - 0.11005291005291*G0_4_1_0_6_0_0 - 0.0677248677248674*G0_4_1_0_6_1_0 + 0.196825396825396*G0_4_1_0_7_0_0 + 0.0507936507936494*G0_4_1_0_7_1_0 + 0.296296296296298*G0_4_1_0_8_0_0 - 0.0677248677248661*G0_4_1_0_8_1_0 - 0.11005291005291*G0_4_1_0_9_0_0 - 0.152380952380951*G0_4_1_0_9_1_0 + 0.196825396825396*G0_4_1_0_10_0_0 + 0.393650793650794*G0_4_1_0_10_1_0 + 0.296296296296297*G0_4_1_0_11_0_0 + 0.761904761904763*G0_4_1_0_11_1_0 + 0.457142857142857*G0_4_1_0_12_0_0 + 0.406349206349205*G0_4_1_0_12_1_0 - 1.16825396825396*G0_4_1_0_13_0_0 - 2.84444444444444*G0_4_1_0_13_1_0 - 1.16825396825397*G0_4_1_0_14_0_0 + 0.406349206349206*G0_4_1_0_14_1_0 + 0.0423280423280422*G0_4_1_0_15_0_1 + 0.0423280423280419*G0_4_1_0_15_1_1 - 0.425396825396825*G0_4_1_0_16_0_1 + 0.0423280423280416*G0_4_1_0_17_1_1 + 0.821164021164017*G0_4_1_0_18_0_1 - 0.761904761904764*G0_4_1_0_18_1_1 + 0.97142857142857*G0_4_1_0_19_0_1 + 2.45079365079365*G0_4_1_0_19_1_1 - 0.296296296296298*G0_4_1_0_20_0_1 - 0.66031746031746*G0_4_1_0_20_1_1 - 0.11005291005291*G0_4_1_0_21_0_1 - 0.0677248677248674*G0_4_1_0_21_1_1 + 0.196825396825396*G0_4_1_0_22_0_1 + 0.0507936507936494*G0_4_1_0_22_1_1 + 0.296296296296298*G0_4_1_0_23_0_1 - 0.0677248677248661*G0_4_1_0_23_1_1 - 0.11005291005291*G0_4_1_0_24_0_1 - 0.152380952380951*G0_4_1_0_24_1_1 + 0.196825396825396*G0_4_1_0_25_0_1 + 0.393650793650794*G0_4_1_0_25_1_1 + 0.296296296296297*G0_4_1_0_26_0_1 + 0.761904761904763*G0_4_1_0_26_1_1 + 0.457142857142857*G0_4_1_0_27_0_1 + 0.406349206349205*G0_4_1_0_27_1_1 - 1.16825396825396*G0_4_1_0_28_0_1 - 2.84444444444444*G0_4_1_0_28_1_1 - 1.16825396825397*G0_4_1_0_29_0_1 + 0.406349206349206*G0_4_1_0_29_1_1 + 0.0423280423280421*G0_5_0_0_0_0_0 + 0.0423280423280423*G0_5_0_0_0_1_0 - 0.0423280423280424*G0_5_0_0_1_0_0 + 0.77883597883598*G0_5_0_0_2_1_0 + 0.338624338624337*G0_5_0_0_3_0_0 + 0.101587301587301*G0_5_0_0_3_1_0 - 0.761904761904761*G0_5_0_0_4_0_0 - 0.296296296296298*G0_5_0_0_4_1_0 + 2.0994708994709*G0_5_0_0_5_0_0 + 1.04973544973545*G0_5_0_0_5_1_0 - 0.338624338624338*G0_5_0_0_6_0_0 - 0.237037037037037*G0_5_0_0_6_1_0 + 0.761904761904763*G0_5_0_0_7_0_0 + 0.465608465608465*G0_5_0_0_7_1_0 - 2.0994708994709*G0_5_0_0_8_0_0 - 1.04973544973545*G0_5_0_0_8_1_0 - 0.101587301587302*G0_5_0_0_9_1_0 - 0.0423280423280415*G0_5_0_0_10_1_0 - 0.101587301587301*G0_5_0_0_11_1_0 + 0.338624338624339*G0_5_0_0_12_1_0 + 0.338624338624338*G0_5_0_0_13_1_0 - 1.28677248677249*G0_5_0_0_14_1_0 + 0.0423280423280421*G0_5_0_0_15_0_1 + 0.0423280423280423*G0_5_0_0_15_1_1 - 0.0423280423280424*G0_5_0_0_16_0_1 + 0.77883597883598*G0_5_0_0_17_1_1 + 0.338624338624337*G0_5_0_0_18_0_1 + 0.101587301587301*G0_5_0_0_18_1_1 - 0.761904761904761*G0_5_0_0_19_0_1 - 0.296296296296298*G0_5_0_0_19_1_1 + 2.0994708994709*G0_5_0_0_20_0_1 + 1.04973544973545*G0_5_0_0_20_1_1 - 0.338624338624338*G0_5_0_0_21_0_1 - 0.237037037037037*G0_5_0_0_21_1_1 + 0.761904761904763*G0_5_0_0_22_0_1 + 0.465608465608465*G0_5_0_0_22_1_1 - 2.0994708994709*G0_5_0_0_23_0_1 - 1.04973544973545*G0_5_0_0_23_1_1 - 0.101587301587302*G0_5_0_0_24_1_1 - 0.0423280423280415*G0_5_0_0_25_1_1 - 0.101587301587301*G0_5_0_0_26_1_1 + 0.338624338624339*G0_5_0_0_27_1_1 + 0.338624338624338*G0_5_0_0_28_1_1 - 1.28677248677249*G0_5_0_0_29_1_1 + 0.042328042328042*G0_5_1_0_0_0_0 + 0.0423280423280419*G0_5_1_0_0_1_0 + 0.203174603174603*G0_5_1_0_1_0_0 + 0.126984126984128*G0_5_1_0_2_1_0 - 0.440211640211642*G0_5_1_0_3_0_0 + 0.338624338624338*G0_5_1_0_3_1_0 + 0.821164021164022*G0_5_1_0_4_0_0 - 0.66031746031746*G0_5_1_0_4_1_0 + 1.04973544973545*G0_5_1_0_5_0_0 + 1.82857142857143*G0_5_1_0_5_1_0 - 0.237037037037037*G0_5_1_0_6_0_0 - 0.135449735449735*G0_5_1_0_6_1_0 + 0.465608465608468*G0_5_1_0_7_0_0 + 0.101587301587302*G0_5_1_0_7_1_0 - 1.04973544973545*G0_5_1_0_8_0_0 - 0.135449735449737*G0_5_1_0_8_1_0 - 0.101587301587301*G0_5_1_0_9_0_0 - 0.203174603174604*G0_5_1_0_9_1_0 - 0.0423280423280416*G0_5_1_0_10_0_0 - 0.152380952380953*G0_5_1_0_10_1_0 - 0.101587301587302*G0_5_1_0_11_0_0 - 0.338624338624338*G0_5_1_0_11_1_0 + 0.338624338624338*G0_5_1_0_12_0_0 + 0.812698412698416*G0_5_1_0_12_1_0 + 0.33862433862434*G0_5_1_0_13_0_0 + 0.812698412698412*G0_5_1_0_13_1_0 - 1.28677248677249*G0_5_1_0_14_0_0 - 2.43809523809524*G0_5_1_0_14_1_0 + 0.042328042328042*G0_5_1_0_15_0_1 + 0.0423280423280419*G0_5_1_0_15_1_1 + 0.203174603174603*G0_5_1_0_16_0_1 + 0.126984126984128*G0_5_1_0_17_1_1 - 0.440211640211642*G0_5_1_0_18_0_1 + 0.338624338624338*G0_5_1_0_18_1_1 + 0.821164021164022*G0_5_1_0_19_0_1 - 0.66031746031746*G0_5_1_0_19_1_1 + 1.04973544973545*G0_5_1_0_20_0_1 + 1.82857142857143*G0_5_1_0_20_1_1 - 0.237037037037037*G0_5_1_0_21_0_1 - 0.135449735449735*G0_5_1_0_21_1_1 + 0.465608465608468*G0_5_1_0_22_0_1 + 0.101587301587302*G0_5_1_0_22_1_1 - 1.04973544973545*G0_5_1_0_23_0_1 - 0.135449735449737*G0_5_1_0_23_1_1 - 0.101587301587301*G0_5_1_0_24_0_1 - 0.203174603174604*G0_5_1_0_24_1_1 - 0.0423280423280416*G0_5_1_0_25_0_1 - 0.152380952380953*G0_5_1_0_25_1_1 - 0.101587301587302*G0_5_1_0_26_0_1 - 0.338624338624338*G0_5_1_0_26_1_1 + 0.338624338624338*G0_5_1_0_27_0_1 + 0.812698412698416*G0_5_1_0_27_1_1 + 0.33862433862434*G0_5_1_0_28_0_1 + 0.812698412698412*G0_5_1_0_28_1_1 - 1.28677248677249*G0_5_1_0_29_0_1 - 2.43809523809524*G0_5_1_0_29_1_1 + 0.126984126984127*G0_6_0_0_0_0_0 + 0.126984126984126*G0_6_0_0_0_1_0 + 0.0423280423280421*G0_6_0_0_1_0_0 - 0.203174603174603*G0_6_0_0_2_1_0 - 0.203174603174603*G0_6_0_0_3_0_0 - 0.101587301587301*G0_6_0_0_3_1_0 - 0.152380952380952*G0_6_0_0_4_0_0 - 0.11005291005291*G0_6_0_0_4_1_0 - 0.338624338624338*G0_6_0_0_5_0_0 - 0.237037037037037*G0_6_0_0_5_1_0 + 1.82857142857143*G0_6_0_0_6_0_0 + 0.778835978835979*G0_6_0_0_6_1_0 - 0.66031746031746*G0_6_0_0_7_0_0 - 1.48148148148148*G0_6_0_0_7_1_0 + 0.338624338624339*G0_6_0_0_8_0_0 + 0.778835978835979*G0_6_0_0_8_1_0 - 0.135449735449736*G0_6_0_0_9_0_0 + 0.914285714285713*G0_6_0_0_9_1_0 + 0.101587301587303*G0_6_0_0_10_0_0 - 0.364021164021164*G0_6_0_0_10_1_0 - 0.135449735449735*G0_6_0_0_11_0_0 + 0.101587301587301*G0_6_0_0_11_1_0 - 2.43809523809524*G0_6_0_0_12_0_0 - 1.15132275132275*G0_6_0_0_12_1_0 + 0.812698412698413*G0_6_0_0_13_0_0 + 0.474074074074074*G0_6_0_0_13_1_0 + 0.812698412698412*G0_6_0_0_14_0_0 + 0.474074074074071*G0_6_0_0_14_1_0 + 0.126984126984127*G0_6_0_0_15_0_1 + 0.126984126984126*G0_6_0_0_15_1_1 + 0.0423280423280421*G0_6_0_0_16_0_1 - 0.203174603174603*G0_6_0_0_17_1_1 - 0.203174603174603*G0_6_0_0_18_0_1 - 0.101587301587301*G0_6_0_0_18_1_1 - 0.152380952380952*G0_6_0_0_19_0_1 - 0.11005291005291*G0_6_0_0_19_1_1 - 0.338624338624338*G0_6_0_0_20_0_1 - 0.237037037037037*G0_6_0_0_20_1_1 + 1.82857142857143*G0_6_0_0_21_0_1 + 0.778835978835979*G0_6_0_0_21_1_1 - 0.66031746031746*G0_6_0_0_22_0_1 - 1.48148148148148*G0_6_0_0_22_1_1 + 0.338624338624339*G0_6_0_0_23_0_1 + 0.778835978835979*G0_6_0_0_23_1_1 - 0.135449735449736*G0_6_0_0_24_0_1 + 0.914285714285713*G0_6_0_0_24_1_1 + 0.101587301587303*G0_6_0_0_25_0_1 - 0.364021164021164*G0_6_0_0_25_1_1 - 0.135449735449735*G0_6_0_0_26_0_1 + 0.101587301587301*G0_6_0_0_26_1_1 - 2.43809523809524*G0_6_0_0_27_0_1 - 1.15132275132275*G0_6_0_0_27_1_1 + 0.812698412698413*G0_6_0_0_28_0_1 + 0.474074074074074*G0_6_0_0_28_1_1 + 0.812698412698412*G0_6_0_0_29_0_1 + 0.474074074074071*G0_6_0_0_29_1_1 - 0.651851851851852*G0_6_1_0_0_0_0 - 0.651851851851852*G0_6_1_0_0_1_0 - 0.245502645502646*G0_6_1_0_2_1_0 - 0.101587301587302*G0_6_1_0_3_0_0 - 0.11005291005291*G0_6_1_0_4_0_0 - 0.0677248677248674*G0_6_1_0_4_1_0 - 0.237037037037037*G0_6_1_0_5_0_0 - 0.135449735449735*G0_6_1_0_5_1_0 + 0.778835978835979*G0_6_1_0_6_0_0 + 1.82857142857143*G0_6_1_0_6_1_0 - 0.364021164021164*G0_6_1_0_7_0_0 - 1.94708994708995*G0_6_1_0_7_1_0 + 0.237037037037037*G0_6_1_0_8_0_0 + 1.01587301587302*G0_6_1_0_8_1_0 + 0.914285714285714*G0_6_1_0_9_0_0 - 0.135449735449736*G0_6_1_0_9_1_0 - 0.364021164021165*G0_6_1_0_10_0_0 - 0.0677248677248697*G0_6_1_0_10_1_0 + 0.101587301587303*G0_6_1_0_11_0_0 - 1.15132275132275*G0_6_1_0_12_0_0 + 0.135449735449738*G0_6_1_0_12_1_0 + 0.474074074074073*G0_6_1_0_13_0_0 + 0.135449735449738*G0_6_1_0_13_1_0 + 0.474074074074073*G0_6_1_0_14_0_0 + 0.135449735449731*G0_6_1_0_14_1_0 - 0.651851851851852*G0_6_1_0_15_0_1 - 0.651851851851852*G0_6_1_0_15_1_1 - 0.245502645502646*G0_6_1_0_17_1_1 - 0.101587301587302*G0_6_1_0_18_0_1 - 0.11005291005291*G0_6_1_0_19_0_1 - 0.0677248677248674*G0_6_1_0_19_1_1 - 0.237037037037037*G0_6_1_0_20_0_1 - 0.135449735449735*G0_6_1_0_20_1_1 + 0.778835978835979*G0_6_1_0_21_0_1 + 1.82857142857143*G0_6_1_0_21_1_1 - 0.364021164021164*G0_6_1_0_22_0_1 - 1.94708994708995*G0_6_1_0_22_1_1 + 0.237037037037037*G0_6_1_0_23_0_1 + 1.01587301587302*G0_6_1_0_23_1_1 + 0.914285714285714*G0_6_1_0_24_0_1 - 0.135449735449736*G0_6_1_0_24_1_1 - 0.364021164021165*G0_6_1_0_25_0_1 - 0.0677248677248697*G0_6_1_0_25_1_1 + 0.101587301587303*G0_6_1_0_26_0_1 - 1.15132275132275*G0_6_1_0_27_0_1 + 0.135449735449738*G0_6_1_0_27_1_1 + 0.474074074074073*G0_6_1_0_28_0_1 + 0.135449735449738*G0_6_1_0_28_1_1 + 0.474074074074073*G0_6_1_0_29_0_1 + 0.135449735449731*G0_6_1_0_29_1_1 + 0.0423280423280421*G0_7_0_0_0_0_0 + 0.0423280423280421*G0_7_0_0_0_1_0 + 0.0423280423280423*G0_7_0_0_1_0_0 + 0.425396825396825*G0_7_0_0_2_1_0 - 0.152380952380956*G0_7_0_0_3_0_0 - 0.042328042328044*G0_7_0_0_3_1_0 + 0.393650793650795*G0_7_0_0_4_0_0 + 0.196825396825396*G0_7_0_0_4_1_0 + 0.761904761904763*G0_7_0_0_5_0_0 + 0.465608465608468*G0_7_0_0_5_1_0 - 0.66031746031746*G0_7_0_0_6_0_0 - 0.364021164021164*G0_7_0_0_6_1_0 + 2.45079365079364*G0_7_0_0_7_0_0 + 1.47936507936508*G0_7_0_0_7_1_0 - 0.761904761904763*G0_7_0_0_8_0_0 - 1.58306878306878*G0_7_0_0_8_1_0 - 0.0677248677248674*G0_7_0_0_9_0_0 - 0.364021164021165*G0_7_0_0_9_1_0 + 0.0507936507936525*G0_7_0_0_10_0_0 - 0.146031746031743*G0_7_0_0_10_1_0 - 0.0677248677248697*G0_7_0_0_11_0_0 + 0.0423280423280434*G0_7_0_0_11_1_0 + 0.406349206349206*G0_7_0_0_12_0_0 + 1.57460317460317*G0_7_0_0_12_1_0 + 0.40634920634921*G0_7_0_0_13_0_0 - 0.0507936507936531*G0_7_0_0_13_1_0 - 2.84444444444444*G0_7_0_0_14_0_0 - 1.67619047619047*G0_7_0_0_14_1_0 + 0.0423280423280421*G0_7_0_0_15_0_1 + 0.0423280423280421*G0_7_0_0_15_1_1 + 0.0423280423280423*G0_7_0_0_16_0_1 + 0.425396825396825*G0_7_0_0_17_1_1 - 0.152380952380956*G0_7_0_0_18_0_1 - 0.042328042328044*G0_7_0_0_18_1_1 + 0.393650793650795*G0_7_0_0_19_0_1 + 0.196825396825396*G0_7_0_0_19_1_1 + 0.761904761904763*G0_7_0_0_20_0_1 + 0.465608465608468*G0_7_0_0_20_1_1 - 0.66031746031746*G0_7_0_0_21_0_1 - 0.364021164021164*G0_7_0_0_21_1_1 + 2.45079365079364*G0_7_0_0_22_0_1 + 1.47936507936508*G0_7_0_0_22_1_1 - 0.761904761904763*G0_7_0_0_23_0_1 - 1.58306878306878*G0_7_0_0_23_1_1 - 0.0677248677248674*G0_7_0_0_24_0_1 - 0.364021164021165*G0_7_0_0_24_1_1 + 0.0507936507936525*G0_7_0_0_25_0_1 - 0.146031746031743*G0_7_0_0_25_1_1 - 0.0677248677248697*G0_7_0_0_26_0_1 + 0.0423280423280434*G0_7_0_0_26_1_1 + 0.406349206349206*G0_7_0_0_27_0_1 + 1.57460317460317*G0_7_0_0_27_1_1 + 0.40634920634921*G0_7_0_0_28_0_1 - 0.0507936507936531*G0_7_0_0_28_1_1 - 2.84444444444444*G0_7_0_0_29_0_1 - 1.67619047619047*G0_7_0_0_29_1_1 + 0.467724867724868*G0_7_1_0_0_0_0 + 0.467724867724868*G0_7_1_0_0_1_0 + 0.467724867724867*G0_7_1_0_2_1_0 - 0.0423280423280443*G0_7_1_0_3_0_0 + 0.196825396825397*G0_7_1_0_4_0_0 + 0.0507936507936494*G0_7_1_0_4_1_0 + 0.465608465608465*G0_7_1_0_5_0_0 + 0.101587301587302*G0_7_1_0_5_1_0 - 1.48148148148148*G0_7_1_0_6_0_0 - 1.94708994708995*G0_7_1_0_6_1_0 + 1.47936507936508*G0_7_1_0_7_0_0 + 2.95873015873016*G0_7_1_0_7_1_0 - 0.465608465608465*G0_7_1_0_8_0_0 - 1.94708994708995*G0_7_1_0_8_1_0 - 0.364021164021164*G0_7_1_0_9_0_0 + 0.101587301587302*G0_7_1_0_9_1_0 - 0.146031746031745*G0_7_1_0_10_0_0 + 0.0507936507936527*G0_7_1_0_10_1_0 + 0.0423280423280407*G0_7_1_0_11_0_0 + 1.57460317460317*G0_7_1_0_12_0_0 - 0.101587301587306*G0_7_1_0_12_1_0 - 0.0507936507936475*G0_7_1_0_13_0_0 - 0.101587301587303*G0_7_1_0_13_1_0 - 1.67619047619047*G0_7_1_0_14_0_0 - 0.101587301587297*G0_7_1_0_14_1_0 + 0.467724867724868*G0_7_1_0_15_0_1 + 0.467724867724868*G0_7_1_0_15_1_1 + 0.467724867724867*G0_7_1_0_17_1_1 - 0.0423280423280443*G0_7_1_0_18_0_1 + 0.196825396825397*G0_7_1_0_19_0_1 + 0.0507936507936494*G0_7_1_0_19_1_1 + 0.465608465608465*G0_7_1_0_20_0_1 + 0.101587301587302*G0_7_1_0_20_1_1 - 1.48148148148148*G0_7_1_0_21_0_1 - 1.94708994708995*G0_7_1_0_21_1_1 + 1.47936507936508*G0_7_1_0_22_0_1 + 2.95873015873016*G0_7_1_0_22_1_1 - 0.465608465608465*G0_7_1_0_23_0_1 - 1.94708994708995*G0_7_1_0_23_1_1 - 0.364021164021164*G0_7_1_0_24_0_1 + 0.101587301587302*G0_7_1_0_24_1_1 - 0.146031746031745*G0_7_1_0_25_0_1 + 0.0507936507936527*G0_7_1_0_25_1_1 + 0.0423280423280407*G0_7_1_0_26_0_1 + 1.57460317460317*G0_7_1_0_27_0_1 - 0.101587301587306*G0_7_1_0_27_1_1 - 0.0507936507936475*G0_7_1_0_28_0_1 - 0.101587301587303*G0_7_1_0_28_1_1 - 1.67619047619047*G0_7_1_0_29_0_1 - 0.101587301587297*G0_7_1_0_29_1_1 - 0.042328042328042*G0_8_0_0_0_0_0 - 0.0423280423280422*G0_8_0_0_0_1_0 + 0.0423280423280424*G0_8_0_0_1_0_0 - 0.77883597883598*G0_8_0_0_2_1_0 - 0.338624338624337*G0_8_0_0_3_0_0 - 0.101587301587301*G0_8_0_0_3_1_0 + 0.761904761904761*G0_8_0_0_4_0_0 + 0.296296296296298*G0_8_0_0_4_1_0 - 2.0994708994709*G0_8_0_0_5_0_0 - 1.04973544973545*G0_8_0_0_5_1_0 + 0.338624338624339*G0_8_0_0_6_0_0 + 0.237037037037037*G0_8_0_0_6_1_0 - 0.761904761904763*G0_8_0_0_7_0_0 - 0.465608465608465*G0_8_0_0_7_1_0 + 2.0994708994709*G0_8_0_0_8_0_0 + 1.04973544973545*G0_8_0_0_8_1_0 + 0.101587301587302*G0_8_0_0_9_1_0 + 0.0423280423280414*G0_8_0_0_10_1_0 + 0.101587301587301*G0_8_0_0_11_1_0 - 0.338624338624339*G0_8_0_0_12_1_0 - 0.338624338624338*G0_8_0_0_13_1_0 + 1.28677248677249*G0_8_0_0_14_1_0 - 0.042328042328042*G0_8_0_0_15_0_1 - 0.0423280423280422*G0_8_0_0_15_1_1 + 0.0423280423280424*G0_8_0_0_16_0_1 - 0.77883597883598*G0_8_0_0_17_1_1 - 0.338624338624337*G0_8_0_0_18_0_1 - 0.101587301587301*G0_8_0_0_18_1_1 + 0.761904761904761*G0_8_0_0_19_0_1 + 0.296296296296298*G0_8_0_0_19_1_1 - 2.0994708994709*G0_8_0_0_20_0_1 - 1.04973544973545*G0_8_0_0_20_1_1 + 0.338624338624339*G0_8_0_0_21_0_1 + 0.237037037037037*G0_8_0_0_21_1_1 - 0.761904761904763*G0_8_0_0_22_0_1 - 0.465608465608465*G0_8_0_0_22_1_1 + 2.0994708994709*G0_8_0_0_23_0_1 + 1.04973544973545*G0_8_0_0_23_1_1 + 0.101587301587302*G0_8_0_0_24_1_1 + 0.0423280423280414*G0_8_0_0_25_1_1 + 0.101587301587301*G0_8_0_0_26_1_1 - 0.338624338624339*G0_8_0_0_27_1_1 - 0.338624338624338*G0_8_0_0_28_1_1 + 1.28677248677249*G0_8_0_0_29_1_1 - 0.245502645502646*G0_8_1_0_0_0_0 - 0.245502645502646*G0_8_1_0_0_1_0 - 0.651851851851852*G0_8_1_0_2_1_0 - 0.101587301587299*G0_8_1_0_3_0_0 + 0.296296296296296*G0_8_1_0_4_0_0 - 0.0677248677248661*G0_8_1_0_4_1_0 - 1.04973544973545*G0_8_1_0_5_0_0 - 0.135449735449737*G0_8_1_0_5_1_0 + 0.778835978835979*G0_8_1_0_6_0_0 + 1.01587301587302*G0_8_1_0_6_1_0 - 1.58306878306878*G0_8_1_0_7_0_0 - 1.94708994708995*G0_8_1_0_7_1_0 + 1.04973544973545*G0_8_1_0_8_0_0 + 1.82857142857143*G0_8_1_0_8_1_0 + 0.101587301587302*G0_8_1_0_9_0_0 - 0.135449735449736*G0_8_1_0_9_1_0 + 0.0423280423280414*G0_8_1_0_10_0_0 - 0.0677248677248693*G0_8_1_0_10_1_0 + 0.101587301587303*G0_8_1_0_11_0_0 - 0.338624338624338*G0_8_1_0_12_0_0 + 0.135449735449738*G0_8_1_0_12_1_0 - 0.338624338624342*G0_8_1_0_13_0_0 + 0.135449735449736*G0_8_1_0_13_1_0 + 1.28677248677248*G0_8_1_0_14_0_0 + 0.135449735449733*G0_8_1_0_14_1_0 - 0.245502645502646*G0_8_1_0_15_0_1 - 0.245502645502646*G0_8_1_0_15_1_1 - 0.651851851851852*G0_8_1_0_17_1_1 - 0.101587301587299*G0_8_1_0_18_0_1 + 0.296296296296296*G0_8_1_0_19_0_1 - 0.0677248677248661*G0_8_1_0_19_1_1 - 1.04973544973545*G0_8_1_0_20_0_1 - 0.135449735449737*G0_8_1_0_20_1_1 + 0.778835978835979*G0_8_1_0_21_0_1 + 1.01587301587302*G0_8_1_0_21_1_1 - 1.58306878306878*G0_8_1_0_22_0_1 - 1.94708994708995*G0_8_1_0_22_1_1 + 1.04973544973545*G0_8_1_0_23_0_1 + 1.82857142857143*G0_8_1_0_23_1_1 + 0.101587301587302*G0_8_1_0_24_0_1 - 0.135449735449736*G0_8_1_0_24_1_1 + 0.0423280423280414*G0_8_1_0_25_0_1 - 0.0677248677248693*G0_8_1_0_25_1_1 + 0.101587301587303*G0_8_1_0_26_0_1 - 0.338624338624338*G0_8_1_0_27_0_1 + 0.135449735449738*G0_8_1_0_27_1_1 - 0.338624338624342*G0_8_1_0_28_0_1 + 0.135449735449736*G0_8_1_0_28_1_1 + 1.28677248677248*G0_8_1_0_29_0_1 + 0.135449735449733*G0_8_1_0_29_1_1 - 0.651851851851852*G0_9_0_0_0_0_0 - 0.651851851851852*G0_9_0_0_0_1_0 - 0.245502645502645*G0_9_0_0_1_0_0 - 0.135449735449733*G0_9_0_0_3_0_0 - 0.237037037037036*G0_9_0_0_3_1_0 - 0.0677248677248682*G0_9_0_0_4_0_0 - 0.110052910052909*G0_9_0_0_4_1_0 - 0.101587301587301*G0_9_0_0_5_1_0 - 0.135449735449736*G0_9_0_0_6_0_0 + 0.914285714285714*G0_9_0_0_6_1_0 - 0.0677248677248674*G0_9_0_0_7_0_0 - 0.364021164021164*G0_9_0_0_7_1_0 + 0.101587301587302*G0_9_0_0_8_1_0 + 1.82857142857143*G0_9_0_0_9_0_0 + 0.778835978835977*G0_9_0_0_9_1_0 - 1.94708994708995*G0_9_0_0_10_0_0 - 0.364021164021163*G0_9_0_0_10_1_0 + 1.01587301587301*G0_9_0_0_11_0_0 + 0.237037037037036*G0_9_0_0_11_1_0 + 0.135449735449737*G0_9_0_0_12_0_0 - 1.15132275132275*G0_9_0_0_12_1_0 + 0.135449735449732*G0_9_0_0_13_0_0 + 0.474074074074072*G0_9_0_0_13_1_0 + 0.135449735449736*G0_9_0_0_14_0_0 + 0.474074074074072*G0_9_0_0_14_1_0 - 0.651851851851852*G0_9_0_0_15_0_1 - 0.651851851851852*G0_9_0_0_15_1_1 - 0.245502645502645*G0_9_0_0_16_0_1 - 0.135449735449733*G0_9_0_0_18_0_1 - 0.237037037037036*G0_9_0_0_18_1_1 - 0.0677248677248682*G0_9_0_0_19_0_1 - 0.110052910052909*G0_9_0_0_19_1_1 - 0.101587301587301*G0_9_0_0_20_1_1 - 0.135449735449736*G0_9_0_0_21_0_1 + 0.914285714285714*G0_9_0_0_21_1_1 - 0.0677248677248674*G0_9_0_0_22_0_1 - 0.364021164021164*G0_9_0_0_22_1_1 + 0.101587301587302*G0_9_0_0_23_1_1 + 1.82857142857143*G0_9_0_0_24_0_1 + 0.778835978835977*G0_9_0_0_24_1_1 - 1.94708994708995*G0_9_0_0_25_0_1 - 0.364021164021163*G0_9_0_0_25_1_1 + 1.01587301587301*G0_9_0_0_26_0_1 + 0.237037037037036*G0_9_0_0_26_1_1 + 0.135449735449737*G0_9_0_0_27_0_1 - 1.15132275132275*G0_9_0_0_27_1_1 + 0.135449735449732*G0_9_0_0_28_0_1 + 0.474074074074072*G0_9_0_0_28_1_1 + 0.135449735449736*G0_9_0_0_29_0_1 + 0.474074074074072*G0_9_0_0_29_1_1 + 0.126984126984127*G0_9_1_0_0_0_0 + 0.126984126984127*G0_9_1_0_0_1_0 - 0.203174603174603*G0_9_1_0_1_0_0 + 0.0423280423280423*G0_9_1_0_2_1_0 - 0.237037037037035*G0_9_1_0_3_0_0 - 0.338624338624337*G0_9_1_0_3_1_0 - 0.110052910052911*G0_9_1_0_4_0_0 - 0.152380952380951*G0_9_1_0_4_1_0 - 0.101587301587302*G0_9_1_0_5_0_0 - 0.203174603174604*G0_9_1_0_5_1_0 + 0.914285714285713*G0_9_1_0_6_0_0 - 0.135449735449736*G0_9_1_0_6_1_0 - 0.364021164021165*G0_9_1_0_7_0_0 + 0.101587301587302*G0_9_1_0_7_1_0 + 0.101587301587302*G0_9_1_0_8_0_0 - 0.135449735449736*G0_9_1_0_8_1_0 + 0.778835978835977*G0_9_1_0_9_0_0 + 1.82857142857143*G0_9_1_0_9_1_0 - 1.48148148148148*G0_9_1_0_10_0_0 - 0.660317460317458*G0_9_1_0_10_1_0 + 0.778835978835978*G0_9_1_0_11_0_0 + 0.338624338624337*G0_9_1_0_11_1_0 - 1.15132275132275*G0_9_1_0_12_0_0 - 2.43809523809524*G0_9_1_0_12_1_0 + 0.47407407407407*G0_9_1_0_13_0_0 + 0.812698412698409*G0_9_1_0_13_1_0 + 0.474074074074076*G0_9_1_0_14_0_0 + 0.812698412698413*G0_9_1_0_14_1_0 + 0.126984126984127*G0_9_1_0_15_0_1 + 0.126984126984127*G0_9_1_0_15_1_1 - 0.203174603174603*G0_9_1_0_16_0_1 + 0.0423280423280423*G0_9_1_0_17_1_1 - 0.237037037037035*G0_9_1_0_18_0_1 - 0.338624338624337*G0_9_1_0_18_1_1 - 0.110052910052911*G0_9_1_0_19_0_1 - 0.152380952380951*G0_9_1_0_19_1_1 - 0.101587301587302*G0_9_1_0_20_0_1 - 0.203174603174604*G0_9_1_0_20_1_1 + 0.914285714285713*G0_9_1_0_21_0_1 - 0.135449735449736*G0_9_1_0_21_1_1 - 0.364021164021165*G0_9_1_0_22_0_1 + 0.101587301587302*G0_9_1_0_22_1_1 + 0.101587301587302*G0_9_1_0_23_0_1 - 0.135449735449736*G0_9_1_0_23_1_1 + 0.778835978835977*G0_9_1_0_24_0_1 + 1.82857142857143*G0_9_1_0_24_1_1 - 1.48148148148148*G0_9_1_0_25_0_1 - 0.660317460317458*G0_9_1_0_25_1_1 + 0.778835978835978*G0_9_1_0_26_0_1 + 0.338624338624337*G0_9_1_0_26_1_1 - 1.15132275132275*G0_9_1_0_27_0_1 - 2.43809523809524*G0_9_1_0_27_1_1 + 0.47407407407407*G0_9_1_0_28_0_1 + 0.812698412698409*G0_9_1_0_28_1_1 + 0.474074074074076*G0_9_1_0_29_0_1 + 0.812698412698413*G0_9_1_0_29_1_1 + 0.467724867724868*G0_10_0_0_0_0_0 + 0.467724867724868*G0_10_0_0_0_1_0 + 0.467724867724867*G0_10_0_0_1_0_0 + 0.1015873015873*G0_10_0_0_3_0_0 + 0.465608465608464*G0_10_0_0_3_1_0 + 0.050793650793651*G0_10_0_0_4_0_0 + 0.196825396825396*G0_10_0_0_4_1_0 - 0.0423280423280416*G0_10_0_0_5_1_0 + 0.101587301587302*G0_10_0_0_6_0_0 - 0.364021164021165*G0_10_0_0_6_1_0 + 0.0507936507936526*G0_10_0_0_7_0_0 - 0.146031746031745*G0_10_0_0_7_1_0 + 0.0423280423280414*G0_10_0_0_8_1_0 - 1.94708994708995*G0_10_0_0_9_0_0 - 1.48148148148148*G0_10_0_0_9_1_0 + 2.95873015873015*G0_10_0_0_10_0_0 + 1.47936507936508*G0_10_0_0_10_1_0 - 1.94708994708994*G0_10_0_0_11_0_0 - 0.465608465608464*G0_10_0_0_11_1_0 - 0.101587301587304*G0_10_0_0_12_0_0 + 1.57460317460317*G0_10_0_0_12_1_0 - 0.101587301587298*G0_10_0_0_13_0_0 - 1.67619047619047*G0_10_0_0_13_1_0 - 0.101587301587304*G0_10_0_0_14_0_0 - 0.0507936507936524*G0_10_0_0_14_1_0 + 0.467724867724868*G0_10_0_0_15_0_1 + 0.467724867724868*G0_10_0_0_15_1_1 + 0.467724867724867*G0_10_0_0_16_0_1 + 0.1015873015873*G0_10_0_0_18_0_1 + 0.465608465608464*G0_10_0_0_18_1_1 + 0.050793650793651*G0_10_0_0_19_0_1 + 0.196825396825396*G0_10_0_0_19_1_1 - 0.0423280423280416*G0_10_0_0_20_1_1 + 0.101587301587302*G0_10_0_0_21_0_1 - 0.364021164021165*G0_10_0_0_21_1_1 + 0.0507936507936526*G0_10_0_0_22_0_1 - 0.146031746031745*G0_10_0_0_22_1_1 + 0.0423280423280414*G0_10_0_0_23_1_1 - 1.94708994708995*G0_10_0_0_24_0_1 - 1.48148148148148*G0_10_0_0_24_1_1 + 2.95873015873015*G0_10_0_0_25_0_1 + 1.47936507936508*G0_10_0_0_25_1_1 - 1.94708994708994*G0_10_0_0_26_0_1 - 0.465608465608464*G0_10_0_0_26_1_1 - 0.101587301587304*G0_10_0_0_27_0_1 + 1.57460317460317*G0_10_0_0_27_1_1 - 0.101587301587298*G0_10_0_0_28_0_1 - 1.67619047619047*G0_10_0_0_28_1_1 - 0.101587301587304*G0_10_0_0_29_0_1 - 0.0507936507936524*G0_10_0_0_29_1_1 + 0.0423280423280424*G0_10_1_0_0_0_0 + 0.0423280423280427*G0_10_1_0_0_1_0 + 0.425396825396824*G0_10_1_0_1_0_0 + 0.042328042328043*G0_10_1_0_2_1_0 + 0.465608465608465*G0_10_1_0_3_0_0 + 0.761904761904759*G0_10_1_0_3_1_0 + 0.196825396825396*G0_10_1_0_4_0_0 + 0.393650793650794*G0_10_1_0_4_1_0 - 0.0423280423280415*G0_10_1_0_5_0_0 - 0.152380952380953*G0_10_1_0_5_1_0 - 0.364021164021164*G0_10_1_0_6_0_0 - 0.0677248677248697*G0_10_1_0_6_1_0 - 0.146031746031743*G0_10_1_0_7_0_0 + 0.0507936507936526*G0_10_1_0_7_1_0 + 0.0423280423280414*G0_10_1_0_8_0_0 - 0.0677248677248692*G0_10_1_0_8_1_0 - 0.364021164021163*G0_10_1_0_9_0_0 - 0.660317460317458*G0_10_1_0_9_1_0 + 1.47936507936508*G0_10_1_0_10_0_0 + 2.45079365079365*G0_10_1_0_10_1_0 - 1.58306878306878*G0_10_1_0_11_0_0 - 0.761904761904759*G0_10_1_0_11_1_0 + 1.57460317460317*G0_10_1_0_12_0_0 + 0.406349206349205*G0_10_1_0_12_1_0 - 1.67619047619047*G0_10_1_0_13_0_0 - 2.84444444444444*G0_10_1_0_13_1_0 - 0.0507936507936526*G0_10_1_0_14_0_0 + 0.406349206349206*G0_10_1_0_14_1_0 + 0.0423280423280424*G0_10_1_0_15_0_1 + 0.0423280423280427*G0_10_1_0_15_1_1 + 0.425396825396824*G0_10_1_0_16_0_1 + 0.042328042328043*G0_10_1_0_17_1_1 + 0.465608465608465*G0_10_1_0_18_0_1 + 0.761904761904759*G0_10_1_0_18_1_1 + 0.196825396825396*G0_10_1_0_19_0_1 + 0.393650793650794*G0_10_1_0_19_1_1 - 0.0423280423280415*G0_10_1_0_20_0_1 - 0.152380952380953*G0_10_1_0_20_1_1 - 0.364021164021164*G0_10_1_0_21_0_1 - 0.0677248677248697*G0_10_1_0_21_1_1 - 0.146031746031743*G0_10_1_0_22_0_1 + 0.0507936507936526*G0_10_1_0_22_1_1 + 0.0423280423280414*G0_10_1_0_23_0_1 - 0.0677248677248692*G0_10_1_0_23_1_1 - 0.364021164021163*G0_10_1_0_24_0_1 - 0.660317460317458*G0_10_1_0_24_1_1 + 1.47936507936508*G0_10_1_0_25_0_1 + 2.45079365079365*G0_10_1_0_25_1_1 - 1.58306878306878*G0_10_1_0_26_0_1 - 0.761904761904759*G0_10_1_0_26_1_1 + 1.57460317460317*G0_10_1_0_27_0_1 + 0.406349206349205*G0_10_1_0_27_1_1 - 1.67619047619047*G0_10_1_0_28_0_1 - 2.84444444444444*G0_10_1_0_28_1_1 - 0.0507936507936526*G0_10_1_0_29_0_1 + 0.406349206349206*G0_10_1_0_29_1_1 - 0.245502645502646*G0_11_0_0_0_0_0 - 0.245502645502646*G0_11_0_0_0_1_0 - 0.651851851851851*G0_11_0_0_1_0_0 - 0.135449735449735*G0_11_0_0_3_0_0 - 1.04973544973545*G0_11_0_0_3_1_0 - 0.0677248677248674*G0_11_0_0_4_0_0 + 0.296296296296297*G0_11_0_0_4_1_0 - 0.101587301587302*G0_11_0_0_5_1_0 - 0.135449735449735*G0_11_0_0_6_0_0 + 0.101587301587303*G0_11_0_0_6_1_0 - 0.0677248677248697*G0_11_0_0_7_0_0 + 0.0423280423280407*G0_11_0_0_7_1_0 + 0.101587301587303*G0_11_0_0_8_1_0 + 1.01587301587301*G0_11_0_0_9_0_0 + 0.778835978835978*G0_11_0_0_9_1_0 - 1.94708994708994*G0_11_0_0_10_0_0 - 1.58306878306878*G0_11_0_0_10_1_0 + 1.82857142857143*G0_11_0_0_11_0_0 + 1.04973544973545*G0_11_0_0_11_1_0 + 0.135449735449736*G0_11_0_0_12_0_0 - 0.338624338624339*G0_11_0_0_12_1_0 + 0.135449735449734*G0_11_0_0_13_0_0 + 1.28677248677248*G0_11_0_0_13_1_0 + 0.135449735449737*G0_11_0_0_14_0_0 - 0.338624338624336*G0_11_0_0_14_1_0 - 0.245502645502646*G0_11_0_0_15_0_1 - 0.245502645502646*G0_11_0_0_15_1_1 - 0.651851851851851*G0_11_0_0_16_0_1 - 0.135449735449735*G0_11_0_0_18_0_1 - 1.04973544973545*G0_11_0_0_18_1_1 - 0.0677248677248674*G0_11_0_0_19_0_1 + 0.296296296296297*G0_11_0_0_19_1_1 - 0.101587301587302*G0_11_0_0_20_1_1 - 0.135449735449735*G0_11_0_0_21_0_1 + 0.101587301587303*G0_11_0_0_21_1_1 - 0.0677248677248697*G0_11_0_0_22_0_1 + 0.0423280423280407*G0_11_0_0_22_1_1 + 0.101587301587303*G0_11_0_0_23_1_1 + 1.01587301587301*G0_11_0_0_24_0_1 + 0.778835978835978*G0_11_0_0_24_1_1 - 1.94708994708994*G0_11_0_0_25_0_1 - 1.58306878306878*G0_11_0_0_25_1_1 + 1.82857142857143*G0_11_0_0_26_0_1 + 1.04973544973545*G0_11_0_0_26_1_1 + 0.135449735449736*G0_11_0_0_27_0_1 - 0.338624338624339*G0_11_0_0_27_1_1 + 0.135449735449734*G0_11_0_0_28_0_1 + 1.28677248677248*G0_11_0_0_28_1_1 + 0.135449735449737*G0_11_0_0_29_0_1 - 0.338624338624336*G0_11_0_0_29_1_1 - 0.0423280423280419*G0_11_1_0_0_0_0 - 0.0423280423280425*G0_11_1_0_0_1_0 - 0.778835978835978*G0_11_1_0_1_0_0 + 0.0423280423280428*G0_11_1_0_2_1_0 - 1.04973544973545*G0_11_1_0_3_0_0 - 2.0994708994709*G0_11_1_0_3_1_0 + 0.296296296296296*G0_11_1_0_4_0_0 + 0.761904761904763*G0_11_1_0_4_1_0 - 0.101587301587301*G0_11_1_0_5_0_0 - 0.338624338624338*G0_11_1_0_5_1_0 + 0.101587301587301*G0_11_1_0_6_0_0 + 0.0423280423280434*G0_11_1_0_7_0_0 + 0.101587301587301*G0_11_1_0_8_0_0 + 0.237037037037036*G0_11_1_0_9_0_0 + 0.338624338624337*G0_11_1_0_9_1_0 - 0.465608465608464*G0_11_1_0_10_0_0 - 0.761904761904759*G0_11_1_0_10_1_0 + 1.04973544973545*G0_11_1_0_11_0_0 + 2.0994708994709*G0_11_1_0_11_1_0 - 0.338624338624338*G0_11_1_0_12_0_0 + 1.28677248677249*G0_11_1_0_13_0_0 - 0.338624338624339*G0_11_1_0_14_0_0 - 0.0423280423280419*G0_11_1_0_15_0_1 - 0.0423280423280425*G0_11_1_0_15_1_1 - 0.778835978835978*G0_11_1_0_16_0_1 + 0.0423280423280428*G0_11_1_0_17_1_1 - 1.04973544973545*G0_11_1_0_18_0_1 - 2.0994708994709*G0_11_1_0_18_1_1 + 0.296296296296296*G0_11_1_0_19_0_1 + 0.761904761904763*G0_11_1_0_19_1_1 - 0.101587301587301*G0_11_1_0_20_0_1 - 0.338624338624338*G0_11_1_0_20_1_1 + 0.101587301587301*G0_11_1_0_21_0_1 + 0.0423280423280434*G0_11_1_0_22_0_1 + 0.101587301587301*G0_11_1_0_23_0_1 + 0.237037037037036*G0_11_1_0_24_0_1 + 0.338624338624337*G0_11_1_0_24_1_1 - 0.465608465608464*G0_11_1_0_25_0_1 - 0.761904761904759*G0_11_1_0_25_1_1 + 1.04973544973545*G0_11_1_0_26_0_1 + 2.0994708994709*G0_11_1_0_26_1_1 - 0.338624338624338*G0_11_1_0_27_0_1 + 1.28677248677249*G0_11_1_0_28_0_1 - 0.338624338624339*G0_11_1_0_29_0_1 - 0.0846560846560848*G0_12_0_0_0_0_0 - 0.0846560846560841*G0_12_0_0_0_1_0 - 0.084656084656084*G0_12_0_0_1_0_0 + 0.812698412698412*G0_12_0_0_3_0_0 + 0.338624338624338*G0_12_0_0_3_1_0 + 0.406349206349207*G0_12_0_0_4_0_0 + 0.457142857142857*G0_12_0_0_4_1_0 + 0.338624338624338*G0_12_0_0_5_1_0 - 2.43809523809524*G0_12_0_0_6_0_0 - 1.15132275132275*G0_12_0_0_6_1_0 + 0.406349206349206*G0_12_0_0_7_0_0 + 1.57460317460317*G0_12_0_0_7_1_0 - 0.338624338624338*G0_12_0_0_8_1_0 + 0.135449735449737*G0_12_0_0_9_0_0 - 1.15132275132275*G0_12_0_0_9_1_0 - 0.101587301587304*G0_12_0_0_10_0_0 + 1.57460317460317*G0_12_0_0_10_1_0 + 0.135449735449736*G0_12_0_0_11_0_0 - 0.338624338624338*G0_12_0_0_11_1_0 + 5.68888888888888*G0_12_0_0_12_0_0 + 2.84444444444444*G0_12_0_0_12_1_0 - 4.06349206349206*G0_12_0_0_13_0_0 - 2.03174603174603*G0_12_0_0_13_1_0 - 0.812698412698412*G0_12_0_0_14_0_0 - 2.03174603174602*G0_12_0_0_14_1_0 - 0.0846560846560848*G0_12_0_0_15_0_1 - 0.0846560846560841*G0_12_0_0_15_1_1 - 0.084656084656084*G0_12_0_0_16_0_1 + 0.812698412698412*G0_12_0_0_18_0_1 + 0.338624338624338*G0_12_0_0_18_1_1 + 0.406349206349207*G0_12_0_0_19_0_1 + 0.457142857142857*G0_12_0_0_19_1_1 + 0.338624338624338*G0_12_0_0_20_1_1 - 2.43809523809524*G0_12_0_0_21_0_1 - 1.15132275132275*G0_12_0_0_21_1_1 + 0.406349206349206*G0_12_0_0_22_0_1 + 1.57460317460317*G0_12_0_0_22_1_1 - 0.338624338624338*G0_12_0_0_23_1_1 + 0.135449735449737*G0_12_0_0_24_0_1 - 1.15132275132275*G0_12_0_0_24_1_1 - 0.101587301587304*G0_12_0_0_25_0_1 + 1.57460317460317*G0_12_0_0_25_1_1 + 0.135449735449736*G0_12_0_0_26_0_1 - 0.338624338624338*G0_12_0_0_26_1_1 + 5.68888888888888*G0_12_0_0_27_0_1 + 2.84444444444444*G0_12_0_0_27_1_1 - 4.06349206349206*G0_12_0_0_28_0_1 - 2.03174603174603*G0_12_0_0_28_1_1 - 0.812698412698412*G0_12_0_0_29_0_1 - 2.03174603174602*G0_12_0_0_29_1_1 - 0.084656084656085*G0_12_1_0_0_0_0 - 0.0846560846560851*G0_12_1_0_0_1_0 - 0.084656084656085*G0_12_1_0_2_1_0 + 0.338624338624334*G0_12_1_0_3_0_0 + 0.457142857142862*G0_12_1_0_4_0_0 + 0.406349206349205*G0_12_1_0_4_1_0 + 0.338624338624339*G0_12_1_0_5_0_0 + 0.812698412698416*G0_12_1_0_5_1_0 - 1.15132275132275*G0_12_1_0_6_0_0 + 0.135449735449738*G0_12_1_0_6_1_0 + 1.57460317460317*G0_12_1_0_7_0_0 - 0.101587301587305*G0_12_1_0_7_1_0 - 0.338624338624339*G0_12_1_0_8_0_0 + 0.135449735449738*G0_12_1_0_8_1_0 - 1.15132275132275*G0_12_1_0_9_0_0 - 2.43809523809524*G0_12_1_0_9_1_0 + 1.57460317460317*G0_12_1_0_10_0_0 + 0.406349206349205*G0_12_1_0_10_1_0 - 0.33862433862434*G0_12_1_0_11_0_0 + 2.84444444444443*G0_12_1_0_12_0_0 + 5.68888888888888*G0_12_1_0_12_1_0 - 2.03174603174602*G0_12_1_0_13_0_0 - 0.812698412698409*G0_12_1_0_13_1_0 - 2.03174603174603*G0_12_1_0_14_0_0 - 4.06349206349206*G0_12_1_0_14_1_0 - 0.084656084656085*G0_12_1_0_15_0_1 - 0.0846560846560851*G0_12_1_0_15_1_1 - 0.084656084656085*G0_12_1_0_17_1_1 + 0.338624338624334*G0_12_1_0_18_0_1 + 0.457142857142862*G0_12_1_0_19_0_1 + 0.406349206349205*G0_12_1_0_19_1_1 + 0.338624338624339*G0_12_1_0_20_0_1 + 0.812698412698416*G0_12_1_0_20_1_1 - 1.15132275132275*G0_12_1_0_21_0_1 + 0.135449735449738*G0_12_1_0_21_1_1 + 1.57460317460317*G0_12_1_0_22_0_1 - 0.101587301587305*G0_12_1_0_22_1_1 - 0.338624338624339*G0_12_1_0_23_0_1 + 0.135449735449738*G0_12_1_0_23_1_1 - 1.15132275132275*G0_12_1_0_24_0_1 - 2.43809523809524*G0_12_1_0_24_1_1 + 1.57460317460317*G0_12_1_0_25_0_1 + 0.406349206349205*G0_12_1_0_25_1_1 - 0.33862433862434*G0_12_1_0_26_0_1 + 2.84444444444443*G0_12_1_0_27_0_1 + 5.68888888888888*G0_12_1_0_27_1_1 - 2.03174603174602*G0_12_1_0_28_0_1 - 0.812698412698409*G0_12_1_0_28_1_1 - 2.03174603174603*G0_12_1_0_29_0_1 - 4.06349206349206*G0_12_1_0_29_1_1 - 0.0846560846560835*G0_13_0_0_0_0_0 - 0.084656084656084*G0_13_0_0_0_1_0 - 0.0846560846560853*G0_13_0_0_1_0_0 - 2.43809523809524*G0_13_0_0_3_0_0 - 1.28677248677249*G0_13_0_0_3_1_0 + 0.406349206349207*G0_13_0_0_4_0_0 - 1.16825396825396*G0_13_0_0_4_1_0 + 0.33862433862434*G0_13_0_0_5_1_0 + 0.812698412698413*G0_13_0_0_6_0_0 + 0.474074074074073*G0_13_0_0_6_1_0 + 0.40634920634921*G0_13_0_0_7_0_0 - 0.0507936507936475*G0_13_0_0_7_1_0 - 0.338624338624342*G0_13_0_0_8_1_0 + 0.135449735449732*G0_13_0_0_9_0_0 + 0.47407407407407*G0_13_0_0_9_1_0 - 0.101587301587298*G0_13_0_0_10_0_0 - 1.67619047619047*G0_13_0_0_10_1_0 + 0.135449735449734*G0_13_0_0_11_0_0 + 1.28677248677249*G0_13_0_0_11_1_0 - 4.06349206349206*G0_13_0_0_12_0_0 - 2.03174603174602*G0_13_0_0_12_1_0 + 5.68888888888888*G0_13_0_0_13_0_0 + 2.84444444444444*G0_13_0_0_13_1_0 - 0.812698412698418*G0_13_0_0_14_0_0 + 1.21904761904761*G0_13_0_0_14_1_0 - 0.0846560846560835*G0_13_0_0_15_0_1 - 0.084656084656084*G0_13_0_0_15_1_1 - 0.0846560846560853*G0_13_0_0_16_0_1 - 2.43809523809524*G0_13_0_0_18_0_1 - 1.28677248677249*G0_13_0_0_18_1_1 + 0.406349206349207*G0_13_0_0_19_0_1 - 1.16825396825396*G0_13_0_0_19_1_1 + 0.33862433862434*G0_13_0_0_20_1_1 + 0.812698412698413*G0_13_0_0_21_0_1 + 0.474074074074073*G0_13_0_0_21_1_1 + 0.40634920634921*G0_13_0_0_22_0_1 - 0.0507936507936475*G0_13_0_0_22_1_1 - 0.338624338624342*G0_13_0_0_23_1_1 + 0.135449735449732*G0_13_0_0_24_0_1 + 0.47407407407407*G0_13_0_0_24_1_1 - 0.101587301587298*G0_13_0_0_25_0_1 - 1.67619047619047*G0_13_0_0_25_1_1 + 0.135449735449734*G0_13_0_0_26_0_1 + 1.28677248677249*G0_13_0_0_26_1_1 - 4.06349206349206*G0_13_0_0_27_0_1 - 2.03174603174602*G0_13_0_0_27_1_1 + 5.68888888888888*G0_13_0_0_28_0_1 + 2.84444444444444*G0_13_0_0_28_1_1 - 0.812698412698418*G0_13_0_0_29_0_1 + 1.21904761904761*G0_13_0_0_29_1_1 - 0.0846560846560848*G0_13_1_0_0_0_0 - 0.0846560846560849*G0_13_1_0_0_1_0 - 0.0846560846560848*G0_13_1_0_2_1_0 - 1.28677248677248*G0_13_1_0_3_0_0 - 1.16825396825396*G0_13_1_0_4_0_0 - 2.84444444444444*G0_13_1_0_4_1_0 + 0.338624338624338*G0_13_1_0_5_0_0 + 0.812698412698412*G0_13_1_0_5_1_0 + 0.474074074074074*G0_13_1_0_6_0_0 + 0.135449735449738*G0_13_1_0_6_1_0 - 0.0507936507936531*G0_13_1_0_7_0_0 - 0.101587301587303*G0_13_1_0_7_1_0 - 0.338624338624338*G0_13_1_0_8_0_0 + 0.135449735449736*G0_13_1_0_8_1_0 + 0.474074074074072*G0_13_1_0_9_0_0 + 0.812698412698409*G0_13_1_0_9_1_0 - 1.67619047619047*G0_13_1_0_10_0_0 - 2.84444444444444*G0_13_1_0_10_1_0 + 1.28677248677248*G0_13_1_0_11_0_0 - 2.03174603174603*G0_13_1_0_12_0_0 - 0.812698412698409*G0_13_1_0_12_1_0 + 2.84444444444444*G0_13_1_0_13_0_0 + 5.68888888888887*G0_13_1_0_13_1_0 + 1.21904761904762*G0_13_1_0_14_0_0 - 0.812698412698411*G0_13_1_0_14_1_0 - 0.0846560846560848*G0_13_1_0_15_0_1 - 0.0846560846560849*G0_13_1_0_15_1_1 - 0.0846560846560848*G0_13_1_0_17_1_1 - 1.28677248677248*G0_13_1_0_18_0_1 - 1.16825396825396*G0_13_1_0_19_0_1 - 2.84444444444444*G0_13_1_0_19_1_1 + 0.338624338624338*G0_13_1_0_20_0_1 + 0.812698412698412*G0_13_1_0_20_1_1 + 0.474074074074074*G0_13_1_0_21_0_1 + 0.135449735449738*G0_13_1_0_21_1_1 - 0.0507936507936531*G0_13_1_0_22_0_1 - 0.101587301587303*G0_13_1_0_22_1_1 - 0.338624338624338*G0_13_1_0_23_0_1 + 0.135449735449736*G0_13_1_0_23_1_1 + 0.474074074074072*G0_13_1_0_24_0_1 + 0.812698412698409*G0_13_1_0_24_1_1 - 1.67619047619047*G0_13_1_0_25_0_1 - 2.84444444444444*G0_13_1_0_25_1_1 + 1.28677248677248*G0_13_1_0_26_0_1 - 2.03174603174603*G0_13_1_0_27_0_1 - 0.812698412698409*G0_13_1_0_27_1_1 + 2.84444444444444*G0_13_1_0_28_0_1 + 5.68888888888887*G0_13_1_0_28_1_1 + 1.21904761904762*G0_13_1_0_29_0_1 - 0.812698412698411*G0_13_1_0_29_1_1 - 0.0846560846560841*G0_14_0_0_0_0_0 - 0.0846560846560838*G0_14_0_0_0_1_0 - 0.0846560846560845*G0_14_0_0_1_0_0 + 0.812698412698417*G0_14_0_0_3_0_0 + 0.338624338624341*G0_14_0_0_3_1_0 - 2.84444444444444*G0_14_0_0_4_0_0 - 1.16825396825397*G0_14_0_0_4_1_0 - 1.28677248677249*G0_14_0_0_5_1_0 + 0.812698412698412*G0_14_0_0_6_0_0 + 0.474074074074073*G0_14_0_0_6_1_0 - 2.84444444444444*G0_14_0_0_7_0_0 - 1.67619047619047*G0_14_0_0_7_1_0 + 1.28677248677248*G0_14_0_0_8_1_0 + 0.135449735449736*G0_14_0_0_9_0_0 + 0.474074074074076*G0_14_0_0_9_1_0 - 0.101587301587304*G0_14_0_0_10_0_0 - 0.0507936507936526*G0_14_0_0_10_1_0 + 0.135449735449737*G0_14_0_0_11_0_0 - 0.338624338624339*G0_14_0_0_11_1_0 - 0.812698412698412*G0_14_0_0_12_0_0 - 2.03174603174603*G0_14_0_0_12_1_0 - 0.812698412698418*G0_14_0_0_13_0_0 + 1.21904761904762*G0_14_0_0_13_1_0 + 5.68888888888888*G0_14_0_0_14_0_0 + 2.84444444444444*G0_14_0_0_14_1_0 - 0.0846560846560841*G0_14_0_0_15_0_1 - 0.0846560846560838*G0_14_0_0_15_1_1 - 0.0846560846560845*G0_14_0_0_16_0_1 + 0.812698412698417*G0_14_0_0_18_0_1 + 0.338624338624341*G0_14_0_0_18_1_1 - 2.84444444444444*G0_14_0_0_19_0_1 - 1.16825396825397*G0_14_0_0_19_1_1 - 1.28677248677249*G0_14_0_0_20_1_1 + 0.812698412698412*G0_14_0_0_21_0_1 + 0.474074074074073*G0_14_0_0_21_1_1 - 2.84444444444444*G0_14_0_0_22_0_1 - 1.67619047619047*G0_14_0_0_22_1_1 + 1.28677248677248*G0_14_0_0_23_1_1 + 0.135449735449736*G0_14_0_0_24_0_1 + 0.474074074074076*G0_14_0_0_24_1_1 - 0.101587301587304*G0_14_0_0_25_0_1 - 0.0507936507936526*G0_14_0_0_25_1_1 + 0.135449735449737*G0_14_0_0_26_0_1 - 0.338624338624339*G0_14_0_0_26_1_1 - 0.812698412698412*G0_14_0_0_27_0_1 - 2.03174603174603*G0_14_0_0_27_1_1 - 0.812698412698418*G0_14_0_0_28_0_1 + 1.21904761904762*G0_14_0_0_28_1_1 + 5.68888888888888*G0_14_0_0_29_0_1 + 2.84444444444444*G0_14_0_0_29_1_1 - 0.0846560846560835*G0_14_1_0_0_0_0 - 0.0846560846560833*G0_14_1_0_0_1_0 - 0.0846560846560851*G0_14_1_0_2_1_0 + 0.338624338624343*G0_14_1_0_3_0_0 - 1.16825396825397*G0_14_1_0_4_0_0 + 0.406349206349206*G0_14_1_0_4_1_0 - 1.28677248677249*G0_14_1_0_5_0_0 - 2.43809523809524*G0_14_1_0_5_1_0 + 0.474074074074071*G0_14_1_0_6_0_0 + 0.135449735449731*G0_14_1_0_6_1_0 - 1.67619047619047*G0_14_1_0_7_0_0 - 0.101587301587297*G0_14_1_0_7_1_0 + 1.28677248677249*G0_14_1_0_8_0_0 + 0.135449735449733*G0_14_1_0_8_1_0 + 0.474074074074072*G0_14_1_0_9_0_0 + 0.812698412698413*G0_14_1_0_9_1_0 - 0.0507936507936525*G0_14_1_0_10_0_0 + 0.406349206349206*G0_14_1_0_10_1_0 - 0.338624338624336*G0_14_1_0_11_0_0 - 2.03174603174602*G0_14_1_0_12_0_0 - 4.06349206349206*G0_14_1_0_12_1_0 + 1.21904761904761*G0_14_1_0_13_0_0 - 0.812698412698411*G0_14_1_0_13_1_0 + 2.84444444444444*G0_14_1_0_14_0_0 + 5.68888888888889*G0_14_1_0_14_1_0 - 0.0846560846560835*G0_14_1_0_15_0_1 - 0.0846560846560833*G0_14_1_0_15_1_1 - 0.0846560846560851*G0_14_1_0_17_1_1 + 0.338624338624343*G0_14_1_0_18_0_1 - 1.16825396825397*G0_14_1_0_19_0_1 + 0.406349206349206*G0_14_1_0_19_1_1 - 1.28677248677249*G0_14_1_0_20_0_1 - 2.43809523809524*G0_14_1_0_20_1_1 + 0.474074074074071*G0_14_1_0_21_0_1 + 0.135449735449731*G0_14_1_0_21_1_1 - 1.67619047619047*G0_14_1_0_22_0_1 - 0.101587301587297*G0_14_1_0_22_1_1 + 1.28677248677249*G0_14_1_0_23_0_1 + 0.135449735449733*G0_14_1_0_23_1_1 + 0.474074074074072*G0_14_1_0_24_0_1 + 0.812698412698413*G0_14_1_0_24_1_1 - 0.0507936507936525*G0_14_1_0_25_0_1 + 0.406349206349206*G0_14_1_0_25_1_1 - 0.338624338624336*G0_14_1_0_26_0_1 - 2.03174603174602*G0_14_1_0_27_0_1 - 4.06349206349206*G0_14_1_0_27_1_1 + 1.21904761904761*G0_14_1_0_28_0_1 - 0.812698412698411*G0_14_1_0_28_1_1 + 2.84444444444444*G0_14_1_0_29_0_1 + 5.68888888888889*G0_14_1_0_29_1_1 + 0.373015873015873*G0_15_0_1_0_0_0 + 0.373015873015873*G0_15_0_1_0_1_0 + 0.0566137566137565*G0_15_0_1_1_0_0 + 0.0566137566137566*G0_15_0_1_2_1_0 + 0.0423280423280418*G0_15_0_1_3_0_0 + 0.042328042328042*G0_15_0_1_3_1_0 + 0.0423280423280421*G0_15_0_1_4_0_0 + 0.0423280423280422*G0_15_0_1_4_1_0 + 0.0423280423280421*G0_15_0_1_5_0_0 + 0.042328042328042*G0_15_0_1_5_1_0 + 0.126984126984127*G0_15_0_1_6_0_0 - 0.651851851851852*G0_15_0_1_6_1_0 + 0.0423280423280421*G0_15_0_1_7_0_0 + 0.467724867724868*G0_15_0_1_7_1_0 - 0.042328042328042*G0_15_0_1_8_0_0 - 0.245502645502646*G0_15_0_1_8_1_0 - 0.651851851851852*G0_15_0_1_9_0_0 + 0.126984126984127*G0_15_0_1_9_1_0 + 0.467724867724868*G0_15_0_1_10_0_0 + 0.0423280423280423*G0_15_0_1_10_1_0 - 0.245502645502646*G0_15_0_1_11_0_0 - 0.0423280423280419*G0_15_0_1_11_1_0 - 0.0846560846560848*G0_15_0_1_12_0_0 - 0.084656084656085*G0_15_0_1_12_1_0 - 0.0846560846560835*G0_15_0_1_13_0_0 - 0.0846560846560848*G0_15_0_1_13_1_0 - 0.0846560846560841*G0_15_0_1_14_0_0 - 0.0846560846560835*G0_15_0_1_14_1_0 + 0.373015873015873*G0_15_0_1_15_0_1 + 0.373015873015873*G0_15_0_1_15_1_1 + 0.0566137566137565*G0_15_0_1_16_0_1 + 0.0566137566137566*G0_15_0_1_17_1_1 + 0.0423280423280418*G0_15_0_1_18_0_1 + 0.042328042328042*G0_15_0_1_18_1_1 + 0.0423280423280421*G0_15_0_1_19_0_1 + 0.0423280423280422*G0_15_0_1_19_1_1 + 0.0423280423280421*G0_15_0_1_20_0_1 + 0.042328042328042*G0_15_0_1_20_1_1 + 0.126984126984127*G0_15_0_1_21_0_1 - 0.651851851851852*G0_15_0_1_21_1_1 + 0.0423280423280421*G0_15_0_1_22_0_1 + 0.467724867724868*G0_15_0_1_22_1_1 - 0.042328042328042*G0_15_0_1_23_0_1 - 0.245502645502646*G0_15_0_1_23_1_1 - 0.651851851851852*G0_15_0_1_24_0_1 + 0.126984126984127*G0_15_0_1_24_1_1 + 0.467724867724868*G0_15_0_1_25_0_1 + 0.0423280423280423*G0_15_0_1_25_1_1 - 0.245502645502646*G0_15_0_1_26_0_1 - 0.0423280423280419*G0_15_0_1_26_1_1 - 0.0846560846560848*G0_15_0_1_27_0_1 - 0.084656084656085*G0_15_0_1_27_1_1 - 0.0846560846560835*G0_15_0_1_28_0_1 - 0.0846560846560848*G0_15_0_1_28_1_1 - 0.0846560846560841*G0_15_0_1_29_0_1 - 0.0846560846560835*G0_15_0_1_29_1_1 + 0.373015873015873*G0_15_1_1_0_0_0 + 0.373015873015873*G0_15_1_1_0_1_0 + 0.0566137566137567*G0_15_1_1_1_0_0 + 0.0566137566137567*G0_15_1_1_2_1_0 + 0.0423280423280421*G0_15_1_1_3_0_0 + 0.0423280423280425*G0_15_1_1_3_1_0 + 0.0423280423280417*G0_15_1_1_4_0_0 + 0.0423280423280419*G0_15_1_1_4_1_0 + 0.0423280423280423*G0_15_1_1_5_0_0 + 0.0423280423280419*G0_15_1_1_5_1_0 + 0.126984126984126*G0_15_1_1_6_0_0 - 0.651851851851852*G0_15_1_1_6_1_0 + 0.0423280423280421*G0_15_1_1_7_0_0 + 0.467724867724868*G0_15_1_1_7_1_0 - 0.0423280423280422*G0_15_1_1_8_0_0 - 0.245502645502646*G0_15_1_1_8_1_0 - 0.651851851851852*G0_15_1_1_9_0_0 + 0.126984126984127*G0_15_1_1_9_1_0 + 0.467724867724868*G0_15_1_1_10_0_0 + 0.0423280423280427*G0_15_1_1_10_1_0 - 0.245502645502646*G0_15_1_1_11_0_0 - 0.0423280423280424*G0_15_1_1_11_1_0 - 0.0846560846560841*G0_15_1_1_12_0_0 - 0.0846560846560852*G0_15_1_1_12_1_0 - 0.084656084656084*G0_15_1_1_13_0_0 - 0.0846560846560849*G0_15_1_1_13_1_0 - 0.0846560846560838*G0_15_1_1_14_0_0 - 0.0846560846560832*G0_15_1_1_14_1_0 + 0.373015873015873*G0_15_1_1_15_0_1 + 0.373015873015873*G0_15_1_1_15_1_1 + 0.0566137566137567*G0_15_1_1_16_0_1 + 0.0566137566137567*G0_15_1_1_17_1_1 + 0.0423280423280421*G0_15_1_1_18_0_1 + 0.0423280423280425*G0_15_1_1_18_1_1 + 0.0423280423280417*G0_15_1_1_19_0_1 + 0.0423280423280419*G0_15_1_1_19_1_1 + 0.0423280423280423*G0_15_1_1_20_0_1 + 0.0423280423280419*G0_15_1_1_20_1_1 + 0.126984126984126*G0_15_1_1_21_0_1 - 0.651851851851852*G0_15_1_1_21_1_1 + 0.0423280423280421*G0_15_1_1_22_0_1 + 0.467724867724868*G0_15_1_1_22_1_1 - 0.0423280423280422*G0_15_1_1_23_0_1 - 0.245502645502646*G0_15_1_1_23_1_1 - 0.651851851851852*G0_15_1_1_24_0_1 + 0.126984126984127*G0_15_1_1_24_1_1 + 0.467724867724868*G0_15_1_1_25_0_1 + 0.0423280423280427*G0_15_1_1_25_1_1 - 0.245502645502646*G0_15_1_1_26_0_1 - 0.0423280423280424*G0_15_1_1_26_1_1 - 0.0846560846560841*G0_15_1_1_27_0_1 - 0.0846560846560852*G0_15_1_1_27_1_1 - 0.084656084656084*G0_15_1_1_28_0_1 - 0.0846560846560849*G0_15_1_1_28_1_1 - 0.0846560846560838*G0_15_1_1_29_0_1 - 0.0846560846560832*G0_15_1_1_29_1_1 + 0.0566137566137565*G0_16_0_1_0_0_0 + 0.0566137566137567*G0_16_0_1_0_1_0 + 0.373015873015873*G0_16_0_1_1_0_0 - 0.0566137566137567*G0_16_0_1_2_1_0 + 0.126984126984127*G0_16_0_1_3_0_0 + 0.778835978835978*G0_16_0_1_3_1_0 + 0.0423280423280423*G0_16_0_1_4_0_0 - 0.425396825396825*G0_16_0_1_4_1_0 - 0.0423280423280424*G0_16_0_1_5_0_0 + 0.203174603174603*G0_16_0_1_5_1_0 + 0.0423280423280421*G0_16_0_1_6_0_0 + 0.0423280423280423*G0_16_0_1_7_0_0 + 0.0423280423280424*G0_16_0_1_8_0_0 - 0.245502645502645*G0_16_0_1_9_0_0 - 0.203174603174603*G0_16_0_1_9_1_0 + 0.467724867724867*G0_16_0_1_10_0_0 + 0.425396825396824*G0_16_0_1_10_1_0 - 0.651851851851851*G0_16_0_1_11_0_0 - 0.778835978835978*G0_16_0_1_11_1_0 - 0.084656084656084*G0_16_0_1_12_0_0 - 0.0846560846560853*G0_16_0_1_13_0_0 - 0.0846560846560846*G0_16_0_1_14_0_0 + 0.0566137566137565*G0_16_0_1_15_0_1 + 0.0566137566137567*G0_16_0_1_15_1_1 + 0.373015873015873*G0_16_0_1_16_0_1 - 0.0566137566137567*G0_16_0_1_17_1_1 + 0.126984126984127*G0_16_0_1_18_0_1 + 0.778835978835978*G0_16_0_1_18_1_1 + 0.0423280423280423*G0_16_0_1_19_0_1 - 0.425396825396825*G0_16_0_1_19_1_1 - 0.0423280423280424*G0_16_0_1_20_0_1 + 0.203174603174603*G0_16_0_1_20_1_1 + 0.0423280423280421*G0_16_0_1_21_0_1 + 0.0423280423280423*G0_16_0_1_22_0_1 + 0.0423280423280424*G0_16_0_1_23_0_1 - 0.245502645502645*G0_16_0_1_24_0_1 - 0.203174603174603*G0_16_0_1_24_1_1 + 0.467724867724867*G0_16_0_1_25_0_1 + 0.425396825396824*G0_16_0_1_25_1_1 - 0.651851851851851*G0_16_0_1_26_0_1 - 0.778835978835978*G0_16_0_1_26_1_1 - 0.084656084656084*G0_16_0_1_27_0_1 - 0.0846560846560853*G0_16_0_1_28_0_1 - 0.0846560846560846*G0_16_0_1_29_0_1 + 0.0566137566137566*G0_17_1_1_0_0_0 + 0.0566137566137567*G0_17_1_1_0_1_0 - 0.0566137566137566*G0_17_1_1_1_0_0 + 0.373015873015873*G0_17_1_1_2_1_0 + 0.203174603174602*G0_17_1_1_3_0_0 - 0.0423280423280427*G0_17_1_1_3_1_0 - 0.425396825396825*G0_17_1_1_4_0_0 + 0.0423280423280416*G0_17_1_1_4_1_0 + 0.77883597883598*G0_17_1_1_5_0_0 + 0.126984126984128*G0_17_1_1_5_1_0 - 0.203174603174603*G0_17_1_1_6_0_0 - 0.245502645502646*G0_17_1_1_6_1_0 + 0.425396825396825*G0_17_1_1_7_0_0 + 0.467724867724867*G0_17_1_1_7_1_0 - 0.77883597883598*G0_17_1_1_8_0_0 - 0.651851851851852*G0_17_1_1_8_1_0 + 0.0423280423280423*G0_17_1_1_9_1_0 + 0.042328042328043*G0_17_1_1_10_1_0 + 0.0423280423280428*G0_17_1_1_11_1_0 - 0.084656084656085*G0_17_1_1_12_1_0 - 0.0846560846560848*G0_17_1_1_13_1_0 - 0.0846560846560851*G0_17_1_1_14_1_0 + 0.0566137566137566*G0_17_1_1_15_0_1 + 0.0566137566137567*G0_17_1_1_15_1_1 - 0.0566137566137566*G0_17_1_1_16_0_1 + 0.373015873015873*G0_17_1_1_17_1_1 + 0.203174603174602*G0_17_1_1_18_0_1 - 0.0423280423280427*G0_17_1_1_18_1_1 - 0.425396825396825*G0_17_1_1_19_0_1 + 0.0423280423280416*G0_17_1_1_19_1_1 + 0.77883597883598*G0_17_1_1_20_0_1 + 0.126984126984128*G0_17_1_1_20_1_1 - 0.203174603174603*G0_17_1_1_21_0_1 - 0.245502645502646*G0_17_1_1_21_1_1 + 0.425396825396825*G0_17_1_1_22_0_1 + 0.467724867724867*G0_17_1_1_22_1_1 - 0.77883597883598*G0_17_1_1_23_0_1 - 0.651851851851852*G0_17_1_1_23_1_1 + 0.0423280423280423*G0_17_1_1_24_1_1 + 0.042328042328043*G0_17_1_1_25_1_1 + 0.0423280423280428*G0_17_1_1_26_1_1 - 0.084656084656085*G0_17_1_1_27_1_1 - 0.0846560846560848*G0_17_1_1_28_1_1 - 0.0846560846560851*G0_17_1_1_29_1_1 + 0.0423280423280418*G0_18_0_1_0_0_0 + 0.0423280423280421*G0_18_0_1_0_1_0 + 0.126984126984127*G0_18_0_1_1_0_0 + 0.203174603174602*G0_18_0_1_2_1_0 + 1.82857142857143*G0_18_0_1_3_0_0 + 1.04973544973545*G0_18_0_1_3_1_0 - 0.660317460317462*G0_18_0_1_4_0_0 + 0.821164021164017*G0_18_0_1_4_1_0 + 0.338624338624337*G0_18_0_1_5_0_0 - 0.440211640211641*G0_18_0_1_5_1_0 - 0.203174603174603*G0_18_0_1_6_0_0 - 0.101587301587302*G0_18_0_1_6_1_0 - 0.152380952380956*G0_18_0_1_7_0_0 - 0.0423280423280443*G0_18_0_1_7_1_0 - 0.338624338624337*G0_18_0_1_8_0_0 - 0.101587301587299*G0_18_0_1_8_1_0 - 0.135449735449733*G0_18_0_1_9_0_0 - 0.237037037037035*G0_18_0_1_9_1_0 + 0.1015873015873*G0_18_0_1_10_0_0 + 0.465608465608465*G0_18_0_1_10_1_0 - 0.135449735449735*G0_18_0_1_11_0_0 - 1.04973544973545*G0_18_0_1_11_1_0 + 0.812698412698412*G0_18_0_1_12_0_0 + 0.338624338624334*G0_18_0_1_12_1_0 - 2.43809523809524*G0_18_0_1_13_0_0 - 1.28677248677248*G0_18_0_1_13_1_0 + 0.812698412698417*G0_18_0_1_14_0_0 + 0.338624338624343*G0_18_0_1_14_1_0 + 0.0423280423280418*G0_18_0_1_15_0_1 + 0.0423280423280421*G0_18_0_1_15_1_1 + 0.126984126984127*G0_18_0_1_16_0_1 + 0.203174603174602*G0_18_0_1_17_1_1 + 1.82857142857143*G0_18_0_1_18_0_1 + 1.04973544973545*G0_18_0_1_18_1_1 - 0.660317460317462*G0_18_0_1_19_0_1 + 0.821164021164017*G0_18_0_1_19_1_1 + 0.338624338624337*G0_18_0_1_20_0_1 - 0.440211640211641*G0_18_0_1_20_1_1 - 0.203174603174603*G0_18_0_1_21_0_1 - 0.101587301587302*G0_18_0_1_21_1_1 - 0.152380952380956*G0_18_0_1_22_0_1 - 0.0423280423280443*G0_18_0_1_22_1_1 - 0.338624338624337*G0_18_0_1_23_0_1 - 0.101587301587299*G0_18_0_1_23_1_1 - 0.135449735449733*G0_18_0_1_24_0_1 - 0.237037037037035*G0_18_0_1_24_1_1 + 0.1015873015873*G0_18_0_1_25_0_1 + 0.465608465608465*G0_18_0_1_25_1_1 - 0.135449735449735*G0_18_0_1_26_0_1 - 1.04973544973545*G0_18_0_1_26_1_1 + 0.812698412698412*G0_18_0_1_27_0_1 + 0.338624338624334*G0_18_0_1_27_1_1 - 2.43809523809524*G0_18_0_1_28_0_1 - 1.28677248677248*G0_18_0_1_28_1_1 + 0.812698412698417*G0_18_0_1_29_0_1 + 0.338624338624343*G0_18_0_1_29_1_1 + 0.042328042328042*G0_18_1_1_0_0_0 + 0.0423280423280425*G0_18_1_1_0_1_0 + 0.778835978835978*G0_18_1_1_1_0_0 - 0.0423280423280427*G0_18_1_1_2_1_0 + 1.04973544973545*G0_18_1_1_3_0_0 + 2.0994708994709*G0_18_1_1_3_1_0 - 0.296296296296297*G0_18_1_1_4_0_0 - 0.761904761904764*G0_18_1_1_4_1_0 + 0.101587301587301*G0_18_1_1_5_0_0 + 0.338624338624338*G0_18_1_1_5_1_0 - 0.101587301587301*G0_18_1_1_6_0_0 - 0.042328042328044*G0_18_1_1_7_0_0 - 0.101587301587301*G0_18_1_1_8_0_0 - 0.237037037037036*G0_18_1_1_9_0_0 - 0.338624338624337*G0_18_1_1_9_1_0 + 0.465608465608464*G0_18_1_1_10_0_0 + 0.761904761904759*G0_18_1_1_10_1_0 - 1.04973544973545*G0_18_1_1_11_0_0 - 2.0994708994709*G0_18_1_1_11_1_0 + 0.338624338624338*G0_18_1_1_12_0_0 - 1.28677248677249*G0_18_1_1_13_0_0 + 0.338624338624341*G0_18_1_1_14_0_0 + 0.042328042328042*G0_18_1_1_15_0_1 + 0.0423280423280425*G0_18_1_1_15_1_1 + 0.778835978835978*G0_18_1_1_16_0_1 - 0.0423280423280427*G0_18_1_1_17_1_1 + 1.04973544973545*G0_18_1_1_18_0_1 + 2.0994708994709*G0_18_1_1_18_1_1 - 0.296296296296297*G0_18_1_1_19_0_1 - 0.761904761904764*G0_18_1_1_19_1_1 + 0.101587301587301*G0_18_1_1_20_0_1 + 0.338624338624338*G0_18_1_1_20_1_1 - 0.101587301587301*G0_18_1_1_21_0_1 - 0.042328042328044*G0_18_1_1_22_0_1 - 0.101587301587301*G0_18_1_1_23_0_1 - 0.237037037037036*G0_18_1_1_24_0_1 - 0.338624338624337*G0_18_1_1_24_1_1 + 0.465608465608464*G0_18_1_1_25_0_1 + 0.761904761904759*G0_18_1_1_25_1_1 - 1.04973544973545*G0_18_1_1_26_0_1 - 2.0994708994709*G0_18_1_1_26_1_1 + 0.338624338624338*G0_18_1_1_27_0_1 - 1.28677248677249*G0_18_1_1_28_0_1 + 0.338624338624341*G0_18_1_1_29_0_1 + 0.0423280423280421*G0_19_0_1_0_0_0 + 0.0423280423280417*G0_19_0_1_0_1_0 + 0.0423280423280423*G0_19_0_1_1_0_0 - 0.425396825396825*G0_19_0_1_2_1_0 - 0.660317460317462*G0_19_0_1_3_0_0 - 0.296296296296297*G0_19_0_1_3_1_0 + 2.45079365079365*G0_19_0_1_4_0_0 + 0.97142857142857*G0_19_0_1_4_1_0 - 0.761904761904761*G0_19_0_1_5_0_0 + 0.821164021164023*G0_19_0_1_5_1_0 - 0.152380952380952*G0_19_0_1_6_0_0 - 0.11005291005291*G0_19_0_1_6_1_0 + 0.393650793650795*G0_19_0_1_7_0_0 + 0.196825396825397*G0_19_0_1_7_1_0 + 0.761904761904761*G0_19_0_1_8_0_0 + 0.296296296296296*G0_19_0_1_8_1_0 - 0.0677248677248682*G0_19_0_1_9_0_0 - 0.110052910052911*G0_19_0_1_9_1_0 + 0.050793650793651*G0_19_0_1_10_0_0 + 0.196825396825396*G0_19_0_1_10_1_0 - 0.0677248677248674*G0_19_0_1_11_0_0 + 0.296296296296296*G0_19_0_1_11_1_0 + 0.406349206349208*G0_19_0_1_12_0_0 + 0.457142857142862*G0_19_0_1_12_1_0 + 0.406349206349207*G0_19_0_1_13_0_0 - 1.16825396825396*G0_19_0_1_13_1_0 - 2.84444444444444*G0_19_0_1_14_0_0 - 1.16825396825397*G0_19_0_1_14_1_0 + 0.0423280423280421*G0_19_0_1_15_0_1 + 0.0423280423280417*G0_19_0_1_15_1_1 + 0.0423280423280423*G0_19_0_1_16_0_1 - 0.425396825396825*G0_19_0_1_17_1_1 - 0.660317460317462*G0_19_0_1_18_0_1 - 0.296296296296297*G0_19_0_1_18_1_1 + 2.45079365079365*G0_19_0_1_19_0_1 + 0.97142857142857*G0_19_0_1_19_1_1 - 0.761904761904761*G0_19_0_1_20_0_1 + 0.821164021164023*G0_19_0_1_20_1_1 - 0.152380952380952*G0_19_0_1_21_0_1 - 0.11005291005291*G0_19_0_1_21_1_1 + 0.393650793650795*G0_19_0_1_22_0_1 + 0.196825396825397*G0_19_0_1_22_1_1 + 0.761904761904761*G0_19_0_1_23_0_1 + 0.296296296296296*G0_19_0_1_23_1_1 - 0.0677248677248682*G0_19_0_1_24_0_1 - 0.110052910052911*G0_19_0_1_24_1_1 + 0.050793650793651*G0_19_0_1_25_0_1 + 0.196825396825396*G0_19_0_1_25_1_1 - 0.0677248677248674*G0_19_0_1_26_0_1 + 0.296296296296296*G0_19_0_1_26_1_1 + 0.406349206349208*G0_19_0_1_27_0_1 + 0.457142857142862*G0_19_0_1_27_1_1 + 0.406349206349207*G0_19_0_1_28_0_1 - 1.16825396825396*G0_19_0_1_28_1_1 - 2.84444444444444*G0_19_0_1_29_0_1 - 1.16825396825397*G0_19_0_1_29_1_1 + 0.0423280423280422*G0_19_1_1_0_0_0 + 0.0423280423280419*G0_19_1_1_0_1_0 - 0.425396825396825*G0_19_1_1_1_0_0 + 0.0423280423280416*G0_19_1_1_2_1_0 + 0.821164021164017*G0_19_1_1_3_0_0 - 0.761904761904764*G0_19_1_1_3_1_0 + 0.97142857142857*G0_19_1_1_4_0_0 + 2.45079365079365*G0_19_1_1_4_1_0 - 0.296296296296298*G0_19_1_1_5_0_0 - 0.66031746031746*G0_19_1_1_5_1_0 - 0.11005291005291*G0_19_1_1_6_0_0 - 0.0677248677248674*G0_19_1_1_6_1_0 + 0.196825396825396*G0_19_1_1_7_0_0 + 0.0507936507936494*G0_19_1_1_7_1_0 + 0.296296296296298*G0_19_1_1_8_0_0 - 0.0677248677248661*G0_19_1_1_8_1_0 - 0.11005291005291*G0_19_1_1_9_0_0 - 0.152380952380951*G0_19_1_1_9_1_0 + 0.196825396825396*G0_19_1_1_10_0_0 + 0.393650793650794*G0_19_1_1_10_1_0 + 0.296296296296297*G0_19_1_1_11_0_0 + 0.761904761904763*G0_19_1_1_11_1_0 + 0.457142857142857*G0_19_1_1_12_0_0 + 0.406349206349205*G0_19_1_1_12_1_0 - 1.16825396825396*G0_19_1_1_13_0_0 - 2.84444444444444*G0_19_1_1_13_1_0 - 1.16825396825397*G0_19_1_1_14_0_0 + 0.406349206349206*G0_19_1_1_14_1_0 + 0.0423280423280422*G0_19_1_1_15_0_1 + 0.0423280423280419*G0_19_1_1_15_1_1 - 0.425396825396825*G0_19_1_1_16_0_1 + 0.0423280423280416*G0_19_1_1_17_1_1 + 0.821164021164017*G0_19_1_1_18_0_1 - 0.761904761904764*G0_19_1_1_18_1_1 + 0.97142857142857*G0_19_1_1_19_0_1 + 2.45079365079365*G0_19_1_1_19_1_1 - 0.296296296296298*G0_19_1_1_20_0_1 - 0.66031746031746*G0_19_1_1_20_1_1 - 0.11005291005291*G0_19_1_1_21_0_1 - 0.0677248677248674*G0_19_1_1_21_1_1 + 0.196825396825396*G0_19_1_1_22_0_1 + 0.0507936507936494*G0_19_1_1_22_1_1 + 0.296296296296298*G0_19_1_1_23_0_1 - 0.0677248677248661*G0_19_1_1_23_1_1 - 0.11005291005291*G0_19_1_1_24_0_1 - 0.152380952380951*G0_19_1_1_24_1_1 + 0.196825396825396*G0_19_1_1_25_0_1 + 0.393650793650794*G0_19_1_1_25_1_1 + 0.296296296296297*G0_19_1_1_26_0_1 + 0.761904761904763*G0_19_1_1_26_1_1 + 0.457142857142857*G0_19_1_1_27_0_1 + 0.406349206349205*G0_19_1_1_27_1_1 - 1.16825396825396*G0_19_1_1_28_0_1 - 2.84444444444444*G0_19_1_1_28_1_1 - 1.16825396825397*G0_19_1_1_29_0_1 + 0.406349206349206*G0_19_1_1_29_1_1 + 0.0423280423280421*G0_20_0_1_0_0_0 + 0.0423280423280423*G0_20_0_1_0_1_0 - 0.0423280423280424*G0_20_0_1_1_0_0 + 0.77883597883598*G0_20_0_1_2_1_0 + 0.338624338624337*G0_20_0_1_3_0_0 + 0.101587301587301*G0_20_0_1_3_1_0 - 0.761904761904761*G0_20_0_1_4_0_0 - 0.296296296296298*G0_20_0_1_4_1_0 + 2.0994708994709*G0_20_0_1_5_0_0 + 1.04973544973545*G0_20_0_1_5_1_0 - 0.338624338624338*G0_20_0_1_6_0_0 - 0.237037037037037*G0_20_0_1_6_1_0 + 0.761904761904763*G0_20_0_1_7_0_0 + 0.465608465608465*G0_20_0_1_7_1_0 - 2.0994708994709*G0_20_0_1_8_0_0 - 1.04973544973545*G0_20_0_1_8_1_0 - 0.101587301587302*G0_20_0_1_9_1_0 - 0.0423280423280415*G0_20_0_1_10_1_0 - 0.101587301587301*G0_20_0_1_11_1_0 + 0.338624338624339*G0_20_0_1_12_1_0 + 0.338624338624338*G0_20_0_1_13_1_0 - 1.28677248677249*G0_20_0_1_14_1_0 + 0.0423280423280421*G0_20_0_1_15_0_1 + 0.0423280423280423*G0_20_0_1_15_1_1 - 0.0423280423280424*G0_20_0_1_16_0_1 + 0.77883597883598*G0_20_0_1_17_1_1 + 0.338624338624337*G0_20_0_1_18_0_1 + 0.101587301587301*G0_20_0_1_18_1_1 - 0.761904761904761*G0_20_0_1_19_0_1 - 0.296296296296298*G0_20_0_1_19_1_1 + 2.0994708994709*G0_20_0_1_20_0_1 + 1.04973544973545*G0_20_0_1_20_1_1 - 0.338624338624338*G0_20_0_1_21_0_1 - 0.237037037037037*G0_20_0_1_21_1_1 + 0.761904761904763*G0_20_0_1_22_0_1 + 0.465608465608465*G0_20_0_1_22_1_1 - 2.0994708994709*G0_20_0_1_23_0_1 - 1.04973544973545*G0_20_0_1_23_1_1 - 0.101587301587302*G0_20_0_1_24_1_1 - 0.0423280423280415*G0_20_0_1_25_1_1 - 0.101587301587301*G0_20_0_1_26_1_1 + 0.338624338624339*G0_20_0_1_27_1_1 + 0.338624338624338*G0_20_0_1_28_1_1 - 1.28677248677249*G0_20_0_1_29_1_1 + 0.042328042328042*G0_20_1_1_0_0_0 + 0.0423280423280419*G0_20_1_1_0_1_0 + 0.203174603174603*G0_20_1_1_1_0_0 + 0.126984126984128*G0_20_1_1_2_1_0 - 0.440211640211642*G0_20_1_1_3_0_0 + 0.338624338624338*G0_20_1_1_3_1_0 + 0.821164021164022*G0_20_1_1_4_0_0 - 0.66031746031746*G0_20_1_1_4_1_0 + 1.04973544973545*G0_20_1_1_5_0_0 + 1.82857142857143*G0_20_1_1_5_1_0 - 0.237037037037037*G0_20_1_1_6_0_0 - 0.135449735449735*G0_20_1_1_6_1_0 + 0.465608465608468*G0_20_1_1_7_0_0 + 0.101587301587302*G0_20_1_1_7_1_0 - 1.04973544973545*G0_20_1_1_8_0_0 - 0.135449735449737*G0_20_1_1_8_1_0 - 0.101587301587301*G0_20_1_1_9_0_0 - 0.203174603174604*G0_20_1_1_9_1_0 - 0.0423280423280416*G0_20_1_1_10_0_0 - 0.152380952380953*G0_20_1_1_10_1_0 - 0.101587301587302*G0_20_1_1_11_0_0 - 0.338624338624338*G0_20_1_1_11_1_0 + 0.338624338624338*G0_20_1_1_12_0_0 + 0.812698412698416*G0_20_1_1_12_1_0 + 0.33862433862434*G0_20_1_1_13_0_0 + 0.812698412698412*G0_20_1_1_13_1_0 - 1.28677248677249*G0_20_1_1_14_0_0 - 2.43809523809524*G0_20_1_1_14_1_0 + 0.042328042328042*G0_20_1_1_15_0_1 + 0.0423280423280419*G0_20_1_1_15_1_1 + 0.203174603174603*G0_20_1_1_16_0_1 + 0.126984126984128*G0_20_1_1_17_1_1 - 0.440211640211642*G0_20_1_1_18_0_1 + 0.338624338624338*G0_20_1_1_18_1_1 + 0.821164021164022*G0_20_1_1_19_0_1 - 0.66031746031746*G0_20_1_1_19_1_1 + 1.04973544973545*G0_20_1_1_20_0_1 + 1.82857142857143*G0_20_1_1_20_1_1 - 0.237037037037037*G0_20_1_1_21_0_1 - 0.135449735449735*G0_20_1_1_21_1_1 + 0.465608465608468*G0_20_1_1_22_0_1 + 0.101587301587302*G0_20_1_1_22_1_1 - 1.04973544973545*G0_20_1_1_23_0_1 - 0.135449735449737*G0_20_1_1_23_1_1 - 0.101587301587301*G0_20_1_1_24_0_1 - 0.203174603174604*G0_20_1_1_24_1_1 - 0.0423280423280416*G0_20_1_1_25_0_1 - 0.152380952380953*G0_20_1_1_25_1_1 - 0.101587301587302*G0_20_1_1_26_0_1 - 0.338624338624338*G0_20_1_1_26_1_1 + 0.338624338624338*G0_20_1_1_27_0_1 + 0.812698412698416*G0_20_1_1_27_1_1 + 0.33862433862434*G0_20_1_1_28_0_1 + 0.812698412698412*G0_20_1_1_28_1_1 - 1.28677248677249*G0_20_1_1_29_0_1 - 2.43809523809524*G0_20_1_1_29_1_1 + 0.126984126984127*G0_21_0_1_0_0_0 + 0.126984126984126*G0_21_0_1_0_1_0 + 0.0423280423280421*G0_21_0_1_1_0_0 - 0.203174603174603*G0_21_0_1_2_1_0 - 0.203174603174603*G0_21_0_1_3_0_0 - 0.101587301587301*G0_21_0_1_3_1_0 - 0.152380952380952*G0_21_0_1_4_0_0 - 0.11005291005291*G0_21_0_1_4_1_0 - 0.338624338624338*G0_21_0_1_5_0_0 - 0.237037037037037*G0_21_0_1_5_1_0 + 1.82857142857143*G0_21_0_1_6_0_0 + 0.778835978835979*G0_21_0_1_6_1_0 - 0.66031746031746*G0_21_0_1_7_0_0 - 1.48148148148148*G0_21_0_1_7_1_0 + 0.338624338624339*G0_21_0_1_8_0_0 + 0.778835978835979*G0_21_0_1_8_1_0 - 0.135449735449736*G0_21_0_1_9_0_0 + 0.914285714285713*G0_21_0_1_9_1_0 + 0.101587301587303*G0_21_0_1_10_0_0 - 0.364021164021164*G0_21_0_1_10_1_0 - 0.135449735449735*G0_21_0_1_11_0_0 + 0.101587301587301*G0_21_0_1_11_1_0 - 2.43809523809524*G0_21_0_1_12_0_0 - 1.15132275132275*G0_21_0_1_12_1_0 + 0.812698412698413*G0_21_0_1_13_0_0 + 0.474074074074074*G0_21_0_1_13_1_0 + 0.812698412698412*G0_21_0_1_14_0_0 + 0.474074074074071*G0_21_0_1_14_1_0 + 0.126984126984127*G0_21_0_1_15_0_1 + 0.126984126984126*G0_21_0_1_15_1_1 + 0.0423280423280421*G0_21_0_1_16_0_1 - 0.203174603174603*G0_21_0_1_17_1_1 - 0.203174603174603*G0_21_0_1_18_0_1 - 0.101587301587301*G0_21_0_1_18_1_1 - 0.152380952380952*G0_21_0_1_19_0_1 - 0.11005291005291*G0_21_0_1_19_1_1 - 0.338624338624338*G0_21_0_1_20_0_1 - 0.237037037037037*G0_21_0_1_20_1_1 + 1.82857142857143*G0_21_0_1_21_0_1 + 0.778835978835979*G0_21_0_1_21_1_1 - 0.66031746031746*G0_21_0_1_22_0_1 - 1.48148148148148*G0_21_0_1_22_1_1 + 0.338624338624339*G0_21_0_1_23_0_1 + 0.778835978835979*G0_21_0_1_23_1_1 - 0.135449735449736*G0_21_0_1_24_0_1 + 0.914285714285713*G0_21_0_1_24_1_1 + 0.101587301587303*G0_21_0_1_25_0_1 - 0.364021164021164*G0_21_0_1_25_1_1 - 0.135449735449735*G0_21_0_1_26_0_1 + 0.101587301587301*G0_21_0_1_26_1_1 - 2.43809523809524*G0_21_0_1_27_0_1 - 1.15132275132275*G0_21_0_1_27_1_1 + 0.812698412698413*G0_21_0_1_28_0_1 + 0.474074074074074*G0_21_0_1_28_1_1 + 0.812698412698412*G0_21_0_1_29_0_1 + 0.474074074074071*G0_21_0_1_29_1_1 - 0.651851851851852*G0_21_1_1_0_0_0 - 0.651851851851852*G0_21_1_1_0_1_0 - 0.245502645502646*G0_21_1_1_2_1_0 - 0.101587301587302*G0_21_1_1_3_0_0 - 0.11005291005291*G0_21_1_1_4_0_0 - 0.0677248677248674*G0_21_1_1_4_1_0 - 0.237037037037037*G0_21_1_1_5_0_0 - 0.135449735449735*G0_21_1_1_5_1_0 + 0.778835978835979*G0_21_1_1_6_0_0 + 1.82857142857143*G0_21_1_1_6_1_0 - 0.364021164021164*G0_21_1_1_7_0_0 - 1.94708994708995*G0_21_1_1_7_1_0 + 0.237037037037037*G0_21_1_1_8_0_0 + 1.01587301587302*G0_21_1_1_8_1_0 + 0.914285714285714*G0_21_1_1_9_0_0 - 0.135449735449736*G0_21_1_1_9_1_0 - 0.364021164021165*G0_21_1_1_10_0_0 - 0.0677248677248697*G0_21_1_1_10_1_0 + 0.101587301587303*G0_21_1_1_11_0_0 - 1.15132275132275*G0_21_1_1_12_0_0 + 0.135449735449738*G0_21_1_1_12_1_0 + 0.474074074074073*G0_21_1_1_13_0_0 + 0.135449735449738*G0_21_1_1_13_1_0 + 0.474074074074073*G0_21_1_1_14_0_0 + 0.135449735449731*G0_21_1_1_14_1_0 - 0.651851851851852*G0_21_1_1_15_0_1 - 0.651851851851852*G0_21_1_1_15_1_1 - 0.245502645502646*G0_21_1_1_17_1_1 - 0.101587301587302*G0_21_1_1_18_0_1 - 0.11005291005291*G0_21_1_1_19_0_1 - 0.0677248677248674*G0_21_1_1_19_1_1 - 0.237037037037037*G0_21_1_1_20_0_1 - 0.135449735449735*G0_21_1_1_20_1_1 + 0.778835978835979*G0_21_1_1_21_0_1 + 1.82857142857143*G0_21_1_1_21_1_1 - 0.364021164021164*G0_21_1_1_22_0_1 - 1.94708994708995*G0_21_1_1_22_1_1 + 0.237037037037037*G0_21_1_1_23_0_1 + 1.01587301587302*G0_21_1_1_23_1_1 + 0.914285714285714*G0_21_1_1_24_0_1 - 0.135449735449736*G0_21_1_1_24_1_1 - 0.364021164021165*G0_21_1_1_25_0_1 - 0.0677248677248697*G0_21_1_1_25_1_1 + 0.101587301587303*G0_21_1_1_26_0_1 - 1.15132275132275*G0_21_1_1_27_0_1 + 0.135449735449738*G0_21_1_1_27_1_1 + 0.474074074074073*G0_21_1_1_28_0_1 + 0.135449735449738*G0_21_1_1_28_1_1 + 0.474074074074073*G0_21_1_1_29_0_1 + 0.135449735449731*G0_21_1_1_29_1_1 + 0.0423280423280421*G0_22_0_1_0_0_0 + 0.0423280423280421*G0_22_0_1_0_1_0 + 0.0423280423280423*G0_22_0_1_1_0_0 + 0.425396825396825*G0_22_0_1_2_1_0 - 0.152380952380956*G0_22_0_1_3_0_0 - 0.042328042328044*G0_22_0_1_3_1_0 + 0.393650793650795*G0_22_0_1_4_0_0 + 0.196825396825396*G0_22_0_1_4_1_0 + 0.761904761904763*G0_22_0_1_5_0_0 + 0.465608465608468*G0_22_0_1_5_1_0 - 0.66031746031746*G0_22_0_1_6_0_0 - 0.364021164021164*G0_22_0_1_6_1_0 + 2.45079365079364*G0_22_0_1_7_0_0 + 1.47936507936508*G0_22_0_1_7_1_0 - 0.761904761904763*G0_22_0_1_8_0_0 - 1.58306878306878*G0_22_0_1_8_1_0 - 0.0677248677248674*G0_22_0_1_9_0_0 - 0.364021164021165*G0_22_0_1_9_1_0 + 0.0507936507936525*G0_22_0_1_10_0_0 - 0.146031746031743*G0_22_0_1_10_1_0 - 0.0677248677248697*G0_22_0_1_11_0_0 + 0.0423280423280434*G0_22_0_1_11_1_0 + 0.406349206349206*G0_22_0_1_12_0_0 + 1.57460317460317*G0_22_0_1_12_1_0 + 0.40634920634921*G0_22_0_1_13_0_0 - 0.0507936507936531*G0_22_0_1_13_1_0 - 2.84444444444444*G0_22_0_1_14_0_0 - 1.67619047619047*G0_22_0_1_14_1_0 + 0.0423280423280421*G0_22_0_1_15_0_1 + 0.0423280423280421*G0_22_0_1_15_1_1 + 0.0423280423280423*G0_22_0_1_16_0_1 + 0.425396825396825*G0_22_0_1_17_1_1 - 0.152380952380956*G0_22_0_1_18_0_1 - 0.042328042328044*G0_22_0_1_18_1_1 + 0.393650793650795*G0_22_0_1_19_0_1 + 0.196825396825396*G0_22_0_1_19_1_1 + 0.761904761904763*G0_22_0_1_20_0_1 + 0.465608465608468*G0_22_0_1_20_1_1 - 0.66031746031746*G0_22_0_1_21_0_1 - 0.364021164021164*G0_22_0_1_21_1_1 + 2.45079365079364*G0_22_0_1_22_0_1 + 1.47936507936508*G0_22_0_1_22_1_1 - 0.761904761904763*G0_22_0_1_23_0_1 - 1.58306878306878*G0_22_0_1_23_1_1 - 0.0677248677248674*G0_22_0_1_24_0_1 - 0.364021164021165*G0_22_0_1_24_1_1 + 0.0507936507936525*G0_22_0_1_25_0_1 - 0.146031746031743*G0_22_0_1_25_1_1 - 0.0677248677248697*G0_22_0_1_26_0_1 + 0.0423280423280434*G0_22_0_1_26_1_1 + 0.406349206349206*G0_22_0_1_27_0_1 + 1.57460317460317*G0_22_0_1_27_1_1 + 0.40634920634921*G0_22_0_1_28_0_1 - 0.0507936507936531*G0_22_0_1_28_1_1 - 2.84444444444444*G0_22_0_1_29_0_1 - 1.67619047619047*G0_22_0_1_29_1_1 + 0.467724867724868*G0_22_1_1_0_0_0 + 0.467724867724868*G0_22_1_1_0_1_0 + 0.467724867724867*G0_22_1_1_2_1_0 - 0.0423280423280443*G0_22_1_1_3_0_0 + 0.196825396825397*G0_22_1_1_4_0_0 + 0.0507936507936494*G0_22_1_1_4_1_0 + 0.465608465608465*G0_22_1_1_5_0_0 + 0.101587301587302*G0_22_1_1_5_1_0 - 1.48148148148148*G0_22_1_1_6_0_0 - 1.94708994708995*G0_22_1_1_6_1_0 + 1.47936507936508*G0_22_1_1_7_0_0 + 2.95873015873016*G0_22_1_1_7_1_0 - 0.465608465608465*G0_22_1_1_8_0_0 - 1.94708994708995*G0_22_1_1_8_1_0 - 0.364021164021164*G0_22_1_1_9_0_0 + 0.101587301587302*G0_22_1_1_9_1_0 - 0.146031746031745*G0_22_1_1_10_0_0 + 0.0507936507936527*G0_22_1_1_10_1_0 + 0.0423280423280407*G0_22_1_1_11_0_0 + 1.57460317460317*G0_22_1_1_12_0_0 - 0.101587301587306*G0_22_1_1_12_1_0 - 0.0507936507936475*G0_22_1_1_13_0_0 - 0.101587301587303*G0_22_1_1_13_1_0 - 1.67619047619047*G0_22_1_1_14_0_0 - 0.101587301587297*G0_22_1_1_14_1_0 + 0.467724867724868*G0_22_1_1_15_0_1 + 0.467724867724868*G0_22_1_1_15_1_1 + 0.467724867724867*G0_22_1_1_17_1_1 - 0.0423280423280443*G0_22_1_1_18_0_1 + 0.196825396825397*G0_22_1_1_19_0_1 + 0.0507936507936494*G0_22_1_1_19_1_1 + 0.465608465608465*G0_22_1_1_20_0_1 + 0.101587301587302*G0_22_1_1_20_1_1 - 1.48148148148148*G0_22_1_1_21_0_1 - 1.94708994708995*G0_22_1_1_21_1_1 + 1.47936507936508*G0_22_1_1_22_0_1 + 2.95873015873016*G0_22_1_1_22_1_1 - 0.465608465608465*G0_22_1_1_23_0_1 - 1.94708994708995*G0_22_1_1_23_1_1 - 0.364021164021164*G0_22_1_1_24_0_1 + 0.101587301587302*G0_22_1_1_24_1_1 - 0.146031746031745*G0_22_1_1_25_0_1 + 0.0507936507936527*G0_22_1_1_25_1_1 + 0.0423280423280407*G0_22_1_1_26_0_1 + 1.57460317460317*G0_22_1_1_27_0_1 - 0.101587301587306*G0_22_1_1_27_1_1 - 0.0507936507936475*G0_22_1_1_28_0_1 - 0.101587301587303*G0_22_1_1_28_1_1 - 1.67619047619047*G0_22_1_1_29_0_1 - 0.101587301587297*G0_22_1_1_29_1_1 - 0.042328042328042*G0_23_0_1_0_0_0 - 0.0423280423280422*G0_23_0_1_0_1_0 + 0.0423280423280424*G0_23_0_1_1_0_0 - 0.77883597883598*G0_23_0_1_2_1_0 - 0.338624338624337*G0_23_0_1_3_0_0 - 0.101587301587301*G0_23_0_1_3_1_0 + 0.761904761904761*G0_23_0_1_4_0_0 + 0.296296296296298*G0_23_0_1_4_1_0 - 2.0994708994709*G0_23_0_1_5_0_0 - 1.04973544973545*G0_23_0_1_5_1_0 + 0.338624338624339*G0_23_0_1_6_0_0 + 0.237037037037037*G0_23_0_1_6_1_0 - 0.761904761904763*G0_23_0_1_7_0_0 - 0.465608465608465*G0_23_0_1_7_1_0 + 2.0994708994709*G0_23_0_1_8_0_0 + 1.04973544973545*G0_23_0_1_8_1_0 + 0.101587301587302*G0_23_0_1_9_1_0 + 0.0423280423280414*G0_23_0_1_10_1_0 + 0.101587301587301*G0_23_0_1_11_1_0 - 0.338624338624339*G0_23_0_1_12_1_0 - 0.338624338624338*G0_23_0_1_13_1_0 + 1.28677248677249*G0_23_0_1_14_1_0 - 0.042328042328042*G0_23_0_1_15_0_1 - 0.0423280423280422*G0_23_0_1_15_1_1 + 0.0423280423280424*G0_23_0_1_16_0_1 - 0.77883597883598*G0_23_0_1_17_1_1 - 0.338624338624337*G0_23_0_1_18_0_1 - 0.101587301587301*G0_23_0_1_18_1_1 + 0.761904761904761*G0_23_0_1_19_0_1 + 0.296296296296298*G0_23_0_1_19_1_1 - 2.0994708994709*G0_23_0_1_20_0_1 - 1.04973544973545*G0_23_0_1_20_1_1 + 0.338624338624339*G0_23_0_1_21_0_1 + 0.237037037037037*G0_23_0_1_21_1_1 - 0.761904761904763*G0_23_0_1_22_0_1 - 0.465608465608465*G0_23_0_1_22_1_1 + 2.0994708994709*G0_23_0_1_23_0_1 + 1.04973544973545*G0_23_0_1_23_1_1 + 0.101587301587302*G0_23_0_1_24_1_1 + 0.0423280423280414*G0_23_0_1_25_1_1 + 0.101587301587301*G0_23_0_1_26_1_1 - 0.338624338624339*G0_23_0_1_27_1_1 - 0.338624338624338*G0_23_0_1_28_1_1 + 1.28677248677249*G0_23_0_1_29_1_1 - 0.245502645502646*G0_23_1_1_0_0_0 - 0.245502645502646*G0_23_1_1_0_1_0 - 0.651851851851852*G0_23_1_1_2_1_0 - 0.101587301587299*G0_23_1_1_3_0_0 + 0.296296296296296*G0_23_1_1_4_0_0 - 0.0677248677248661*G0_23_1_1_4_1_0 - 1.04973544973545*G0_23_1_1_5_0_0 - 0.135449735449737*G0_23_1_1_5_1_0 + 0.778835978835979*G0_23_1_1_6_0_0 + 1.01587301587302*G0_23_1_1_6_1_0 - 1.58306878306878*G0_23_1_1_7_0_0 - 1.94708994708995*G0_23_1_1_7_1_0 + 1.04973544973545*G0_23_1_1_8_0_0 + 1.82857142857143*G0_23_1_1_8_1_0 + 0.101587301587302*G0_23_1_1_9_0_0 - 0.135449735449736*G0_23_1_1_9_1_0 + 0.0423280423280414*G0_23_1_1_10_0_0 - 0.0677248677248693*G0_23_1_1_10_1_0 + 0.101587301587303*G0_23_1_1_11_0_0 - 0.338624338624338*G0_23_1_1_12_0_0 + 0.135449735449738*G0_23_1_1_12_1_0 - 0.338624338624342*G0_23_1_1_13_0_0 + 0.135449735449736*G0_23_1_1_13_1_0 + 1.28677248677248*G0_23_1_1_14_0_0 + 0.135449735449733*G0_23_1_1_14_1_0 - 0.245502645502646*G0_23_1_1_15_0_1 - 0.245502645502646*G0_23_1_1_15_1_1 - 0.651851851851852*G0_23_1_1_17_1_1 - 0.101587301587299*G0_23_1_1_18_0_1 + 0.296296296296296*G0_23_1_1_19_0_1 - 0.0677248677248661*G0_23_1_1_19_1_1 - 1.04973544973545*G0_23_1_1_20_0_1 - 0.135449735449737*G0_23_1_1_20_1_1 + 0.778835978835979*G0_23_1_1_21_0_1 + 1.01587301587302*G0_23_1_1_21_1_1 - 1.58306878306878*G0_23_1_1_22_0_1 - 1.94708994708995*G0_23_1_1_22_1_1 + 1.04973544973545*G0_23_1_1_23_0_1 + 1.82857142857143*G0_23_1_1_23_1_1 + 0.101587301587302*G0_23_1_1_24_0_1 - 0.135449735449736*G0_23_1_1_24_1_1 + 0.0423280423280414*G0_23_1_1_25_0_1 - 0.0677248677248693*G0_23_1_1_25_1_1 + 0.101587301587303*G0_23_1_1_26_0_1 - 0.338624338624338*G0_23_1_1_27_0_1 + 0.135449735449738*G0_23_1_1_27_1_1 - 0.338624338624342*G0_23_1_1_28_0_1 + 0.135449735449736*G0_23_1_1_28_1_1 + 1.28677248677248*G0_23_1_1_29_0_1 + 0.135449735449733*G0_23_1_1_29_1_1 - 0.651851851851852*G0_24_0_1_0_0_0 - 0.651851851851852*G0_24_0_1_0_1_0 - 0.245502645502645*G0_24_0_1_1_0_0 - 0.135449735449733*G0_24_0_1_3_0_0 - 0.237037037037036*G0_24_0_1_3_1_0 - 0.0677248677248682*G0_24_0_1_4_0_0 - 0.110052910052909*G0_24_0_1_4_1_0 - 0.101587301587301*G0_24_0_1_5_1_0 - 0.135449735449736*G0_24_0_1_6_0_0 + 0.914285714285714*G0_24_0_1_6_1_0 - 0.0677248677248674*G0_24_0_1_7_0_0 - 0.364021164021164*G0_24_0_1_7_1_0 + 0.101587301587302*G0_24_0_1_8_1_0 + 1.82857142857143*G0_24_0_1_9_0_0 + 0.778835978835977*G0_24_0_1_9_1_0 - 1.94708994708995*G0_24_0_1_10_0_0 - 0.364021164021163*G0_24_0_1_10_1_0 + 1.01587301587301*G0_24_0_1_11_0_0 + 0.237037037037036*G0_24_0_1_11_1_0 + 0.135449735449737*G0_24_0_1_12_0_0 - 1.15132275132275*G0_24_0_1_12_1_0 + 0.135449735449732*G0_24_0_1_13_0_0 + 0.474074074074072*G0_24_0_1_13_1_0 + 0.135449735449736*G0_24_0_1_14_0_0 + 0.474074074074072*G0_24_0_1_14_1_0 - 0.651851851851852*G0_24_0_1_15_0_1 - 0.651851851851852*G0_24_0_1_15_1_1 - 0.245502645502645*G0_24_0_1_16_0_1 - 0.135449735449733*G0_24_0_1_18_0_1 - 0.237037037037036*G0_24_0_1_18_1_1 - 0.0677248677248682*G0_24_0_1_19_0_1 - 0.110052910052909*G0_24_0_1_19_1_1 - 0.101587301587301*G0_24_0_1_20_1_1 - 0.135449735449736*G0_24_0_1_21_0_1 + 0.914285714285714*G0_24_0_1_21_1_1 - 0.0677248677248674*G0_24_0_1_22_0_1 - 0.364021164021164*G0_24_0_1_22_1_1 + 0.101587301587302*G0_24_0_1_23_1_1 + 1.82857142857143*G0_24_0_1_24_0_1 + 0.778835978835977*G0_24_0_1_24_1_1 - 1.94708994708995*G0_24_0_1_25_0_1 - 0.364021164021163*G0_24_0_1_25_1_1 + 1.01587301587301*G0_24_0_1_26_0_1 + 0.237037037037036*G0_24_0_1_26_1_1 + 0.135449735449737*G0_24_0_1_27_0_1 - 1.15132275132275*G0_24_0_1_27_1_1 + 0.135449735449732*G0_24_0_1_28_0_1 + 0.474074074074072*G0_24_0_1_28_1_1 + 0.135449735449736*G0_24_0_1_29_0_1 + 0.474074074074072*G0_24_0_1_29_1_1 + 0.126984126984127*G0_24_1_1_0_0_0 + 0.126984126984127*G0_24_1_1_0_1_0 - 0.203174603174603*G0_24_1_1_1_0_0 + 0.0423280423280423*G0_24_1_1_2_1_0 - 0.237037037037035*G0_24_1_1_3_0_0 - 0.338624338624337*G0_24_1_1_3_1_0 - 0.110052910052911*G0_24_1_1_4_0_0 - 0.152380952380951*G0_24_1_1_4_1_0 - 0.101587301587302*G0_24_1_1_5_0_0 - 0.203174603174604*G0_24_1_1_5_1_0 + 0.914285714285713*G0_24_1_1_6_0_0 - 0.135449735449736*G0_24_1_1_6_1_0 - 0.364021164021165*G0_24_1_1_7_0_0 + 0.101587301587302*G0_24_1_1_7_1_0 + 0.101587301587302*G0_24_1_1_8_0_0 - 0.135449735449736*G0_24_1_1_8_1_0 + 0.778835978835977*G0_24_1_1_9_0_0 + 1.82857142857143*G0_24_1_1_9_1_0 - 1.48148148148148*G0_24_1_1_10_0_0 - 0.660317460317458*G0_24_1_1_10_1_0 + 0.778835978835978*G0_24_1_1_11_0_0 + 0.338624338624337*G0_24_1_1_11_1_0 - 1.15132275132275*G0_24_1_1_12_0_0 - 2.43809523809524*G0_24_1_1_12_1_0 + 0.47407407407407*G0_24_1_1_13_0_0 + 0.812698412698409*G0_24_1_1_13_1_0 + 0.474074074074076*G0_24_1_1_14_0_0 + 0.812698412698413*G0_24_1_1_14_1_0 + 0.126984126984127*G0_24_1_1_15_0_1 + 0.126984126984127*G0_24_1_1_15_1_1 - 0.203174603174603*G0_24_1_1_16_0_1 + 0.0423280423280423*G0_24_1_1_17_1_1 - 0.237037037037035*G0_24_1_1_18_0_1 - 0.338624338624337*G0_24_1_1_18_1_1 - 0.110052910052911*G0_24_1_1_19_0_1 - 0.152380952380951*G0_24_1_1_19_1_1 - 0.101587301587302*G0_24_1_1_20_0_1 - 0.203174603174604*G0_24_1_1_20_1_1 + 0.914285714285713*G0_24_1_1_21_0_1 - 0.135449735449736*G0_24_1_1_21_1_1 - 0.364021164021165*G0_24_1_1_22_0_1 + 0.101587301587302*G0_24_1_1_22_1_1 + 0.101587301587302*G0_24_1_1_23_0_1 - 0.135449735449736*G0_24_1_1_23_1_1 + 0.778835978835977*G0_24_1_1_24_0_1 + 1.82857142857143*G0_24_1_1_24_1_1 - 1.48148148148148*G0_24_1_1_25_0_1 - 0.660317460317458*G0_24_1_1_25_1_1 + 0.778835978835978*G0_24_1_1_26_0_1 + 0.338624338624337*G0_24_1_1_26_1_1 - 1.15132275132275*G0_24_1_1_27_0_1 - 2.43809523809524*G0_24_1_1_27_1_1 + 0.47407407407407*G0_24_1_1_28_0_1 + 0.812698412698409*G0_24_1_1_28_1_1 + 0.474074074074076*G0_24_1_1_29_0_1 + 0.812698412698413*G0_24_1_1_29_1_1 + 0.467724867724868*G0_25_0_1_0_0_0 + 0.467724867724868*G0_25_0_1_0_1_0 + 0.467724867724867*G0_25_0_1_1_0_0 + 0.1015873015873*G0_25_0_1_3_0_0 + 0.465608465608464*G0_25_0_1_3_1_0 + 0.050793650793651*G0_25_0_1_4_0_0 + 0.196825396825396*G0_25_0_1_4_1_0 - 0.0423280423280416*G0_25_0_1_5_1_0 + 0.101587301587302*G0_25_0_1_6_0_0 - 0.364021164021165*G0_25_0_1_6_1_0 + 0.0507936507936526*G0_25_0_1_7_0_0 - 0.146031746031745*G0_25_0_1_7_1_0 + 0.0423280423280414*G0_25_0_1_8_1_0 - 1.94708994708995*G0_25_0_1_9_0_0 - 1.48148148148148*G0_25_0_1_9_1_0 + 2.95873015873015*G0_25_0_1_10_0_0 + 1.47936507936508*G0_25_0_1_10_1_0 - 1.94708994708994*G0_25_0_1_11_0_0 - 0.465608465608464*G0_25_0_1_11_1_0 - 0.101587301587304*G0_25_0_1_12_0_0 + 1.57460317460317*G0_25_0_1_12_1_0 - 0.101587301587298*G0_25_0_1_13_0_0 - 1.67619047619047*G0_25_0_1_13_1_0 - 0.101587301587304*G0_25_0_1_14_0_0 - 0.0507936507936524*G0_25_0_1_14_1_0 + 0.467724867724868*G0_25_0_1_15_0_1 + 0.467724867724868*G0_25_0_1_15_1_1 + 0.467724867724867*G0_25_0_1_16_0_1 + 0.1015873015873*G0_25_0_1_18_0_1 + 0.465608465608464*G0_25_0_1_18_1_1 + 0.050793650793651*G0_25_0_1_19_0_1 + 0.196825396825396*G0_25_0_1_19_1_1 - 0.0423280423280416*G0_25_0_1_20_1_1 + 0.101587301587302*G0_25_0_1_21_0_1 - 0.364021164021165*G0_25_0_1_21_1_1 + 0.0507936507936526*G0_25_0_1_22_0_1 - 0.146031746031745*G0_25_0_1_22_1_1 + 0.0423280423280414*G0_25_0_1_23_1_1 - 1.94708994708995*G0_25_0_1_24_0_1 - 1.48148148148148*G0_25_0_1_24_1_1 + 2.95873015873015*G0_25_0_1_25_0_1 + 1.47936507936508*G0_25_0_1_25_1_1 - 1.94708994708994*G0_25_0_1_26_0_1 - 0.465608465608464*G0_25_0_1_26_1_1 - 0.101587301587304*G0_25_0_1_27_0_1 + 1.57460317460317*G0_25_0_1_27_1_1 - 0.101587301587298*G0_25_0_1_28_0_1 - 1.67619047619047*G0_25_0_1_28_1_1 - 0.101587301587304*G0_25_0_1_29_0_1 - 0.0507936507936524*G0_25_0_1_29_1_1 + 0.0423280423280424*G0_25_1_1_0_0_0 + 0.0423280423280427*G0_25_1_1_0_1_0 + 0.425396825396824*G0_25_1_1_1_0_0 + 0.042328042328043*G0_25_1_1_2_1_0 + 0.465608465608465*G0_25_1_1_3_0_0 + 0.761904761904759*G0_25_1_1_3_1_0 + 0.196825396825396*G0_25_1_1_4_0_0 + 0.393650793650794*G0_25_1_1_4_1_0 - 0.0423280423280415*G0_25_1_1_5_0_0 - 0.152380952380953*G0_25_1_1_5_1_0 - 0.364021164021164*G0_25_1_1_6_0_0 - 0.0677248677248697*G0_25_1_1_6_1_0 - 0.146031746031743*G0_25_1_1_7_0_0 + 0.0507936507936526*G0_25_1_1_7_1_0 + 0.0423280423280414*G0_25_1_1_8_0_0 - 0.0677248677248692*G0_25_1_1_8_1_0 - 0.364021164021163*G0_25_1_1_9_0_0 - 0.660317460317458*G0_25_1_1_9_1_0 + 1.47936507936508*G0_25_1_1_10_0_0 + 2.45079365079365*G0_25_1_1_10_1_0 - 1.58306878306878*G0_25_1_1_11_0_0 - 0.761904761904759*G0_25_1_1_11_1_0 + 1.57460317460317*G0_25_1_1_12_0_0 + 0.406349206349205*G0_25_1_1_12_1_0 - 1.67619047619047*G0_25_1_1_13_0_0 - 2.84444444444444*G0_25_1_1_13_1_0 - 0.0507936507936526*G0_25_1_1_14_0_0 + 0.406349206349206*G0_25_1_1_14_1_0 + 0.0423280423280424*G0_25_1_1_15_0_1 + 0.0423280423280427*G0_25_1_1_15_1_1 + 0.425396825396824*G0_25_1_1_16_0_1 + 0.042328042328043*G0_25_1_1_17_1_1 + 0.465608465608465*G0_25_1_1_18_0_1 + 0.761904761904759*G0_25_1_1_18_1_1 + 0.196825396825396*G0_25_1_1_19_0_1 + 0.393650793650794*G0_25_1_1_19_1_1 - 0.0423280423280415*G0_25_1_1_20_0_1 - 0.152380952380953*G0_25_1_1_20_1_1 - 0.364021164021164*G0_25_1_1_21_0_1 - 0.0677248677248697*G0_25_1_1_21_1_1 - 0.146031746031743*G0_25_1_1_22_0_1 + 0.0507936507936526*G0_25_1_1_22_1_1 + 0.0423280423280414*G0_25_1_1_23_0_1 - 0.0677248677248692*G0_25_1_1_23_1_1 - 0.364021164021163*G0_25_1_1_24_0_1 - 0.660317460317458*G0_25_1_1_24_1_1 + 1.47936507936508*G0_25_1_1_25_0_1 + 2.45079365079365*G0_25_1_1_25_1_1 - 1.58306878306878*G0_25_1_1_26_0_1 - 0.761904761904759*G0_25_1_1_26_1_1 + 1.57460317460317*G0_25_1_1_27_0_1 + 0.406349206349205*G0_25_1_1_27_1_1 - 1.67619047619047*G0_25_1_1_28_0_1 - 2.84444444444444*G0_25_1_1_28_1_1 - 0.0507936507936526*G0_25_1_1_29_0_1 + 0.406349206349206*G0_25_1_1_29_1_1 - 0.245502645502646*G0_26_0_1_0_0_0 - 0.245502645502646*G0_26_0_1_0_1_0 - 0.651851851851851*G0_26_0_1_1_0_0 - 0.135449735449735*G0_26_0_1_3_0_0 - 1.04973544973545*G0_26_0_1_3_1_0 - 0.0677248677248674*G0_26_0_1_4_0_0 + 0.296296296296297*G0_26_0_1_4_1_0 - 0.101587301587302*G0_26_0_1_5_1_0 - 0.135449735449735*G0_26_0_1_6_0_0 + 0.101587301587303*G0_26_0_1_6_1_0 - 0.0677248677248697*G0_26_0_1_7_0_0 + 0.0423280423280407*G0_26_0_1_7_1_0 + 0.101587301587303*G0_26_0_1_8_1_0 + 1.01587301587301*G0_26_0_1_9_0_0 + 0.778835978835978*G0_26_0_1_9_1_0 - 1.94708994708994*G0_26_0_1_10_0_0 - 1.58306878306878*G0_26_0_1_10_1_0 + 1.82857142857143*G0_26_0_1_11_0_0 + 1.04973544973545*G0_26_0_1_11_1_0 + 0.135449735449736*G0_26_0_1_12_0_0 - 0.338624338624339*G0_26_0_1_12_1_0 + 0.135449735449734*G0_26_0_1_13_0_0 + 1.28677248677248*G0_26_0_1_13_1_0 + 0.135449735449737*G0_26_0_1_14_0_0 - 0.338624338624336*G0_26_0_1_14_1_0 - 0.245502645502646*G0_26_0_1_15_0_1 - 0.245502645502646*G0_26_0_1_15_1_1 - 0.651851851851851*G0_26_0_1_16_0_1 - 0.135449735449735*G0_26_0_1_18_0_1 - 1.04973544973545*G0_26_0_1_18_1_1 - 0.0677248677248674*G0_26_0_1_19_0_1 + 0.296296296296297*G0_26_0_1_19_1_1 - 0.101587301587302*G0_26_0_1_20_1_1 - 0.135449735449735*G0_26_0_1_21_0_1 + 0.101587301587303*G0_26_0_1_21_1_1 - 0.0677248677248697*G0_26_0_1_22_0_1 + 0.0423280423280407*G0_26_0_1_22_1_1 + 0.101587301587303*G0_26_0_1_23_1_1 + 1.01587301587301*G0_26_0_1_24_0_1 + 0.778835978835978*G0_26_0_1_24_1_1 - 1.94708994708994*G0_26_0_1_25_0_1 - 1.58306878306878*G0_26_0_1_25_1_1 + 1.82857142857143*G0_26_0_1_26_0_1 + 1.04973544973545*G0_26_0_1_26_1_1 + 0.135449735449736*G0_26_0_1_27_0_1 - 0.338624338624339*G0_26_0_1_27_1_1 + 0.135449735449734*G0_26_0_1_28_0_1 + 1.28677248677248*G0_26_0_1_28_1_1 + 0.135449735449737*G0_26_0_1_29_0_1 - 0.338624338624336*G0_26_0_1_29_1_1 - 0.0423280423280419*G0_26_1_1_0_0_0 - 0.0423280423280425*G0_26_1_1_0_1_0 - 0.778835978835978*G0_26_1_1_1_0_0 + 0.0423280423280428*G0_26_1_1_2_1_0 - 1.04973544973545*G0_26_1_1_3_0_0 - 2.0994708994709*G0_26_1_1_3_1_0 + 0.296296296296296*G0_26_1_1_4_0_0 + 0.761904761904763*G0_26_1_1_4_1_0 - 0.101587301587301*G0_26_1_1_5_0_0 - 0.338624338624338*G0_26_1_1_5_1_0 + 0.101587301587301*G0_26_1_1_6_0_0 + 0.0423280423280434*G0_26_1_1_7_0_0 + 0.101587301587301*G0_26_1_1_8_0_0 + 0.237037037037036*G0_26_1_1_9_0_0 + 0.338624338624337*G0_26_1_1_9_1_0 - 0.465608465608464*G0_26_1_1_10_0_0 - 0.761904761904759*G0_26_1_1_10_1_0 + 1.04973544973545*G0_26_1_1_11_0_0 + 2.0994708994709*G0_26_1_1_11_1_0 - 0.338624338624338*G0_26_1_1_12_0_0 + 1.28677248677249*G0_26_1_1_13_0_0 - 0.338624338624339*G0_26_1_1_14_0_0 - 0.0423280423280419*G0_26_1_1_15_0_1 - 0.0423280423280425*G0_26_1_1_15_1_1 - 0.778835978835978*G0_26_1_1_16_0_1 + 0.0423280423280428*G0_26_1_1_17_1_1 - 1.04973544973545*G0_26_1_1_18_0_1 - 2.0994708994709*G0_26_1_1_18_1_1 + 0.296296296296296*G0_26_1_1_19_0_1 + 0.761904761904763*G0_26_1_1_19_1_1 - 0.101587301587301*G0_26_1_1_20_0_1 - 0.338624338624338*G0_26_1_1_20_1_1 + 0.101587301587301*G0_26_1_1_21_0_1 + 0.0423280423280434*G0_26_1_1_22_0_1 + 0.101587301587301*G0_26_1_1_23_0_1 + 0.237037037037036*G0_26_1_1_24_0_1 + 0.338624338624337*G0_26_1_1_24_1_1 - 0.465608465608464*G0_26_1_1_25_0_1 - 0.761904761904759*G0_26_1_1_25_1_1 + 1.04973544973545*G0_26_1_1_26_0_1 + 2.0994708994709*G0_26_1_1_26_1_1 - 0.338624338624338*G0_26_1_1_27_0_1 + 1.28677248677249*G0_26_1_1_28_0_1 - 0.338624338624339*G0_26_1_1_29_0_1 - 0.0846560846560848*G0_27_0_1_0_0_0 - 0.0846560846560841*G0_27_0_1_0_1_0 - 0.084656084656084*G0_27_0_1_1_0_0 + 0.812698412698412*G0_27_0_1_3_0_0 + 0.338624338624338*G0_27_0_1_3_1_0 + 0.406349206349207*G0_27_0_1_4_0_0 + 0.457142857142857*G0_27_0_1_4_1_0 + 0.338624338624338*G0_27_0_1_5_1_0 - 2.43809523809524*G0_27_0_1_6_0_0 - 1.15132275132275*G0_27_0_1_6_1_0 + 0.406349206349206*G0_27_0_1_7_0_0 + 1.57460317460317*G0_27_0_1_7_1_0 - 0.338624338624338*G0_27_0_1_8_1_0 + 0.135449735449737*G0_27_0_1_9_0_0 - 1.15132275132275*G0_27_0_1_9_1_0 - 0.101587301587304*G0_27_0_1_10_0_0 + 1.57460317460317*G0_27_0_1_10_1_0 + 0.135449735449736*G0_27_0_1_11_0_0 - 0.338624338624338*G0_27_0_1_11_1_0 + 5.68888888888888*G0_27_0_1_12_0_0 + 2.84444444444444*G0_27_0_1_12_1_0 - 4.06349206349206*G0_27_0_1_13_0_0 - 2.03174603174603*G0_27_0_1_13_1_0 - 0.812698412698412*G0_27_0_1_14_0_0 - 2.03174603174602*G0_27_0_1_14_1_0 - 0.0846560846560848*G0_27_0_1_15_0_1 - 0.0846560846560841*G0_27_0_1_15_1_1 - 0.084656084656084*G0_27_0_1_16_0_1 + 0.812698412698412*G0_27_0_1_18_0_1 + 0.338624338624338*G0_27_0_1_18_1_1 + 0.406349206349207*G0_27_0_1_19_0_1 + 0.457142857142857*G0_27_0_1_19_1_1 + 0.338624338624338*G0_27_0_1_20_1_1 - 2.43809523809524*G0_27_0_1_21_0_1 - 1.15132275132275*G0_27_0_1_21_1_1 + 0.406349206349206*G0_27_0_1_22_0_1 + 1.57460317460317*G0_27_0_1_22_1_1 - 0.338624338624338*G0_27_0_1_23_1_1 + 0.135449735449737*G0_27_0_1_24_0_1 - 1.15132275132275*G0_27_0_1_24_1_1 - 0.101587301587304*G0_27_0_1_25_0_1 + 1.57460317460317*G0_27_0_1_25_1_1 + 0.135449735449736*G0_27_0_1_26_0_1 - 0.338624338624338*G0_27_0_1_26_1_1 + 5.68888888888888*G0_27_0_1_27_0_1 + 2.84444444444444*G0_27_0_1_27_1_1 - 4.06349206349206*G0_27_0_1_28_0_1 - 2.03174603174603*G0_27_0_1_28_1_1 - 0.812698412698412*G0_27_0_1_29_0_1 - 2.03174603174602*G0_27_0_1_29_1_1 - 0.084656084656085*G0_27_1_1_0_0_0 - 0.0846560846560851*G0_27_1_1_0_1_0 - 0.084656084656085*G0_27_1_1_2_1_0 + 0.338624338624334*G0_27_1_1_3_0_0 + 0.457142857142862*G0_27_1_1_4_0_0 + 0.406349206349205*G0_27_1_1_4_1_0 + 0.338624338624339*G0_27_1_1_5_0_0 + 0.812698412698416*G0_27_1_1_5_1_0 - 1.15132275132275*G0_27_1_1_6_0_0 + 0.135449735449738*G0_27_1_1_6_1_0 + 1.57460317460317*G0_27_1_1_7_0_0 - 0.101587301587305*G0_27_1_1_7_1_0 - 0.338624338624339*G0_27_1_1_8_0_0 + 0.135449735449738*G0_27_1_1_8_1_0 - 1.15132275132275*G0_27_1_1_9_0_0 - 2.43809523809524*G0_27_1_1_9_1_0 + 1.57460317460317*G0_27_1_1_10_0_0 + 0.406349206349205*G0_27_1_1_10_1_0 - 0.33862433862434*G0_27_1_1_11_0_0 + 2.84444444444443*G0_27_1_1_12_0_0 + 5.68888888888888*G0_27_1_1_12_1_0 - 2.03174603174602*G0_27_1_1_13_0_0 - 0.812698412698409*G0_27_1_1_13_1_0 - 2.03174603174603*G0_27_1_1_14_0_0 - 4.06349206349206*G0_27_1_1_14_1_0 - 0.084656084656085*G0_27_1_1_15_0_1 - 0.0846560846560851*G0_27_1_1_15_1_1 - 0.084656084656085*G0_27_1_1_17_1_1 + 0.338624338624334*G0_27_1_1_18_0_1 + 0.457142857142862*G0_27_1_1_19_0_1 + 0.406349206349205*G0_27_1_1_19_1_1 + 0.338624338624339*G0_27_1_1_20_0_1 + 0.812698412698416*G0_27_1_1_20_1_1 - 1.15132275132275*G0_27_1_1_21_0_1 + 0.135449735449738*G0_27_1_1_21_1_1 + 1.57460317460317*G0_27_1_1_22_0_1 - 0.101587301587305*G0_27_1_1_22_1_1 - 0.338624338624339*G0_27_1_1_23_0_1 + 0.135449735449738*G0_27_1_1_23_1_1 - 1.15132275132275*G0_27_1_1_24_0_1 - 2.43809523809524*G0_27_1_1_24_1_1 + 1.57460317460317*G0_27_1_1_25_0_1 + 0.406349206349205*G0_27_1_1_25_1_1 - 0.33862433862434*G0_27_1_1_26_0_1 + 2.84444444444443*G0_27_1_1_27_0_1 + 5.68888888888888*G0_27_1_1_27_1_1 - 2.03174603174602*G0_27_1_1_28_0_1 - 0.812698412698409*G0_27_1_1_28_1_1 - 2.03174603174603*G0_27_1_1_29_0_1 - 4.06349206349206*G0_27_1_1_29_1_1 - 0.0846560846560835*G0_28_0_1_0_0_0 - 0.084656084656084*G0_28_0_1_0_1_0 - 0.0846560846560853*G0_28_0_1_1_0_0 - 2.43809523809524*G0_28_0_1_3_0_0 - 1.28677248677249*G0_28_0_1_3_1_0 + 0.406349206349207*G0_28_0_1_4_0_0 - 1.16825396825396*G0_28_0_1_4_1_0 + 0.33862433862434*G0_28_0_1_5_1_0 + 0.812698412698413*G0_28_0_1_6_0_0 + 0.474074074074073*G0_28_0_1_6_1_0 + 0.40634920634921*G0_28_0_1_7_0_0 - 0.0507936507936475*G0_28_0_1_7_1_0 - 0.338624338624342*G0_28_0_1_8_1_0 + 0.135449735449732*G0_28_0_1_9_0_0 + 0.47407407407407*G0_28_0_1_9_1_0 - 0.101587301587298*G0_28_0_1_10_0_0 - 1.67619047619047*G0_28_0_1_10_1_0 + 0.135449735449734*G0_28_0_1_11_0_0 + 1.28677248677249*G0_28_0_1_11_1_0 - 4.06349206349206*G0_28_0_1_12_0_0 - 2.03174603174602*G0_28_0_1_12_1_0 + 5.68888888888888*G0_28_0_1_13_0_0 + 2.84444444444444*G0_28_0_1_13_1_0 - 0.812698412698418*G0_28_0_1_14_0_0 + 1.21904761904761*G0_28_0_1_14_1_0 - 0.0846560846560835*G0_28_0_1_15_0_1 - 0.084656084656084*G0_28_0_1_15_1_1 - 0.0846560846560853*G0_28_0_1_16_0_1 - 2.43809523809524*G0_28_0_1_18_0_1 - 1.28677248677249*G0_28_0_1_18_1_1 + 0.406349206349207*G0_28_0_1_19_0_1 - 1.16825396825396*G0_28_0_1_19_1_1 + 0.33862433862434*G0_28_0_1_20_1_1 + 0.812698412698413*G0_28_0_1_21_0_1 + 0.474074074074073*G0_28_0_1_21_1_1 + 0.40634920634921*G0_28_0_1_22_0_1 - 0.0507936507936475*G0_28_0_1_22_1_1 - 0.338624338624342*G0_28_0_1_23_1_1 + 0.135449735449732*G0_28_0_1_24_0_1 + 0.47407407407407*G0_28_0_1_24_1_1 - 0.101587301587298*G0_28_0_1_25_0_1 - 1.67619047619047*G0_28_0_1_25_1_1 + 0.135449735449734*G0_28_0_1_26_0_1 + 1.28677248677249*G0_28_0_1_26_1_1 - 4.06349206349206*G0_28_0_1_27_0_1 - 2.03174603174602*G0_28_0_1_27_1_1 + 5.68888888888888*G0_28_0_1_28_0_1 + 2.84444444444444*G0_28_0_1_28_1_1 - 0.812698412698418*G0_28_0_1_29_0_1 + 1.21904761904761*G0_28_0_1_29_1_1 - 0.0846560846560848*G0_28_1_1_0_0_0 - 0.0846560846560849*G0_28_1_1_0_1_0 - 0.0846560846560848*G0_28_1_1_2_1_0 - 1.28677248677248*G0_28_1_1_3_0_0 - 1.16825396825396*G0_28_1_1_4_0_0 - 2.84444444444444*G0_28_1_1_4_1_0 + 0.338624338624338*G0_28_1_1_5_0_0 + 0.812698412698412*G0_28_1_1_5_1_0 + 0.474074074074074*G0_28_1_1_6_0_0 + 0.135449735449738*G0_28_1_1_6_1_0 - 0.0507936507936531*G0_28_1_1_7_0_0 - 0.101587301587303*G0_28_1_1_7_1_0 - 0.338624338624338*G0_28_1_1_8_0_0 + 0.135449735449736*G0_28_1_1_8_1_0 + 0.474074074074072*G0_28_1_1_9_0_0 + 0.812698412698409*G0_28_1_1_9_1_0 - 1.67619047619047*G0_28_1_1_10_0_0 - 2.84444444444444*G0_28_1_1_10_1_0 + 1.28677248677248*G0_28_1_1_11_0_0 - 2.03174603174603*G0_28_1_1_12_0_0 - 0.812698412698409*G0_28_1_1_12_1_0 + 2.84444444444444*G0_28_1_1_13_0_0 + 5.68888888888887*G0_28_1_1_13_1_0 + 1.21904761904762*G0_28_1_1_14_0_0 - 0.812698412698411*G0_28_1_1_14_1_0 - 0.0846560846560848*G0_28_1_1_15_0_1 - 0.0846560846560849*G0_28_1_1_15_1_1 - 0.0846560846560848*G0_28_1_1_17_1_1 - 1.28677248677248*G0_28_1_1_18_0_1 - 1.16825396825396*G0_28_1_1_19_0_1 - 2.84444444444444*G0_28_1_1_19_1_1 + 0.338624338624338*G0_28_1_1_20_0_1 + 0.812698412698412*G0_28_1_1_20_1_1 + 0.474074074074074*G0_28_1_1_21_0_1 + 0.135449735449738*G0_28_1_1_21_1_1 - 0.0507936507936531*G0_28_1_1_22_0_1 - 0.101587301587303*G0_28_1_1_22_1_1 - 0.338624338624338*G0_28_1_1_23_0_1 + 0.135449735449736*G0_28_1_1_23_1_1 + 0.474074074074072*G0_28_1_1_24_0_1 + 0.812698412698409*G0_28_1_1_24_1_1 - 1.67619047619047*G0_28_1_1_25_0_1 - 2.84444444444444*G0_28_1_1_25_1_1 + 1.28677248677248*G0_28_1_1_26_0_1 - 2.03174603174603*G0_28_1_1_27_0_1 - 0.812698412698409*G0_28_1_1_27_1_1 + 2.84444444444444*G0_28_1_1_28_0_1 + 5.68888888888887*G0_28_1_1_28_1_1 + 1.21904761904762*G0_28_1_1_29_0_1 - 0.812698412698411*G0_28_1_1_29_1_1 - 0.0846560846560841*G0_29_0_1_0_0_0 - 0.0846560846560838*G0_29_0_1_0_1_0 - 0.0846560846560845*G0_29_0_1_1_0_0 + 0.812698412698417*G0_29_0_1_3_0_0 + 0.338624338624341*G0_29_0_1_3_1_0 - 2.84444444444444*G0_29_0_1_4_0_0 - 1.16825396825397*G0_29_0_1_4_1_0 - 1.28677248677249*G0_29_0_1_5_1_0 + 0.812698412698412*G0_29_0_1_6_0_0 + 0.474074074074073*G0_29_0_1_6_1_0 - 2.84444444444444*G0_29_0_1_7_0_0 - 1.67619047619047*G0_29_0_1_7_1_0 + 1.28677248677248*G0_29_0_1_8_1_0 + 0.135449735449736*G0_29_0_1_9_0_0 + 0.474074074074076*G0_29_0_1_9_1_0 - 0.101587301587304*G0_29_0_1_10_0_0 - 0.0507936507936526*G0_29_0_1_10_1_0 + 0.135449735449737*G0_29_0_1_11_0_0 - 0.338624338624339*G0_29_0_1_11_1_0 - 0.812698412698412*G0_29_0_1_12_0_0 - 2.03174603174603*G0_29_0_1_12_1_0 - 0.812698412698418*G0_29_0_1_13_0_0 + 1.21904761904762*G0_29_0_1_13_1_0 + 5.68888888888888*G0_29_0_1_14_0_0 + 2.84444444444444*G0_29_0_1_14_1_0 - 0.0846560846560841*G0_29_0_1_15_0_1 - 0.0846560846560838*G0_29_0_1_15_1_1 - 0.0846560846560845*G0_29_0_1_16_0_1 + 0.812698412698417*G0_29_0_1_18_0_1 + 0.338624338624341*G0_29_0_1_18_1_1 - 2.84444444444444*G0_29_0_1_19_0_1 - 1.16825396825397*G0_29_0_1_19_1_1 - 1.28677248677249*G0_29_0_1_20_1_1 + 0.812698412698412*G0_29_0_1_21_0_1 + 0.474074074074073*G0_29_0_1_21_1_1 - 2.84444444444444*G0_29_0_1_22_0_1 - 1.67619047619047*G0_29_0_1_22_1_1 + 1.28677248677248*G0_29_0_1_23_1_1 + 0.135449735449736*G0_29_0_1_24_0_1 + 0.474074074074076*G0_29_0_1_24_1_1 - 0.101587301587304*G0_29_0_1_25_0_1 - 0.0507936507936526*G0_29_0_1_25_1_1 + 0.135449735449737*G0_29_0_1_26_0_1 - 0.338624338624339*G0_29_0_1_26_1_1 - 0.812698412698412*G0_29_0_1_27_0_1 - 2.03174603174603*G0_29_0_1_27_1_1 - 0.812698412698418*G0_29_0_1_28_0_1 + 1.21904761904762*G0_29_0_1_28_1_1 + 5.68888888888888*G0_29_0_1_29_0_1 + 2.84444444444444*G0_29_0_1_29_1_1 - 0.0846560846560835*G0_29_1_1_0_0_0 - 0.0846560846560833*G0_29_1_1_0_1_0 - 0.0846560846560851*G0_29_1_1_2_1_0 + 0.338624338624343*G0_29_1_1_3_0_0 - 1.16825396825397*G0_29_1_1_4_0_0 + 0.406349206349206*G0_29_1_1_4_1_0 - 1.28677248677249*G0_29_1_1_5_0_0 - 2.43809523809524*G0_29_1_1_5_1_0 + 0.474074074074071*G0_29_1_1_6_0_0 + 0.135449735449731*G0_29_1_1_6_1_0 - 1.67619047619047*G0_29_1_1_7_0_0 - 0.101587301587297*G0_29_1_1_7_1_0 + 1.28677248677249*G0_29_1_1_8_0_0 + 0.135449735449733*G0_29_1_1_8_1_0 + 0.474074074074072*G0_29_1_1_9_0_0 + 0.812698412698413*G0_29_1_1_9_1_0 - 0.0507936507936525*G0_29_1_1_10_0_0 + 0.406349206349206*G0_29_1_1_10_1_0 - 0.338624338624336*G0_29_1_1_11_0_0 - 2.03174603174602*G0_29_1_1_12_0_0 - 4.06349206349206*G0_29_1_1_12_1_0 + 1.21904761904761*G0_29_1_1_13_0_0 - 0.812698412698411*G0_29_1_1_13_1_0 + 2.84444444444444*G0_29_1_1_14_0_0 + 5.68888888888889*G0_29_1_1_14_1_0 - 0.0846560846560835*G0_29_1_1_15_0_1 - 0.0846560846560833*G0_29_1_1_15_1_1 - 0.0846560846560851*G0_29_1_1_17_1_1 + 0.338624338624343*G0_29_1_1_18_0_1 - 1.16825396825397*G0_29_1_1_19_0_1 + 0.406349206349206*G0_29_1_1_19_1_1 - 1.28677248677249*G0_29_1_1_20_0_1 - 2.43809523809524*G0_29_1_1_20_1_1 + 0.474074074074071*G0_29_1_1_21_0_1 + 0.135449735449731*G0_29_1_1_21_1_1 - 1.67619047619047*G0_29_1_1_22_0_1 - 0.101587301587297*G0_29_1_1_22_1_1 + 1.28677248677249*G0_29_1_1_23_0_1 + 0.135449735449733*G0_29_1_1_23_1_1 + 0.474074074074072*G0_29_1_1_24_0_1 + 0.812698412698413*G0_29_1_1_24_1_1 - 0.0507936507936525*G0_29_1_1_25_0_1 + 0.406349206349206*G0_29_1_1_25_1_1 - 0.338624338624336*G0_29_1_1_26_0_1 - 2.03174603174602*G0_29_1_1_27_0_1 - 4.06349206349206*G0_29_1_1_27_1_1 + 1.21904761904761*G0_29_1_1_28_0_1 - 0.812698412698411*G0_29_1_1_28_1_1 + 2.84444444444444*G0_29_1_1_29_0_1 + 5.68888888888889*G0_29_1_1_29_1_1;
}

/// Constructor
UFC_P4VecDivL2Functional::UFC_P4VecDivL2Functional() : ufc::form()
{
    // Do nothing
}

/// Destructor
UFC_P4VecDivL2Functional::~UFC_P4VecDivL2Functional()
{
    // Do nothing
}

/// Return a string identifying the form
const char* UFC_P4VecDivL2Functional::signature() const
{
    return "w0_a0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29]w0_a3[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29](dXa1[0, 1]/dxa2[0, 1])(dXa4[0, 1]/dxa5[0, 1]) | ((d/dXa1[0, 1])va0[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29][a2[0, 1]])*((d/dXa4[0, 1])va3[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29][a5[0, 1]])*dX(0)";
}

/// Return the rank of the global tensor (r)
unsigned int UFC_P4VecDivL2Functional::rank() const
{
    return 0;
}

/// Return the number of coefficients (n)
unsigned int UFC_P4VecDivL2Functional::num_coefficients() const
{
    return 1;
}

/// Return the number of cell integrals
unsigned int UFC_P4VecDivL2Functional::num_cell_integrals() const
{
    return 1;
}
  
/// Return the number of exterior facet integrals
unsigned int UFC_P4VecDivL2Functional::num_exterior_facet_integrals() const
{
    return 0;
}
  
/// Return the number of interior facet integrals
unsigned int UFC_P4VecDivL2Functional::num_interior_facet_integrals() const
{
    return 0;
}

/// Create a new finite element for argument function i
ufc::finite_element* UFC_P4VecDivL2Functional::create_finite_element(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_finite_element_0();
}

/// Create a new dof map for argument function i
ufc::dof_map* UFC_P4VecDivL2Functional::create_dof_map(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_dof_map_0();
}

/// Create a new cell integral on sub domain i
ufc::cell_integral* UFC_P4VecDivL2Functional::create_cell_integral(unsigned int i) const
{
    return new UFC_P4VecDivL2Functional_cell_integral_0();
}

/// Create a new exterior facet integral on sub domain i
ufc::exterior_facet_integral* UFC_P4VecDivL2Functional::create_exterior_facet_integral(unsigned int i) const
{
    return 0;
}

/// Create a new interior facet integral on sub domain i
ufc::interior_facet_integral* UFC_P4VecDivL2Functional::create_interior_facet_integral(unsigned int i) const
{
    return 0;
}

